Package: r4pde
Type: Package
Title: Tools for Quantitative Plant Disease Epidemiology
Version: 0.2.2
Authors@R: 
    person(given = "Emerson", family = "Del Ponte", email = "delponte@ufv.br", 
           role = c("aut", "cre"), comment = c(ORCID = "0000-0003-4398-409X"))
Description: Tools for quantitative plant disease epidemiology, including functional analysis
    of disease progress curves, spatial epidemiology, disease quantification,
    and weather-driven epidemic analysis. The package also serves as an educational
    resource and companion to the book "R for Plant Disease Epidemiology" (R4PDE).
    Several functions are based on classical and contemporary methods, including those
    discussed in Laurence V. Madden, Gareth Hughes, and Frank van den Bosch (2007) <doi:10.1094/9780890545058>.
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Imports: boot, car, cluster, dplyr, ggplot2, igraph, lubridate, mgcv,
        magrittr, nasapower, progress, purrr, rlang, stats, survival,
        tidyr, tibble, httr, jsonlite, terra
Suggests: interval, testthat, knitr, rmarkdown, ggdendro, patchwork,
        cowplot, ncdf4, ggrepel
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.3
BugReports: https://github.com/emdelponte/r4pde/issues
URL: https://emdelponte.github.io/r4pde/,
        https://github.com/emdelponte/r4pde
NeedsCompilation: no
Packaged: 2026-09-07 17:21:42 UTC; edelp
Author: Emerson Del Ponte [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4398-409X>)
Maintainer: Emerson Del Ponte <delponte@ufv.br>
Repository: CRAN
Date/Publication: 2026-09-07 17:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,r4pde.oruns_test)
export(AFSD)
export(BPL)
export(CompMuCens)
export(DSI)
export(DSI2)
export(count_subareas)
export(count_subareas_random)
export(fit_gradients)
export(get_nasapower)
export(join_count)
export(oruns_test)
export(oruns_test_boustrophedon)
export(oruns_test_byrowcol)
export(plot_AFSD)
export(theme_r4pde)
export(windowpane)
export(windowpane_tests)
import(tidyr)
importFrom(car,linearHypothesis)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(igraph,components)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(interval,ictest)
importFrom(lubridate,days)
importFrom(nasapower,get_power)
importFrom(progress,progress_bar)
importFrom(purrr,map_dfr)
importFrom(rlang,as_label)
importFrom(rlang,enquo)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor.test)
importFrom(stats,dist)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(survival,Surv)
importFrom(tidyr,pivot_longer)
importFrom(utils,install.packages)

# r4pde <img src="man/figures/logo.png" width="200px" align="right"/>
An R package developed as a companion to the book [R for Plant Disease Epidemiology](https://r4pde.netlify.app/). It provides access to a suite of specialized R functions and datasets tailored for teaching and analyzing plant disease epidemiology. This package supports researchers, students, and practitioners by offering tools for disease quantification, spatial analysis and predictive modeling.

## Installation

The development version of {r4pde} is available from GitHub. To install it along with its dependencies (including Bioconductor packages), use the [`pak`](https://pak.r-lib.org/) package:

``` r
if (!requireNamespace("pak", quietly = TRUE)) install.packages("pak")
pak::pkg_install("emdelponte/r4pde")
```

Alternatively, using remotes:

``` r
if (!requireNamespace("remotes", quietly = TRUE)) install.packages("remotes")
remotes::install_github("emdelponte/r4pde")
```

> Note: The package depends on `Icens`, which is available from Bioconductor. Both `pak` and `remotes` will install it automatically if needed.

## Reporting Issues

Please [report any bugs or issues](https://github.com/emdelponte/r4pde/issues) via the GitHub issue tracker.

## License and Citation

-   This package is released under the MIT license.
-   To cite the package, please use:

``` r
citation("r4pde")
```

# r4pde <img src="man/figures/logo.png" width="200px" align="right"/>

[![CRAN](https://www.r-pkg.org/badges/version/r4pde)](https://CRAN.R-project.org/package=r4pde) [![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/r4pde)](https://CRAN.R-project.org/package=r4pde)\
An R package developed as a companion to the book [R for Plant Disease Epidemiology](https://r4pde.netlify.app/). It provides access to a suite of specialized R functions and datasets tailored for teaching and analyzing plant disease epidemiology. This package supports researchers, students, and practitioners by offering tools for disease quantification, spatial analysis and predictive modeling.

## Installation

Install the stable release from CRAN.

``` r
install.packages("r4pde")
```

The development version of {r4pde} is available from GitHub. To install it along with its dependencies (including Bioconductor packages), use the [`pak`](https://pak.r-lib.org/) package:

``` r
if (!requireNamespace("pak", quietly = TRUE)) install.packages("pak")
pak::pkg_install("emdelponte/r4pde")
```

Alternatively, using remotes:

``` r
if (!requireNamespace("remotes", quietly = TRUE)) install.packages("remotes")
remotes::install_github("emdelponte/r4pde")
```

## Reporting Issues

Please [report any bugs or issues](https://github.com/emdelponte/r4pde/issues) via the GitHub issue tracker.

## License and Citation

-   This package is released under the MIT license.
-   To cite the package, please use:

``` r
citation("r4pde")
```

#' @title Boustrophedon Run Test for Binary Matrix
#' @description Applies the ordinary runs test to a binary matrix using boustrophedon-style traversal.
#' The function supports two modes: row-wise and column-wise boustrophedon. Each traversal flattens the matrix into a 1D sequence
#' which is then tested using `oruns_test`.
#'
#' @param mat A binary matrix (containing 0s and 1s, and possibly NAs).
#' @return A list with two elements:
#' 	\item{rowwise_boustrophedon}{List containing the sequence and result of `oruns_test` for row-wise traversal.}
#' 	\item{colwise_boustrophedon}{List containing the sequence and result of `oruns_test` for column-wise traversal.}
#' @export
#' @seealso \code{\link{oruns_test}}
#' @importFrom stats na.omit
#' @family Spatial analysis

oruns_test_boustrophedon <- function(mat) {
  if (!is.matrix(mat) || !all(mat %in% c(0, 1, NA))) {
    stop("Input must be a binary matrix (0/1), possibly with NAs.")
  }

  # Row-wise boustrophedon
  row_seq <- unlist(lapply(seq_len(nrow(mat)), function(i) {
    row_i <- mat[i, ]
    if (i %% 2 == 0) rev(row_i) else row_i
  }))
  row_seq <- na.omit(row_seq)

  # Column-wise boustrophedon
  col_seq <- unlist(lapply(seq_len(ncol(mat)), function(j) {
    col_j <- mat[, j]
    if (j %% 2 == 0) rev(col_j) else col_j
  }))
  col_seq <- na.omit(col_seq)

  row_run <- oruns_test(row_seq)
  col_run <- oruns_test(col_seq)

  list(
    rowwise_boustrophedon = list(
      sequence = row_seq,
      test = row_run
    ),
    colwise_boustrophedon = list(
      sequence = col_seq,
      test = col_run
    )
  )
}

#' Functional suppression profiles of disease control treatments
#'
#' @description
#' Computes disease suppression trajectories relative to a reference treatment, estimates functional distances among suppression curves, clusters treatments according to their temporal suppression profiles, and summarizes each profile using functional suppression metrics.
#'
#' @details 
#' A Functional Suppression Profile (FSP) is the temporal trajectory of disease 
#' suppression produced by a treatment relative to an untreated or reference control. 
#' Functional distances and hierarchical clustering are computed from smoothed disease 
#' suppression trajectories rather than raw observed suppression values. Functional 
#' summary metrics are subsequently used to interpret the resulting suppression profiles.
#'
#' The workflow includes contrasting treatments against a reference, fitting functional 
#' curves, calculating distances, clustering, and summarizing the suppression using 
#' metrics like protected area, maximum suppression, and persistence.
#' 
#' Treatments belonging to the same functional suppression profile are displayed using 
#' the same colour throughout all graphical summaries. This visual consistency facilitates 
#' interpretation of the relationship between suppression trajectories, functional clustering, 
#' and suppression metrics.
#' 
#' The \code{classification} table combines functional profile membership, suppression metrics, 
#' and mean functional rank. Functional profiles are defined from distances among smoothed 
#' suppression trajectories, whereas mean rank summarizes the overall performance across 
#' functional suppression metrics.
#'
#' @param data A data frame or tibble containing the disease progress data.
#' @param reference Character string specifying the reference treatment name.
#' @param time Character string specifying the time column. Default is \code{"time"}.
#' @param response Character string specifying the response column. Default is \code{"severity"}.
#' @param treatment Character string specifying the treatment column. Default is \code{"treatment"}.
#' @param environment Character string specifying the environment column (optional).
#' @param by_environment Logical; if \code{TRUE}, the analysis is run separately for each environment. Default is \code{FALSE}.
#' @param env_ref Character string specifying the reference environment for joint analysis (optional).
#' @param threshold Numeric threshold for the persistence metric. Default is \code{5}.
#' @param metrics Character vector of metrics to include in the ranking. Default is \code{c("protected_area", "max_suppression", "persistence", "centroid")}.
#' @param dist_method Character string specifying the distance method. Default is \code{"euclidean"}.
#' @param hclust_method Character string specifying the hierarchical clustering method. Default is \code{"average"}.
#' @param k Integer specifying the number of functional profiles to create, or \code{"auto"} (default). When \code{"auto"}, the optimal number of profiles is estimated by maximizing the average silhouette width.
#' @param cut_height Numeric specifying the cut height for clustering. If provided, overrides \code{k}.
#' @param ... Additional arguments passed to \code{\link{functional_contrast}}.
#'
#' @return A list of class \code{"functional_suppression_profiles"} containing:
#' \itemize{
#'   \item \code{call}: The matched call.
#'   \item \code{reference}: The reference treatment name.
#'   \item \code{contrast}: The contrast data from \code{functional_contrast}.
#'   \item \code{dsp_curves}: The fitted curves from \code{functional_curves}.
#'   \item \code{distances}: The distances object from \code{functional_distances}.
#'   \item \code{hclust}: The hierarchical clustering object.
#'   \item \code{clusters}: A tibble with treatment cluster assignments (internal).
#'   \item \code{profiles}: A lightweight tibble mapping treatments to profiles.
#'   \item \code{classification}: A tibble with the final classification combining profiles, mean rank, and functional metrics, ordered by mean rank.
#'   \item \code{summary}: A tibble with functional summary metrics.
#'   \item \code{ranking}: A tibble with metric rankings.
#'   \item \code{cluster_summary}: A tibble combining metrics, ranks, and cluster assignments.
#'   \item \code{parameters}: A list of input parameters.
#' }
#'
#' @export
#' @seealso \code{\link{functional_contrast}}, \code{\link{functional_curves}}, \code{\link{functional_distances}}, \code{\link{functional_summary}}, \code{\link{rank_dsp}}, \code{\link{plot_dendrogram}}
#'
#' @examples
#' \dontrun{
#' sim_dat <- tibble::tibble(
#'   treatment = rep(c("Control", "A", "B", "C"), each = 6),
#'   time = rep(seq(0, 25, by = 5), times = 4),
#'   severity = c(
#'     c(5, 10, 20, 35, 50, 65),
#'     c(3, 5, 10, 18, 30, 40),
#'     c(4, 6, 12, 22, 35, 45),
#'     c(2, 4, 8, 14, 22, 32)
#'   )
#' )
#' fsp <- functional_suppression_profiles(
#'   data = sim_dat,
#'   reference = "Control",
#'   time = "time",
#'   response = "severity",
#'   treatment = "treatment",
#'   threshold = 5,
#'   k = 2
#' )
#'
#' fsp
#' summary(fsp)
#' plot(fsp, type = "dendrogram")
#' plot(fsp, type = "profiles")
#' plot(fsp, type = "heatmap")
#' plot(fsp, type = "rank")
#' }
functional_suppression_profiles <- function(
    data,
    reference,
    time = "time",
    response = "severity",
    treatment = "treatment",
    environment = NULL,
    by_environment = FALSE,
    env_ref = NULL,
    threshold = 5,
    metrics = c("protected_area", "max_suppression", "persistence", "centroid"),
    dist_method = "euclidean",
    hclust_method = "average",
    k = "auto",
    cut_height = NULL,
    ...
) {
  
  if (!requireNamespace("tibble", quietly = TRUE)) {
    stop("Package 'tibble' is required.")
  }
  
  if (!requireNamespace("dplyr", quietly = TRUE)) {
    stop("Package 'dplyr' is required.")
  }
  
  if (!requireNamespace("stats", quietly = TRUE)) {
    stop("Package 'stats' is required.")
  }

  if (isTRUE(by_environment)) {
    if (is.null(environment)) {
      stop("`environment` must be specified if `by_environment = TRUE`.")
    }
    if (!environment %in% names(data)) {
      stop(sprintf("Environment column '%s' not found in data.", environment))
    }
    
    env_levels <- unique(as.character(data[[environment]]))
    res_list <- lapply(env_levels, function(env_level) {
      env_data <- data[data[[environment]] == env_level, , drop = FALSE]
      functional_suppression_profiles(
        data = env_data,
        reference = reference,
        time = time,
        response = response,
        treatment = treatment,
        environment = NULL,
        by_environment = FALSE,
        env_ref = NULL,
        threshold = threshold,
        metrics = metrics,
        dist_method = dist_method,
        hclust_method = hclust_method,
        k = k,
        cut_height = cut_height,
        ...
      )
    })
    names(res_list) <- env_levels
    class(res_list) <- "functional_suppression_profiles_list"
    return(res_list)
  }

  # Ensure treatment is character to avoid issues
  if (is.factor(data[[treatment]])) {
    data[[treatment]] <- as.character(data[[treatment]])
  }
  
  # Step 1: Functional contrast
  contrast <- functional_contrast(
    data = data,
    reference = reference,
    time = time,
    response = response,
    treatment = treatment,
    group = environment,
    ...
  )
  
  # Step 2: Functional curves for DSP
  # functional_curves uses character variables for column names
  dsp_curves <- functional_curves(
    data = contrast,
    time = time,
    response = "DSP",
    treatment = treatment,
    environment = environment,
    env_ref = env_ref,
    ...
  )
  
  # Determine k for functional_distances and cutree
  n_treatments <- length(unique(dsp_curves$curves[[treatment]]))
  
  if (is.null(cut_height)) {
    if (is.null(k) || identical(k, "auto")) {
      if (n_treatments >= 6) {
        pass_k <- 4
      } else {
        pass_k <- min(3, max(1, n_treatments - 1))
      }
    } else {
      pass_k <- min(k, n_treatments - 1)
    }
    pass_k <- max(1, pass_k)
  } else {
    pass_k <- min(4, n_treatments)
  }

  # Step 3: Functional distances
  distances <- functional_distances(
    dsp_curves,
    hc_method = hclust_method,
    cluster_k = pass_k
  )
  
  # Step 4: Hierarchical clustering
  dist_mat <- distances$distance_matrix
  if (is.null(dist_mat) && !is.null(distances$curve_distance)) dist_mat <- distances$curve_distance
  if (is.null(dist_mat)) dist_mat <- as.matrix(stats::dist(dsp_curves$curves_summary))
  
  if (!is.null(distances$hc)) {
    hc <- distances$hc
  } else if (!is.null(distances$hclust)) {
    hc <- distances$hclust
  } else {
    hc <- stats::hclust(stats::as.dist(dist_mat), method = hclust_method)
  }
  
  # Step 5: Cluster assignment
  sil_obj <- NULL
  sil_avg <- NA
  
  if (!is.null(cut_height)) {
    clusters <- stats::cutree(hc, h = cut_height)
    final_k <- length(unique(clusters))
  } else if (identical(k, "auto")) {
    if (!requireNamespace("cluster", quietly = TRUE)) {
      warning("Package 'cluster' is required for automatic profile estimation. Defaulting to k = 3.")
      final_k <- min(3, max(1, n_treatments - 1))
      clusters <- stats::cutree(hc, k = final_k)
    } else {
      max_k <- min(10, n_treatments - 1)
      if (max_k < 2) {
        final_k <- 1
        clusters <- stats::cutree(hc, k = final_k)
      } else {
        k_values <- 2:max_k
        avg_widths <- numeric(length(k_values))
        for (i in seq_along(k_values)) {
          clust <- stats::cutree(hc, k = k_values[i])
          sil <- cluster::silhouette(clust, stats::as.dist(dist_mat))
          avg_widths[i] <- mean(sil[, "sil_width"])
        }
        best_idx <- which.max(avg_widths)
        final_k <- k_values[best_idx]
        clusters <- stats::cutree(hc, k = final_k)
        sil_obj <- cluster::silhouette(clusters, stats::as.dist(dist_mat))
        sil_avg <- avg_widths[best_idx]
      }
    }
  } else {
    final_k <- k
    clusters <- stats::cutree(hc, k = final_k)
    if (final_k > 1 && requireNamespace("cluster", quietly = TRUE)) {
      sil_obj <- cluster::silhouette(clusters, stats::as.dist(dist_mat))
      sil_avg <- mean(sil_obj[, "sil_width"])
    }
  }
  
  cluster_table <- tibble::tibble(
    !!treatment := names(clusters),
    cluster = clusters
  )
  
  # Convert numeric clusters to Roman numerals
  k_unique <- length(unique(cluster_table$cluster))
  cluster_labels <- paste("Profile", utils::as.roman(cluster_table$cluster))
  cluster_table$cluster <- factor(cluster_labels, levels = paste("Profile", utils::as.roman(sort(unique(cluster_table$cluster)))))
  
  # Generate palette
  cluster_palette <- grDevices::hcl(h = seq(15, 375, length.out = k_unique + 1)[1:k_unique], l = 65, c = 100)
  names(cluster_palette) <- levels(cluster_table$cluster)
  
  # Update distances object to use roman numerals
  if (!is.null(distances$clusters)) {
    distances$clusters$cluster <- cluster_table$cluster[match(distances$clusters[[treatment]], cluster_table[[treatment]])]
  }
  
  # Ensure treatment is character in cluster table
  cluster_table$treatment <- as.character(cluster_table$treatment)
  
  # Step 6: Functional summary
  if (!is.null(environment)) {
    summary_df <- functional_summary(
      object = dsp_curves$curves,
      time = time,
      response = "mu",
      treatment = treatment,
      threshold = threshold
    )
  } else {
    summary_df <- functional_summary(
      object = contrast,
      time = time,
      response = "DSP",
      treatment = treatment,
      threshold = threshold
    )
  }
  
  # Step 7: Ranking
  # Only use metrics that exist in summary_df
  available_metrics <- intersect(metrics, names(summary_df))
  if (length(available_metrics) < length(metrics)) {
    missing <- setdiff(metrics, names(summary_df))
    warning("Some metrics are missing from the summary and will be dropped: ", paste(missing, collapse = ", "))
  }
  
  if (length(available_metrics) > 0) {
    ranking <- rank_dsp(
      summary_data = summary_df,
      metrics = available_metrics
    )
    
    # Extract only ranking variables and rename average_rank to mean_rank
    rank_cols <- intersect(names(ranking), c(paste0("rank_", available_metrics), "average_rank"))
    ranking <- ranking |>
      dplyr::select(dplyr::all_of(c(treatment, rank_cols)))
    
    if ("average_rank" %in% names(ranking)) {
      ranking <- ranking |> dplyr::rename(mean_rank = "average_rank")
    }
  } else {
    ranking <- tibble::tibble(!!treatment := summary_df[[treatment]])
  }
  
  # Ensure treatment column is character in all tables before joining
  summary_df[[treatment]] <- as.character(summary_df[[treatment]])
  if (treatment %in% names(ranking)) {
     ranking[[treatment]] <- as.character(ranking[[treatment]])
  }
  
  # Step 8: Cluster-level summary
  cluster_summary <- summary_df |>
    dplyr::inner_join(ranking, by = treatment) |>
    dplyr::inner_join(cluster_table, by = "treatment")
  
  # Create lightweight profiles table
  profiles_df <- tibble::tibble(
    !!treatment := cluster_table[[treatment]],
    profile = cluster_table$cluster
  )
  
  # Step 9: Final classification table
  # Join profile, mean_rank, and functional metrics
  class_cols <- c(treatment, "profile", "mean_rank", available_metrics)
  
  classification_df <- profiles_df |>
    dplyr::inner_join(ranking |> dplyr::select(dplyr::all_of(c(treatment, "mean_rank"))), by = treatment) |>
    dplyr::inner_join(summary_df |> dplyr::select(dplyr::all_of(c(treatment, available_metrics))), by = treatment) |>
    dplyr::arrange(.data[["mean_rank"]])
  
  # Return object
  res <- structure(
    list(
      call = match.call(),
      reference = reference,
      contrast = contrast,
      dsp_curves = dsp_curves,
      distances = distances,
      hclust = hc,
      k = final_k,
      silhouette_avg = sil_avg,
      silhouette = sil_obj,
      classification = classification_df,
      clusters = cluster_table,
      profiles = profiles_df,
      cluster_palette = cluster_palette,
      summary = summary_df,
      ranking = ranking,
      cluster_summary = cluster_summary,
      parameters = list(
        time = time,
        response = response,
        treatment = treatment,
        environment = environment,
        by_environment = by_environment,
        env_ref = env_ref,
        threshold = threshold,
        metrics = metrics,
        dist_method = dist_method,
        hclust_method = hclust_method,
        k = k,
        cut_height = cut_height
      )
    ),
    class = "functional_suppression_profiles"
  )
}

test_that("functional_suppression_profiles works correctly", {
  sim_dat <- tibble::tibble(
    treatment = rep(c("Control", "A", "B", "C"), each = 6),
    time = rep(seq(0, 25, by = 5), times = 4),
    severity = c(
      c(5, 10, 20, 35, 50, 65),
      c(3, 5, 10, 18, 30, 40),
      c(4, 6, 12, 22, 35, 45),
      c(2, 4, 8, 14, 22, 32)
    )
  )

  fsp <- functional_suppression_profiles(
    data = sim_dat,
    reference = "Control",
    time = "time",
    response = "severity",
    treatment = "treatment",
    threshold = 5,
    k = 2
  )

  # Check outputs
  expect_s3_class(fsp, "functional_suppression_profiles")
  expect_true("classification" %in% names(fsp))
  expect_true(all(c("treatment", "profile", "mean_rank") %in% names(fsp$classification)))
  
  # Check mean_rank order
  expect_false(is.unsorted(fsp$classification$mean_rank))
  
  # Check profile names
  expect_true(all(grepl("^Profile ", as.character(fsp$classification$profile))))
  
  # 1. Check object class
  expect_s3_class(fsp, "functional_suppression_profiles")
  
  # 2. Check components
  expect_true(!is.null(fsp$contrast))
  expect_true(!is.null(fsp$dsp_curves))
  expect_true(!is.null(fsp$distances))
  expect_true(!is.null(fsp$hclust))
  expect_true(!is.null(fsp$clusters))
  expect_true(!is.null(fsp$summary))
  expect_true(!is.null(fsp$ranking))
  
  # Check independent ranking and summary
  expect_true("protected_area" %in% names(fsp$summary))
  expect_false("protected_area" %in% names(fsp$ranking))
  expect_true("rank_protected_area" %in% names(fsp$ranking))
  expect_true("mean_rank" %in% names(fsp$ranking))
  expect_false("average_rank" %in% names(fsp$ranking))
  
  # Check distances uses smoothed curves
  expect_identical(fsp$distances$functional_curves, fsp$dsp_curves)
  
  # 3. Check cluster table columns
  expect_true(all(c("treatment", "cluster") %in% names(fsp$clusters)))
  
  # 4. Check print method returns invisible object
  expect_invisible(print(fsp))
  
  # 5. Check summary method returns expected components
  summ <- summary(fsp)
  expect_s3_class(summ, "summary.functional_suppression_profiles")
  expect_true(all(c("classification", "profile_summary", "ranking", "metrics", "silhouette") %in% names(summ)))
  
  # 6. Check plot method returns a ggplot object
  p1 <- plot(fsp, type = "dendrogram")
  expect_s3_class(p1, "ggplot")
  
  p2 <- plot(fsp, type = "profiles")
  expect_s3_class(p2, "ggplot")
  expect_true("mu" %in% names(p2$data)) # smoothed curves
  
  p3 <- plot(fsp, type = "heatmap")
  expect_s3_class(p3, "ggplot")
  expect_true("rank" %in% names(p3$data)) # only ranks
  
  p4 <- plot(fsp, type = "rank")
  expect_s3_class(p4, "ggplot")
  expect_true("mean_rank" %in% names(p4$data))
  
  if (requireNamespace("patchwork", quietly = TRUE)) {
    p_all <- plot(fsp, type = "all")
    expect_s3_class(p_all, "patchwork")
  }
  
  # 7. missing metric handling works
  fsp_missing_metric <- functional_suppression_profiles(
    data = sim_dat,
    reference = "Control",
    time = "time",
    response = "severity",
    treatment = "treatment",
    metrics = c("protected_area", "non_existent_metric"),
    k = 2
  )
  expect_true("rank_protected_area" %in% names(fsp_missing_metric$ranking))
  expect_false("rank_non_existent_metric" %in% names(fsp_missing_metric$ranking))
  
  # 8. non-numeric treatment labels work
  sim_dat_char <- sim_dat
  sim_dat_char$treatment <- paste0("Trt_", sim_dat_char$treatment)
  fsp_char <- functional_suppression_profiles(
    data = sim_dat_char,
    reference = "Trt_Control",
    time = "time",
    response = "severity",
    treatment = "treatment",
    threshold = 5,
    k = 2
  )
  expect_true(all(grepl("Trt_", fsp_char$clusters$treatment)))
})

test_that("functional_suppression_profiles supports multiple environments", {
  # Create data with two environments
  sim_dat1 <- tibble::tibble(
    treatment = rep(c("Control", "A", "B", "C"), each = 6),
    time = rep(seq(0, 25, by = 5), times = 4),
    severity = c(
      c(5, 10, 20, 35, 50, 65),
      c(3, 5, 10, 18, 30, 40),
      c(4, 6, 12, 22, 35, 45),
      c(2, 4, 8, 14, 22, 32)
    ),
    env = "Env1"
  )
  
  sim_dat2 <- tibble::tibble(
    treatment = rep(c("Control", "A", "B", "C"), each = 6),
    time = rep(seq(0, 25, by = 5), times = 4),
    severity = c(
      c(6, 12, 24, 38, 54, 70),
      c(4, 6, 12, 20, 32, 44),
      c(5, 7, 14, 24, 38, 50),
      c(3, 5, 9, 16, 25, 36)
    ),
    env = "Env2"
  )
  
  sim_dat_multi <- dplyr::bind_rows(sim_dat1, sim_dat2)
  
  # Test Joint Mode
  fsp_joint <- functional_suppression_profiles(
    data = sim_dat_multi,
    reference = "Control",
    time = "time",
    response = "severity",
    treatment = "treatment",
    environment = "env",
    by_environment = FALSE,
    threshold = 5,
    k = 2
  )
  
  expect_s3_class(fsp_joint, "functional_suppression_profiles")
  expect_equal(nrow(fsp_joint$classification), 3) # A, B, C (reference is excluded)
  expect_true("mean_rank" %in% names(fsp_joint$classification))
  expect_false(any(duplicated(fsp_joint$classification$treatment)))
  
  # Test Separate Mode
  fsp_sep <- functional_suppression_profiles(
    data = sim_dat_multi,
    reference = "Control",
    time = "time",
    response = "severity",
    treatment = "treatment",
    environment = "env",
    by_environment = TRUE,
    threshold = 5,
    k = 2
  )
  
  expect_s3_class(fsp_sep, "functional_suppression_profiles_list")
  expect_length(fsp_sep, 2)
  expect_identical(names(fsp_sep), c("Env1", "Env2"))
  expect_s3_class(fsp_sep$Env1, "functional_suppression_profiles")
  expect_s3_class(fsp_sep$Env2, "functional_suppression_profiles")
  
  # Test print method for the list
  expect_output(print(fsp_sep), "List of Functional Suppression Profiles per Environment")
  
  # Test summary method for the list
  summ_sep <- summary(fsp_sep)
  expect_s3_class(summ_sep, "summary.functional_suppression_profiles_list")
  expect_length(summ_sep, 2)
  expect_output(print(summ_sep), "Environment: Env1")
  
  # Test plot method for the list
  plots_sep <- plot(fsp_sep, type = "profiles")
  expect_type(plots_sep, "list")
  expect_length(plots_sep, 2)
  expect_s3_class(plots_sep$Env1, "ggplot")
  # Test Joint Mode with response_scale = "percent" to ensure ... works and no warning is thrown
  expect_no_warning({
    functional_suppression_profiles(
      data = sim_dat_multi,
      reference = "Control",
      time = "time",
      response = "severity",
      treatment = "treatment",
      environment = "env",
      by_environment = FALSE,
      threshold = 5,
      k = 2,
      response_scale = "percent"
    )
  })
})

#' @title Random Subgrid Sampling of a Binary Matrix
#' @description Randomly samples submatrices (quadrats) of specified size from a binary matrix,
#' and returns the positions, submatrices, and count of 1s in each sampled quadrat.
#'
#' @param matrix_data A binary matrix of 0s and 1s.
#' @param sub_rows Number of rows in each subgrid sample.
#' @param sub_cols Number of columns in each subgrid sample.
#' @param n_samples Number of subgrid samples to draw.
#'
#' @return A list of sampled subgrids. Each element is a list with:
#' \item{position}{Row and column start position of the sample.}
#' \item{submatrix}{The sampled subgrid matrix.}
#' \item{count}{Number of 1s in the sampled submatrix.}
#' @export
#' @family Spatial analysis

count_subareas_random <- function(matrix_data, sub_rows = 3, sub_cols = 3, n_samples = 100) {
  total_rows <- nrow(matrix_data)
  total_cols <- ncol(matrix_data)
  max_row_start <- total_rows - sub_rows + 1
  max_col_start <- total_cols - sub_cols + 1

  samples <- lapply(1:n_samples, function(i) {
    row_start <- sample(1:max_row_start, 1)
    col_start <- sample(1:max_col_start, 1)

    submatrix <- matrix_data[
      row_start:(row_start + sub_rows - 1),
      col_start:(col_start + sub_cols - 1)
    ]

    list(
      position = c(row_start, col_start),
      submatrix = submatrix,
      count = sum(submatrix == 1, na.rm = TRUE)
    )
  })

  list(
    samples = samples,
    counts = sapply(samples, function(x) x$count)
  )
}

# zzz.R

# Suppress NOTES from R CMD check about non-standard evaluation (NSE) variables
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".data", "across", "as_label", "bind_cols", "distinct", "element_rect",
    "enquo", "linearHypothesis", "pull", "rename_with", "row_number", "theme",
    "ungroup", "field", "i", "n", "p", "V", "Vbin", "x", "y", "Y", "mod_x", "focus_id",
    "start_day", "end_day", "column_name", "m", "simes_threshold", "score",
    "mk", "treat", "treatment", "Slow", "Sup", ":="
  ))
}
utils::globalVariables(c(
  ".mu_hat", ".res_dev", ".res_pear", ".y",
  "time", "mu", "cluster", "trt_lab",
  "label", "label_short", "xend", "yend",
  # functional_instability variables
  "geno", "env", "mean_mu", "sq_dev", "int_sq_dev",
  "FI", "nFI", "FI_space", "mean_energy_space", "nFI_space",
  "FI_time", "mean_energy_time", "nFI_time",
  "location", "year",
  # plot_functional_instability variables
  "nFI", "nFI_space", "nFI_time"
))
.onLoad <- function(libname, pkgname) {
  utils::globalVariables(c(
    "curve_id_chr", "trt_chr", "y_hat", "u_chr", "id", ".iter",
    "FPC", "prop_var", "cum_var", "curve_id", "DSP", "adjust_group"
  ))
}
# Import from base and recommended packages to suppress NOTES on undefined functions
#' @importFrom stats aggregate coef complete.cases cor.test lm median sd var
#' @importFrom utils install.packages head tail
#' @importFrom car linearHypothesis
#' @importFrom ggplot2 theme element_rect
#' @importFrom dplyr across distinct ungroup pull rename_with bind_cols row_number
#' @importFrom rlang enquo as_label
NULL

#' Fetch ERA5 Data from Open-Meteo for Multiple Locations
#'
#' This function downloads ERA5 historical weather data from the Open-Meteo API
#' for specified locations and dates. It mimics the functionality of `get_nasapower`
#' by fetching weather variables and returning a daily-aggregated data frame.
#' It uses the Open-Meteo Archive API and aggregates hourly data to daily values
#' to ensure all variables (including relative humidity and dew point) are available.
#'
#' @param data A data frame containing the input data, including columns for `latitude`, `longitude`,
#' and the date column specified by `date_col`.
#' @param days_around An integer specifying the number of days before and after the date in the date column
#' to download data.
#' @param date_col A character string specifying the name of the date column in the data frame.
#' @param study_col A character string specifying the name of the column containing the study identifier
#' in the input data frame (default: "study").
#' @param pars A character vector specifying the weather variables to fetch from Open-Meteo.
#' These are mapped to `nasapower` equivalents:
#' "temperature_2m" (T2M), "relative_humidity_2m" (RH2M), "precipitation" (PRECTOTCORR), etc.
#' @param models A character string specifying the reanalysis model(s) to use. If `NULL` (default),
#' Open-Meteo uses its "best_match" logic (ERA5, ERA5-Land, etc.).
#' @param direction Character string specifying the direction of the date range relative to the reference date.
#' Options are "both" (default), "back", or "forth".
#' "back" retrieves data from `date - days_around` to `date`.
#' "forth" retrieves data from `date` to `date + days_around`.
#' "both" retrieves data from `date - days_around` to `date + days_around`.
#'
#' @return A data frame (tibble) with daily weather data. Columns are named to match
#' `nasapower` conventions: `date`, `T2M`, `T2M_MAX`, `T2M_MIN`, `RH2M`, `PRECTOTCORR`,
#' `T2MDEW`, `longitude`, `latitude`, and `study`.
#'
#' @details
#' Since the Open-Meteo Archive API `daily` endpoint does not provide all variables
#' (like mean relative humidity) directly, this function fetches `hourly` data and
#' performs the daily aggregation manually:
#' - `T2M`: Mean of hourly `temperature_2m`
#' - `T2M_MAX`: Max of hourly `temperature_2m`
#' - `T2M_MIN`: Min of hourly `temperature_2m`
#' - `RH2M`: Mean of hourly `relative_humidity_2m`
#' - `PRECTOTCORR`: Sum of hourly `precipitation`
#' - `T2MDEW`: Mean of hourly `dewpoint_2m`
#'
#' @importFrom dplyr mutate group_by summarize ungroup as_tibble select everything
#' @importFrom lubridate days as_date
#' @importFrom purrr map_dfr
#' @importFrom progress progress_bar
#' @importFrom httr GET content
#' @importFrom jsonlite fromJSON
#' @export
#' @family Disease modeling
get_era5 <- function(data, days_around, date_col, study_col = "study",
                     pars = c(
                       "temperature_2m", "relative_humidity_2m",
                       "precipitation", "dewpoint_2m"
                     ),
                     models = NULL, direction = "both") {
  # Helper function to fetch and aggregate data for a single location
  get_single_era5 <- function(data_row, days_around, date_col, study_col, pars, models, direction) {
    # Dynamically extract the date column
    given_date <- as.Date(data_row[[date_col]])

    # Calculate start and end dates
    # Calculate start and end dates
    if (direction == "back") {
      start_date <- given_date - lubridate::days(days_around)
      end_date <- given_date
    } else if (direction == "forth") {
      start_date <- given_date
      end_date <- given_date + lubridate::days(days_around)
    } else {
      # "both" or default
      start_date <- given_date - lubridate::days(days_around)
      end_date <- given_date + lubridate::days(days_around)
    }

    # Open-Meteo Archive API URL
    url <- "https://archive-api.open-meteo.com/v1/archive"

    # Identify coordinate columns (flexible matching)
    lon_col <- if ("longitude" %in% names(data_row)) "longitude" else if ("lon" %in% names(data_row)) "lon" else stop("Longitude column not found.")
    lat_col <- if ("latitude" %in% names(data_row)) "latitude" else if ("lat" %in% names(data_row)) "lat" else stop("Latitude column not found.")

    # Query parameters - requesting HOURLY to ensure we can calculate all means
    query <- list(
      latitude = data_row[[lat_col]],
      longitude = data_row[[lon_col]],
      start_date = as.character(start_date),
      end_date = as.character(end_date),
      hourly = paste(pars, collapse = ","),
      timezone = "auto"
    )

    if (!is.null(models)) query$models <- models

    # Fetch data
    res <- tryCatch(
      {
        httr::GET(url, query = query)
      },
      error = function(e) {
        warning(paste("Failed to fetch data for location:", data_row[[lat_col]], data_row[[lon_col]], "-", e$message))
        return(NULL)
      }
    )

    if (is.null(res)) {
      return(data.frame())
    }

    if (httr::status_code(res) != 200) {
      error_msg <- tryCatch(
        {
          jsonlite::fromJSON(httr::content(res, as = "text"))$reason
        },
        error = function(e) "Unknown error"
      )

      warning(paste("Open-Meteo API error", httr::status_code(res), " (", error_msg, ") for location:", data_row[[lat_col]], data_row[[lon_col]]))
      return(data.frame())
    }

    content_json <- httr::content(res, as = "text", encoding = "UTF-8")
    json_data <- jsonlite::fromJSON(content_json)

    if (is.null(json_data$hourly)) {
      return(data.frame())
    }

    # Convert hourly results to a data frame
    hourly_df <- as.data.frame(json_data$hourly)
    hourly_df$date <- as.Date(hourly_df$time)

    # Aggregate to daily
    # We check available columns first to avoid issues with names(.)
    avail_cols <- names(hourly_df)

    daily_df <- hourly_df %>%
      dplyr::group_by(.data$date) %>%
      dplyr::summarize(
        T2M = if ("temperature_2m" %in% avail_cols) mean(.data$temperature_2m, na.rm = TRUE) else NA_real_,
        T2M_MAX = if ("temperature_2m" %in% avail_cols) max(.data$temperature_2m, na.rm = TRUE) else NA_real_,
        T2M_MIN = if ("temperature_2m" %in% avail_cols) min(.data$temperature_2m, na.rm = TRUE) else NA_real_,
        RH2M = if ("relative_humidity_2m" %in% avail_cols) mean(.data$relative_humidity_2m, na.rm = TRUE) else NA_real_,
        PRECTOTCORR = if ("precipitation" %in% avail_cols) sum(.data$precipitation, na.rm = TRUE) else NA_real_,
        T2MDEW = if ("dewpoint_2m" %in% avail_cols) mean(.data$dewpoint_2m, na.rm = TRUE) else NA_real_,
        .groups = "drop"
      )

    # Add metadata
    daily_df$longitude <- json_data$longitude
    daily_df$latitude <- json_data$latitude
    daily_df$study <- data_row[[study_col]]

    return(daily_df)
  }

  # Initialize a progress bar
  pb <- progress_bar$new(
    format = "  Downloading ERA5 [:bar] :percent in :elapsed",
    total = nrow(data), clear = FALSE, width = 60
  )

  # Use map_dfr with progress bar
  results <- purrr::map_dfr(1:nrow(data), function(i) {
    pb$tick()
    get_single_era5(data[i, , drop = FALSE], days_around, date_col, study_col, pars, models, direction)
  })

  if (nrow(results) == 0) {
    return(dplyr::as_tibble(data.frame()))
  }

  return(dplyr::as_tibble(results))
}

test_that("get_era5 validates input coordinates and column names without network call", {
  df_no_coords <- data.frame(
    date = as.Date("2021-01-01"),
    study = 1
  )

  expect_error(
    get_era5(df_no_coords, days_around = 1, date_col = "date"),
    "Longitude column not found"
  )

  df_no_lat <- data.frame(
    longitude = -45.0,
    date = as.Date("2021-01-01"),
    study = 1
  )

  expect_error(
    get_era5(df_no_lat, days_around = 1, date_col = "date"),
    "Latitude column not found"
  )
})

test_that("get_brdwgd validates coordinate columns and file path", {
  df_no_coords <- data.frame(
    date = as.Date("2021-01-01"),
    study = 1
  )

  expect_error(
    get_brdwgd(df_no_coords, days_around = 1, date_col = "date", path = "nonexistent_dir"),
    "Longitude column not found"
  )

  df_with_coords <- data.frame(
    longitude = -45.0,
    latitude = -20.0,
    date = as.Date("2021-01-01"),
    study = 1
  )

  expect_warning(
    res <- get_brdwgd(df_with_coords, days_around = 1, date_col = "date", path = "nonexistent_dir"),
    "No files for"
  )
  expect_s3_class(res, "tbl_df")
  expect_equal(nrow(res), 0)
})

test_that("get_nasapower validates coordinate columns without network call", {
  df_no_coords <- data.frame(
    date = as.Date("2021-01-01"),
    study = 1
  )

  expect_error(
    get_nasapower(df_no_coords, days_around = 1, date_col = "date"),
    "Longitude column not found"
  )

  df_no_lat <- data.frame(
    longitude = -45.0,
    date = as.Date("2021-01-01"),
    study = 1
  )

  expect_error(
    get_nasapower(df_no_lat, days_around = 1, date_col = "date"),
    "Latitude column not found"
  )
})

#' Summarize functional suppression profiles
#'
#' @param object An object of class \code{"functional_suppression_profiles"}.
#' @param ... Additional arguments.
#'
#' @return A list of class \code{"summary.functional_suppression_profiles"}.
#'
#' @export
summary.functional_suppression_profiles <- function(object, ...) {
  if (!requireNamespace("dplyr", quietly = TRUE)) {
    stop("Package 'dplyr' is required.")
  }
  
  metrics <- object$parameters$metrics
  available_metrics <- intersect(metrics, names(object$summary))
  
  class_df <- object$classification
  trt_var <- object$parameters$treatment
  
  profile_summary <- class_df |>
    dplyr::group_by(.data$profile) |>
    dplyr::summarise(
      n_treatments = dplyr::n(),
      treatments = paste(.data[[trt_var]], collapse = ", "),
      mean_rank = mean(.data$mean_rank, na.rm = TRUE),
      best_treatment = .data[[trt_var]][which.min(.data$mean_rank)],
      .groups = "drop"
    )
    
  res <- list(
    classification = class_df,
    profile_summary = profile_summary,
    ranking = object$ranking,
    metrics = object$summary,
    silhouette = object$silhouette_avg
  )
  
  class(res) <- "summary.functional_suppression_profiles"
  res
}

#' Print summary of functional suppression profiles
#'
#' @param x An object of class \code{"summary.functional_suppression_profiles"}.
#' @param ... Additional arguments.
#'
#' @export
print.summary.functional_suppression_profiles <- function(x, ...) {
  cat("----------------------------------------\n")
  cat("Functional Suppression Profiles\n")
  cat("----------------------------------------\n\n")
  
  cat("Number of treatments:", nrow(x$classification) + 1, "\n") # +1 for reference
  cat("Estimated functional profiles:", nrow(x$profile_summary), "\n")
  
  if (!is.null(x$silhouette) && !is.na(x$silhouette)) {
    cat("Average silhouette width:", round(x$silhouette, 2), "\n")
  }
  
  cat("\n")
  
  # Print treatments for each profile
  for (i in 1:nrow(x$profile_summary)) {
    row <- x$profile_summary[i, ]
    cat(as.character(row$profile), "|", row$n_treatments, "|", row$treatments, "|", round(row$mean_rank, 1), "|", as.character(row$best_treatment), "\n")
  }
  
  invisible(x)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5,
  fig.align = "center"
)

## ----setup, message = FALSE---------------------------------------------------
library(r4pde)
library(ggplot2)

## ----data---------------------------------------------------------------------
data("BudBlightSoybean", package = "r4pde")
head(BudBlightSoybean)

## ----curves, fig.alt = "Fitted mean disease progress curves over time for soybean bud blight treatments"----
fc <- functional_curves(
  data = BudBlightSoybean,
  time = "time",
  response = "y",
  treatment = "treat",
  block = "block",
  min_points = 3,
  family_try = "quasibinomial",
  show_progress = FALSE
)

# Plot fitted mean curves
plot(fc)

## ----rate, fig.alt = "Instantaneous rate of disease progress and velocity phenotypes across treatments"----
fr <- functional_rate(fc, scale = "response", n_grid = 100)

# Summary of rate phenotypes (r_max, t_r_max, growth duration)
summary(fr)

# Visualize instantaneous progress rates over time
plot(fr)

## ----distances, fig.alt = c("Adjusted functional curves colored by functional cluster", "Hierarchical clustering dendrogram based on L2 functional distances")----
fd <- functional_distances(fc, cluster_k = 2, show_progress = FALSE)

# Plot environment-adjusted curves colored by functional cluster
plot_curves(fd)

# Plot hierarchical clustering dendrogram
plot_dendrogram(fd)

## ----fpca, fig.alt = "Biplot of treatment trajectories in functional principal component score space"----
# Retain 2 components for score biplot
fpca <- functional_pca(fc, n_components = 2)

# Print variance explained by the functional principal components
print(fpca)

# Biplot of treatments in FPCA score space
plot(fpca, type = "scores")

## ----resistance---------------------------------------------------------------
# Using 'PD1' as the reference susceptible treatment
fres <- functional_resistance(
  fc,
  reference = "PD1",
  group_method = "quantile",
  n_groups = 2
)

# View resistance scores and rankings
fres$table

#' Plot method for functional instability results
#'
#' @param object Output from [functional_instability()].
#' @param type One of `"overall"`, `"space"`, or `"time"`.
#' @param ... Further arguments passed to methods.
#'
#' @return A ggplot object.
#' @export
plot_functional_instability <- function(object, type = c("overall", "space", "time"), ...) {
  type <- match.arg(type)

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package `ggplot2` is required for plot_functional_instability().", call. = FALSE)
  }

  dat <- tibble::as_tibble(object)
  trt_col <- names(dat)[1]
  names(dat)[1] <- "geno"

  if (type == "overall") {
    p <- dat |>
      dplyr::mutate(geno = stats::reorder(geno, nFI)) |>
      ggplot2::ggplot(ggplot2::aes(geno, nFI)) +
      ggplot2::geom_col() +
      ggplot2::coord_flip() +
      ggplot2::labs(
        x = trt_col,
        y = "Normalized functional instability (nFI)"
      ) +
      ggplot2::theme_minimal()
    return(p)
  }

  if (type == "space") {
    if (!"nFI_space" %in% names(dat)) {
      stop("`nFI_space` not found. Re-run `functional_instability()` with `env_sep`.", call. = FALSE)
    }

    p <- dat |>
      dplyr::mutate(geno = stats::reorder(geno, nFI_space)) |>
      ggplot2::ggplot(ggplot2::aes(geno, nFI_space)) +
      ggplot2::geom_col() +
      ggplot2::coord_flip() +
      ggplot2::labs(
        x = trt_col,
        y = "Spatial normalized functional instability (nFI_space)"
      ) +
      ggplot2::theme_minimal()
    return(p)
  }

  if (!"nFI_time" %in% names(dat)) {
    stop("`nFI_time` not found. Re-run `functional_instability()` with `env_sep`.", call. = FALSE)
  }

  dat |>
    dplyr::mutate(geno = stats::reorder(geno, nFI_time)) |>
    ggplot2::ggplot(ggplot2::aes(geno, nFI_time)) +
    ggplot2::geom_col() +
    ggplot2::coord_flip() +
    ggplot2::labs(
      x = trt_col,
      y = "Temporal normalized functional instability (nFI_time)"
    ) +
    ggplot2::theme_minimal()
}

#' Binary Power Law Analysis for Spatial Disease Patterns
#'
#' This function calculates the Binary Power Law (BPL) parameters for spatial disease patterns,
#' fits a linear model, and performs a hypothesis test for the slope.
#'
#' @param data A data frame containing the following columns:
#'   - \code{field}: The field identifier.
#'   - \code{n}: The number of observations in each quadrat.
#'   - \code{i}: The incidence count in each quadrat.
#'
#' @return A list containing the following elements:
#'   - \code{summary}: A data frame summarizing the input data by field, including total observations (\code{n_total}),
#'     mean incidence (\code{incidence_mean}), observed variance (\code{V}), and binomial variance (\code{Vbin}).
#'   - \code{model_summary}: A summary of the linear model fitted to the log-transformed variances.
#'   - \code{hypothesis_test}: The result of the hypothesis test for the slope being equal to 1.
#'   - \code{ln_Ap}: The intercept of the linear model, representing the natural logarithm of the parameter \( A_p \).
#'   - \code{slope}: The slope of the linear model.
#'
#' @details The function performs the following steps:
#'   1. Summarizes the data by field to calculate the total number of observations (\code{n_total}),
#'      mean incidence (\code{incidence_mean}), observed variance (\code{V}), and binomial variance (\code{Vbin}).
#'   2. Log-transforms the variances.
#'   3. Fits a linear model to the log-transformed variances.
#'   4. Tests the hypothesis that the slope of the linear model is equal to 1.
#' @examples
#' \donttest{
#' # Example usage with a sample data frame
#' result <- BPL(FHBWheat)
#' print(result$summary)
#' print(result$model_summary)
#' print(result$hypothesis_test)
#' print(paste("ln(Ap):", result$ln_Ap))
#' print(paste("Slope (b):", result$slope))
#' }
#' @export
#' @family Spatial analysis

BPL <- function(data) {
  # Summarize the data by field
  df_summary <- data %>%
    group_by(field) %>%
    mutate(p = i / n) %>%
    summarise(
      n_total = sum(n),
      incidence_mean = mean(p, na.rm = TRUE),
      V = var(p, na.rm = TRUE),
      Vbin = (mean(p, na.rm = TRUE) * (1 - mean(p, na.rm = TRUE))) / (mean(n, na.rm = TRUE))
    )

  # Log-transform the variances
  df_summary <- df_summary %>%
    mutate(
      log_V = log(V),
      log_Vbin = log(Vbin)
    )

  # Fit the linear model
  model <- lm(log_V ~ log_Vbin, data = df_summary)

  # Summary of the model
  model_summary <- summary(model)

  # Hypothesis test for slope = 1
  hypothesis_test <- linearHypothesis(model, "log_Vbin = 1")

  # Extract coefficients
  intercept <- coef(model)[1]
  slope <- coef(model)[2]

  # Calculate ln(Ap)
  ln_Ap <- intercept

  return(list(
    summary = df_summary,
    model_summary = model_summary,
    hypothesis_test = hypothesis_test,
    ln_Ap = ln_Ap,
    slope = slope
  ))
}

test_that("AFSD detects foci structure and returns expected components", {
  df <- data.frame(
    x = c(1, 2, 3, 10, 11),
    y = c(1, 1, 1, 10, 10),
    i = c(1, 1, 1, 1, 1)
  )

  res <- AFSD(df)
  expect_type(res, "list")
  expect_length(res, 3)

  # Check cluster df (second element) has 2 distinct foci
  focus_df <- res[[2]]
  expect_equal(nrow(focus_df), 2)
  expect_equal(focus_df$size, c(3, 2))

  # Error on missing columns
  expect_error(AFSD(data.frame(x = 1, y = 1)), "must contain 'x', 'y', and 'i'")
  # Error on empty infected
  expect_error(AFSD(data.frame(x = 1, y = 1, i = 0)), "no rows where i == 1")
})

test_that("BPL fits binary power law and returns model parameters", {
  data("FHBWheat", package = "r4pde")
  res <- BPL(FHBWheat)

  expect_type(res, "list")
  expect_named(res, c("summary", "model_summary", "hypothesis_test", "ln_Ap", "slope"))

  # Slope should be close to 1 for FHBWheat
  expect_true(is.numeric(res$slope))
  expect_true(res$slope > 0.5 && res$slope < 1.5)
  expect_s3_class(res$model_summary, "summary.lm")
})

test_that("join_count calculates join count statistics correctly", {
  # 2x2 matrix with diagonal ones
  m <- matrix(c(0, 1, 1, 0), nrow = 2, ncol = 2)
  jc <- join_count(m, verbose = FALSE)

  expect_type(jc, "list")
  expect_named(jc, c("observed", "expected", "sd", "zscore", "pattern"))
  expect_true(is.numeric(jc$observed$HD))
  expect_true(is.numeric(jc$observed$DD))
})

test_that("oruns_test calculates ordinary runs test and prints output", {
  # Sequence of runs: 1, 0, 1, 1, 0, 1, 0, 0, 1, 1
  # Runs: [1], [0], [1,1], [0], [1], [0,0], [1,1] -> 7 runs
  s <- c(1, 0, 1, 1, 0, 1, 0, 0, 1, 1)
  res <- oruns_test(s)

  expect_s3_class(res, "r4pde.oruns_test")
  expect_equal(res$U, 7)
  expect_true(res$EU > 0)
  expect_true(is.numeric(res$Z))
  expect_true(res$pvalue >= 0 && res$pvalue <= 1)
  expect_true(res$result %in% c("aggregation or clustering", "randomness"))

  # Print method output check
  expect_message(print(res), "Total Number of Runs")
})

test_that("windowpane aggregates variables over date windows", {
  df <- data.frame(
    date = as.Date("2021-01-01") + 0:19,
    end_date = as.Date("2021-01-20"),
    val = 1:20
  )

  wp <- windowpane(
    data = df,
    end_date_col = end_date,
    date_col = date,
    variable = val,
    summary_type = "mean",
    window_lengths = c(5, 10)
  )

  expect_s3_class(wp, "data.frame")
  expect_equal(nrow(wp), 1)
  expect_true(any(grepl("bw_length5", names(wp))))
  expect_true(any(grepl("bw_length10", names(wp))))
})

#' Fetch NASA POWER Data for Multiple Locations with a Progress Bar
#'
#' This function downloads daily NASA POWER data for specified weather variables over a specified number of days
#' around a given date column for multiple locations. It includes a progress bar to show the download progress.
#'
#' @param data A data frame containing the input data, including columns for latitude, longitude, and
#' the date column.
#' @param days_around An integer specifying the number of days before and after the date in the date column
#' to download data.
#' @param date_col A character string specifying the name of the date column in the data frame.
#' @param study_col A character string specifying the name of the column containing the study identifier
#' in the input data frame (default: "study").
#' @param pars A character vector specifying the weather variables to fetch from NASA POWER
#' (default: c("T2M", "RH2M", "PRECTOTCORR", "T2M_MAX", "T2M_MIN", "T2MDEW")).
#' @param direction Character string specifying the direction of the date range relative to the reference date.
#' Options are "both" (default), "back", or "forth".
#' "back" retrieves data from `date - days_around` to `date`.
#' "forth" retrieves data from `date` to `date + days_around`.
#' "both" retrieves data from `date - days_around` to `date + days_around`.
#'
#' @return A data frame with the downloaded weather data from NASA POWER, combined for all specified locations.
#' Includes a variable `study` indicating the study identifier from the input data.
#' Returns an empty data frame if no data is retrieved.
#'
#' @details
#' The function uses the `get_power` function from the `nasapower` package to fetch weather data for a range of
#' dates around the specified date column for each location. A progress bar is shown during the data download
#' process, and the results are combined into a single data frame.
#'
#' @importFrom dplyr mutate bind_rows
#' @importFrom lubridate days
#' @importFrom nasapower get_power
#' @importFrom purrr map_dfr
#' @importFrom progress progress_bar
#' @export
#' @family Disease modeling
get_nasapower <- function(data, days_around, date_col, study_col = "study",
                          pars = c("T2M", "RH2M", "PRECTOTCORR", "T2M_MAX", "T2M_MIN", "T2MDEW"),
                          direction = "both") {
  # Helper function to fetch data for a single location
  get_nasa_data <- function(data, index, days_around, date_col, study_col, pars, direction) {
    # Dynamically extract the date column
    given_date <- as.Date(data[[date_col]][index])

    # Calculate start and end dates based on the specified given date and days_around
    if (direction == "back") {
      start_date <- given_date - days(days_around)
      end_date <- given_date
    } else if (direction == "forth") {
      start_date <- given_date
      end_date <- given_date + days(days_around)
    } else {
      # "both" or default
      start_date <- given_date - days(days_around)
      end_date <- given_date + days(days_around)
    }

    # Identify coordinate columns (flexible matching)
    lon_col <- if ("longitude" %in% names(data)) "longitude" else if ("lon" %in% names(data)) "lon" else stop("Longitude column not found.")
    lat_col <- if ("latitude" %in% names(data)) "latitude" else if ("lat" %in% names(data)) "lat" else stop("Latitude column not found.")

    # Fetch data using get_power
    response <- get_power(
      community = "ag",
      temporal_api = "daily",
      dates = c(start_date, end_date),
      lonlat = c(data[[lon_col]][index], data[[lat_col]][index]),
      pars = pars
    )

    if (nrow(response) > 0) {
      # Use the specified study_col to assign the study identifier
      response <- response %>% mutate(study = data[[study_col]][index])
    }

    return(response)
  }

  # Initialize a progress bar
  pb <- progress_bar$new(
    format = "  Downloading NASA POWER [:bar] :percent in :elapsed",
    total = nrow(data), clear = FALSE, width = 60
  )

  # Use map_dfr with progress bar
  results <- map_dfr(1:nrow(data), function(index) {
    pb$tick() # Update the progress bar
    get_nasa_data(data, index, days_around, date_col, study_col, pars, direction)
  })

  return(results)
}

#' Window Pane for Epidemiological Analysis
#'
#' This function calculates summary statistics within specified windows around a given end date
#' in a dataset, facilitating epidemiological analysis. It allows backward, forward, or both
#' directions of window calculations based on a user-defined variable and window lengths.
#'
#' @param data A data frame containing the input data.
#' @param end_date_col A string specifying the name of the column representing the end date.
#' @param date_col A string specifying the name of the column representing the date variable.
#' @param variable A string specifying the name of the column for which summary statistics are calculated.
#' @param summary_type A string specifying the type of summary to calculate. Options are "mean", "sum",
#'   "above_threshold", or "below_threshold".
#' @param threshold Optional numeric value used when \code{summary_type} is "above_threshold" or "below_threshold".
#' @param window_lengths A numeric vector specifying the window lengths (in days) for the calculations.
#' @param direction A string specifying the direction of the window. Options are "backward" (default),
#'   "forward", or "both".
#' @param group_by_cols Optional vector of strings specifying column names for grouping the data.
#' @param date_format A string specifying the format of the date columns. Default is "%Y-%m-%d".
#'
#' @return A data frame with the calculated summary values for each window.
#'
#' @importFrom tidyr unite pivot_wider
#' @importFrom dplyr mutate group_by summarize ungroup as_tibble select everything filter distinct pull bind_rows bind_cols rename_with across all_of
#' @importFrom rlang enquo !! as_label
#' @importFrom magrittr %>%
#' @importFrom tibble tibble
#' @export
#' @family Disease modeling
windowpane <- function(data,
                       end_date_col,
                       date_col,
                       variable,
                       summary_type,
                       threshold = NULL,
                       window_lengths,
                       direction = "backward",
                       group_by_cols = NULL,
                       date_format = "%Y-%m-%d") {
  # Convert columns to symbols for tidy evaluation
  end_date_col <- enquo(end_date_col)
  date_col <- enquo(date_col)
  variable <- enquo(variable)

  # Prepare data and convert date columns
  data <- tibble::as_tibble(data) %>%
    mutate(
      {{ date_col }} := as.Date(as.character({{ date_col }}), format = date_format),
      {{ end_date_col }} := as.Date(as.character({{ end_date_col }}), format = date_format)
    )

  # Handle grouping
  if (!is.null(group_by_cols)) {
    grouped_data <- data %>% group_by(across(all_of(group_by_cols)))
  } else {
    grouped_data <- data
  }

  # Get unique combinations of grouping variables and end_date_col
  unique_combinations <- grouped_data %>%
    distinct(across(c(dplyr::any_of(group_by_cols), !!end_date_col))) %>%
    ungroup()

  results_list <- list()

  # Iterate over each unique combination
  for (i in seq_len(nrow(unique_combinations))) {
    current_group <- unique_combinations[i, ]
    end_date <- pull(current_group, !!end_date_col)

    # Filter data for the current group
    subset_data <- data
    if (!is.null(group_by_cols)) {
      for (col in group_by_cols) {
        subset_data <- subset_data %>% dplyr::filter(.data[[col]] == current_group[[col]])
      }
    }
    subset_data <- subset_data %>% dplyr::filter((!!end_date_col) == end_date)

    start_date <- end_date - max(window_lengths)

    # Temporary list to store results for each window size
    temp_results <- list()

    # Backward window calculations
    if (direction == "backward" || direction == "both") {
      for (window_size in window_lengths) {
        window_results <- tibble(
          start_day = numeric(),
          end_day = numeric(),
          value = numeric()
        )

        for (j in 0:(max(window_lengths) - window_size)) {
          window_end_date <- end_date - j
          window_start_date <- window_end_date - (window_size - 1)

          # Filter data for the current window
          window_data <- subset_data %>%
            dplyr::filter((!!date_col) >= window_start_date & (!!date_col) <= window_end_date)

          # Calculate summary value
          value <- switch(summary_type,
            "mean" = mean(pull(window_data, !!variable), na.rm = TRUE),
            "sum" = sum(pull(window_data, !!variable), na.rm = TRUE),
            "above_threshold" = if (!is.null(threshold)) sum(pull(window_data, !!variable) > threshold, na.rm = TRUE),
            "below_threshold" = if (!is.null(threshold)) sum(pull(window_data, !!variable) < threshold, na.rm = TRUE),
            NA
          )

          # Save results
          window_results <- bind_rows(window_results, tibble(
            start_day = -j,
            end_day = -(j + window_size - 1),
            value = value
          ))
        }

        # Pivot to wide format for current window size
        window_results_wide <- window_results %>%
          unite("column_name", start_day, end_day, sep = "_") %>%
          pivot_wider(names_from = column_name, values_from = value)

        # Add prefix to column names for backward direction
        window_results_wide <- window_results_wide %>%
          rename_with(~ paste0("bw_length", window_size, "_", as_label(variable), "_", summary_type, "_", .))

        # Store results in temporary list
        temp_results[[paste0("bw_", window_size)]] <- window_results_wide
      }
    }

    # Forward window calculations
    if (direction == "forward" || direction == "both") {
      for (window_size in window_lengths) {
        window_results <- tibble(
          start_day = numeric(),
          end_day = numeric(),
          value = numeric()
        )

        for (j in 0:(max(window_lengths) - window_size)) {
          window_start_date <- start_date + j
          window_end_date <- window_start_date + (window_size - 1)

          # Filter data for the current window
          window_data <- subset_data %>%
            dplyr::filter((!!date_col) >= window_start_date & (!!date_col) <= window_end_date)

          # Calculate summary value
          value <- switch(summary_type,
            "mean" = mean(pull(window_data, !!variable), na.rm = TRUE),
            "sum" = sum(pull(window_data, !!variable), na.rm = TRUE),
            "above_threshold" = if (!is.null(threshold)) sum(pull(window_data, !!variable) > threshold, na.rm = TRUE),
            "below_threshold" = if (!is.null(threshold)) sum(pull(window_data, !!variable) < threshold, na.rm = TRUE),
            NA
          )

          # Save results
          window_results <- bind_rows(window_results, tibble(
            start_day = j,
            end_day = j + window_size - 1,
            value = value
          ))
        }

        # Pivot to wide format for current window size
        window_results_wide <- window_results %>%
          unite("column_name", start_day, end_day, sep = "_") %>%
          pivot_wider(names_from = column_name, values_from = value)

        # Add prefix to column names for forward direction
        window_results_wide <- window_results_wide %>%
          rename_with(~ paste0("fw_length", window_size, "_", as_label(variable), "_", summary_type, "_", .))

        # Store results in temporary list
        temp_results[[paste0("fw_", window_size)]] <- window_results_wide
      }
    }

    # Combine results for all window sizes for the current group
    combined_results <- bind_cols(temp_results)

    # Add grouping columns and end_date column
    if (!is.null(group_by_cols)) {
      for (col in group_by_cols) {
        combined_results[[col]] <- current_group[[col]]
      }
    }
    combined_results[[as_label(end_date_col)]] <- as.character(end_date)

    # Store the results for the current group
    results_list[[i]] <- combined_results
  }

  # Combine all results into a single dataframe
  final_results <- bind_rows(results_list)

  return(final_results)
}

##' Calculate the Disease Severity Index (DSI) (class for each unit)
#'
#' This function calculates the Disease Severity Index (DSI) based on the provided unit,
#' class, and maximum class value. The DSI is computed by aggregating the classes,
#' calculating weights by multiplying the frequency of each class by the class itself,
#' and then dividing the sum of these weights by the product of the total number of
#' entries and the maximum class value, then multiplying by 100.
#'
#' @param unit A vector representing the units.
#' @param class A vector representing the classes corresponding to the units.
#' @param max A numeric value representing the maximum possible class value.
#'
#' @return Returns a single numeric value representing the DSI.
#'
#' @examples
#' # Example usage:
#' unit <- c(1, 2, 3, 4, 5, 6)
#' class <- c(1, 2, 1, 2, 3, 1)
#' max <- 3
#' DSI(unit, class, max)
#'
#' @export
#' @family Disease quantification
DSI <- function(unit, class, max) {
  df <- data.frame(unit, class)
  tab <- aggregate(df$class, list(num=class), length)
  tab$weight = tab$num * tab$x
  (sum(as.numeric(tab$weight)))/(nrow(df)*max)*100
}

test_that("DSI calculates disease severity index correctly", {
  # Known example from documentation
  unit <- c(1, 2, 3, 4, 5, 6)
  class <- c(1, 2, 1, 2, 3, 1)
  max <- 3

  # Expected: sum of weights = (1*3 + 2*2 + 3*1) = 10
  # DSI = 10 / (6 * 3) * 100 = 55.55555...
  res <- DSI(unit, class, max)
  expect_equal(res, (10 / 18) * 100, tolerance = 1e-6)

  # Edge cases
  # All zero class
  expect_equal(DSI(1:4, rep(0, 4), max = 5), 0)
  # All max class
  expect_equal(DSI(1:4, rep(5, 4), max = 5), 100)
})

test_that("DSI2 calculates disease severity index from frequencies correctly", {
  # Example from documentation
  res <- DSI2(c(0, 1, 2, 3, 4), c(2, 0, 5, 0, 5), 4)
  # Weighted sum: 0*2 + 1*0 + 2*5 + 3*0 + 4*5 = 30
  # Total freq: 2 + 0 + 5 + 0 + 5 = 12
  # Max: 4 -> 30 / (12 * 4) * 100 = 62.5
  expect_equal(res, 62.5)

  # Edge case: All zeros
  expect_equal(DSI2(c(0, 1, 2), c(10, 0, 0), max = 2), 0)
  # Edge case: All max
  expect_equal(DSI2(c(0, 1, 2), c(0, 0, 10), max = 2), 100)
})

test_that("CompMuCens executes when interval package is available", {
  skip_if_not_installed("interval")

  inputData <- data.frame(
    treatment = c(rep("A", 10), rep("B", 10)),
    x = c(rep(2, 5), rep(3, 5), rep(4, 5), rep(5, 5))
  )
  res <- CompMuCens(
    dat = inputData,
    scale = c(0, 3, 6, 12, 25, 50, 75, 88, 94, 97, 100, 100),
    ckData = TRUE
  )
  expect_type(res, "list")
  expect_true(!is.null(res$Test_result))
})

#' Compute functional resistance and stability-adjusted functional resistance
#'
#' @description
#' Computes a functional resistance index (FRI) relative to a reference genotype curve.
#' Optionally adjusts FRI by a functional instability penalty to calculate a
#' stability-adjusted functional resistance index (SAFRI). Supports grouping
#' genotypes into classes based on quantiles, clustering, or bootstrap-supported differences.
#'
#' @param object An object of class \code{functional_curves}.
#' @param instability Optional output from \code{functional_instability()}.
#' @param reference Character string naming the reference genotype, or a named vector if \code{reference_within_group = TRUE}.
#' @param lambda Numeric penalty weight for instability. Default is 1.
#' @param method Character string for distance method.
#' @param scale_nfi Logical; if \code{TRUE}, normalizes nFI to a 0-1 scale before applying penalty.
#' @param n_groups Integer number of classes to group into.
#' @param group_method Character string for the grouping method.
#' @param time_var Optional variable name overrides.
#' @param genotype_var Optional variable name overrides.
#' @param n_boot Integer number of bootstrap iterations.
#' @param ci_level Numeric confidence level for bootstrap intervals.
#' @param clustering_method Character string for clustering method if \code{group_method = "clustering"}.
#' @param adjust_by Optional character string naming a genotype-level covariate to stratify by.
#' @param reference_within_group Logical; if \code{TRUE}, the reference is evaluated within each \code{adjust_by} group.
#' @param group_within_adjust_by Logical; if \code{TRUE}, resistance classes are assigned within each \code{adjust_by} group.
#' @param ... Additional arguments.
#'
#' @return A list with a \code{table} containing genotype, FRI, nFI, SAFRI, rank, and classes,
#' and optionally bootstrap results.
#'
#' @examples
#' \dontrun{
#' fr <- functional_resistance(
#'   fc,
#'   reference = "Susceptible",
#'   group_method = "bootstrap"
#' )
#' print(fr)
#' }
#' @export
functional_resistance <- function(
    object,
    instability = NULL,
    reference,
    lambda = 1,
    method = c("positive_area", "l2_difference"),
    scale_nfi = FALSE,
    n_groups = 4,
    group_method = c("none", "quantile", "bootstrap", "clustering"),
    time_var = NULL,
    genotype_var = NULL,
    n_boot = 1000,
    ci_level = 0.95,
    clustering_method = c("hclust", "kmeans"),
    adjust_by = NULL,
    reference_within_group = FALSE,
    group_within_adjust_by = TRUE,
    ...
) {
  if (!inherits(object, "functional_curves")) {
    stop("`object` must be of class 'functional_curves'")
  }

  method <- match.arg(method)
  group_method <- match.arg(group_method)
  clustering_method <- match.arg(clustering_method)

  .time <- if(is.null(time_var)) object$vars$time else time_var
  .trt <- if(is.null(genotype_var)) object$vars$treatment else genotype_var

  pred_trt <- object$curves
  t_grid <- object$grid
  dt_grid <- mean(diff(t_grid))

  if (!is.null(adjust_by)) {
    if (!adjust_by %in% names(object$genotype_info)) {
      stop(sprintf("`adjust_by` variable '%s' not found in object$genotype_info.", adjust_by))
    }
    geno_groups <- object$genotype_info |> dplyr::select(dplyr::all_of(c(.trt, adjust_by)))
    names(geno_groups)[2] <- "adjust_group"
  } else {
    geno_groups <- tibble::tibble(!!.trt := unique(pred_trt[[.trt]]), adjust_group = "All")
    adjust_by <- "All"
  }

  groups <- unique(geno_groups$adjust_group)
  ref_curves <- list()

  if (reference_within_group && adjust_by != "All") {
    for (g in groups) {
      ref_g <- if (length(reference) > 1 && !is.null(names(reference))) reference[as.character(g)] else reference[1]
      if (is.na(ref_g) || !ref_g %in% geno_groups[[.trt]][geno_groups$adjust_group == g]) {
        stop(sprintf("Reference genotype '%s' not found in group '%s'. When reference_within_group = TRUE, each group must have a valid reference genotype.", ref_g, g))
      }
      ref_curves[[as.character(g)]] <- pred_trt |> dplyr::filter(.data[[.trt]] == ref_g) |> dplyr::arrange(.data[[.time]]) |> dplyr::pull("mu")
    }
  } else {
    ref_g <- reference[1]
    if (!ref_g %in% pred_trt[[.trt]]) stop(sprintf("Global reference genotype '%s' not found in the treatments of `object`.", ref_g))
    global_ref <- pred_trt |> dplyr::filter(.data[[.trt]] == ref_g) |> dplyr::arrange(.data[[.time]]) |> dplyr::pull("mu")
    for (g in groups) {
      ref_curves[[as.character(g)]] <- global_ref
    }
  }

  mat <- pred_trt |>
    dplyr::select(.data[[.time]], .data[[.trt]], .data[["mu"]]) |>
    tidyr::pivot_wider(names_from = .data[[.trt]], values_from = .data[["mu"]]) |>
    dplyr::arrange(.data[[.time]])

  X <- as.matrix(mat[, setdiff(names(mat), .time), drop = FALSE])
  trts <- colnames(X)

  fri_res <- numeric(length(trts))
  names(fri_res) <- trts

  for (i in seq_along(trts)) {
    y_g <- X[, i]
    g_val <- as.character(geno_groups$adjust_group[geno_groups[[.trt]] == trts[i]])
    ref_curve <- ref_curves[[g_val]]
    
    if (method == "positive_area") {
      fri_res[i] <- sum(pmax(ref_curve - y_g, 0)) * dt_grid
    } else {
      fri_res[i] <- sqrt(sum((ref_curve - y_g)^2) * dt_grid)
    }
  }

  res_df <- tibble::tibble(!!.trt := names(fri_res), FRI = fri_res)

  if (!is.null(instability)) {
    inst_df <- if(is.data.frame(instability)) instability else instability$nfi_table
    trt_col_inst <- names(inst_df)[apply(inst_df, 2, function(col) all(res_df[[.trt]] %in% col))][1]
    if (is.na(trt_col_inst)) stop("Could not match genotype names in `instability`.")

    inst_sub <- inst_df |> dplyr::select(!!trt_col_inst, nFI = dplyr::any_of("nFI"))
    res_df <- res_df |> dplyr::left_join(inst_sub, by = stats::setNames(trt_col_inst, .trt))
    
    if(scale_nfi && max(res_df$nFI, na.rm=TRUE) > 0) res_df$nFI <- res_df$nFI / max(res_df$nFI, na.rm=TRUE)
    res_df$SAFRI <- res_df$FRI / (1 + lambda * res_df$nFI)
  } else {
    res_df$SAFRI <- res_df$FRI
  }

  res_df <- res_df |>
    dplyr::arrange(dplyr::desc(.data[["SAFRI"]])) |>
    dplyr::mutate(rank_global = dplyr::row_number())

  if (adjust_by != "All") {
    res_df <- res_df |>
      dplyr::left_join(geno_groups |> dplyr::rename(!!adjust_by := adjust_group), by = .trt) |>
      dplyr::group_by(.data[[adjust_by]]) |>
      dplyr::arrange(dplyr::desc(.data[["SAFRI"]])) |>
      dplyr::mutate(rank_within_group = dplyr::row_number()) |>
      dplyr::ungroup()
  }

  default_labels <- c("high stable resistance", "moderate stable resistance", "low stable resistance", "susceptible or unstable")
  grp_labels <- if (n_groups == 4) default_labels else paste("Class", seq_len(n_groups))
  class_names <- if (n_groups == 4) c("A", "B", "C", "D") else LETTERS[1:n_groups]

  assign_classes <- function(sub_df) {
    if (group_method == "quantile") {
      c_vec <- as.character(cut(sub_df$SAFRI, breaks = stats::quantile(sub_df$SAFRI, probs = seq(0, 1, length.out = n_groups + 1), na.rm = TRUE), labels = rev(grp_labels), include.lowest = TRUE))
      sub_df$resistance_class <- factor(c_vec, levels = grp_labels)
    } else if (group_method == "clustering") {
      if (nrow(sub_df) < n_groups) {
         warning("Not enough genotypes for clustering. Skipping class assignment.")
         sub_df$resistance_class <- NA
         return(sub_df)
      }
      cl <- if (clustering_method == "hclust") {
        stats::cutree(stats::hclust(stats::dist(sub_df$SAFRI), method = "ward.D2"), k = n_groups)
      } else {
        stats::kmeans(sub_df$SAFRI, centers = n_groups)$cluster
      }
      cl_means <- tapply(sub_df$SAFRI, cl, mean)
      map_cl <- match(cl, order(cl_means, decreasing = TRUE))
      sub_df$resistance_class <- factor(grp_labels[map_cl], levels = grp_labels)
    }
    return(sub_df)
  }

  if (group_method %in% c("quantile", "clustering")) {
    if (adjust_by != "All" && group_within_adjust_by) {
      res_list <- split(res_df, res_df[[adjust_by]])
      res_list <- lapply(res_list, assign_classes)
      res_df <- dplyr::bind_rows(res_list) |> dplyr::arrange(dplyr::desc(.data[["SAFRI"]]))
    } else {
      res_df <- assign_classes(res_df)
    }
  }

  boot_out <- NULL
  if (group_method == "bootstrap") {
    df_boot <- object$observed_data
    m_gam <- object$gam
    .blk <- object$vars$block
    unit_used <- object$vars$unit
    ids0 <- if(!is.null(.blk)) levels(df_boot[[.blk]]) else levels(df_boot[[unit_used]])
    id_col <- if(!is.null(.blk)) .blk else unit_used
    
    curve_meta <- df_boot |> dplyr::distinct(.data[[unit_used]], .data[[.trt]], .data[[id_col]]) |>
      dplyr::mutate(curve_id_chr = as.character(.data[[unit_used]]), trt_chr = as.character(.data[[.trt]]))
      
    new_curve_grid <- tidyr::crossing(curve_id_chr = curve_meta$curve_id_chr, !!.time := t_grid) |>
      dplyr::left_join(curve_meta, by = "curve_id_chr") |>
      dplyr::mutate(!!unit_used := factor(curve_id_chr, levels = levels(df_boot[[unit_used]])), !!.trt := factor(trt_chr, levels = levels(df_boot[[.trt]])))
      
    if (!is.null(object$vars$covariates)) {
      for (cov in object$vars$covariates) {
        cov_vals <- object$genotype_info[[cov]][match(new_curve_grid[[.trt]], object$genotype_info[[.trt]])]
        new_curve_grid[[cov]] <- cov_vals
      }
    }
      
    excl_c <- character()
    if(any(grepl(sprintf("s\\(%s\\)", unit_used), names(m_gam$smooth), fixed = TRUE))) excl_c <- sprintf("s(%s)", unit_used)
    
    new_curve_grid$y_hat <- as.numeric(mgcv::predict.gam(m_gam, newdata = new_curve_grid, type = "response", exclude = excl_c))
    boot_safri_list <- vector("list", n_boot)
    
    for (b in seq_len(n_boot)) {
      ids_b <- sample(ids0, replace = TRUE)
      w_draw <- tibble::tibble(id = factor(ids_b, levels = ids0)) |> dplyr::count(id, name = "w")
      pred_b <- new_curve_grid |> dplyr::mutate(id = factor(.data[[id_col]], levels = ids0)) |>
        dplyr::left_join(w_draw, by = "id") |> dplyr::filter(!is.na(.data[["w"]])) |>
        dplyr::group_by(.data[[.trt]], .data[[.time]]) |>
        dplyr::summarise(mu = stats::weighted.mean(.data[["y_hat"]], w = .data[["w"]], na.rm = TRUE), .groups = "drop")
        
      mat_b <- pred_b |> tidyr::pivot_wider(names_from = .data[[.trt]], values_from = .data[["mu"]]) |> dplyr::arrange(.data[[.time]])
      X_b <- as.matrix(mat_b[, setdiff(names(mat_b), .time), drop = FALSE])
      
      safri_b <- numeric(ncol(X_b))
      names(safri_b) <- colnames(X_b)
      
      for (j in seq_len(ncol(X_b))) {
        trt_j <- colnames(X_b)[j]
        g_val <- as.character(geno_groups$adjust_group[geno_groups[[.trt]] == trt_j])
        ref_b_nm <- if (reference_within_group && adjust_by != "All") {
           if (length(reference) > 1 && !is.null(names(reference))) reference[as.character(g_val)] else reference[1]
        } else reference[1]
        
        if (!ref_b_nm %in% colnames(X_b)) {
           safri_b[trt_j] <- NA
           next
        }
        ref_b <- X_b[, ref_b_nm]
        
        fri_b <- if(method == "positive_area") sum(pmax(ref_b - X_b[, j], 0)) * dt_grid else sqrt(sum((ref_b - X_b[, j])^2) * dt_grid)
        safri_b[trt_j] <- fri_b
        
        if (!is.null(instability)) {
          nfi_val <- res_df$nFI[res_df[[.trt]] == trt_j]
          if(length(nfi_val)>0) safri_b[trt_j] <- fri_b / (1 + lambda * nfi_val[1])
        }
      }
      
      safri_b <- safri_b[!is.na(safri_b)]
      
      if (adjust_by != "All") {
         cov_b <- geno_groups$adjust_group[match(names(safri_b), geno_groups[[.trt]])]
         boot_safri_list[[b]] <- tibble::tibble(!!.trt := names(safri_b), SAFRI_b = safri_b, !!adjust_by := cov_b, iter = b)
      } else {
         boot_safri_list[[b]] <- tibble::tibble(!!.trt := names(safri_b), SAFRI_b = safri_b, iter = b)
      }
    }
    
    boot_safri_df <- dplyr::bind_rows(boot_safri_list)
    boot_ci_tbl <- boot_safri_df |> dplyr::group_by(.data[[.trt]]) |>
      dplyr::summarise(SAFRI_lower = stats::quantile(.data[["SAFRI_b"]], probs = (1-ci_level)/2, na.rm = TRUE),
                       SAFRI_upper = stats::quantile(.data[["SAFRI_b"]], probs = 1-(1-ci_level)/2, na.rm = TRUE), .groups = "drop")
    res_df <- res_df |> dplyr::left_join(boot_ci_tbl, by = .trt)
    
    assign_boot_classes <- function(sub_res, sub_boot) {
      trts_ordered <- sub_res[[.trt]]
      n_t <- length(trts_ordered)
      mat_boot <- sub_boot |> tidyr::pivot_wider(names_from = .data[[.trt]], values_from = .data[["SAFRI_b"]], id_cols = "iter")
      
      group_assign <- numeric(n_t)
      current_class <- 1
      for (i in seq_len(n_t)) {
        if (group_assign[i] > 0) next
        group_assign[i] <- current_class
        for (j in seq_len(n_t)[-(1:i)]) {
          if (group_assign[j] > 0) next
          diff_ij <- mat_boot[[trts_ordered[i]]] - mat_boot[[trts_ordered[j]]]
          ci_diff <- stats::quantile(diff_ij, probs = c((1-ci_level)/2, 1-(1-ci_level)/2), na.rm = TRUE)
          if (ci_diff[1] <= 0 && ci_diff[2] >= 0) group_assign[j] <- current_class
        }
        current_class <- current_class + 1
      }
      group_assign <- pmin(group_assign, n_groups)
      sub_res$bootstrap_group <- class_names[group_assign]
      sub_res$resistance_class <- factor(sub_res$bootstrap_group, levels = class_names)
      sub_res
    }

    if (adjust_by != "All" && group_within_adjust_by) {
      res_list <- split(res_df, res_df[[adjust_by]])
      boot_list <- split(boot_safri_df, boot_safri_df[[adjust_by]])
      for (g in names(res_list)) res_list[[g]] <- assign_boot_classes(res_list[[g]], boot_list[[g]])
      res_df <- dplyr::bind_rows(res_list) |> dplyr::arrange(dplyr::desc(.data[["SAFRI"]]))
    } else {
      res_df <- assign_boot_classes(res_df, boot_safri_df)
    }
    
    res_df$n_boot <- n_boot
    boot_out <- list(boot_distributions = boot_safri_df)
  }

  out <- list(table = res_df, settings = list(lambda = lambda, method = method, scale_nfi = scale_nfi, group_method = group_method), bootstrap = boot_out)
  class(out) <- "functional_resistance"
  out
}

#' Print functional_resistance
#' @param x An object of class \code{"functional_resistance"}.
#' @param ... Additional arguments.
#' @export
print.functional_resistance <- function(x, ...) {
  cat("A functional_resistance object\n")
  print(x$table)
  invisible(x)
}

test_that("functional_curves returns expected class and components", {
  # Generate some dummy data
  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 4),
    treatment = rep(c("A", "B"), each = 10),
    block = rep(rep(1:2, each = 5), 2),
    y = pmax(0, pmin(1, rep(c(0.1, 0.3, 0.5, 0.7, 0.9), 4) + rnorm(20, 0, 0.05)))
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  expect_s3_class(fc, "functional_curves")
  expect_true(all(c("gam", "curves", "grid", "observed_data", "vars", "settings", "plot_mean", "scores") %in% names(fc)))
  expect_s3_class(fc$plot_mean, "ggplot")
})

test_that("functional_distances returns symmetric distance matrix with zero diagonal", {
  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 4),
    treatment = rep(c("A", "B"), each = 10),
    block = rep(rep(1:2, each = 5), 2),
    y = pmax(0, pmin(1, rep(c(0.1, 0.3, 0.5, 0.7, 0.9), 4) + rnorm(20, 0, 0.05)))
  )
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  fd <- functional_distances(fc, cluster_k = 2, show_progress = FALSE)
  
  expect_s3_class(fd, "functional_distances")
  
  # Symmetric
  expect_equal(fd$distance_matrix, t(fd$distance_matrix))
  # Zero diagonal
  expect_true(all(diag(fd$distance_matrix) == 0))
})

test_that("functional_resistance rankings and instability penalties", {
  set.seed(123)
  # A is low disease, B is medium, C is high (Reference)
  df <- data.frame(
    time = rep(1:5, 6),
    treatment = rep(c("A", "B", "C"), each = 10),
    block = rep(rep(1:2, each = 5), 3),
    y = pmax(0, pmin(1, c(
      rep(c(0.01, 0.05, 0.1, 0.15, 0.2), 2), # A
      rep(c(0.1, 0.2, 0.3, 0.4, 0.5), 2),   # B
      rep(c(0.2, 0.4, 0.6, 0.8, 0.9), 2)    # C
    ) + rnorm(30, 0, 0.01)))
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr_no_inst <- functional_resistance(fc, reference = "C", group_method = "quantile", n_groups = 3)
  expect_s3_class(fr_no_inst, "functional_resistance")
  
  # A should have lower disease than C, so higher FRI than B
  safri_a <- fr_no_inst$table$SAFRI[fr_no_inst$table$treatment == "A"]
  safri_b <- fr_no_inst$table$SAFRI[fr_no_inst$table$treatment == "B"]
  expect_true(safri_a > safri_b)
  
  # Instability penalty
  inst_mock <- data.frame(treatment = c("A", "B", "C"), nFI = c(1, 0, 0.5))
  fr_inst <- functional_resistance(fc, instability = inst_mock, reference = "C", lambda = 1)
  
  safri_a_inst <- fr_inst$table$SAFRI[fr_inst$table$treatment == "A"]
  # SAFRI should decrease when nFI increases (lambda=1)
  expect_true(safri_a_inst < safri_a)
})

test_that("modular functional workflow, plotting, and diagnostics work without legacy wrappers", {
  expect_false("compare_curves" %in% getNamespaceExports("r4pde"))

  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 4),
    treatment = rep(c("A", "B"), each = 10),
    block = rep(rep(1:2, each = 5), 2),
    y = pmax(0, pmin(1, rep(c(0.1, 0.3, 0.5, 0.7, 0.9), 4) + rnorm(20, 0, 0.05)))
  )
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  fd <- functional_distances(fc, show_progress = FALSE)

  expect_s3_class(fc, "functional_curves")
  expect_s3_class(fd, "functional_distances")
  expect_s3_class(plot_curves(fd), "ggplot")
  expect_s3_class(plot_dendrogram(fd), "ggplot")
  expect_s3_class(diagnose_curves(fc), "r4pde_curve_diagnostics")
})

test_that("functional_resistance bootstrap output", {
  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 6),
    treatment = rep(c("A", "B", "C"), each = 10),
    block = rep(rep(1:2, each = 5), 3),
    y = pmax(0, pmin(1, c(
      rep(c(0.01, 0.05, 0.1, 0.15, 0.2), 2), # A
      rep(c(0.1, 0.2, 0.3, 0.4, 0.5), 2),   # B
      rep(c(0.2, 0.4, 0.6, 0.8, 0.9), 2)    # C
    ) + rnorm(30, 0, 0.01)))
  )
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  fr <- functional_resistance(fc, reference = "C", group_method = "bootstrap", n_boot = 50, n_groups = 3)
  
  # output returns one distribution per genotype
  expect_true(!is.null(fr$bootstrap$boot_distributions))
  dist_counts <- table(fr$bootstrap$boot_distributions$treatment)
  expect_equal(as.numeric(dist_counts["A"]), 50)
  
  # pairwise bootstrap differences separate clearly different genotypes
  # A and B are very different
  grp_a <- fr$table$bootstrap_group[fr$table$treatment == "A"]
  grp_b <- fr$table$bootstrap_group[fr$table$treatment == "B"]
  expect_true(grp_a != grp_b)
  
  # grouping labels are ordered from most resistant to least
  # A > B > C in resistance
  expect_true(as.numeric(factor(grp_a, levels = LETTERS)) < as.numeric(factor(grp_b, levels = LETTERS)))
})

test_that("functional_curves processes covariates correctly", {
  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 4),
    treatment = rep(c("A", "B"), each = 10),
    block = rep(rep(1:2, each = 5), 2),
    covar = rep(c("early", "late"), each = 10),
    y = pmax(0, pmin(1, rep(c(0.1, 0.3, 0.5, 0.7, 0.9), 4) + rnorm(20, 0, 0.05)))
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", covariates = "covar", include_covariates = TRUE,
    covariate_smooths = TRUE, min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  expect_true(!is.null(fc$genotype_info))
  expect_true("covar" %in% names(fc$genotype_info))
  expect_equal(as.character(fc$genotype_info$covar), c("early", "late"))
  
  # Error if covariate varies within genotype
  df_bad <- df
  df_bad$covar[1] <- "late"
  expect_error(functional_curves(
    data = df_bad, time = "time", response = "y", treatment = "treatment",
    block = "block", covariates = "covar", include_covariates = TRUE,
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  ), "multiple values")
})

test_that("functional_resistance works with adjust_by and reference_within_group", {
  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 8),
    treatment = rep(c("A_E", "B_E", "A_L", "B_L"), each = 10),
    block = rep(rep(1:2, each = 5), 4),
    covar = rep(c("E", "E", "L", "L"), each = 10),
    y = pmax(0, pmin(1, c(
      rep(c(0.1, 0.2, 0.3, 0.4, 0.5), 2), # A_E
      rep(c(0.2, 0.4, 0.6, 0.8, 0.9), 2), # B_E (Susc Early)
      rep(c(0.1, 0.2, 0.3, 0.4, 0.5), 2), # A_L
      rep(c(0.2, 0.4, 0.6, 0.8, 0.9), 2)  # B_L (Susc Late)
    ) + rnorm(40, 0, 0.01)))
  )
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", covariates = "covar", include_covariates = FALSE,
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  # adjust_by = covar, reference_within_group = TRUE
  refs <- c(E = "B_E", L = "B_L")
  fr <- functional_resistance(fc, reference = refs, adjust_by = "covar", reference_within_group = TRUE, group_method = "quantile", n_groups = 2, group_within_adjust_by = TRUE)
  
  expect_true(!is.null(fr$table$covar))
  expect_true("rank_within_group" %in% names(fr$table))
  
  # reference missing
  refs_bad <- c(E = "B_E", L = "Missing")
  expect_error(functional_resistance(fc, reference = refs_bad, adjust_by = "covar", reference_within_group = TRUE), "not found in group")
})

#' Diagnostic tools for functional epidemic curve models
#'
#' @description
#' Diagnostic tools for functional epidemic curve models fitted with \code{\link{functional_curves}()}.
#' Produces diagnostic plots without invoking base graphics.
#'
#' @param x An object of class \code{"functional_curves"}.
#' @param grid_n Number of points used for the diagnostic smooth curve.
#'
#' @return An object of class \code{"r4pde_curve_diagnostics"} containing:
#' \itemize{
#'   \item diagnostic data,
#'   \item k-index checks,
#'   \item concurvity estimates,
#'   \item ggplot-based diagnostic panels.
#' }
#'
#' @export
diagnose_curves <- function(x, grid_n = 200){
  if (!inherits(x, "functional_curves")) {
    stop("`x` must be an object returned by `functional_curves()`.",
         call. = FALSE)
  }

  req <- c("mgcv","ggplot2","dplyr","tibble")
  miss <- req[!vapply(req, requireNamespace, logical(1), quietly = TRUE)]
  if(length(miss)) stop("Missing packages: ", paste(miss, collapse = ", "))

  df  <- x$observed_data
  tim <- x$vars$time
  trt <- x$vars$treatment

  # fitted values and residuals (ensure same length as df)
  mu_hat   <- as.numeric(mgcv::predict.gam(x$gam, type = "response"))
  res_pear <- as.numeric(stats::residuals(x$gam, type = "pearson"))
  res_dev  <- suppressWarnings(as.numeric(stats::residuals(x$gam, type = "deviance")))

  if(length(mu_hat) != nrow(df) || length(res_pear) != nrow(df)) {
    stop("Diagnostics length mismatch: fitted/residuals not aligned with x$observed_data (nrow).")
  }
  if(length(res_dev) != nrow(df)) res_dev <- rep(NA_real_, nrow(df))

  diag_df <- dplyr::mutate(
    df,
    .mu_hat   = mu_hat,
    .res_pear = res_pear,
    .res_dev  = res_dev
  )

  summ <- list(
    family_used = x$family_used %||% NA_character_,
    n = nrow(df),
    n_treatments = nlevels(df[[trt]]),
    n_units = nlevels(df[[x$vars$unit]]),
    time_range = range(df[[tim]], na.rm = TRUE),
    y_range = range(df$.y, na.rm = TRUE),
    edf = sum(x$gam$edf),
    aic = tryCatch(stats::AIC(x$gam), error = function(e) NA_real_),
    warnings_betar = x$warnings_betar %||% character()
  )

  kcheck <- tryCatch(
    suppressWarnings(mgcv::k.check(x$gam)),
    error = function(e) NULL
  )

  conc <- tryCatch(
    suppressWarnings(mgcv::concurvity(x$gam, estimate = "worst")),
    error = function(e) NULL
  )

  # ---- plots ----
  p_fit <- ggplot2::ggplot(diag_df, ggplot2::aes(x = .mu_hat, y = .y)) +
    ggplot2::geom_point(alpha = 0.35) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed") +
    ggplot2::labs(x = "Fitted mean", y = "Observed (transformed proportion)") +
    ggplot2::theme_classic(base_size = 12)

  p_res_time <- ggplot2::ggplot(diag_df, ggplot2::aes(x = .data[[tim]], y = .res_pear)) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
    ggplot2::geom_point(alpha = 0.35) +
    ggplot2::geom_smooth(method = "loess", se = FALSE) +
    ggplot2::labs(x = tim, y = "Pearson residual") +
    ggplot2::theme_classic(base_size = 12)

  p_res_fit <- ggplot2::ggplot(diag_df, ggplot2::aes(x = .mu_hat, y = .res_pear)) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
    ggplot2::geom_point(alpha = 0.35) +
    ggplot2::geom_smooth(method = "loess", se = FALSE) +
    ggplot2::labs(x = "Fitted mean", y = "Pearson residual") +
    ggplot2::theme_classic(base_size = 12)

  p_qq <- ggplot2::ggplot(
    diag_df |> dplyr::filter(is.finite(.res_dev)),
    ggplot2::aes(sample = .res_dev)
  ) +
    ggplot2::stat_qq(alpha = 0.35) +
    ggplot2::stat_qq_line() +
    ggplot2::labs(x = "Theoretical quantiles", y = "Deviance residuals") +
    ggplot2::theme_classic(base_size = 12)

  # ---- sanity curve ----
  trt0 <- levels(df[[trt]])[1]
  t_grid <- seq(min(df[[tim]]), max(df[[tim]]), length.out = grid_n)

  newd <- tibble::tibble(.tmp = t_grid)
  names(newd)[1] <- tim

  newd[[trt]] <- factor(trt0, levels = levels(df[[trt]]))

  if(!is.null(x$vars$environment)){
    env <- x$vars$environment
    env_ref <- x$settings$env_ref %||% levels(df[[env]])[1]
    newd[[env]] <- factor(env_ref, levels = levels(df[[env]]))
  }

  newd[[x$vars$unit]] <- df[[x$vars$unit]][1]

  excl <- c(sprintf("s(%s)", x$vars$unit))
  if(!is.null(x$vars$environment)){
    excl <- c(excl, sprintf("s(%s,%s)", tim, x$vars$environment))
  }

  mu_grid <- mgcv::predict.gam(
    x$gam, newdata = newd, type = "response", exclude = excl
  )

  p_curve <- ggplot2::ggplot(
    tibble::tibble(time = t_grid, mu = as.numeric(mu_grid)),
    ggplot2::aes(x = time, y = mu)
  ) +
    ggplot2::geom_line(linewidth = 1.1) +
    ggplot2::expand_limits(y = c(0, 1)) +
    ggplot2::labs(
      x = tim,
      y = paste0("Adjusted mean (", trt0, ")"),
      title = "Sanity check: one adjusted curve"
    ) +
    ggplot2::theme_classic(base_size = 12)

  out <- list(
    summary = summ,
    kcheck = kcheck,
    concurvity = conc,
    diag_data = diag_df,
    plots = list(
      fitted_vs_observed = p_fit,
      residuals_vs_time = p_res_time,
      residuals_vs_fitted = p_res_fit,
      qq_deviance = p_qq,
      one_adjusted_curve = p_curve
    )
  )
  class(out) <- "r4pde_curve_diagnostics"
  out
}

#' Plot diagnostics for functional epidemic curve models
#'
#' @description
#' Combines multiple diagnostic plots from
#' \code{\link{diagnose_curves}} into a multi-panel layout.
#'
#' @param diag An object of class \code{"r4pde_curve_diagnostics"}.
#' @param layout Layout of diagnostic panels: \code{"2x2"}, \code{"vertical"},
#'   or \code{"horizontal"}.
#' @param show_curve Logical; whether to include the example adjusted curve.
#' @param rel_heights Relative heights for panels when stacking.
#'
#' @return A composite plot object generated using \pkg{cowplot}.
#'
#' @export
plot_diagnostics <- function(
    diag,
    layout = c("2x2", "vertical", "horizontal"),
    show_curve = FALSE,
    rel_heights = c(1, 0.7)
){
  stopifnot(inherits(diag, "r4pde_curve_diagnostics"))

  req <- c("cowplot")
  miss <- req[!vapply(req, requireNamespace, logical(1), quietly = TRUE)]
  if(length(miss)) stop("Missing package: ", paste(miss, collapse = ", "))

  layout <- match.arg(layout)

  p_fit  <- diag$plots$fitted_vs_observed
  p_rt   <- diag$plots$residuals_vs_time
  p_rf   <- diag$plots$residuals_vs_fitted
  p_qq   <- diag$plots$qq_deviance
  p_curv <- diag$plots$one_adjusted_curve

  subtitle <- paste0(
    "Family: ", diag$summary$family_used,
    " | EDF = ", signif(diag$summary$edf, 4),
    " | AIC = ", signif(diag$summary$aic, 6)
  )

  title_g <- cowplot::ggdraw() +
    cowplot::draw_label("Model diagnostics", x = 0, hjust = 0, fontface = "bold", size = 13) +
    cowplot::draw_label(subtitle, x = 0, y = 0.25, hjust = 0, size = 10)

  fig <- switch(
    layout,

    "2x2" = {
      top <- cowplot::plot_grid(p_fit, p_rt, nrow = 1, align = "hv")
      bot <- cowplot::plot_grid(p_rf,  p_qq, nrow = 1, align = "hv")
      core <- cowplot::plot_grid(top, bot, ncol = 1, align = "hv")

      if(show_curve){
        cowplot::plot_grid(core, p_curv, ncol = 1, rel_heights = rel_heights, align = "hv")
      } else core
    },

    "vertical" = {
      if(show_curve){
        cowplot::plot_grid(p_fit, p_rt, p_rf, p_qq, p_curv, ncol = 1, align = "hv")
      } else {
        cowplot::plot_grid(p_fit, p_rt, p_rf, p_qq, ncol = 1, align = "hv")
      }
    },

    "horizontal" = {
      if(show_curve){
        cowplot::plot_grid(p_fit, p_rt, p_rf, p_qq, p_curv, nrow = 1, align = "hv")
      } else {
        cowplot::plot_grid(p_fit, p_rt, p_rf, p_qq, nrow = 1, align = "hv")
      }
    }
  )

  cowplot::plot_grid(title_g, fig, ncol = 1, rel_heights = c(0.13, 1))
}

#' Print method for curve diagnostics
#'
#' @param x An object of class \code{"r4pde_curve_diagnostics"}.
#' @param ... Additional arguments (ignored).
#'
#' @return Invisibly returns \code{x}.
#'
#' @export
print.r4pde_curve_diagnostics <- function(x, ...){
  s <- x$summary
  cat("Curve model diagnostics\n")
  cat("  family_used: ", s$family_used, "\n", sep = "")
  cat("  n (rows): ", s$n, "\n", sep = "")
  cat("  treatments: ", s$n_treatments, "\n", sep = "")
  cat("  units: ", s$n_units, "\n", sep = "")
  cat("  time range: ", paste(s$time_range, collapse = " to "), "\n", sep = "")
  cat("  y range: ", paste(signif(s$y_range, 3), collapse = " to "), "\n", sep = "")
  cat("  EDF sum: ", signif(s$edf, 4), "\n", sep = "")
  cat("  AIC: ", signif(s$aic, 6), "\n", sep = "")
  if(length(s$warnings_betar)) {
    cat("  captured warnings (betar attempt):\n")
    for(w in s$warnings_betar) cat("   - ", w, "\n", sep = "")
  }
  invisible(x)
}

#' BlastWheat dataset
#'
#' Wheat blast dataset with severity and weather covariates.
#'
#' @format A data frame with the following columns:
#' \describe{
#'   \item{heading}{Date of heading}
#'   \item{inc_mean}{Mean incidence}
#'   \item{index_mean}{FHB index mean}
#'   \item{latitude}{Latitude coordinate}
#'   \item{location}{Experimental site name}
#'   \item{longitude}{Longitude coordinate}
#'   \item{state}{Brazilian state}
#'   \item{study}{Study ID or code}
#'   \item{year}{Crop year}
#'   \item{yld_mean}{Mean yield}
#' }
#' @source Del Ponte Lab internal data
"BlastWheat"
#' BudBlightSoybean dataset
#'
#' Soybean bud blight incidence in experimental blocks.
#'
#' @format A data frame with the following columns:
#' \describe{
#'   \item{block}{Block number}
#'   \item{time}{Time point of assessment}
#'   \item{treat}{Treatment name}
#'   \item{y}{Incidence or severity value}
#' }
#' @source Del Ponte Lab internal data
"BudBlightSoybean"
#' DidymellaWatermelon dataset
#'
#' Assessment of Didymella symptoms in watermelon plots.
#'
#' @format A data frame with:
#' \describe{
#'   \item{EW_row}{Row position (east–west)}
#'   \item{NS_col}{Column position (north–south)}
#'   \item{dap}{Days after planting}
#'   \item{severity}{Disease severity}
#' }
#' @source Del Ponte Lab internal data
"DidymellaWatermelon"
#' FHBWheat dataset
#'
#' Fusarium head blight quadrat assessments in wheat.
#'
#' @format A data frame with:
#' \describe{
#'   \item{field}{Field identifier}
#'   \item{i}{Row position}
#'   \item{n}{Column position}
#'   \item{quadrat}{Quadrat ID}
#'   \item{season}{Crop season}
#' }
#' @source Del Ponte Lab internal data
"FHBWheat"
#' FusariumBanana dataset
#'
#' Observations of Fusarium symptoms in banana fields.
#'
#' @format A data frame with:
#' \describe{
#'   \item{field}{Field ID}
#'   \item{lat}{Latitude}
#'   \item{lon}{Longitude}
#'   \item{marker}{Infection marker presence}
#' }
#' @source Del Ponte Lab internal data
"FusariumBanana"
#' RustSoybean dataset
#'
#' Soybean rust severity and field metadata.
#'
#' @format A data frame with:
#' \describe{
#'   \item{detection}{Detection score or date}
#'   \item{epidemia}{Epidemic phase}
#'   \item{latitude}{Latitude}
#'   \item{local}{Location name}
#'   \item{longitude}{Longitude}
#'   \item{planting}{Planting date or stage}
#'   \item{severity}{Disease severity}
#' }
#' @source Del Ponte Lab internal data
"RustSoybean"
#' WhiteMoldSoybean dataset
#'
#' National dataset of white mold severity and yield.
#'
#' @format A data frame with:
#' \describe{
#'   \item{country}{Country name}
#'   \item{elevation}{Field elevation}
#'   \item{elevation_class}{Elevation class}
#'   \item{harvest_year}{Year of harvest}
#'   \item{inc}{Incidence}
#'   \item{inc_check}{Check plot incidence}
#'   \item{inc_class}{Incidence class}
#'   \item{location}{Location name}
#'   \item{region}{Geographical region}
#'   \item{scl}{Soybean canopy layer}
#'   \item{season}{Crop season}
#'   \item{state}{State name}
#'   \item{study}{Study identifier}
#'   \item{treat}{Treatment applied}
#'   \item{yld}{Yield}
#'   \item{yld_check}{Yield of untreated check}
#'   \item{yld_class}{Yield class}
#' }
#' @source Del Ponte Lab internal data
"WhiteMoldSoybean"
#' SpatialAggregated dataset
#'
#' Simulated aggregated spatial binary disease pattern.
#'
#' @format A data frame with:
#' \describe{
#'   \item{x}{x-coordinate}
#'   \item{y}{y-coordinate}
#' }
#' @source Simulated example
"SpatialAggregated"

#' SpatialRandom dataset
#'
#' Simulated random spatial binary disease pattern.
#'
#' @format A data frame with:
#' \describe{
#'   \item{x}{x-coordinate}
#'   \item{y}{y-coordinate}
#' }
#' @source Simulated example
"SpatialRandom"

#' FHBWheatBrazil dataset
#'
#' Fusarium head blight (FHB) index in wheat in Brazil.
#'
#' @format A data frame with the following columns:
#' \describe{
#'   \item{study}{Study ID or code}
#'   \item{year}{Crop year}
#'   \item{location}{Experimental site name}
#'   \item{state}{Brazilian state}
#'   \item{lat}{Latitude coordinate}
#'   \item{lon}{Longitude coordinate}
#'   \item{cultivar}{Wheat cultivar}
#'   \item{planting_date}{Planting date}
#'   \item{flowering}{Flowering date}
#'   \item{index}{FHB index}
#' }
#' @source Del Ponte Lab internal data
"FHBWheatBrazil"

#' Normalized functional instability from fitted functional disease curves
#'
#' Computes normalized functional instability (NFI) for each treatment
#' based on genotype-by-environment predicted curves extracted from a
#' \code{\link{functional_curves}} object. Optionally, instability can be decomposed
#' into spatial and temporal components if the environment identifier can
#' be split into location and year.
#'
#' The overall instability metric is defined as the mean integrated squared
#' deviation of each genotype-by-environment curve from the genotype-specific
#' mean curve across environments, normalized by the integrated squared mean
#' curve.
#'
#' @param x An object returned by \code{\link{functional_curves}} or a \code{functional_dsp} object.
#' @param n_time Number of points in the prediction grid over the time domain.
#' @param env_sep Optional separator used to split `env` into spatial and
#'   temporal components, for example `"_"` or `"-"`. If `NULL`, only the
#'   overall NFI is returned.
#' @param env_names A character vector of length 2 giving the names of the
#'   components obtained after splitting `env`. Default is
#'   `c("location", "year")`.
#' @param return_curves Logical. If `TRUE`, the predicted genotype-by-environment
#'   curves are also returned.
#' @param ... Additional arguments.
#'
#' @return If `return_curves = FALSE`, a tibble with one row per genotype and
#'   columns:
#'   \describe{
#'     \item{geno}{Treatment or genotype identifier.}
#'     \item{n_env}{Number of environments used in the calculation.}
#'     \item{FI}{Absolute functional instability.}
#'     \item{mean_energy}{Integrated squared mean epidemic curve.}
#'     \item{nFI}{Normalized functional instability. Lower values indicate
#'       greater stability.}
#'   }
#'
#'   If `env_sep` is provided, the returned tibble also includes:
#'   \describe{
#'     \item{FI_space, mean_energy_space, nFI_space}{Spatial component of
#'       instability.}
#'     \item{FI_time, mean_energy_time, nFI_time}{Temporal component of
#'       instability.}
#'   }
#'
#'   If `return_curves = TRUE`, a list is returned with two elements:
#'   \describe{
#'     \item{metrics}{The tibble described above.}
#'     \item{curves}{A tibble of predicted genotype-by-environment curves with
#'       columns `geno`, `env`, `time`, `mu`, and `eta`.}
#'   }
#'
#' @details
#' Let \eqn{f_{ge}(t)} denote the predicted epidemic trajectory of genotype
#' \eqn{g} in environment \eqn{e}, and let \eqn{\bar f_g(t)} denote the mean
#' trajectory of genotype \eqn{g} across environments. Functional instability
#' is computed as:
#'
#' \deqn{
#' FI_g = \frac{1}{E_g}\sum_{e=1}^{E_g} \int_T \left(f_{ge}(t)-\bar f_g(t)\right)^2 dt
#' }
#'
#' and normalized as:
#'
#' \deqn{
#' nFI_g = \frac{FI_g}{\int_T \bar f_g(t)^2 dt}
#' }
#'
#' Numerical integration is performed with the trapezoidal rule on a regular
#' prediction grid over the observed time domain.
#'
#' @seealso \code{\link{functional_curves}}
#'
#' @examples
#' \dontrun{
#' m1 <- r4pde::functional_curves(
#'   data = dat_ready,
#'   time = "time",
#'   response = "y",
#'   treatment = "geno",
#'   environment = "env"
#' )
#'
#' # Overall instability
#' res <- functional_instability(m1)
#' res
#'
#' # Overall + spatial and temporal components
#' # Assuming env has values such as "PF_2021"
#' res2 <- functional_instability(m1, env_sep = "_")
#' res2
#'
#' # Return curves used in the calculations
#' out <- functional_instability(m1, env_sep = "_", return_curves = TRUE)
#' out$metrics
#' head(out$curves)
#' }
#'
#' @export
functional_instability <- function(x, ...) {
  UseMethod("functional_instability")
}

#' @export
#' @rdname functional_instability
functional_instability.functional_curves <- function(x,
                n_time = 200,
                env_sep = NULL,
                env_names = c("location", "year"),
                return_curves = FALSE,
                ...) {

  if (!inherits(x, "functional_curves") || is.null(names(x)) || !("gam" %in% names(x)) || !("observed_data" %in% names(x))) {
    stop("`x` must be a valid `functional_curves()` object containing `gam` and `observed_data`.",
         call. = FALSE)
  }

  dat <- x$observed_data

  trt_col <- if (!is.null(x$vars$treatment)) x$vars$treatment else "geno"
  env_col <- if (!is.null(x$vars$environment)) x$vars$environment else "env"
  time_col <- if (!is.null(x$vars$time)) x$vars$time else "time"

  if (!all(c(trt_col, env_col, time_col) %in% names(dat))) {
    stop(sprintf("`x$observed_data` must contain columns `%s`, `%s`, and `%s`.", trt_col, env_col, time_col), call. = FALSE)
  }

 trapz_vec <- function(x, y) {
  ok <- !(is.na(x) | is.na(y))
  x <- x[ok]
  y <- y[ok]

  if (length(x) < 2) return(NA_real_)

  ord <- order(x)
  x <- x[ord]
  y <- y[ord]

  sum(diff(x) * (head(y, -1) + tail(y, -1)) / 2)
}

  predict_geno_env_curves <- function(dat, mod, trt_c, env_c, time_c, n_time = 200) {
    pairs <- dat %>%
      dplyr::distinct(.data[[trt_c]], .data[[env_c]])

    time_seq <- seq(
      min(dat[[time_c]], na.rm = TRUE),
      max(dat[[time_c]], na.rm = TRUE),
      length.out = n_time
    )

    time_df <- tibble::tibble(time_seq)
    names(time_df) <- time_c

    newdata <- tidyr::expand_grid(pairs, time_df)

    mu  <- stats::predict(mod, newdata = newdata, type = "response")
    eta <- stats::predict(mod, newdata = newdata, type = "link")

    newdata <- newdata %>%
      dplyr::mutate(mu = as.numeric(mu), eta = as.numeric(eta))

    names(newdata)[names(newdata) == trt_c] <- "geno"
    names(newdata)[names(newdata) == env_c] <- "env"
    names(newdata)[names(newdata) == time_c] <- "time"

    newdata
  }

  calc_nfi_general <- function(curves) {
    mean_curve <- curves %>%
      dplyr::group_by(geno, time) %>%
      dplyr::summarise(mean_mu = mean(mu, na.rm = TRUE), .groups = "drop")

    fi_env <- curves %>%
      dplyr::left_join(mean_curve, by = c("geno", "time")) %>%
      dplyr::mutate(sq_dev = (mu - mean_mu)^2) %>%
      dplyr::group_by(geno, env) %>%
      dplyr::summarise(
        int_sq_dev = trapz_vec(time, sq_dev),
        .groups = "drop"
      )

    fi_geno <- fi_env %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        n_env = dplyr::n(),
        FI = mean(int_sq_dev, na.rm = TRUE),
        .groups = "drop"
      )

    mean_energy <- mean_curve %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        mean_energy = trapz_vec(time, mean_mu^2),
        .groups = "drop"
      )

    fi_geno %>%
      dplyr::left_join(mean_energy, by = "geno") %>%
      dplyr::mutate(nFI = FI / mean_energy) %>%
      dplyr::arrange(nFI)
  }

  calc_nfi_space <- function(curves) {
    mean_curve <- curves %>%
      dplyr::group_by(geno, year, time) %>%
      dplyr::summarise(mean_mu = mean(mu, na.rm = TRUE), .groups = "drop")

    fi_units <- curves %>%
      dplyr::left_join(mean_curve, by = c("geno", "year", "time")) %>%
      dplyr::mutate(sq_dev = (mu - mean_mu)^2) %>%
      dplyr::group_by(geno, year, location) %>%
      dplyr::summarise(
        int_sq_dev = trapz_vec(time, sq_dev),
        .groups = "drop"
      )

    fi_geno <- fi_units %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        FI_space = mean(int_sq_dev, na.rm = TRUE),
        .groups = "drop"
      )

    mean_energy <- mean_curve %>%
      dplyr::group_by(geno, year) %>%
      dplyr::summarise(
        mean_energy_space = trapz_vec(time, mean_mu^2),
        .groups = "drop"
      ) %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        mean_energy_space = mean(mean_energy_space, na.rm = TRUE),
        .groups = "drop"
      )

    fi_geno %>%
      dplyr::left_join(mean_energy, by = "geno") %>%
      dplyr::mutate(nFI_space = FI_space / mean_energy_space)
  }

  calc_nfi_time <- function(curves) {
    mean_curve <- curves %>%
      dplyr::group_by(geno, location, time) %>%
      dplyr::summarise(mean_mu = mean(mu, na.rm = TRUE), .groups = "drop")

    fi_units <- curves %>%
      dplyr::left_join(mean_curve, by = c("geno", "location", "time")) %>%
      dplyr::mutate(sq_dev = (mu - mean_mu)^2) %>%
      dplyr::group_by(geno, location, year) %>%
      dplyr::summarise(
        int_sq_dev = trapz_vec(time, sq_dev),
        .groups = "drop"
      )

    fi_geno <- fi_units %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        FI_time = mean(int_sq_dev, na.rm = TRUE),
        .groups = "drop"
      )

    mean_energy <- mean_curve %>%
      dplyr::group_by(geno, location) %>%
      dplyr::summarise(
        mean_energy_time = trapz_vec(time, mean_mu^2),
        .groups = "drop"
      ) %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        mean_energy_time = mean(mean_energy_time, na.rm = TRUE),
        .groups = "drop"
      )

    fi_geno %>%
      dplyr::left_join(mean_energy, by = "geno") %>%
      dplyr::mutate(nFI_time = FI_time / pmax(mean_energy_time, 1e-8)) %>%
      dplyr::mutate(nFI_time = pmax(nFI_time, 0))
  }

  curves <- predict_geno_env_curves(dat, x$gam, trt_col, env_col, time_col, n_time = n_time)
  out <- calc_nfi_general(curves)

  if (!is.null(env_sep)) {
    if (length(env_names) != 2) {
      stop("`env_names` must have length 2.", call. = FALSE)
    }

    curves2 <- curves %>%
      tidyr::separate(
        env,
        into = env_names,
        sep = env_sep,
        remove = FALSE
      )

    names(curves2)[names(curves2) == env_names[1]] <- "location"
    names(curves2)[names(curves2) == env_names[2]] <- "year"

    out <- out %>%
      dplyr::left_join(calc_nfi_space(curves2), by = "geno") %>%
      dplyr::left_join(calc_nfi_time(curves2), by = "geno")
  }

  class(out) <- c("fi_tbl", class(out))

  if ("geno" %in% names(out)) {
    names(out)[names(out) == "geno"] <- trt_col
  }

  if (return_curves) {
    names(curves)[names(curves) == "geno"] <- trt_col
    names(curves)[names(curves) == "env"] <- env_col
    names(curves)[names(curves) == "time"] <- time_col
    return(list(metrics = out, curves = curves))
  }

  out
}

#' @export
#' @rdname functional_instability
functional_instability.functional_dsp <- function(x,
                env_sep = NULL,
                env_names = c("location", "year"),
                return_curves = FALSE,
                ...) {

  vars <- attr(x, "dsp_vars")
  if (is.null(vars)) stop("`x` is missing `dsp_vars` attribute.")
  
  trt_col <- vars$treatment
  time_col <- vars$time
  resp_col <- vars$response
  group_cols <- vars$group
  
  if (length(group_cols) == 0) {
    stop("functional_dsp object must have grouping variables (environments) to calculate instability.")
  }
  
  # For instability, we need exactly one environment column. If there are multiple, combine them.
  if (length(group_cols) > 1) {
    x <- x |> dplyr::mutate(env = interaction(dplyr::select(x, dplyr::all_of(group_cols)), sep = "_"))
    env_col <- "env"
  } else {
    env_col <- group_cols[1]
  }

  curves <- x |>
    dplyr::rename(geno = dplyr::all_of(trt_col), env = dplyr::all_of(env_col), time = dplyr::all_of(time_col), mu = dplyr::all_of(resp_col)) |>
    dplyr::select(geno, env, time, mu)
    
  trapz_vec <- function(x, y) {
    ok <- !(is.na(x) | is.na(y))
    x <- x[ok]
    y <- y[ok]
    if (length(x) < 2) return(NA_real_)
    ord <- order(x)
    x <- x[ord]
    y <- y[ord]
    sum(diff(x) * (head(y, -1) + tail(y, -1)) / 2)
  }

  calc_nfi_general <- function(curves) {
    mean_curve <- curves %>%
      dplyr::group_by(geno, time) %>%
      dplyr::summarise(mean_mu = mean(mu, na.rm = TRUE), .groups = "drop")

    fi_env <- curves %>%
      dplyr::left_join(mean_curve, by = c("geno", "time")) %>%
      dplyr::mutate(sq_dev = (mu - mean_mu)^2) %>%
      dplyr::group_by(geno, env) %>%
      dplyr::summarise(
        int_sq_dev = trapz_vec(time, sq_dev),
        .groups = "drop"
      )

    fi_geno <- fi_env %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        n_env = dplyr::n(),
        FI = mean(int_sq_dev, na.rm = TRUE),
        .groups = "drop"
      )

    mean_energy <- mean_curve %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        mean_energy = trapz_vec(time, mean_mu^2),
        .groups = "drop"
      )

    fi_geno %>%
      dplyr::left_join(mean_energy, by = "geno") %>%
      dplyr::mutate(nFI = FI / mean_energy) %>%
      dplyr::arrange(nFI)
  }

  calc_nfi_space <- function(curves) {
    mean_curve <- curves %>%
      dplyr::group_by(geno, year, time) %>%
      dplyr::summarise(mean_mu = mean(mu, na.rm = TRUE), .groups = "drop")

    fi_units <- curves %>%
      dplyr::left_join(mean_curve, by = c("geno", "year", "time")) %>%
      dplyr::mutate(sq_dev = (mu - mean_mu)^2) %>%
      dplyr::group_by(geno, year, location) %>%
      dplyr::summarise(
        int_sq_dev = trapz_vec(time, sq_dev),
        .groups = "drop"
      )

    fi_geno <- fi_units %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        FI_space = mean(int_sq_dev, na.rm = TRUE),
        .groups = "drop"
      )

    mean_energy <- mean_curve %>%
      dplyr::group_by(geno, year) %>%
      dplyr::summarise(
        mean_energy_space = trapz_vec(time, mean_mu^2),
        .groups = "drop"
      ) %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        mean_energy_space = mean(mean_energy_space, na.rm = TRUE),
        .groups = "drop"
      )

    fi_geno %>%
      dplyr::left_join(mean_energy, by = "geno") %>%
      dplyr::mutate(nFI_space = FI_space / mean_energy_space)
  }

  calc_nfi_time <- function(curves) {
    mean_curve <- curves %>%
      dplyr::group_by(geno, location, time) %>%
      dplyr::summarise(mean_mu = mean(mu, na.rm = TRUE), .groups = "drop")

    fi_units <- curves %>%
      dplyr::left_join(mean_curve, by = c("geno", "location", "time")) %>%
      dplyr::mutate(sq_dev = (mu - mean_mu)^2) %>%
      dplyr::group_by(geno, location, year) %>%
      dplyr::summarise(
        int_sq_dev = trapz_vec(time, sq_dev),
        .groups = "drop"
      )

    fi_geno <- fi_units %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        FI_time = mean(int_sq_dev, na.rm = TRUE),
        .groups = "drop"
      )

    mean_energy <- mean_curve %>%
      dplyr::group_by(geno, location) %>%
      dplyr::summarise(
        mean_energy_time = trapz_vec(time, mean_mu^2),
        .groups = "drop"
      ) %>%
      dplyr::group_by(geno) %>%
      dplyr::summarise(
        mean_energy_time = mean(mean_energy_time, na.rm = TRUE),
        .groups = "drop"
      )

    fi_geno %>%
      dplyr::left_join(mean_energy, by = "geno") %>%
      dplyr::mutate(nFI_time = FI_time / pmax(mean_energy_time, 1e-8)) %>%
      dplyr::mutate(nFI_time = pmax(nFI_time, 0))
  }

  out <- calc_nfi_general(curves)

  if (!is.null(env_sep)) {
    if (length(env_names) != 2) {
      stop("`env_names` must have length 2.", call. = FALSE)
    }

    curves2 <- curves %>%
      tidyr::separate(
        env,
        into = env_names,
        sep = env_sep,
        remove = FALSE
      )

    names(curves2)[names(curves2) == env_names[1]] <- "location"
    names(curves2)[names(curves2) == env_names[2]] <- "year"

    out <- out %>%
      dplyr::left_join(calc_nfi_space(curves2), by = "geno") %>%
      dplyr::left_join(calc_nfi_time(curves2), by = "geno")
  }

  class(out) <- c("fi_tbl", class(out))

  if ("geno" %in% names(out)) {
    names(out)[names(out) == "geno"] <- trt_col
  }

  if (return_curves) {
    names(curves)[names(curves) == "geno"] <- trt_col
    names(curves)[names(curves) == "env"] <- env_col
    names(curves)[names(curves) == "time"] <- time_col
    return(list(metrics = out, curves = curves))
  }

  out
}

#' @title Runs Test for Each Row and Column of a Binary Matrix
#' @description Applies the ordinary runs test to each row and column of a binary matrix individually.
#'
#' @param mat A binary matrix (containing 0s and 1s, and possibly NAs).
#' @return A list with four elements:
#' 	\item{row_results}{Data frame with test results for each row.}
#' 	\item{col_results}{Data frame with test results for each column.}
#' 	\item{row_summary}{Percentage summary of interpretation for rows.}
#' 	\item{col_summary}{Percentage summary of interpretation for columns.}
#' @export
#' @seealso \code{\link{oruns_test}}
#' @importFrom stats na.omit
#' @family Spatial analysis

oruns_test_byrowcol <- function(mat) {
  if (!is.matrix(mat) || !all(mat %in% c(0, 1, NA)))
    stop("Input must be a binary matrix (0/1), possibly with NAs.")

  apply_oruns <- function(x) {
    x <- na.omit(x)
    if (length(x) < 2 || all(x == x[1])) return(NULL)
    oruns_test(x)
  }

  build_df <- function(res) {
    if (is.null(res)) {
      return(data.frame(U = NA, EU = NA, sU = NA, Z = NA, pvalue = NA, result = NA))
    } else {
      return(data.frame(U = res$U, EU = res$EU, sU = res$sU, Z = res$Z, pvalue = res$pvalue, result = res$result))
    }
  }

  row_tests <- lapply(seq_len(nrow(mat)), function(i) apply_oruns(mat[i, ]))
  col_tests <- lapply(seq_len(ncol(mat)), function(j) apply_oruns(mat[, j]))

  row_df <- do.call(rbind, lapply(row_tests, build_df))
  col_df <- do.call(rbind, lapply(col_tests, build_df))

  row_summary <- round(prop.table(table(na.omit(row_df$result))) * 100, 2)
  col_summary <- round(prop.table(table(na.omit(col_df$result))) * 100, 2)

  list(
    row_results = row_df,
    col_results = col_df,
    row_summary = as.data.frame(row_summary),
    col_summary = as.data.frame(col_summary)
  )
}

#' Print a list of functional suppression profiles
#'
#' @param x An object of class \code{"functional_suppression_profiles_list"}.
#' @param ... Additional arguments.
#'
#' @export
print.functional_suppression_profiles_list <- function(x, ...) {
  cat("List of Functional Suppression Profiles per Environment\n")
  cat("----------------------------------------------------\n")
  cat("Environments represented (", length(x), "):\n", sep = "")
  for (name in names(x)) {
    cat("  - ", name, "\n", sep = "")
  }
  cat("\nUse x$<environment_name> to inspect an individual profile.\n")
  invisible(x)
}

#' Summarize a list of functional suppression profiles
#'
#' @param object An object of class \code{"functional_suppression_profiles_list"}.
#' @param ... Additional arguments.
#'
#' @return A list of summaries, one for each environment.
#' @export
summary.functional_suppression_profiles_list <- function(object, ...) {
  res <- lapply(object, summary, ...)
  class(res) <- "summary.functional_suppression_profiles_list"
  res
}

#' Print summary of a list of functional suppression profiles
#'
#' @param x An object of class \code{"summary.functional_suppression_profiles_list"}.
#' @param ... Additional arguments.
#'
#' @export
print.summary.functional_suppression_profiles_list <- function(x, ...) {
  for (name in names(x)) {
    cat("========================================\n")
    cat("Environment: ", name, "\n", sep = "")
    cat("========================================\n")
    print(x[[name]])
    cat("\n")
  }
  invisible(x)
}

#' Plot a list of functional suppression profiles
#'
#' @param x An object of class \code{"functional_suppression_profiles_list"}.
#' @param ... Additional arguments passed to \code{plot.functional_suppression_profiles}.
#'
#' @return A list of ggplot objects, one for each environment.
#' @export
plot.functional_suppression_profiles_list <- function(x, ...) {
  lapply(x, plot, ...)
}

#' Analysis of foci structure and dynamics (AFSD)
#'
#' This function performs the analysis of a simple method introduced by Nelson (1996) and expanded by Laranjeira et al. (1998).
#' The function assumes
#' the dataframe supplied as input has columns 'x', 'y', and 'i', where 'x' and 'y' are spatial coordinates
#' and 'i' is a disease indicator variable (1 if diseased, otherwise 0). The function performs several steps
#' including filtering rows where 'i' is 1, converting to an adjacency matrix, and creating foci using igraph.
#' It then calculates various statistics about the foci and returns these in a list.
#'
#' @param df A dataframe containing at least three columns: 'x', 'y', and 'i'. 'x' and 'y' represent spatial coordinates
#'           and 'i' is a disease indicator (1 if diseased, otherwise 0).
#'
#' @return A list containing:
#'         cluster_summary2: a dataframe summarizing the number and size of foci, and proportions of diseased plants.
#'         cluster_df: a dataframe containing foci information, including size and number of rows and columns in each foci.
#'         df_clustered: the original dataframe with an added 'focus_id' column, showing which foci each row belongs to.
#'
#' @examples
#' # Generate a sample dataframe
#' set.seed(123)
#' df <- data.frame(x = sample(1:100, 500, replace = TRUE),
#'                  y = sample(1:100, 500, replace = TRUE),
#'                  i = sample(0:1, 500, replace = TRUE, prob = c(0.7, 0.3)))
#'
#' # Perform the AFSD
#' result <- AFSD(df)
#'
#' @importFrom igraph graph_from_adjacency_matrix components
#' @importFrom tidyr pivot_longer
#' @importFrom stats dist
#'
#' @export
#' @family Spatial analysis
AFSD <- function(df) {
  # Check that the necessary columns are present in the dataframe
  if (!all(c("x", "y", "i") %in% colnames(df))) {
    stop("Dataframe must contain 'x', 'y', and 'i' columns.")
  }

  # Check for required packages
  if (!requireNamespace("igraph", quietly = TRUE)) {
    stop("The 'igraph' package is required. Please install it with install.packages('igraph').")
  }
  if (!requireNamespace("tidyr", quietly = TRUE)) {
    stop("The 'tidyr' package is required. Please install it with install.packages('tidyr').")
  }

  # Filter for rows where i == 1
  df_filtered <- df[df$i == 1, c("x", "y")]

  # Check that filtered dataframe is not empty
  if (nrow(df_filtered) == 0) {
    stop("There are no rows where i == 1.")
  }

  # Convert to adjacency matrix
  adj <- as.matrix(dist(df_filtered))

  # Threshold the adjacency matrix
  adj_thresholded <- adj <= sqrt(2)  # Consider cells touching diagonally as well

  # Create graph and find clusters
  g <- igraph::graph_from_adjacency_matrix(adj_thresholded, mode = "undirected")
  clusters <- igraph::components(g)

  # Calculate proportion of diseased plants
  prop_diseased <- sum(df$i) / nrow(df)

  # Calculate the size of each cluster
  cluster_sizes <- table(clusters$membership)

  # Create a dataframe with cluster information and sizes
  cluster_df <- data.frame(
    focus_id = names(cluster_sizes),
    size = as.integer(cluster_sizes)
  )

  cluster_summary <- data.frame(
    NF = length(cluster_sizes),
    NF1000 = length(cluster_sizes) * 1000 / nrow(df),
    NSF = sum(cluster_sizes == 1),
    NSF1000 = sum(cluster_sizes == 1) * 1000 / nrow(df),
    DIS_INC = prop_diseased
  )

  # Add columns for the number of rows and columns in each cluster
  cluster_df$rows <- sapply(cluster_df$focus_id, function(id) {
    nodes_in_cluster <- which(clusters$membership == as.integer(id))
    max(df_filtered[nodes_in_cluster, 'y']) - min(df_filtered[nodes_in_cluster, 'y']) + 1
  })
  cluster_df$cols <- sapply(cluster_df$focus_id, function(id) {
    nodes_in_cluster <- which(clusters$membership == as.integer(id))
    max(df_filtered[nodes_in_cluster, 'x']) - min(df_filtered[nodes_in_cluster, 'x']) + 1
  })

  cluster_df$SIF <- cluster_df$rows / cluster_df$cols
  cluster_df$CIF <- cluster_df$size / (cluster_df$rows * cluster_df$cols)

  # Add mean indices for shape and compactness
  cluster_summary$mean_SIF <- mean(cluster_df$cols / cluster_df$rows)
  cluster_summary$mean_CIF <- mean(cluster_df$size / (cluster_df$rows * cluster_df$cols))

  # Pivot the summary table
  cluster_summary2 <- tidyr::pivot_longer(cluster_summary, cols = 1:7, names_to = "stats", values_to = "value")

  # Add cluster IDs to filtered df
  df_clustered <- df[df$i == 1, ]
  df_clustered$focus_id <- clusters$membership

  return(list(cluster_summary2, cluster_df, df_clustered))
}

#' Plot functional suppression profiles
#'
#' @param x An object of class \code{"functional_suppression_profiles"}.
#' @param type Character string specifying the plot type. One of \code{"dendrogram"}, \code{"profiles"}, \code{"heatmap"}, \code{"rank"}, or \code{"all"}.
#' @param show_cut Logical; whether to display the cluster cut height in dendrogram. Default is \code{TRUE}.
#' @param show_points Logical; whether to overlay observed DSP values as points in the profiles plot. Default is \code{FALSE}.
#' @param show_points Logical; whether to overlay observed DSP values as points in the profiles plot. Default is \code{FALSE}.
#' @param ... Additional arguments passed to specific plot functions.
#'
#' @details 
#' The plot method visualizes the components of a Functional Suppression Profile. 
#' The dendrogram defines the Functional Suppression Profiles based on temporal similarity. 
#' The heatmap is ordered by the dendrogram to help interpret these functional groups 
#' using summary metrics.
#'
#' @return A \code{ggplot} object or a list of \code{ggplot} objects.
#'
#' @export
plot.functional_suppression_profiles <- function(
    x,
    type = c("dendrogram", "profiles", "heatmap", "rank", "all"),
    show_cut = TRUE,
    show_points = FALSE,
    ...
) {
  type <- match.arg(type)
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.")
  }
  
  # Get dendrogram order
  hc <- x$hclust
  dend_order <- hc$labels[hc$order]

  # Dendrogram
  p_dendrogram <- function() {
    p <- plot_dendrogram(x$distances, show_cut = show_cut, palette = x$cluster_palette, ...)
    p + ggplot2::labs(y = "Functional distance")
  }
  
  # Profiles
  p_profiles <- function() {
    df_smooth <- x$dsp_curves$curves
    df_obs <- x$contrast
    time_var <- x$parameters$time
    trt_var <- x$parameters$treatment
    
    if (!time_var %in% names(df_smooth) || !trt_var %in% names(df_smooth)) {
      stop("Time or treatment variables not found in smoothed curves data.")
    }
    
      df_smooth[[trt_var]] <- factor(df_smooth[[trt_var]], levels = dend_order)
      df_smooth <- df_smooth |> dplyr::left_join(x$classification |> dplyr::select(dplyr::all_of(c(trt_var, "profile"))), by = trt_var)
      if (show_points) {
        df_obs[[trt_var]] <- factor(df_obs[[trt_var]], levels = dend_order)
        df_obs <- df_obs |> dplyr::left_join(x$classification |> dplyr::select(dplyr::all_of(c(trt_var, "profile"))), by = trt_var)
      }
      
      curve_labels <- df_smooth |>
        dplyr::group_by(.data[[trt_var]]) |>
        dplyr::slice_max(.data[[time_var]], n = 1, with_ties = FALSE) |>
        dplyr::ungroup()
        
      p <- ggplot2::ggplot(df_smooth, ggplot2::aes(
        x = .data[[time_var]], 
        y = .data[["mu"]], 
        color = .data[["profile"]], 
        group = .data[[trt_var]]
      )) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray50") +
      ggplot2::geom_line(linewidth = 0.8, alpha = 0.85, show.legend = FALSE)
      
    if (requireNamespace("ggrepel", quietly = TRUE)) {
      p <- p + ggrepel::geom_text_repel(
        data = curve_labels,
        ggplot2::aes(label = .data[[trt_var]]),
        nudge_x = 0.5,
        direction = "y",
        hjust = 0,
        segment.size = 0.2,
        segment.alpha = 0.7,
        min.segment.length = 0,
        box.padding = 0.1,
        point.padding = 0.15,
        show.legend = FALSE,
        size = 2.8
      )
    } else {
      p <- p + ggplot2::geom_text(
        data = curve_labels,
        ggplot2::aes(label = .data[[trt_var]]),
        hjust = -0.1,
        size = 2.8,
        show.legend = FALSE
      )
    }
    
    p <- p +
      ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.02, 0.18))) +
      ggplot2::coord_cartesian(clip = "off") +
      ggplot2::theme_minimal() +
      ggplot2::scale_color_manual(values = x$cluster_palette) +
      ggplot2::labs(
        x = "Time",
        y = "Disease suppression",
        color = "Functional Profile"
      )
      
    if (show_points) {
      p <- p + ggplot2::geom_point(
        data = df_obs, 
        ggplot2::aes(x = .data[[time_var]], y = .data[["DSP"]]),
        alpha = 0.5,
        show.legend = FALSE
      )
    }
    p
  }
  
  # Heatmap
  p_heatmap <- function() {
    if (!requireNamespace("tidyr", quietly = TRUE)) {
      stop("Package 'tidyr' is required for heatmap plot.")
    }
    
    rank_df <- x$ranking
    trt_var <- x$parameters$treatment
    n_treatments <- nrow(rank_df)
    
    rank_df[[trt_var]] <- factor(
      rank_df[[trt_var]], 
      levels = rev(dend_order)
    )
    
    rank_long <- rank_df |>
      tidyr::pivot_longer(
        cols = -dplyr::all_of(trt_var),
        names_to = "metric",
        values_to = "rank"
      )
      
    metric_levels <- c("protected_area", "max_suppression", "persistence", "centroid", "mean_rank")
    metric_labels <- c("Area", "Maximum", "Persistence", "Timing", "Rank")
    
    rank_long <- rank_long |>
      dplyr::mutate(
        metric = gsub("^rank_", "", .data[["metric"]]),
        metric = factor(.data[["metric"]], levels = metric_levels, labels = metric_labels)
      )
      
    y_clusters <- x$classification$profile[match(rev(dend_order), x$classification[[trt_var]])]
    y_colors <- x$cluster_palette[as.character(y_clusters)]
    
    ggplot2::ggplot(rank_long, ggplot2::aes(
      x = .data[["metric"]], 
      y = .data[[trt_var]], 
      fill = .data[["rank"]]
    )) +
      ggplot2::geom_tile(color = "white", linewidth = 0.5) +
      ggplot2::scale_fill_gradient(low = "white", high = "steelblue", limits = c(1, n_treatments)) + 
      ggplot2::theme_minimal() +
      ggplot2::labs(
        x = NULL,
        y = "Treatment",
        fill = "Rank"
      ) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 11),
        axis.text.y = ggplot2::element_text(size = 11, color = y_colors)
      )
  }
  
  # Rank
  p_rank <- function() {
    rank_df <- x$classification
    trt_var <- x$parameters$treatment
    
    if (!"mean_rank" %in% names(rank_df)) {
      stop("mean_rank not found in classification data.")
    }
    
    rank_df[[trt_var]] <- factor(
      rank_df[[trt_var]], 
      levels = rank_df[[trt_var]][order(rank_df[["mean_rank"]], decreasing = TRUE)]
    )
    
    ggplot2::ggplot(rank_df, ggplot2::aes(
      x = .data[["mean_rank"]], 
      y = .data[[trt_var]],
      fill = .data[["profile"]]
    )) +
      ggplot2::geom_col(show.legend = FALSE) +
      ggplot2::scale_fill_manual(values = x$cluster_palette) +
      ggplot2::theme_minimal() +
      ggplot2::labs(
        x = "Mean Rank",
        y = "Treatment"
      )
  }
  
  if (type == "dendrogram") return(p_dendrogram())
  if (type == "profiles") return(p_profiles())
  if (type == "heatmap") return(p_heatmap())
  if (type == "rank") return(p_rank())
  
  if (type == "all") {
    if (!requireNamespace("patchwork", quietly = TRUE)) {
      message("Package 'patchwork' is required to combine plots. Returning a list of plots instead.")
      out <- list(
        dendrogram = p_dendrogram(),
        profiles = p_profiles(),
        heatmap = p_heatmap()
      )
      return(out)
    } else {
      p1 <- p_profiles()
      p2 <- p_dendrogram()
      p3 <- p_heatmap()
      p4 <- p_rank()
      
      fig <- (p1 | p2) / (p3 | p4)
      
      return(fig + patchwork::plot_annotation(
        tag_levels = "A"
      ))
    }
  }
}

#' Compute pairwise functional distances and cluster curves
#'
#' @description
#' Computes L2 functional distances among treatment mean trajectories evaluated on a
#' common time grid from a \code{functional_curves} object. Also computes curve-level
#' distances, performs hierarchical clustering, and optionally performs permutation testing.
#'
#' @param object An object of class \code{functional_curves}.
#' @param cluster_k Number of clusters used to cut the hierarchical tree.
#' @param hc_method Clustering linkage method passed to \code{stats::hclust()}.
#' @param test_factor Optional a priori grouping used for a distance-based permutation test.
#' @param n_perm Number of permutations for the distance-based test.
#' @param test_mode Character; permutation test mode.
#' @param min_strata_for_pairwise Minimum number of strata levels required for pairwise tests.
#' @param perm_unit Optional character naming the unit at which group labels are permuted.
#' @param perm_strata Optional character naming a factor defining restricted permutation strata.
#' @param bootstrap Logical; if \code{TRUE}, compute bootstrap envelopes and distance summaries.
#' @param boot_B Integer number of bootstrap replicates.
#' @param boot_seed Integer seed for bootstrap resampling.
#' @param boot_ci Length-2 numeric vector of quantiles for bootstrap intervals.
#' @param show_progress Logical; whether to show progress.
#' @param curve_level Logical; whether to compute curve-level distance matrix.
#' @param ... Additional arguments.
#'
#' @return An object of class \code{"functional_distances"} containing:
#' \itemize{
#'   \item \code{distance}: treatment-by-treatment functional distance matrix;
#'   \item \code{distance_table}: pairwise distances as a data frame;
#'   \item \code{hc}: \code{hclust} object;
#'   \item \code{clusters}: treatment cluster assignments;
#'   \item \code{curve_distance}: curve-by-curve distance matrix;
#'   \item \code{test}: permutation test results;
#'   \item \code{bootstrap}: bootstrap results if requested.
#' }
#'
#' @examples
#' \dontrun{
#' fd <- functional_distances(fc, cluster_k = 4)
#' print(fd)
#' plot_dendrogram(fd)
#' plot_curves(fd)
#' }
#' @export
functional_distances <- function(
    object,
    cluster_k = 4,
    hc_method = "ward.D2",
    test_factor = NULL,
    n_perm = 999,
    test_mode = c("auto","none","global","global_pairwise"),
    min_strata_for_pairwise = 6,
    perm_unit = NULL,
    perm_strata = NULL,
    bootstrap = FALSE,
    boot_B = 399,
    boot_seed = 1,
    boot_ci = c(0.025, 0.975),
    show_progress = TRUE,
    curve_level = NULL,
    ...
) {
  if (!inherits(object, "functional_curves")) {
    stop("`object` must be of class 'functional_curves'")
  }

  test_mode <- match.arg(test_mode)

  df <- object$observed_data
  m_gam <- object$gam
  t_grid <- object$grid
  pred_trt <- object$curves
  trt_score <- object$scores

  .time <- object$vars$time
  .trt <- object$vars$treatment
  .env <- object$vars$environment
  .blk <- object$vars$block
  unit_used <- object$vars$unit

  trt_levels0 <- levels(df[[.trt]])
  unit_levels0 <- levels(df[[unit_used]])
  env_levels0 <- if (!is.null(.env)) levels(df[[.env]]) else NULL
  blk_levels0 <- if (!is.null(.blk)) levels(df[[.blk]]) else NULL

  # ---- functional distance + clustering (treatment means) ----
  mat <- pred_trt |>
    dplyr::select(.data[[.time]], .data[[.trt]], .data[["mu"]]) |>
    tidyr::pivot_wider(names_from = .data[[.trt]], values_from = .data[["mu"]]) |>
    dplyr::arrange(.data[[.time]])

  dt_grid <- mean(diff(mat[[.time]]))
  X <- as.matrix(mat[, setdiff(names(mat), .time), drop = FALSE])

  # L2 distance via trapezoidal (dt_grid scaling is proportional to trapezoidal for regular grid)
  D <- as.matrix(stats::dist(t(X), method = "euclidean")) * sqrt(dt_grid)
  
  # Distance table
  D_dist <- stats::as.dist(D)
  D_mat <- as.matrix(D_dist)
  dist_df <- as.data.frame(as.table(D_mat))
  names(dist_df) <- c("group1", "group2", "distance")
  dist_df <- dist_df[as.character(dist_df$group1) < as.character(dist_df$group2), ]
  rownames(dist_df) <- NULL

  hc <- stats::hclust(stats::as.dist(D), method = hc_method)
  
  cl_raw <- stats::cutree(hc, k = min(cluster_k, length(hc$order)))

  cluster_tbl <- tibble::tibble(
    !!.trt := names(cl_raw),
    cluster_raw = as.integer(cl_raw)
  ) |>
    dplyr::left_join(trt_score, by = .trt) |>
    dplyr::group_by(.data[["cluster_raw"]]) |>
    dplyr::summarise(cluster_mean = mean(.data[["mean_mu"]], na.rm = TRUE), .groups = "drop") |>
    dplyr::arrange(.data[["cluster_mean"]]) |>
    dplyr::mutate(cluster = dplyr::row_number()) |>
    dplyr::select(.data[["cluster_raw"]], .data[["cluster"]])

  trt_cluster <- tibble::tibble(
    !!.trt := names(cl_raw),
    cluster_raw = as.integer(cl_raw)
  ) |>
    dplyr::left_join(cluster_tbl, by = "cluster_raw") |>
    dplyr::left_join(trt_score, by = .trt)

  # ---- Curve-level fitted trajectories (exclude unit RE) + distance matrix ----
  D_curve <- NULL
  new_curve_grid <- NULL
  
  if (is.null(curve_level)) {
    curve_level <- !is.null(test_factor) || isTRUE(bootstrap)
  }

  if (isTRUE(curve_level)) {
    col_name1 <- function(x){
      if (is.null(x)) return(NULL)
      if (is.character(x) && length(x) == 1L) return(x)
      if (rlang::is_quosure(x)) return(rlang::as_name(rlang::quo_get_expr(x)))
      if (rlang::is_symbol(x))  return(rlang::as_name(x))
      if (is.list(x) && length(x) == 1L) return(col_name1(x[[1]]))
      stop("Could not coerce column spec to a single name. Got: ", paste(class(x), collapse = "/"))
    }

    env_col <- col_name1(.env)
    blk_col <- col_name1(.blk)
    ab_cols <- c(env_col, blk_col)
    ab_cols <- ab_cols[!is.na(ab_cols) & nzchar(ab_cols)]

    curve_meta <- df |>
      dplyr::distinct(.data[[unit_used]], .data[[.trt]], dplyr::across(dplyr::all_of(ab_cols))) |>
      dplyr::mutate(
        curve_id_chr = as.character(.data[[unit_used]]),
        trt_chr      = as.character(.data[[.trt]])
      )

    dup <- curve_meta |>
      dplyr::count(curve_id_chr) |>
      dplyr::filter(n > 1)
    if (nrow(dup) > 0) {
      stop("Each curve (unit) must map to one treatment/block/env. Duplicates for: ",
           paste(dup$curve_id_chr, collapse = ", "))
    }

    new_curve_grid <- tidyr::crossing(
      curve_id_chr = curve_meta$curve_id_chr,
      !!.time := t_grid
    ) |>
      dplyr::left_join(curve_meta, by = "curve_id_chr") |>
      dplyr::mutate(
        !!unit_used := factor(curve_id_chr, levels = unit_levels0),
        !!.trt      := factor(trt_chr, levels = trt_levels0)
      )

    if (!is.null(.env) && .env %in% names(new_curve_grid)) {
      new_curve_grid[[.env]] <- factor(new_curve_grid[[.env]], levels = env_levels0)
    }
    if (!is.null(.blk) && .blk %in% names(new_curve_grid)) {
      new_curve_grid[[.blk]] <- factor(new_curve_grid[[.blk]], levels = blk_levels0)
    }

    excl_c <- character()
    if(any(grepl(sprintf("s\\(%s\\)", unit_used), names(m_gam$smooth), fixed = TRUE))) {
       excl_c <- c(excl_c, sprintf("s(%s)", unit_used))
    }

    new_curve_grid$y_hat <- as.numeric(
      mgcv::predict.gam(
        m_gam,
        newdata = new_curve_grid,
        type = "response",
        exclude = excl_c
      )
    )

    Yhat <- new_curve_grid |>
      dplyr::select(curve_id_chr, !!.time, y_hat) |>
      tidyr::pivot_wider(names_from = !!.time, values_from = y_hat) |>
      dplyr::arrange(curve_id_chr)

    curve_ids <- Yhat$curve_id_chr
    Ymat <- as.matrix(Yhat[, -1, drop = FALSE])

    D_curve <- as.matrix(stats::dist(Ymat, method = "euclidean")) * sqrt(dt_grid)
    rownames(D_curve) <- curve_ids
    colnames(D_curve) <- curve_ids

    D_curve <- D_curve / stats::median(D_curve[D_curve > 0])
  }

  # ---- Permutation test ----
  test_res <- NULL
  if (!is.null(test_factor) && test_mode != "none") {
    if (is.null(perm_strata)) perm_strata <- .blk
    perm_unit_eff <- if (is.null(perm_unit)) unit_used else perm_unit

    if (!perm_unit_eff %in% names(df)) {
      stop("`perm_unit` (or default unit) not found in data: ", perm_unit_eff)
    }

    permute_within_strata <- function(g, strata) {
      gperm <- g
      for (s in unique(strata)) {
        idx <- which(strata == s)
        gperm[idx] <- sample(g[idx], length(idx), replace = FALSE)
      }
      gperm
    }

    permanova_F <- function(Dmat, grp) {
      n <- nrow(Dmat)
      grp <- droplevels(factor(grp))
      k <- nlevels(grp)
      if (n < 3 || k < 2) return(list(F = NA_real_, df1 = NA_integer_, df2 = NA_integer_))
      df1 <- k - 1
      df2 <- n - k
      if (df2 <= 0) return(list(F = NA_real_, df1 = df1, df2 = df2))

      A <- -0.5 * (Dmat^2)
      H <- diag(n) - matrix(1, n, n)/n
      G <- H %*% A %*% H
      Xg <- stats::model.matrix(~ grp)
      P  <- Xg %*% solve(t(Xg) %*% Xg) %*% t(Xg)

      SS_between <- sum(diag(P %*% G))
      SS_total   <- sum(diag(G))
      SS_within  <- SS_total - SS_between
      Fstat <- (SS_between/df1) / (SS_within/df2)
      list(F = as.numeric(Fstat), df1 = df1, df2 = df2)
    }

    if (is.character(test_factor) && length(test_factor) == 1L) {
      tf_col <- test_factor
      if (!tf_col %in% names(df)) stop("`test_factor` not found in data: ", tf_col)

      map_u <- df |>
        dplyr::distinct(.data[[perm_unit_eff]], .data[[tf_col]], .keep_all = FALSE) |>
        dplyr::filter(!is.na(.data[[tf_col]])) |>
        dplyr::mutate(u_chr = as.character(.data[[perm_unit_eff]]))

      dup <- map_u |> dplyr::count(u_chr) |> dplyr::filter(n > 1)
      if (nrow(dup) > 0) {
        stop("`test_factor` must be unique per `perm_unit`. Duplicates for: ",
             paste(dup$u_chr, collapse = ", "))
      }

      g_u <- map_u[[tf_col]]
      names(g_u) <- map_u$u_chr

    } else {
      g_u <- test_factor
      if (is.null(names(g_u))) stop("If `test_factor` is a vector, it must be named by `perm_unit` ids.")
    }

    map_curve_to_u <- df |>
      dplyr::distinct(.data[[unit_used]], .data[[perm_unit_eff]], .keep_all = FALSE) |>
      dplyr::mutate(curve_id_chr = as.character(.data[[unit_used]]),
                    u_chr = as.character(.data[[perm_unit_eff]]))

    dup2 <- map_curve_to_u |> dplyr::count(curve_id_chr) |> dplyr::filter(n > 1)
    if (nrow(dup2) > 0) stop("Each curve must map to one perm_unit. Duplicates for: ", paste(dup2$curve_id_chr, collapse = ", "))

    curve_ids2 <- rownames(D_curve)
    u_curve <- map_curve_to_u$u_chr
    names(u_curve) <- map_curve_to_u$curve_id_chr
    u_curve <- u_curve[curve_ids2]

    g_curve <- g_u[u_curve]
    g_curve <- droplevels(factor(g_curve))

    strata_curve <- NULL
    if (!is.null(perm_strata)) {
      if (!perm_strata %in% names(df)) stop("`perm_strata` not found in data: ", perm_strata)
      map_curve_to_s <- df |>
        dplyr::distinct(.data[[unit_used]], .data[[perm_strata]], .keep_all = FALSE) |>
        dplyr::mutate(curve_id_chr = as.character(.data[[unit_used]]))
      
      strata_curve <- map_curve_to_s[[perm_strata]]
      names(strata_curve) <- map_curve_to_s$curve_id_chr
      strata_curve <- droplevels(factor(strata_curve[curve_ids2]))
    }

    pairwise_allowed <- TRUE
    if (test_mode == "auto") {
      if (!is.null(strata_curve)) {
        n_strata <- nlevels(strata_curve)
        if (n_strata < min_strata_for_pairwise) pairwise_allowed <- FALSE
      }
    }
    if (test_mode == "global") pairwise_allowed <- FALSE
    if (test_mode == "global_pairwise") pairwise_allowed <- TRUE

    obs <- permanova_F(D_curve, g_curve)
    set.seed(1)
    
    if (show_progress) {
      pb_glob <- progress::progress_bar$new(
        format = "  Global permutation test [:bar] :percent eta: :eta",
        total = n_perm, clear = FALSE, width = 60, force = TRUE, show_after = 0)
    }
    
    F_perm <- numeric(n_perm)
    for (i in seq_len(n_perm)) {
      if (show_progress) pb_glob$tick()
      gperm <- if (is.null(strata_curve)) {
        sample(g_curve, replace = FALSE)
      } else {
        permute_within_strata(g_curve, strata_curve)
      }
      F_perm[i] <- permanova_F(D_curve, gperm)$F
    }
    
    p_global <- (1 + sum(F_perm >= obs$F, na.rm = TRUE)) / (n_perm + 1)

    pairwise <- NULL
    lev <- levels(g_curve)
    if (pairwise_allowed && length(lev) > 2) {
      pairs <- utils::combn(lev, 2, simplify = FALSE)
      
      if (show_progress) {
        pb_pair <- progress::progress_bar$new(
          format = "  Pairwise permutation tests [:bar] :percent eta: :eta",
          total = length(pairs) * n_perm, clear = FALSE, width = 60, force = TRUE, show_after = 0)
      }

      pairwise <- purrr::map_dfr(pairs, function(pp) {
        keep2 <- g_curve %in% pp
        D2 <- D_curve[keep2, keep2, drop = FALSE]
        g2 <- droplevels(g_curve[keep2])
        s2 <- if (is.null(strata_curve)) NULL else droplevels(strata_curve[keep2])

        ob2 <- permanova_F(D2, g2)
        
        Fp2 <- numeric(n_perm)
        for (j in seq_len(n_perm)) {
          if (show_progress) pb_pair$tick()
          gp <- if (is.null(s2)) {
            sample(g2, replace = FALSE)
          } else {
            permute_within_strata(g2, s2)
          }
          Fp2[j] <- permanova_F(D2, gp)$F
        }
        
        p2 <- (1 + sum(Fp2 >= ob2$F, na.rm = TRUE)) / (n_perm + 1)
        tibble::tibble(group1 = pp[1], group2 = pp[2], F = ob2$F, p = p2)
      }) |>
        dplyr::mutate(p_adj = stats::p.adjust(p, method = "holm"))
    }

    test_res <- list(
      type = "permutation_distance_test",
      mode = test_mode,
      perm_unit = perm_unit_eff,
      perm_strata = perm_strata,
      n_perm = n_perm,
      groups = g_curve,
      strata = strata_curve,
      global = c(F = obs$F, p = p_global, df1 = obs$df1, df2 = obs$df2),
      pairwise = pairwise,
      perm_F = F_perm
    )
  }

  # ---- Bootstrap ----
  boot_res <- NULL
  if (isTRUE(bootstrap)) {
    set.seed(boot_seed)
    if(!is.null(.blk)) {
      ids0 <- blk_levels0
      id_col <- .blk
      boot_unit_label <- "block"
    } else {
      ids0 <- unit_levels0
      id_col <- unit_used
      boot_unit_label <- "unit"
    }

    if (show_progress) {
      pb_boot <- progress::progress_bar$new(
        format = "  Bootstrapping predictions [:bar] :percent eta: :eta",
        total = boot_B, clear = FALSE, width = 60, force = TRUE, show_after = 0)
    }

    boot_pred_list <- vector("list", boot_B)

    for (b in seq_len(boot_B)) {
      if (show_progress) pb_boot$tick()
      ids_b <- sample(ids0, replace = TRUE)

      w_draw <- tibble::tibble(id = factor(ids_b, levels = ids0)) |>
        dplyr::count(id, name = "w")

      grid_b <- new_curve_grid |>
        dplyr::mutate(id = factor(.data[[id_col]], levels = ids0)) |>
        dplyr::left_join(w_draw, by = "id") |>
        dplyr::filter(!is.na(.data[["w"]]))

      boot_pred_list[[b]] <- grid_b |>
        dplyr::group_by(.data[[.trt]], .data[[.time]]) |>
        dplyr::summarise(
          mu = stats::weighted.mean(.data[["y_hat"]], w = .data[["w"]], na.rm = TRUE),
          .groups = "drop"
        ) |>
        dplyr::mutate(.iter = b)
    }

    boot_pred <- dplyr::bind_rows(boot_pred_list)
    trt_order <- object$scores[[.trt]]
    env_tbl <- boot_pred |>
      dplyr::group_by(.data[[.trt]], .data[[.time]]) |>
      dplyr::summarise(
        mu_boot_mean = mean(.data[["mu"]], na.rm = TRUE),
        mu_lo = stats::quantile(.data[["mu"]], probs = boot_ci[1], na.rm = TRUE),
        mu_hi = stats::quantile(.data[["mu"]], probs = boot_ci[2], na.rm = TRUE),
        .groups = "drop"
      ) |>
      dplyr::mutate(!!.trt := factor(.data[[.trt]], levels = trt_order))

    boot_dist_one_iter <- function(df_iter) {
      mat_i <- df_iter |>
        dplyr::select(.data[[.time]], .data[[.trt]], .data[["mu"]]) |>
        tidyr::pivot_wider(names_from = .data[[.trt]], values_from = .data[["mu"]]) |>
        dplyr::arrange(.data[[.time]])

      X_i <- as.matrix(mat_i[, setdiff(names(mat_i), .time), drop = FALSE])
      coln <- colnames(X_i)
      if (ncol(X_i) < 2) return(NULL)

      pairs <- utils::combn(coln, 2, simplify = FALSE)
      purrr::map_dfr(pairs, function(pp){
        d <- sqrt(sum((X_i[, pp[1]] - X_i[, pp[2]])^2) * dt_grid)
        tibble::tibble(group1 = pp[1], group2 = pp[2], dist = as.numeric(d))
      })
    }

    if (show_progress) {
      pb_dist <- progress::progress_bar$new(
        format = "  Calculating bootstrap distances [:bar] :percent eta: :eta",
        total = boot_B, clear = FALSE, width = 60, force = TRUE, show_after = 0)
    }

    dist_boot <- boot_pred |>
      dplyr::group_by(.iter) |>
      dplyr::group_modify(~ {
        if (show_progress) pb_dist$tick()
        boot_dist_one_iter(.x)
      }) |>
      dplyr::ungroup()

    dist_summary <- dist_boot |>
      dplyr::group_by(.data[["group1"]], .data[["group2"]]) |>
      dplyr::summarise(
        dist_mean = mean(.data[["dist"]], na.rm = TRUE),
        dist_med  = stats::median(.data[["dist"]], na.rm = TRUE),
        dist_lo   = stats::quantile(.data[["dist"]], probs = boot_ci[1], na.rm = TRUE),
        dist_hi   = stats::quantile(.data[["dist"]], probs = boot_ci[2], na.rm = TRUE),
        .groups = "drop"
      ) |>
      dplyr::arrange(dplyr::desc(.data[["dist_mean"]]))

    boot_res <- list(
      method = "bootstrap_on_predicted_curves",
      unit = boot_unit_label,
      id_col = id_col,
      B = boot_B,
      seed = boot_seed,
      ci = boot_ci,
      envelopes = env_tbl,
      distances = dist_summary,
      distances_boot = dist_boot
    )
  }

  out <- list(
    distance_matrix = D,
    distance_table = dist_df,
    hc = hc,
    clusters = trt_cluster,
    curve_distance = D_curve,
    test = test_res,
    bootstrap = boot_res,
    functional_curves = object
  )
  class(out) <- "functional_distances"
  out
}

#' Print functional_distances
#' @param x An object of class \code{"functional_distances"}.
#' @param ... Additional arguments.
#' @export
print.functional_distances <- function(x, ...) {
  cat("A functional_distances object\n")
  cat("Dimensions of distance matrix:", nrow(x$distance_matrix), "x", ncol(x$distance_matrix), "\n")
  cat("Number of clusters:", length(unique(x$clusters$cluster)), "\n")
  invisible(x)
}

#' Survival analysis for quantitative ordinal scale data
#'
#' @param dat Data frame containing the data to be processed.
#' @param scale A numeric vector indicating the scale or order of classes.
#' @param grade Logical. If TRUE, uses the class value. If FALSE, uses the NPE (Non-Parametric Estimate).
#' @param ckData Logical. If TRUE, returns the input data along with the results.
#'   If FALSE, returns only the results.
#'
#' @return A list containing the score statistic, hypothesis tests, adjusted significance level,
#'   and conclusion based on pairwise comparisons.
#'
#' @details
#' This function supports analysis of quantitative ordinal scale data via interval-censored methods.
#' The approach follows the workflow described by Chiang et al. (2023) and the reference implementation
#' provided in the CompMuCens repository.
#'
#' @references
#' Chiang, K.S., Chang, Y.M., Liu, H.I., Lee, J.Y., El Jarroudi, M. and Bock, C. (2023).
#' Survival Analysis as a Basis to Test Hypotheses When Using Quantitative Ordinal Scale Disease Severity Data.
#' Phytopathology. <https://apsjournals.apsnet.org/doi/abs/10.1094/PHYTO-02-23-0055-R>
#'
#' @examples
#' if (requireNamespace("interval", quietly = TRUE)) {
#'   trAs <- c(5,4,2,5,5,4,4,2,5,2,2,3,4,3,2,2,6,2,2,4,2,4,2,4,5,3,4,2,2,3)
#'   trBs <- c(5,3,2,4,4,5,4,5,4,4,6,4,5,5,5,2,6,2,3,5,2,6,4,3,2,5,3,5,4,5)
#'   trCs <- c(2,3,1,4,1,1,4,1,1,3,2,1,4,1,1,2,5,2,1,3,1,4,2,2,2,4,2,3,2,2)
#'   trDs <- c(5,5,4,5,5,6,6,4,6,4,3,5,5,6,4,6,5,6,5,4,5,5,5,3,5,6,5,5,5,6)
#'   inputData <- data.frame(
#'     treatment = c(rep("A",30), rep("B",30), rep("C",30), rep("D",30)),
#'     x = c(trAs, trBs, trCs, trDs)
#'   )
#'   CompMuCens(dat = inputData,
#'              scale = c(0,3,6,12,25,50,75,88,94,97,100,100),
#'              ckData = TRUE)
#' }
#'
#' @export
#' @family Disease quantification

CompMuCens <- function(dat, scale, grade = TRUE, ckData = FALSE) {

  if (!requireNamespace("interval", quietly = TRUE)) {
    stop(
      "CompMuCens() requires the CRAN package 'interval'.\n",
      "Note: 'interval' depends on Bioconductor 'Icens'. Install with:\n",
      "  install.packages('BiocManager')\n",
      "  BiocManager::install('Icens')\n",
      "  install.packages('interval')",
      call. = FALSE
    )
  }
  # Converting data into a censored data format
  if (grade) {
    if (scale[1] == 0) {
      if (scale[length(scale) - 1] == 100) {
        Scale <- unique(scale)
        dat$Slow <- ifelse(dat$x == 1, 0,
                           ifelse(dat$x == length(scale), 100,
                                  suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[x - 1])))))
        dat$Sup <- ifelse(dat$x == 1, 0,
                          ifelse(dat$x == length(scale), 100,
                                 suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[x])))))
      } else {
        Scale <- scale
        dat$Slow <- ifelse(dat$x == 1, 0,
                           suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[x - 1]))))
        dat$Sup <- ifelse(dat$x == 1, 0,
                          suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[x]))))
      }
    } else {
      if (scale[length(scale) - 1] == 100) {
        Scale <- c(0, unique(scale))
        dat$Slow <- ifelse(dat$x == length(scale), 100,
                           suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[x]))))
        dat$Sup <- ifelse(dat$x == length(scale), 100,
                          suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[x + 1]))))
      } else {
        if (scale[length(scale)] == 100) {
          Scale <- c(0, scale)
        } else {
          Scale <- c(0, scale, 100)
        }
        dat$Slow <- as.numeric(lapply(dat$x, function(x) Scale[x]))
        dat$Sup <- as.numeric(lapply(dat$x, function(x) Scale[x + 1]))
      }
    }
    outPrintD <- dplyr::rename(dat, ClassValue = x)
    dat <- dplyr::select(dat, treatment, Slow, Sup)
  } else {
    if (scale[1] == 0) {
      if (scale[length(scale) - 1] == 100) {
        Scale <- unique(scale)
        dat$Slow <- ifelse(!dat$x %in% c(0, 100),
                           suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[max(which(x > Scale))]))),
                           dat$x)
        dat$Sup <- as.numeric(lapply(dat$x, function(x) Scale[min(which(x <= Scale))]))
      } else {
        Scale <- scale
        dat$Slow <- ifelse(dat$x != 0,
                           suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[max(which(x > Scale))]))),
                           dat$x)
        dat$Sup <- as.numeric(lapply(dat$x, function(x) Scale[min(which(x <= Scale))]))
      }
    } else {
      if (scale[length(scale) - 1] == 100) {
        Scale <- c(0, unique(scale))
        dat$Slow <- ifelse(!dat$x %in% c(0, 100),
                           suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[max(which(x > Scale))]))),
                           dat$x)
        dat$Sup <- ifelse(dat$x != 0,
                          as.numeric(lapply(dat$x, function(x) Scale[min(which(x <= Scale))])),
                          Scale[2])
      } else {
        if (scale[length(scale)] == 100) {
          Scale <- c(0, scale)
        } else {
          Scale <- c(0, scale, 100)
        }
        dat$Slow <- ifelse(dat$x != 0,
                           suppressWarnings(as.numeric(lapply(dat$x, function(x) Scale[max(which(x > Scale))]))),
                           dat$x)
        dat$Sup <- ifelse(dat$x != 0,
                          as.numeric(lapply(dat$x, function(x) Scale[min(which(x <= Scale))])),
                          Scale[2])
      }
    }
    outPrintD <- dplyr::rename(dat, MidPoint = x)
    dat <- dplyr::select(dat, treatment, Slow, Sup)
  }

  outPrintD$intervals <- as.character(survival::Surv(outPrintD$Slow, outPrintD$Sup, type = "interval2"))
  outPrintD <- dplyr::select(outPrintD, -Slow, -Sup)

  mAll <- interval::ictest(survival::Surv(Slow, Sup, type = "interval2") ~ treatment, scores = "wmw", data = dat)

  anaD <- dat |>
    dplyr::mutate(score = mAll$scores) |>
    dplyr::group_by(treatment) |>
    dplyr::summarise(score = sum(score), .groups = "drop") |>
    dplyr::arrange(dplyr::desc(score)) |>
    dplyr::mutate(mk = dplyr::row_number())

  out <- anaD |>
    dplyr::arrange(dplyr::desc(mk)) |>
    dplyr::mutate(
      V1 = treatment,
      V2 = dplyr::lead(treatment),
      V1n = mk,
      V2n = dplyr::lead(mk),
      pvalue = NA,
      pvalue2 = NA,
      conclusion = "",
      conc1 = ""
    )

  dat1 <- dat |>
    dplyr::rowwise() |>
    dplyr::mutate(treat = which(anaD$treatment %in% treatment))

  for (i in seq_len(nrow(out) - 1)) {
    testD <- dat1 |>
      dplyr::filter(treatment %in% c(out[i, 4], out[i, 5])) |>
      dplyr::select(-treatment) |>
      dplyr::arrange(treat)

    m22 <- interval::ictest(survival::Surv(Slow, Sup, type = "interval2") ~ treat,
                            scores = "wmw", data = testD, alternative = "greater")
    out[i, 8] <- m22$p.value

    if (out[i, 8] > (0.05 / (nrow(out) - 1))) {
      m221 <- interval::ictest(survival::Surv(Slow, Sup, type = "interval2") ~ treat,
                               scores = "wmw", data = testD, alternative = "two.sided")
      out[i, 9] <- m221$p.value
      out[i, 10] <- ifelse(m221$p.value > (0.05 / (nrow(out) - 1)),
                           paste0(out[i, 4], "=", out[i, 5]),
                           paste0(out[i, 4], "<", out[i, 5]))
    } else {
      out[i, 10] <- paste0(out[i, 4], ">", out[i, 5])
    }

    if (i == 1) {
      out[i, 11] <- as.character(out[i, 10])
    } else {
      out[i, 11] <- substr(
        as.character(out[i, 10]),
        start = which(unlist(strsplit(as.character(out[i, 10]), split = "")) %in% c(">", "<", "=")),
        stop = nchar(as.character(out[i, 10]))
      )
    }
  }

  colnames(out)[c(4, 5, 8, 9)] <- c("treat1", "treat2", "p-value for H0: treat1 <= treat2",
                                    "p-value for H0: treat1 = treat2")

  if (!ckData) {
    out1 <- list(
      U.Score = out[, c(1, 2)],
      Hypothesis.test = out[-nrow(out), c(4, 5, 8, 9)],
      adj.Signif = 0.05 / (nrow(out) - 1),
      Conclusion = paste(out[-nrow(out), 11], collapse = "")
    )
  } else {
    out1 <- list(
      inputData = outPrintD,
      U.Score = out[, c(1, 2)],
      Hypothesis.test = out[-nrow(out), c(4, 5, 8, 9)],
      adj.Signif = 0.05 / (nrow(out) - 1),
      Conclusion = paste(out[-nrow(out), 11], collapse = "")
    )
  }

  return(out1)
}

#' Calculate the Disease severity Index (DSI) (frequency of each class)
#'
#' This function calculates the Disease Severity Index (DSI) given a vector of classes,
#' a vector of frequencies, and a maximum possible class value. The DSI is calculated
#' as a weighted sum of class values, where each class is multiplied by its corresponding
#' frequency, then divided by the product of the total frequency and maximum class value,
#' and finally multiplied by 100 to get a percentage.
#'
#' @param class A numeric vector representing the classes.
#' @param freq A numeric vector representing the frequency of each class.
#'     Must be the same length as 'class'.
#' @param max A numeric value representing the maximum possible class value.
#'
#' @return Returns a single numeric value representing the DSI.
#'
#' @examples
#' DSI2(c(0, 1, 2, 3, 4), c(2, 0, 5, 0, 5), 4)
#'
#' @export
#' @family Disease quantification
DSI2 <- function(class, freq, max) {
  df <- data.frame(class, freq)

  # Calculate weighted sum of class * frequency
  weighted_sum <- sum(df$class * df$freq)

  # Calculate total frequency
  total_freq <- sum(df$freq)

  # Calculate DSI
  DSI_value <- (weighted_sum / (total_freq * max)) * 100

  return(DSI_value)
}

#' Fetch BR-DWGD Data for Multiple Locations
#'
#' This function extracts daily weather data from the Brazilian Daily Weather Gridded Data (BR-DWGD)
#' NetCDF files for specified locations and dates. It mimics the functionality of `get_nasapower`
#' and `get_era5` by returning a daily-aggregated data frame with columns named according
#' to `nasapower` conventions.
#'
#' @param data A data frame containing the input data, including columns for `latitude`, `longitude`,
#' and the date column specified by `date_col`.
#' @param days_around An integer specifying the number of days before and after the date in the date column
#' to download data.
#' @param date_col A character string specifying the name of the date column in the data frame.
#' @param study_col A character string specifying the name of the column containing the study identifier
#' in the input data frame (default: "study").
#' @param path A character string specifying the directory where the BR-DWGD NetCDF files are stored.
#' @param vars A character vector specifying the weather variables to fetch.
#' These are expected to be the prefix of the NetCDF files (e.g., "pr", "Tmax").
#' Default: `c("pr", "Tmax", "Tmin", "Rs", "RH")`.
#' @param direction Character string specifying the direction of the date range relative to the reference date.
#' Options are "both" (default), "back", or "forth".
#' "back" retrieves data from `date - days_around` to `date`.
#' "forth" retrieves data from `date` to `date + days_around`.
#' "both" retrieves data from `date - days_around` to `date + days_around`.
#'
#' @return A data frame (tibble) with daily weather data. Columns are named to match
#' `nasapower` conventions where possible: `date`, `PRECTOTCORR` (from `pr`),
#' `T2M_MAX` (from `Tmax`), `T2M_MIN` (from `Tmin`), `ALLSKY_SFC_SW_DWN` (from `Rs`),
#' `RH2M` (from `RH`), `longitude`, `latitude`, and `study`.
#'
#' @details
#' The function requires the `terra` package to read NetCDF files and the `tidyr` package
#' for data reshaping. It expects NetCDF files to be named starting with the variable name
#' (e.g., `pr_*.nc`) and to contain the variable with the same name.
#'
#' @importFrom dplyr mutate bind_rows as_tibble select everything inner_join
#' @importFrom lubridate days as_date
#' @importFrom purrr map_dfr
#' @importFrom progress progress_bar
#' @importFrom tidyr pivot_wider
#' @importFrom terra rast time extract cellFromXY nlyr
#' @importFrom rlang .data
#' @export
#' @family Disease modeling
get_brdwgd <- function(data, days_around, date_col, study_col = "study",
                       path = "netcdf_files",
                       vars = c("pr", "Tmax", "Tmin", "Rs", "RH"),
                       direction = "both") {
  # 1. Flexible coordinate and study column detection
  lon_col <- if ("longitude" %in% names(data)) "longitude" else if ("lon" %in% names(data)) "lon" else stop("Longitude column not found.")
  lat_col <- if ("latitude" %in% names(data)) "latitude" else if ("lat" %in% names(data)) "lat" else stop("Latitude column not found.")

  # 2. Pre-calculate all target dates for all locations to find the global range
  data$..row_id <- 1:nrow(data)
  date_list <- lapply(1:nrow(data), function(i) {
    base_date <- as.Date(data[[date_col]][i])
    if (direction == "back") {
      seq(base_date - days_around, base_date, by = "day")
    } else if (direction == "forth") {
      seq(base_date, base_date + days_around, by = "day")
    } else {
      seq(base_date - days_around, base_date + days_around, by = "day")
    }
  })
  all_needed_dates <- unique(do.call(c, date_list))

  # 3. Load and cache search for variables
  res_all_vars <- list()

  # Mapping of BR-DWGD variable names to NASA POWER/ERA5 conventions
  var_name_map <- c(
    pr = "PRECTOTCORR",
    Tmax = "T2M_MAX",
    Tmin = "T2M_MIN",
    Rs = "ALLSKY_SFC_SW_DWN",
    RH = "RH2M"
  )

  for (v in vars) {
    pattern <- paste0("^", v, ".*\\.nc$")
    files <- list.files(path, pattern = pattern, full.names = TRUE)
    if (length(files) == 0) {
      warning(paste("No files for", v))
      next
    }

    # Message for transparency
    message(paste("Processing", v, "..."))

    # Load once
    r <- terra::rast(files)
    t_vals <- terra::time(r)
    if (is.numeric(t_vals)) t_vals <- as.Date(t_vals, origin = "1961-01-01") else t_vals <- as.Date(t_vals)

    # Identify unique layers needed for this variable across ALL locations
    global_idx <- which(t_vals %in% all_needed_dates)
    if (length(global_idx) == 0) next

    # Subset once
    r_sub <- r[[global_idx]]
    t_sub <- t_vals[global_idx]

    # Extract ALL points at once (Matrix extraction is much faster than point by point)
    pts <- matrix(c(data[[lon_col]], data[[lat_col]]), ncol = 2)
    # This returns a matrix: rows = locations, cols = layers
    extracted_matrix <- terra::extract(r_sub, pts)

    # terra::extract might return a data frame with an ID column if pts is a data frame or SpatVector,
    # but for a matrix it usually returns a matrix. Let's ensure we have just the values.
    if (inherits(extracted_matrix, "data.frame") && "ID" %in% names(extracted_matrix)) {
      extracted_matrix <- as.matrix(extracted_matrix[, -1, drop = FALSE])
    } else {
      extracted_matrix <- as.matrix(extracted_matrix)
    }

    # Map variable name
    mapped_name <- if (v %in% names(var_name_map)) var_name_map[[v]] else v

    # Organize back to a long data frame
    var_results <- list()
    for (i in 1:nrow(data)) {
      # Target dates for this specific row
      row_dates <- date_list[[i]]
      # Column indices in the extracted_matrix that correspond to these dates
      col_idx <- which(t_sub %in% row_dates)

      if (length(col_idx) > 0) {
        var_results[[i]] <- data.frame(
          ..row_id = i,
          date = t_sub[col_idx],
          value = as.numeric(extracted_matrix[i, col_idx]),
          var = mapped_name,
          stringsAsFactors = FALSE
        )
      }
    }
    res_all_vars[[v]] <- do.call(rbind, var_results)
  }

  if (length(res_all_vars) == 0) {
    return(dplyr::as_tibble(data.frame()))
  }

  # Combine all variables
  combined_long <- do.call(rbind, res_all_vars)

  # Final Join with original data to get study and coords
  final_df <- combined_long %>%
    dplyr::inner_join(data, by = "..row_id") %>%
    tidyr::pivot_wider(names_from = "var", values_from = "value") %>%
    dplyr::select(-"..row_id") %>%
    dplyr::mutate(study = .data[[study_col]])

  # Cleanup selection
  final_res <- final_df %>%
    dplyr::select(
      dplyr::any_of(c("date", "study", "latitude", "longitude", lon_col, lat_col)),
      dplyr::everything()
    )

  return(dplyr::as_tibble(final_res))
}

#' Fit genotype-specific epidemic trajectories using GAM
#'
#' @description
#' Fits a generalized additive model (GAM) to replicated disease progress curves,
#' returning adjusted treatment mean curves evaluated on a common time grid.
#'
#' @param data A data.frame containing disease progress observations.
#' @param time Character string naming the time variable.
#' @param response Character string naming the response variable.
#' @param treatment Character string naming the treatment/cultivar factor.
#' @param environment Optional character string naming an environment factor.
#' @param block Optional character string naming a blocking factor.
#' @param unit Optional character string naming a unique experimental unit identifier.
#' @param response_scale Character string specifying the response scale: \code{"proportion"} or \code{"percent"}.
#' @param eps Numeric small constant retained for API compatibility.
#' @param min_points Minimum number of observations required per curve.
#' @param grid_n Number of time points for evaluating predicted curves.
#' @param env_ref Reference environment level used for adjusted predictions.
#' @param k_smooth Basis dimension for the global smooth of time.
#' @param k_env Basis dimension for the environment-specific smooth.
#' @param k_trt Basis dimension for the treatment-specific smooth.
#' @param gamma Penalization parameter passed to \code{mgcv::bam()}.
#' @param discrete Logical; whether to use discrete (approximate) fitting.
#' @param family_try Character string specifying the GAM family to try.
#' @param show_progress Logical; whether to show progress.
#' @param covariates Optional character vector of genotype-level covariates (e.g., \code{c("heading_group")}).
#' @param include_covariates Logical; whether to include covariates as fixed effects in the model.
#' @param covariate_smooths Logical; if TRUE and \code{include_covariates = TRUE}, adds smooth interactions \code{s(time, by=covariate)} for factor covariates.
#' @param global_smooth Logical; if \code{TRUE}, fits a shared global curve plus treatment deviations. If \code{FALSE} (default), fits independent smooths per treatment.
#' @param ... Additional arguments.
#'
#' @details
#' Genotype-level covariates are descriptors that do not vary within a genotype, such as phenological groups.
#' For example, in wheat blast studies, a cultivar's \code{heading_group} (early, intermediate, late)
#' can be supplied. This helps distinguish between true genetic resistance and phenological escape, as
#' cultivars with different heading dates may encounter different infection-risk windows in the same
#' environment. When \code{include_covariates = TRUE}, the model accounts for these covariates,
#' yielding heading-adjusted functional resistance.
#'
#' @return An object of class \code{"functional_curves"} containing:
#' \itemize{
#'   \item \code{gam}: fitted GAM object and \code{family_used};
#'   \item \code{curves}: environment-adjusted treatment mean curves on the common grid;
#'   \item \code{grid}: the time grid used;
#'   \item \code{observed_data}: the processed input data;
#'   \item \code{genotype_info}: a tibble with one row per genotype containing covariates;
#'   \item \code{vars}: variable names used;
#'   \item \code{settings}: model settings;
#'   \item \code{warnings_betar}: any warnings caught during beta fitting;
#'   \item \code{plot_mean}: ggplot object of the mean curves.
#' }
#'
#' @examples
#' \dontrun{
#' fc <- functional_curves(
#'   data = my_data,
#'   time = "time_var",
#'   response = "severity",
#'   treatment = "cultivar",
#'   environment = "env",
#'   covariates = c("heading_group"),
#'   include_covariates = TRUE,
#'   covariate_smooths = TRUE
#' )
#' plot(fc)
#' }
#' @export
functional_curves <- function(
    data,
    time,
    response,
    treatment,
    environment = NULL,
    block = NULL,
    unit = NULL,
    response_scale = c("proportion", "percent"),
    eps = 1e-4,
    min_points = 5,
    grid_n = 140,
    env_ref = NULL,
    k_smooth = 10,
    k_env = 4,
    k_trt = 6,
    gamma = 1.4,
    discrete = TRUE,
    family_try = c("betar","quasibinomial"),
    show_progress = TRUE,
    covariates = NULL,
    include_covariates = FALSE,
    covariate_smooths = FALSE,
    global_smooth = FALSE,
    ...
) {
  response_scale <- match.arg(response_scale)
  family_try <- match.arg(family_try)

  # Validate columns
  dat <- data
  needed <- c(time, response, treatment)
  if(!is.null(environment)) needed <- c(needed, environment)
  if(!is.null(block))       needed <- c(needed, block)
  if(!is.null(unit))        needed <- c(needed, unit)
  if(!is.null(covariates))  needed <- c(needed, covariates)
  needed <- unique(needed)

  bad <- setdiff(needed, names(dat))
  if(length(bad)) stop("These columns are missing from `data`: ", paste(bad, collapse = ", "))

  .time <- time; .y <- response; .trt <- treatment
  .env  <- environment; .blk <- block; .unit <- unit

  df <- tibble::as_tibble(dat)

  df[[.trt]] <- as.factor(df[[.trt]])
  if(!is.null(.env)) df[[.env]] <- as.factor(df[[.env]])
  if(!is.null(.blk)) df[[.blk]] <- as.factor(df[[.blk]])
  if(!is.null(covariates)) {
    for (cov in covariates) {
      if (is.character(df[[cov]])) df[[cov]] <- as.factor(df[[cov]])
    }
  }
  df[[.time]] <- as.numeric(df[[.time]])

  y_raw <- as.numeric(df[[.y]])

  if (response_scale == "percent") {
    y_raw <- y_raw / 100
  } else {
    mx <- suppressWarnings(max(y_raw, na.rm = TRUE))
    if (is.finite(mx) && mx > 1) {
      warning("`response_scale='proportion'` but max(response) > 1; assuming percent and dividing by 100.")
      y_raw <- y_raw / 100
    }
  }

  y_raw <- pmin(pmax(y_raw, 0), 1)
  df$.y_raw <- y_raw

  n_eff <- sum(is.finite(y_raw))
  if (!is.finite(n_eff) || n_eff < 2) stop("Not enough finite response values to fit the model.")
  df$.y <- (y_raw * (n_eff - 1) + 0.5) / n_eff

  unit_used <- .unit
  if(is.null(.unit)){
    parts <- list()
    if(!is.null(.env)) parts <- c(parts, list(df[[.env]]))
    parts <- c(parts, list(df[[.trt]]))
    if(!is.null(.blk)) parts <- c(parts, list(df[[.blk]]))

    unit_id <- if(length(parts) == 1) {
      parts[[1]]
    } else {
      do.call(interaction, c(parts, list(drop = TRUE, sep = "|")))
    }

    df$.unit <- as.factor(unit_id)
    .unit <- ".unit"
    unit_used <- ".unit"
  } else {
    df[[.unit]] <- as.factor(df[[.unit]])
  }

  keep <- is.finite(df[[.time]]) &
    is.finite(df$.y) &
    is.finite(df$.y_raw) &
    !is.na(df[[.trt]]) &
    !is.na(df[[.unit]])
  if(!is.null(.env)) keep <- keep & !is.na(df[[.env]])
  if(!is.null(covariates)) {
    for (cov in covariates) {
      keep <- keep & !is.na(df[[cov]])
    }
  }
  df <- df[keep, , drop = FALSE]

  if(!is.null(covariates)) {
    genotype_info <- df |> dplyr::select(dplyr::all_of(c(.trt, covariates))) |> dplyr::distinct()
    if (nrow(genotype_info) > length(unique(df[[.trt]]))) {
      for (cov in covariates) {
        check <- df |> dplyr::group_by(.data[[.trt]]) |> dplyr::summarise(n = dplyr::n_distinct(.data[[cov]]))
        if (any(check$n > 1)) {
          stop(sprintf("Covariate '%s' has multiple values within at least one genotype. Covariates must be genotype-level.", cov))
        }
      }
    }
  } else {
    genotype_info <- tibble::tibble(!!.trt := unique(df[[.trt]]))
  }

  n_by_unit <- df |>
    dplyr::group_by(.data[[.unit]]) |>
    dplyr::summarise(n = dplyr::n(), .groups = "drop")

  df <- df |>
    dplyr::left_join(n_by_unit, by = .unit) |>
    dplyr::filter(.data[["n"]] >= min_points) |>
    dplyr::select(-.data[["n"]])

  if(nrow(df) == 0) stop("No data left after filtering. Check `min_points` and missingness.")

  trt_levels0  <- levels(df[[.trt]])
  env_levels0  <- if(!is.null(.env)) levels(df[[.env]]) else NULL
  blk_levels0  <- if(!is.null(.blk)) levels(df[[.blk]]) else NULL
  unit_levels0 <- levels(df[[.unit]])

  k_smooth_eff <- min(k_smooth, max(3, length(unique(df[[.time]])) - 1))
  k_trt_eff    <- min(k_trt,    max(2, length(unique(df[[.trt]]))  - 1))
  k_env_eff <- NULL
  if(!is.null(.env)) k_env_eff <- min(k_env, max(2, length(unique(df[[.env]])) - 1))

  if (global_smooth) {
    f_terms <- c(
      sprintf("%s", .trt),
      sprintf("s(%s, k=%s)", .time, k_smooth_eff),
      sprintf("s(%s, by=%s, k=%s)", .time, .trt, k_trt_eff)
    )
  } else {
    f_terms <- c(
      sprintf("%s", .trt),
      sprintf("s(%s, by=%s, k=%s)", .time, .trt, k_smooth_eff)
    )
  }

  if (include_covariates && !is.null(covariates)) {
    for (cov in covariates) {
      f_terms <- c(f_terms, sprintf("%s", cov))
      if (covariate_smooths && is.factor(df[[cov]])) {
        f_terms <- c(f_terms, sprintf("s(%s, by=%s, k=%s)", .time, cov, k_trt_eff))
      }
    }
  }

  if(!is.null(.env)){
    f_terms <- c(f_terms, sprintf("s(%s, bs='re')", .env))
  }

  if(!is.null(.blk) || !is.null(unit)){
    f_terms <- c(f_terms, sprintf("s(%s, bs='re')", .unit))
  }

  f_gam_beta <- stats::as.formula(paste0(".y ~ ", paste(f_terms, collapse = " + ")))
  f_gam_qb   <- stats::as.formula(paste0(".y_raw ~ ", paste(f_terms, collapse = " + ")))

  fit_betar <- function(dat_fit) {
    mgcv::bam(
      formula  = f_gam_beta,
      family   = mgcv::betar(link = "logit"),
      data     = dat_fit,
      method   = "fREML",
      discrete = discrete,
      gamma    = gamma,
      select   = TRUE
    )
  }
  fit_qb <- function(dat_fit) {
    mgcv::bam(
      formula  = f_gam_qb,
      family   = stats::quasibinomial(link = "logit"),
      data     = dat_fit,
      method   = "fREML",
      discrete = discrete,
      gamma    = gamma,
      select   = TRUE
    )
  }

  if (show_progress) message("Fitting GAM model (this may take a moment)...")

  w <- character()

  if (family_try == "quasibinomial") {
    m_gam <- fit_qb(df)
    fam_used <- "quasibinomial"
  } else {
    m_try <- withCallingHandlers(
      try(fit_betar(df), silent = TRUE),
      warning = function(cnd) { w <<- c(w, conditionMessage(cnd)); invokeRestart("muffleWarning") }
    )
    theta_failed <- inherits(m_try, "try-error") || any(grepl("theta estimation", w, fixed = TRUE))

    if(theta_failed){
      warning("beta-GAM precision (theta) estimation failed; refitting with quasibinomial().")
      m_gam <- fit_qb(df)
      fam_used <- "quasibinomial"
    } else {
      m_gam <- m_try
      fam_used <- "betar"
    }
  }

  tmin <- min(df[[.time]], na.rm = TRUE)
  tmax <- max(df[[.time]], na.rm = TRUE)
  t_grid <- seq(tmin, tmax, length.out = grid_n)

  if(!is.null(.env)){
    if(is.null(env_ref)) env_ref <- env_levels0[1]
    if(!(env_ref %in% env_levels0)) stop("`env_ref` not found in environment levels.")
  }

  pred_trt <- purrr::map_dfr(trt_levels0, function(trt_i){
    newd <- tibble::tibble(tmp_time = t_grid)
    names(newd)[1] <- .time
    newd[[.trt]] <- factor(trt_i, levels = trt_levels0)

    if (!is.null(covariates)) {
      cov_vals <- genotype_info |> dplyr::filter(.data[[.trt]] == trt_i)
      for (cov in covariates) {
        newd[[cov]] <- cov_vals[[cov]][1]
      }
    }

    if(!is.null(.env)) newd[[.env]] <- factor(env_levels0[1], levels = env_levels0)

    excl <- character()
    if(!is.null(.env)) excl <- c(excl, sprintf("s(%s)", .env))

    smooth_labels <- if (!is.null(m_gam$smooth)) {
      vapply(m_gam$smooth, function(s) s$label, character(1))
    } else {
      character(0)
    }
    if(any(grepl(sprintf("s(%s)", .unit), smooth_labels, fixed = TRUE))) {
      newd[[.unit]] <- factor(unit_levels0[1], levels = unit_levels0)
      excl <- c(excl, sprintf("s(%s)", .unit))
    }

    mu <- mgcv::predict.gam(m_gam, newdata = newd, type = "response", exclude = excl)

    tibble::tibble(
      !!.trt := trt_i,
      !!.time := t_grid,
      mu = as.numeric(mu)
    )
  })

  trt_score <- pred_trt |>
    dplyr::group_by(.data[[.trt]]) |>
    dplyr::summarise(mean_mu = mean(.data[["mu"]]), .groups = "drop") |>
    dplyr::arrange(.data[["mean_mu"]])

  trt_order <- trt_score[[.trt]]

  plot_mean <- pred_trt |>
    dplyr::mutate(!!.trt := factor(.data[[.trt]], levels = trt_order)) |>
    ggplot2::ggplot(ggplot2::aes(x = .data[[.time]], y = .data[["mu"]], color = .data[[.trt]])) +
    ggplot2::geom_line(linewidth = 1) +
    ggplot2::theme_classic()

  out <- list(
    gam = m_gam,
    curves = pred_trt,
    grid = t_grid,
    observed_data = df,
    genotype_info = genotype_info,
    vars = list(time = .time, response = .y, treatment = .trt,
                environment = .env, block = .blk, unit = unit_used,
                covariates = covariates),
    settings = list(
      response_scale = response_scale, eps = eps, min_points = min_points,
      grid_n = grid_n, env_ref = env_ref,
      k_smooth = k_smooth, k_env = k_env, k_trt = k_trt,
      k_smooth_eff = k_smooth_eff,
      k_env_eff = k_env_eff,
      k_trt_eff = k_trt_eff,
      gamma = gamma, discrete = discrete,
      family_try = family_try
    ),
    warnings_betar = w,
    family_used = fam_used,
    plot_mean = plot_mean,
    scores = trt_score
  )
  class(out) <- "functional_curves"
  out
}

#' Print functional_curves
#' @param x An object of class \code{"functional_curves"}.
#' @param ... Additional arguments.
#' @export
print.functional_curves <- function(x, ...) {
  cat("A functional_curves object\n")
  cat("Time variable:", x$vars$time, "\n")
  cat("Treatment variable:", x$vars$treatment, "\n")
  cat("Grid size:", length(x$grid), "points\n")
  invisible(x)
}

#' Plot functional_curves
#' @param x An object of class \code{"functional_curves"}.
#' @param ... Additional arguments.
#' @export
plot.functional_curves <- function(x, ...) {
  x$plot_mean
}

#' Plot ASFD
#'
#' This function creates a tile plot of the foci (cluster) identified by the AFSD function.
#' It colors each cell in a foci and labels the centroid of each cluster with the foci ID.
#' The 'ggplot2' package is used for the plot, and will be automatically installed if not already present.
#'
#' @param df A dataframe containing at least three columns: 'x', 'y', and 'cluster_id'.
#'           'x' and 'y' are spatial coordinates and 'cluster_id' is the cluster identifier
#'           to which each cell belongs.
#'
#' @return A ggplot object with the scatter plot of foci (clusters).
#'
#' @examples
#' df <- data.frame(x = sample(1:100, 500, replace = TRUE),
#'                  y = sample(1:100, 500, replace = TRUE),
#'                  i = sample(0:1, 500, replace = TRUE, prob = c(0.7, 0.3)))
#'
#' # Perform the AFSD
#' result <- AFSD(df)
#' # Plot the foci
#' plot_AFSD(result[[3]])
#'
#' @importFrom ggplot2 ggplot geom_tile theme_minimal geom_text
#' @importFrom dplyr group_by summarise
#' @export
#' @family Spatial analysis
plot_AFSD <- function(df) {
  # Check required packages
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("The 'ggplot2' package is required. Please install it with install.packages('ggplot2').")
  }
  if (!requireNamespace("dplyr", quietly = TRUE)) {
    stop("The 'dplyr' package is required. Please install it with install.packages('dplyr').")
  }

  # Calculate the centroid of each cluster
  centroids <- dplyr::group_by(df, focus_id) |>
    dplyr::summarise(x = mean(x), y = mean(y), .groups = "drop")

  # Create scatter plot
  ggplot2::ggplot(df, ggplot2::aes(x = x, y = y)) +
    ggplot2::geom_tile(fill = "gray70", color = "black") +
    ggplot2::geom_text(data = centroids, ggplot2::aes(label = focus_id), size = 5, vjust = -0.5) +
    ggplot2::theme_minimal()
}

#' Plot functional dendrogram of epidemic curves
#'
#' @description
#' Visualizes the hierarchical clustering of treatments based on
#' functional distances among epidemic curves.
#'
#' @param x An object of class \code{"functional_distances"}.
#' @param label_fun Optional function to modify treatment labels.
#' @param palette Optional named vector of colors for clusters.
#' @param show_cut Logical; whether to display the cluster cut height.
#' @param label_size Numeric specifying the font size of the labels. Default is \code{2.8}.
#'
#' @return A \code{ggplot} object.
#'
#' @export
plot_dendrogram <- function(
    x,
    label_fun = NULL,
    palette = NULL,
    show_cut = TRUE,
    label_size = 2.8
){
  if (!inherits(x, "functional_distances")) {
    stop("`x` must be an object of class `functional_distances`.", call. = FALSE)
  }
  if(!requireNamespace("ggplot2", quietly = TRUE)) stop("Need ggplot2.")
  if(!requireNamespace("dplyr", quietly = TRUE)) stop("Need dplyr.")
  if(!requireNamespace("ggdendro", quietly = TRUE)) stop("Need ggdendro.")

  fc <- x$functional_curves
  trt <- fc$vars$treatment
  k <- length(unique(x$clusters$cluster))

  if(is.null(label_fun)) label_fun <- function(z) z

  dd <- ggdendro::dendro_data(stats::as.dendrogram(x$hc), type = "rectangle")

  lab_map <- x$clusters |>
    dplyr::mutate(
      label = .data[[trt]],
      label_short = label_fun(.data[[trt]]),
      cluster = factor(cluster)
    ) |>
    dplyr::select(label, label_short, cluster)

  labs_df <- dd$labels |>
    dplyr::left_join(lab_map, by = "label")

  p <- ggplot2::ggplot() +
    ggplot2::geom_segment(
      data = dd$segments,
      ggplot2::aes(x = x, y = y, xend = xend, yend = yend),
      linewidth = 0.45
    ) +
    ggplot2::geom_text(
      data = labs_df,
      ggplot2::aes(x = x, y = y, label = label_short, colour = cluster),
      angle = 90, hjust = 1, vjust = 0.5, size = label_size
    ) +
    ggplot2::labs(x = NULL, y = "Functional distance", colour = "Cluster") +
    ggplot2::theme_classic(base_size = 13) +
    ggplot2::theme(
      axis.line.x  = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      axis.text.x  = ggplot2::element_blank(),
      legend.position = "none",
      plot.margin = ggplot2::margin(t = 5.5, r = 5.5, b = 65, l = 5.5)
    ) +
    ggplot2::coord_cartesian(clip = "off")

  if(show_cut){
    h_cut <- x$hc$height[length(x$hc$height) - (k - 1)]
    p <- p + ggplot2::geom_hline(yintercept = h_cut, linetype = "dashed", linewidth = 0.5) +
      ggplot2::annotate(
        "text",
        x = max(dd$segments$x, na.rm = TRUE),
        y = h_cut,
        label = paste("Profiles =", k),
        hjust = 1,
        vjust = -0.5,
        size = 3.5,
        color = "gray30"
      )
  }

  if(!is.null(palette)) p <- p + ggplot2::scale_colour_manual(values = palette)
  p
}

#' Suggest GAM smoothing parameters for epidemic curve models
#'
#' @description
#' A helper function to guide selection of the \code{k_smooth}, \code{k_trt},
#' \code{k_env}, and \code{gamma} parameters used by \code{\link{functional_curves}}.
#' Can infer the effective replication from a data frame or accept scalar values
#' directly when the data are not yet available.
#'
#' For sparse epidemic data it is strongly recommended to use
#' \code{rule = "minimum"} so that k values are based on the least-replicated
#' treatment-by-environment combination, guarding against over-fitting.
#'
#' @param data Optional data frame.  If supplied, \code{time}, \code{treatment},
#'   and optionally \code{environment} are used to compute summaries of the
#'   number of unique time points and environments per treatment.
#' @param time Unquoted column name for the time variable, or a character
#'   string naming the column.
#' @param treatment Unquoted column name for the treatment / cultivar variable,
#'   or a character string naming the column.
#' @param environment Optional unquoted column name for the environment
#'   variable, or a character string naming the column.
#' @param n_time Integer.  Number of unique time points to use directly
#'   (ignored when \code{data} is supplied).
#' @param n_env Integer.  Number of unique environments to use directly
#'   (ignored when \code{data} is supplied, or when there is no environment
#'   variable).
#' @param rule Character string. How to summarise the distribution of
#'   replication counts across treatment-by-environment combinations:
#'   \code{"minimum"} (default, conservative) or \code{"median"}.
#' @param smoothness Character string controlling how liberal the
#'   recommendations are: \code{"conservative"} (default), \code{"moderate"},
#'   or \code{"flexible"}.
#'
#' @details
#' The function computes, for every treatment-by-environment combination, the
#' number of unique time points at which observations are available.  It then
#' summarises these counts using the chosen \code{rule} to obtain
#' \code{effective_n_time}.  Similarly it computes, per treatment, the number
#' of unique environments, and summarises using the same \code{rule} to obtain
#' \code{effective_n_env}.
#'
#' Recommended k values follow these heuristics:
#' \itemize{
#'   \item \code{k_smooth}: global smooth over time; capped below
#'     \code{effective_n_time - 1}.
#'   \item \code{k_trt}: treatment-specific smooth; capped below
#'     \code{k_smooth}.
#'   \item \code{k_env}: environment random effect; capped below
#'     \code{effective_n_env}.
#'   \item \code{gamma}: penalty multiplier; higher values encourage
#'     smoother fits.
#' }
#'
#' @return A named list with the following elements:
#' \describe{
#'   \item{\code{time_summary}}{Named numeric vector with minimum, median, and
#'     maximum unique time points per treatment-by-environment combination.}
#'   \item{\code{environment_summary}}{Named numeric vector with minimum, median,
#'     and maximum unique environments per treatment (or \code{NULL} when no
#'     environment variable is given).}
#'   \item{\code{effective_n_time}}{The effective number of time points chosen
#'     by \code{rule}.}
#'   \item{\code{effective_n_env}}{The effective number of environments chosen
#'     by \code{rule}, or \code{NULL}.}
#'   \item{\code{k_smooth}}{Recommended basis dimension for the global time smooth.}
#'   \item{\code{k_trt}}{Recommended basis dimension for the treatment-specific smooth.}
#'   \item{\code{k_env}}{Recommended basis dimension for the environment random effect.}
#'   \item{\code{gamma}}{Recommended penalisation multiplier.}
#'   \item{\code{message}}{A short interpretation message.}
#' }
#'
#' @examples
#' # Using explicit values
#' suggest_k(n_time = 5, n_env = 8)
#'
#' # Inferring from a data frame (unquoted column names)
#' df <- data.frame(
#'   time      = rep(1:6, times = 6),
#'   cultivar  = rep(c("A", "B", "C"), each = 12),
#'   env       = rep(rep(c("E1", "E2"), each = 6), times = 3),
#'   severity  = runif(36, 0, 0.5)
#' )
#' suggest_k(
#'   data        = df,
#'   time        = time,
#'   treatment   = cultivar,
#'   environment = env,
#'   rule        = "minimum",
#'   smoothness  = "conservative"
#' )
#'
#' @export
suggest_k <- function(
    data        = NULL,
    time        = NULL,
    treatment   = NULL,
    environment = NULL,
    n_time      = NULL,
    n_env       = NULL,
    rule        = c("minimum", "median"),
    smoothness  = c("conservative", "moderate", "flexible")
) {
  rule       <- match.arg(rule)
  smoothness <- match.arg(smoothness)

  # ---- resolve tidy-eval column names ----------------------------------------
  time_col <- NULL
  trt_col  <- NULL
  env_col  <- NULL

  if (!is.null(data)) {
    time_quo <- rlang::enquo(time)
    trt_quo  <- rlang::enquo(treatment)
    env_quo  <- rlang::enquo(environment)

    time_col <- if (!rlang::quo_is_null(time_quo)) rlang::as_name(time_quo) else NULL
    trt_col  <- if (!rlang::quo_is_null(trt_quo))  rlang::as_name(trt_quo)  else NULL
    env_col  <- if (!rlang::quo_is_null(env_quo))  rlang::as_name(env_quo)  else NULL

    if (is.null(time_col) || is.null(trt_col)) {
      stop("When `data` is supplied, both `time` and `treatment` must be specified.", call. = FALSE)
    }
    bad <- setdiff(c(time_col, trt_col, env_col), names(data))
    if (length(bad)) stop("Column(s) not found in `data`: ", paste(bad, collapse = ", "), call. = FALSE)
  }

  # ---- compute summaries from data -------------------------------------------
  time_summary <- NULL
  env_summary  <- NULL

  if (!is.null(data)) {
    if (!is.null(env_col)) {
      time_counts <- tapply(
        data[[time_col]],
        list(data[[trt_col]], data[[env_col]]),
        function(x) length(unique(x[!is.na(x)]))
      )
      time_vec <- as.numeric(time_counts)
      time_vec <- time_vec[!is.na(time_vec)]

      env_counts <- tapply(
        data[[env_col]],
        data[[trt_col]],
        function(x) length(unique(x[!is.na(x)]))
      )
      env_vec <- as.numeric(env_counts)
    } else {
      time_counts <- tapply(
        data[[time_col]],
        data[[trt_col]],
        function(x) length(unique(x[!is.na(x)]))
      )
      time_vec <- as.numeric(time_counts)
      time_vec <- time_vec[!is.na(time_vec)]
      env_vec  <- NULL
    }

    time_summary <- c(
      minimum = min(time_vec),
      median  = stats::median(time_vec),
      maximum = max(time_vec)
    )

    if (!is.null(env_vec)) {
      env_summary <- c(
        minimum = min(env_vec),
        median  = stats::median(env_vec),
        maximum = max(env_vec)
      )
    }

    n_time <- if (rule == "minimum") time_summary["minimum"] else time_summary["median"]
    n_env  <- if (!is.null(env_summary)) {
      if (rule == "minimum") env_summary["minimum"] else env_summary["median"]
    } else NULL

  } else {
    # use explicit scalars
    if (is.null(n_time)) stop("`n_time` must be provided when `data` is NULL.", call. = FALSE)
  }

  effective_n_time <- as.integer(n_time)
  effective_n_env  <- if (!is.null(n_env)) as.integer(n_env) else NULL

  # ---- recommend k values -----------------------------------------------------
  gamma_tbl <- c(conservative = 1.8, moderate = 1.6, flexible = 1.4)
  gamma_rec <- gamma_tbl[[smoothness]]

  # k_smooth: global smooth over time
  k_smooth_raw <- if (effective_n_time <= 4) 4L
                  else if (effective_n_time <= 6) 5L
                  else if (effective_n_time <= 8) 7L
                  else 10L
  if (smoothness == "flexible")     k_smooth_raw <- min(k_smooth_raw + 2L, effective_n_time - 1L)
  else if (smoothness == "moderate") k_smooth_raw <- min(k_smooth_raw + 1L, effective_n_time - 1L)
  k_smooth_rec <- min(k_smooth_raw, max(3L, effective_n_time - 1L))

  # k_trt: treatment-specific smooth
  k_trt_raw <- if (effective_n_time <= 4) 3L
               else if (effective_n_time <= 6) 4L
               else if (smoothness == "conservative") 5L
               else if (smoothness == "moderate") 5L
               else 6L
  k_trt_rec <- min(k_trt_raw, k_smooth_rec - 1L)
  k_trt_rec <- max(k_trt_rec, 2L)

  # k_env: environment random effect basis dimension
  k_env_rec <- NULL
  if (!is.null(effective_n_env)) {
    k_env_raw <- if (effective_n_env < 6) 3L
                 else if (smoothness == "conservative") 3L
                 else 4L
    k_env_rec <- min(k_env_raw, max(2L, effective_n_env - 1L))
  }

  # ---- interpretation message -------------------------------------------------
  parts <- c(
    sprintf("effective_n_time = %d (%s across combinations)", effective_n_time, rule),
    if (!is.null(effective_n_env)) sprintf("effective_n_env = %d (%s across treatments)", effective_n_env, rule),
    sprintf("smoothness = '%s'", smoothness),
    sprintf("k_smooth = %d, k_trt = %d, k_env = %s, gamma = %.1f",
            k_smooth_rec, k_trt_rec,
            if (!is.null(k_env_rec)) as.character(k_env_rec) else "NA (no environment)",
            gamma_rec)
  )
  msg <- paste(parts, collapse = "\n  ")

  if (effective_n_time <= 4) {
    msg <- paste0(msg, "\n  Warning: very few time points detected. Prefer rule = 'minimum' and smoothness = 'conservative'.")
  }

  # ---- return -----------------------------------------------------------------
  structure(
    list(
      time_summary     = time_summary,
      environment_summary = env_summary,
      effective_n_time = effective_n_time,
      effective_n_env  = effective_n_env,
      k_smooth         = k_smooth_rec,
      k_trt            = k_trt_rec,
      k_env            = k_env_rec,
      gamma            = gamma_rec,
      message          = msg
    ),
    class = "suggest_k"
  )
}

#' Print method for suggest_k
#'
#' @param x An object of class \code{"suggest_k"}.
#' @param ... Additional arguments (ignored).
#' @return Invisibly returns \code{x}.
#' @export
print.suggest_k <- function(x, ...) {
  cat("-- suggest_k: GAM parameter recommendations ----------------------\n")
  cat(" ", x$message, "\n")
  cat("\nSuggested call to functional_curves():\n")
  cat(sprintf(
    "  functional_curves(..., k_smooth = %d, k_trt = %d, k_env = %s, gamma = %.1f)\n",
    x$k_smooth, x$k_trt,
    if (!is.null(x$k_env)) as.character(x$k_env) else "NA",
    x$gamma
  ))
  if (!is.null(x$time_summary)) {
    cat("\nTime points per treatment-by-environment:\n")
    cat(sprintf("  min = %g, median = %g, max = %g\n",
                x$time_summary["minimum"], x$time_summary["median"], x$time_summary["maximum"]))
  }
  if (!is.null(x$environment_summary)) {
    cat("Environments per treatment:\n")
    cat(sprintf("  min = %g, median = %g, max = %g\n",
                x$environment_summary["minimum"], x$environment_summary["median"],
                x$environment_summary["maximum"]))
  }
  invisible(x)
}

test_that("suggest_k works with explicit n_time and n_env", {
  out <- suggest_k(n_time = 5, n_env = 8, rule = "minimum", smoothness = "conservative")

  expect_s3_class(out, "suggest_k")
  expect_equal(out$effective_n_time, 5L)
  expect_equal(out$effective_n_env, 8L)
  expect_true(out$k_smooth >= 4)
  expect_true(out$k_trt < out$k_smooth)
  expect_equal(out$gamma, 1.8)
  expect_null(out$time_summary)
})

test_that("suggest_k infers time points per treatment-environment correctly", {
  df <- data.frame(
    time     = c(1:6, 1:5, 1:6, 1:5),   # trt A env E1: 6, trt A env E2: 5, trt B env E1: 6, trt B env E2: 5
    trt      = rep(c("A", "A", "B", "B"), times = c(6, 5, 6, 5)),
    env      = rep(c("E1", "E2", "E1", "E2"), times = c(6, 5, 6, 5)),
    severity = runif(22, 0, 0.5)
  )

  out_min <- suggest_k(data = df, time = time, treatment = trt,
                       environment = env, rule = "minimum")
  out_med <- suggest_k(data = df, time = time, treatment = trt,
                       environment = env, rule = "median")

  # minimum rule: smallest unique time count across trt x env is 5
  expect_equal(out_min$effective_n_time, 5L)
  # median rule: median of c(6,5,6,5) = 5.5 -> 5 as integer
  expect_equal(out_med$effective_n_time, 5L)

  # time_summary should have min = 5, max = 6
  expect_equal(out_min$time_summary[["minimum"]], 5)
  expect_equal(out_min$time_summary[["maximum"]], 6)
})

test_that("suggest_k infers environments per treatment correctly", {
  df <- data.frame(
    time     = rep(1:5, times = 9),
    trt      = rep(c("A", "B", "C"), each = 15),
    env      = rep(rep(c("E1", "E2", "E3"), each = 5), times = 3),
    severity = runif(45, 0, 0.5)
  )

  out <- suggest_k(data = df, time = time, treatment = trt,
                   environment = env, rule = "minimum")

  # each treatment appears in 3 environments
  expect_equal(out$environment_summary[["minimum"]], 3)
  expect_equal(out$environment_summary[["maximum"]], 3)
  expect_equal(out$effective_n_env, 3L)
})

test_that("rule = 'minimum' uses the smallest replication", {
  # Unbalanced: trt A has env E1=6 pts, E2=3 pts
  df <- data.frame(
    time = c(1:6, 1:3),
    trt  = c(rep("A", 6), rep("A", 3)),
    env  = c(rep("E1", 6), rep("E2", 3)),
    y    = runif(9)
  )
  out_min <- suggest_k(data = df, time = time, treatment = trt, environment = env, rule = "minimum")
  out_med <- suggest_k(data = df, time = time, treatment = trt, environment = env, rule = "median")

  expect_equal(out_min$effective_n_time, 3L)   # minimum is 3
  expect_equal(out_med$effective_n_time, 4L)   # median of c(6, 3) = 4.5 -> 4
})

test_that("rule = 'median' uses median replication", {
  df <- data.frame(
    time = c(1:3, 1:5, 1:7),
    trt  = rep(c("A", "B", "C"), times = c(3, 5, 7)),
    y    = runif(15)
  )
  out <- suggest_k(data = df, time = time, treatment = trt, rule = "median")
  # median of c(3, 5, 7) = 5
  expect_equal(out$effective_n_time, 5L)
})

test_that("sparse data returns low k values", {
  out <- suggest_k(n_time = 4, n_env = 3, smoothness = "conservative")
  expect_lte(out$k_smooth, 4)
  expect_lte(out$k_trt, out$k_smooth)
  expect_lte(out$k_env, 3)
  expect_equal(out$gamma, 1.8)  # most conservative gamma
})

test_that("smoothness levels produce ordered gamma values", {
  o_con <- suggest_k(n_time = 8, n_env = 10, smoothness = "conservative")
  o_mod <- suggest_k(n_time = 8, n_env = 10, smoothness = "moderate")
  o_fle <- suggest_k(n_time = 8, n_env = 10, smoothness = "flexible")

  expect_gt(o_con$gamma, o_mod$gamma)
  expect_gt(o_mod$gamma, o_fle$gamma)
})

test_that("suggest_k works without environment variable", {
  df <- data.frame(
    time = rep(1:6, times = 2),
    trt  = rep(c("A", "B"), each = 6),
    y    = runif(12)
  )
  out <- suggest_k(data = df, time = time, treatment = trt)
  expect_null(out$k_env)
  expect_null(out$environment_summary)
  expect_null(out$effective_n_env)
})

test_that("suggest_k errors informatively on bad inputs", {
  expect_error(suggest_k(), "n_time")
  expect_error(suggest_k(data = data.frame(a = 1), time = missing_col, treatment = a),
               "not found in")
})

test_that("print.suggest_k runs without error", {
  out <- suggest_k(n_time = 6, n_env = 4)
  expect_output(print(out), "functional_curves")
})

#' Functional principal component analysis of disease progress curves
#'
#' @description
#' Performs functional principal component analysis on fitted disease progress curves 
#' returned by \code{\link{functional_curves}}. The function decomposes variation among epidemic 
#' trajectories into orthogonal temporal components and returns curve-level scores, 
#' eigenfunctions, variance explained, and reconstructed curves.
#'
#' @param object An object returned by \code{\link{functional_curves}}.
#' @param n_components Optional integer number of functional principal components to retain.
#' @param var_explained Cumulative variance threshold used when \code{n_components = NULL}.
#' @param center Logical; whether to center curves before PCA. Default TRUE.
#' @param scale Logical; whether to scale grid columns before PCA. Default FALSE.
#' @param method Character; method for FPCA, currently only \code{"pca_on_grid"} is supported.
#' @param ... Additional arguments for future extensions.
#'
#' @details
#' The function uses the fitted curves from \code{functional_curves()} and does not refit 
#' the disease progress model. The first implementation uses PCA on a common prediction grid. 
#' FPC scores can be analyzed as functional epidemiological traits in downstream models.
#' 
#' Interpretation:
#' \itemize{
#'   \item FPC1 often captures the largest mode of variation, commonly overall epidemic intensity or speed.
#'   \item Later FPCs may capture timing of disease onset, curve crossing, late acceleration, or other shape-related deviations.
#'   \item Interpretation must be data-driven and should be based on eigenfunction plots and mean +/- perturbation plots.
#' }
#'
#' @return An object of class \code{"r4pde_functional_pca"} containing:
#' \itemize{
#'   \item \code{scores}: Tibble of curve-level FPC scores.
#'   \item \code{eigenfunctions}: Tibble of eigenfunction values across the time grid.
#'   \item \code{variance}: Tibble of eigenvalues, variance explained, and cumulative variance.
#'   \item \code{mean_curve}: Tibble of the mean curve.
#'   \item \code{reconstructed}: Tibble of reconstructed curves using retained components.
#'   \item \code{input_curves}: Tibble of original fitted curves.
#'   \item \code{pca}: The underlying \code{prcomp} object.
#'   \item \code{settings}: List of settings used.
#'   \item \code{call}: The matched call.
#' }
#'
#' @examples
#' \dontrun{
#' curves <- functional_curves(...)
#' fpca <- functional_pca(curves, var_explained = 0.95)
#' 
#' print(fpca)
#' plot(fpca, type = "scree")
#' plot(fpca, type = "components")
#' plot(fpca, type = "scores", components = c(1, 2))
#' 
#' scores <- get_fpca_scores(fpca)
#' }
#' @export
functional_pca <- function(object, ...) {
  UseMethod("functional_pca")
}

#' @export
functional_pca.default <- function(object, ...) {
  stop("`object` must be of class 'functional_curves'.", call. = FALSE)
}

#' @export
#' @rdname functional_pca
functional_pca.functional_curves <- function(
    object,
    n_components = NULL,
    var_explained = 0.95,
    center = TRUE,
    scale = FALSE,
    method = c("pca_on_grid"),
    ...
) {
  method <- match.arg(method)
  
  if (!inherits(object, "functional_curves")) {
    stop("`object` must be of class 'functional_curves'.")
  }
  
  if (is.null(object$curves)) {
    stop("Fitted curves are not available in the `object`.")
  }
  
  .time <- object$vars$time
  .trt <- object$vars$treatment
  
  if (is.null(.time) || is.null(.trt)) {
    stop("Time or treatment variable information is missing from the object.")
  }
  
  # Extract fitted curve data
  pred_trt <- object$curves
  t_grid <- object$grid
  
  if (length(t_grid) < 3) {
    stop("There must be at least three time points in the common grid.")
  }
  
  # Reshape to a wide matrix: rows = individual curves, columns = common time grid
  wide_mat <- pred_trt |>
    dplyr::select(dplyr::all_of(c(.trt, .time, "mu"))) |>
    tidyr::pivot_wider(names_from = dplyr::all_of(.time), values_from = "mu")
  
  curve_ids <- wide_mat[[.trt]]
  
  if (length(unique(curve_ids)) < 2) {
    stop("There must be at least two curves.")
  }
  
  # Extract curve matrix
  curve_mat <- as.matrix(wide_mat[, -1, drop = FALSE])
  rownames(curve_mat) <- as.character(curve_ids)
  
  if (any(is.na(curve_mat))) {
    stop("Missing fitted values found in the curve matrix.")
  }
  
  # Perform PCA
  pca_res <- stats::prcomp(curve_mat, center = center, scale. = scale)
  
  # Variance explained
  eigenvalues <- pca_res$sdev^2
  prop_var <- eigenvalues / sum(eigenvalues)
  cum_var <- cumsum(prop_var)
  
  # Select retained components
  if (!is.null(n_components)) {
    k <- min(n_components, length(eigenvalues))
  } else {
    k <- which(cum_var >= var_explained)[1]
    if (is.na(k)) k <- length(eigenvalues)
  }
  
  # Create variance tibble
  var_tbl <- tibble::tibble(
    FPC = paste0("FPC", seq_along(eigenvalues)),
    eigenvalue = eigenvalues,
    prop_var = prop_var,
    cum_var = cum_var
  ) |>
    dplyr::slice(1:k)
  
  # Create scores tibble
  scores_mat <- pca_res$x[, 1:k, drop = FALSE]
  colnames(scores_mat) <- gsub("^PC", "FPC", colnames(scores_mat))
  scores_tbl <- tibble::as_tibble(scores_mat)
  scores_tbl <- dplyr::bind_cols(tibble::tibble(curve_id = curve_ids), scores_tbl)
  
  # Create eigenfunctions tibble
  ef_mat <- pca_res$rotation[, 1:k, drop = FALSE]
  ef_tbl <- tibble::as_tibble(ef_mat) |>
    dplyr::mutate(time = t_grid) |>
    tidyr::pivot_longer(
      cols = dplyr::starts_with("PC"),
      names_to = "FPC",
      values_to = "value"
    ) |>
    dplyr::mutate(FPC = gsub("^PC", "FPC", .data$FPC))
  
  # Create mean curve tibble
  if (center) {
    mean_val <- pca_res$center
  } else {
    mean_val <- colMeans(curve_mat)
  }
  
  mean_curve_tbl <- tibble::tibble(
    time = t_grid,
    mean = as.numeric(mean_val)
  )
  
  # Create reconstructed curves using retained components
  recon_mat <- t(t(pca_res$x[, 1:k, drop = FALSE] %*% t(pca_res$rotation[, 1:k, drop = FALSE])) + pca_res$center)
  
  recon_tbl <- tibble::as_tibble(recon_mat) |>
    dplyr::mutate(curve_id = curve_ids) |>
    tidyr::pivot_longer(
      cols = -curve_id,
      names_to = "time",
      values_to = "reconstructed"
    ) |>
    dplyr::mutate(time = as.numeric(.data$time))
  
  # Input curves
  input_tbl <- wide_mat |>
    dplyr::rename(curve_id = !!.trt) |>
    tidyr::pivot_longer(
      cols = -curve_id,
      names_to = "time",
      values_to = "fitted"
    ) |>
    dplyr::mutate(time = as.numeric(.data$time))
  
  # Combine for reconstructed output
  recon_combined <- input_tbl |>
    dplyr::left_join(recon_tbl, by = c("curve_id", "time"))
  
  settings <- list(
    n_components = n_components,
    var_explained = var_explained,
    center = center,
    scale = scale,
    method = method,
    n_retained = k,
    time_var = .time,
    trt_var = .trt
  )
  
  out <- list(
    scores = scores_tbl,
    eigenfunctions = ef_tbl,
    variance = var_tbl,
    mean_curve = mean_curve_tbl,
    reconstructed = recon_combined,
    input_curves = input_tbl,
    pca = pca_res,
    settings = settings,
    call = match.call()
  )
  
  class(out) <- "r4pde_functional_pca"
  return(out)
}

#' @export
print.r4pde_functional_pca <- function(x, ...) {
  cat("A functional_pca object\n")
  cat("Number of curves:", nrow(x$scores), "\n")
  cat("Number of time points:", nrow(x$mean_curve), "\n")
  cat("Number of retained FPCs:", x$settings$n_retained, "\n")
  cat("\nVariance Explained:\n")
  print(x$variance |> dplyr::select(FPC, prop_var, cum_var), n = x$settings$n_retained)
  invisible(x)
}

#' @export
summary.r4pde_functional_pca <- function(object, ...) {
  list(
    settings = object$settings,
    variance = object$variance,
    score_ranges = lapply(object$scores |> dplyr::select(-curve_id), range),
    n_curves = nrow(object$scores),
    n_timepoints = nrow(object$mean_curve)
  )
}

#' Plot functional PCA results
#' 
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @param type Type of plot: "scree", "components", "scores", "reconstruction", or "mean".
#' @param components Integer vector of length 2 indicating which components to plot for scores and mean perturbation.
#' @param curve_id Optional vector of curve IDs to include in reconstruction plot.
#' @param ... Additional arguments.
#' @export
plot.r4pde_functional_pca <- function(
    x,
    type = c("scree", "components", "scores", "reconstruction", "mean"),
    components = c(1, 2),
    curve_id = NULL,
    ...
) {
  type <- match.arg(type)
  
  if (type == "scree") {
    var_data <- x$variance
    p <- ggplot2::ggplot(var_data, ggplot2::aes(x = factor(.data$FPC, levels = .data$FPC), y = .data$prop_var)) +
      ggplot2::geom_col(fill = "steelblue") +
      ggplot2::geom_line(ggplot2::aes(y = .data$cum_var, group = 1), color = "darkred", size = 1) +
      ggplot2::geom_point(ggplot2::aes(y = .data$cum_var), color = "darkred", size = 2) +
      ggplot2::scale_y_continuous(labels = function(x) paste0(x * 100, "%")) +
      ggplot2::labs(x = "Functional Principal Component", y = "Proportion of Variance Explained",
                    title = "Scree Plot") +
      ggplot2::theme_classic()
    return(p)
  }
  
  if (type == "components") {
    ef_data <- x$eigenfunctions
    if (!is.null(components)) {
      ef_data <- ef_data |> dplyr::filter(.data$FPC %in% paste0("FPC", components))
    }
    p <- ggplot2::ggplot(ef_data, ggplot2::aes(x = .data$time, y = .data$value, color = .data$FPC)) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
      ggplot2::geom_line(size = 1) +
      ggplot2::labs(x = "Time", y = "Eigenfunction Value", title = "Functional Principal Components") +
      ggplot2::theme_classic()
    return(p)
  }
  
  if (type == "scores") {
    if (length(components) != 2) stop("Please specify exactly two components for the scores plot.")
    comp_names <- paste0("FPC", components)
    if (!all(comp_names %in% names(x$scores))) stop("Specified components not found in scores.")
    
    p <- ggplot2::ggplot(x$scores, ggplot2::aes(x = .data[[comp_names[1]]], y = .data[[comp_names[2]]], label = .data$curve_id)) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
      ggplot2::geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
      ggplot2::geom_point(color = "steelblue", size = 3) +
      ggplot2::geom_text(size = 3, vjust = -0.5, hjust = 0.5) +
      ggplot2::labs(x = paste0(comp_names[1], " Score"), y = paste0(comp_names[2], " Score"), 
                    title = "FPC Scores") +
      ggplot2::theme_classic()
    return(p)
  }
  
  if (type == "reconstruction") {
    r_data <- x$reconstructed
    if (!is.null(curve_id)) {
      r_data <- r_data |> dplyr::filter(.data$curve_id %in% curve_id)
    } else {
      # Take a sample of 6 curves if none specified
      sample_ids <- unique(r_data$curve_id)[1:min(6, length(unique(r_data$curve_id)))]
      r_data <- r_data |> dplyr::filter(.data$curve_id %in% sample_ids)
    }
    
    p <- ggplot2::ggplot(r_data, ggplot2::aes(x = .data$time)) +
      ggplot2::geom_line(ggplot2::aes(y = .data$fitted, color = "Fitted"), size = 1) +
      ggplot2::geom_line(ggplot2::aes(y = .data$reconstructed, color = "Reconstructed"), linetype = "dashed", size = 1) +
      ggplot2::facet_wrap(~curve_id) +
      ggplot2::labs(x = "Time", y = "Value", title = "Curve Reconstruction", color = "") +
      ggplot2::scale_color_manual(values = c("Fitted" = "black", "Reconstructed" = "red")) +
      ggplot2::theme_classic() +
      ggplot2::theme(legend.position = "bottom")
    return(p)
  }
  
  if (type == "mean") {
    if (length(components) > 2) stop("Please specify at most two components for the mean perturbation plot.")
    comp_names <- paste0("FPC", components)
    
    mean_df <- x$mean_curve
    ef_df <- x$eigenfunctions |> dplyr::filter(.data$FPC %in% comp_names)
    scores_df <- x$scores
    
    plot_data <- list()
    for (comp in comp_names) {
      if (comp %in% names(scores_df)) {
        sd_score <- sd(scores_df[[comp]])
        ef_val <- ef_df |> dplyr::filter(.data$FPC == comp) |> dplyr::pull(.data$value)
        
        tmp <- tibble::tibble(
          time = mean_df$time,
          FPC = comp,
          mean = mean_df$mean,
          plus_2sd = mean_df$mean + 2 * sd_score * ef_val,
          minus_2sd = mean_df$mean - 2 * sd_score * ef_val
        )
        plot_data[[comp]] <- tmp
      }
    }
    plot_data <- dplyr::bind_rows(plot_data)
    
    p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data$time)) +
      ggplot2::geom_line(ggplot2::aes(y = .data$mean), color = "black", size = 1) +
      ggplot2::geom_line(ggplot2::aes(y = .data$plus_2sd), color = "blue", linetype = "dashed", size = 0.8) +
      ggplot2::geom_line(ggplot2::aes(y = .data$minus_2sd), color = "red", linetype = "dashed", size = 0.8) +
      ggplot2::facet_wrap(~FPC) +
      ggplot2::labs(x = "Time", y = "Value", title = "Mean Curve +/- 2 SD Perturbation") +
      ggplot2::theme_classic()
    return(p)
  }
}

#' @rdname plot.r4pde_functional_pca
#' @param object An object of class \code{"r4pde_functional_pca"}.
#' @export
autoplot.r4pde_functional_pca <- function(object, ...) {
  plot.r4pde_functional_pca(object, ...)
}

#' Get FPCA scores
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @param components Optional integer vector of components to return.
#' @export
get_fpca_scores <- function(x, components = NULL) {
  if (!inherits(x, "r4pde_functional_pca")) stop("Object must be of class 'r4pde_functional_pca'.")
  scores <- x$scores
  if (!is.null(components)) {
    cols <- c("curve_id", paste0("FPC", components))
    scores <- scores |> dplyr::select(dplyr::any_of(cols))
  }
  scores
}

#' Get FPCA eigenfunctions
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @param components Optional integer vector of components to return.
#' @export
get_fpca_eigenfunctions <- function(x, components = NULL) {
  if (!inherits(x, "r4pde_functional_pca")) stop("Object must be of class 'r4pde_functional_pca'.")
  ef <- x$eigenfunctions
  if (!is.null(components)) {
    comps <- paste0("FPC", components)
    ef <- ef |> dplyr::filter(.data$FPC %in% comps)
  }
  ef
}

#' Get FPCA variance explained
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @export
get_fpca_variance <- function(x) {
  if (!inherits(x, "r4pde_functional_pca")) stop("Object must be of class 'r4pde_functional_pca'.")
  x$variance
}

#' Augment functional PCA
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @export
augment_functional_pca <- function(x) {
  if (!inherits(x, "r4pde_functional_pca")) stop("Object must be of class 'r4pde_functional_pca'.")
  x$reconstructed |>
    dplyr::mutate(residual = .data$fitted - .data$reconstructed)
}

#' Reconstruct curves using specified FPCA components
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @param components Integer vector of components to use for reconstruction. If NULL, uses all retained components.
#' @export
reconstruct_curves <- function(x, components = NULL) {
  if (!inherits(x, "r4pde_functional_pca")) stop("Object must be of class 'r4pde_functional_pca'.")
  
  pca <- x$pca
  scores <- pca$x
  rot <- pca$rotation
  
  if (is.null(components)) {
    k <- x$settings$n_retained
    components <- 1:k
  } else {
    valid_comps <- components[components <= ncol(scores)]
    if (length(valid_comps) == 0) stop("Invalid components specified.")
    components <- valid_comps
  }
  
  # Ensure components is not out of bounds
  recon_mat <- matrix(0, nrow = nrow(scores), ncol = nrow(rot))
  for (i in components) {
    recon_mat <- recon_mat + outer(scores[, i], rot[, i])
  }
  recon_mat <- t(t(recon_mat) + pca$center)
  
  recon_tbl <- tibble::as_tibble(recon_mat) |>
    dplyr::mutate(curve_id = rownames(scores)) |>
    tidyr::pivot_longer(
      cols = -curve_id,
      names_to = "time",
      values_to = "reconstructed"
    ) |>
    dplyr::mutate(time = as.numeric(.data$time))
  
  recon_tbl
}

#' Cluster curves based on FPCA scores
#' @param x An object of class \code{"r4pde_functional_pca"}.
#' @param k Number of clusters.
#' @param components Integer vector of components to use.
#' @param method Clustering method: "kmeans" or "hclust".
#' @param choose_k Method for suggesting k: "none", "silhouette", or "elbow".
#' @param ... Additional arguments passed to clustering functions.
#' @export
functional_pca_clusters <- function(
    x,
    k = NULL,
    components = NULL,
    method = c("kmeans", "hclust"),
    choose_k = c("none", "silhouette", "elbow"),
    ...
) {
  method <- match.arg(method)
  choose_k <- match.arg(choose_k)
  
  if (!inherits(x, "r4pde_functional_pca")) stop("Object must be of class 'r4pde_functional_pca'.")
  
  scores <- get_fpca_scores(x, components = components)
  curve_ids <- scores$curve_id
  score_mat <- as.matrix(scores |> dplyr::select(-curve_id))
  
  # Standardize scores before clustering
  score_mat <- scale(score_mat)
  
  # Logic for choosing k if requested, though simplified for now
  if (is.null(k) && choose_k == "none") {
    stop("Must specify `k` or a method for `choose_k`.")
  }
  
  if (is.null(k)) {
    k <- 2 # fallback
  }
  
  if (method == "kmeans") {
    res <- stats::kmeans(score_mat, centers = k, ...)
    clusters <- res$cluster
  } else {
    d <- stats::dist(score_mat)
    hc <- stats::hclust(d, ...)
    clusters <- stats::cutree(hc, k = k)
  }
  
  tibble::tibble(
    curve_id = curve_ids,
    cluster = factor(clusters)
  )
}

#' @export
#' @rdname functional_pca
functional_pca.functional_dsp <- function(
    object,
    n_components = NULL,
    var_explained = 0.95,
    center = TRUE,
    scale = FALSE,
    method = c("pca_on_grid"),
    ...
) {
  method <- match.arg(method)
  
  vars <- attr(object, "dsp_vars")
  .time <- vars$time
  .trt <- vars$treatment
  
  if (is.null(.time) || is.null(.trt)) {
    stop("Time or treatment variable information is missing from the functional_dsp object.")
  }
  
  # Ensure only distinct time points exist per treatment for matrix formatting
  # If there are groups, we need to handle them (PCA on DSPs assumes a single set of curves or curves per group*trt)
  if (length(vars$group) > 0) {
    # Combine group and treatment into a single identifier for curves
    object <- object |>
      dplyr::mutate(.curve_id = interaction(dplyr::select(object, dplyr::all_of(c(vars$group, .trt))), sep = "_"))
    curve_id_col <- ".curve_id"
  } else {
    curve_id_col <- .trt
  }
  
  pred_trt <- object |> dplyr::rename(mu = DSP)
  t_grid <- sort(unique(pred_trt[[.time]]))
  
  if (length(t_grid) < 3) {
    stop("There must be at least three time points in the common grid.")
  }
  
  # Reshape to a wide matrix: rows = individual curves, columns = common time grid
  wide_mat <- pred_trt |>
    dplyr::select(dplyr::all_of(c(curve_id_col, .time, "mu"))) |>
    tidyr::pivot_wider(names_from = dplyr::all_of(.time), values_from = "mu")
  
  curve_ids <- wide_mat[[curve_id_col]]
  
  if (length(unique(curve_ids)) < 2) {
    stop("There must be at least two curves.")
  }
  
  # Extract curve matrix
  curve_mat <- as.matrix(wide_mat[, -1, drop = FALSE])
  rownames(curve_mat) <- as.character(curve_ids)
  
  if (any(is.na(curve_mat))) {
    stop("Missing fitted values found in the curve matrix. Ensure times are uniform across treatments.")
  }
  
  # Perform PCA
  pca_res <- stats::prcomp(curve_mat, center = center, scale. = scale)
  
  # Variance explained
  eigenvalues <- pca_res$sdev^2
  prop_var <- eigenvalues / sum(eigenvalues)
  cum_var <- cumsum(prop_var)
  
  # Select retained components
  if (!is.null(n_components)) {
    k <- min(n_components, length(eigenvalues))
  } else {
    k <- which(cum_var >= var_explained)[1]
    if (is.na(k)) k <- length(eigenvalues)
  }
  
  # Create variance tibble
  var_tbl <- tibble::tibble(
    FPC = paste0("FPC", seq_along(eigenvalues)),
    eigenvalue = eigenvalues,
    prop_var = prop_var,
    cum_var = cum_var
  ) |>
    dplyr::slice(1:k)
  
  # Create scores tibble
  scores_mat <- pca_res$x[, 1:k, drop = FALSE]
  colnames(scores_mat) <- gsub("^PC", "FPC", colnames(scores_mat))
  scores_tbl <- tibble::as_tibble(scores_mat)
  scores_tbl <- dplyr::bind_cols(tibble::tibble(curve_id = curve_ids), scores_tbl)
  
  # Create eigenfunctions tibble
  ef_mat <- pca_res$rotation[, 1:k, drop = FALSE]
  ef_tbl <- tibble::as_tibble(ef_mat) |>
    dplyr::mutate(time = t_grid) |>
    tidyr::pivot_longer(
      cols = dplyr::starts_with("PC"),
      names_to = "FPC",
      values_to = "value"
    ) |>
    dplyr::mutate(FPC = gsub("^PC", "FPC", .data$FPC))
  
  # Create mean curve tibble
  if (center) {
    mean_val <- pca_res$center
  } else {
    mean_val <- colMeans(curve_mat)
  }
  
  mean_curve_tbl <- tibble::tibble(
    time = t_grid,
    mean = as.numeric(mean_val)
  )
  
  # Create reconstructed curves using retained components
  recon_mat <- t(t(pca_res$x[, 1:k, drop = FALSE] %*% t(pca_res$rotation[, 1:k, drop = FALSE])) + pca_res$center)
  
  recon_tbl <- tibble::as_tibble(recon_mat) |>
    dplyr::mutate(curve_id = curve_ids) |>
    tidyr::pivot_longer(
      cols = -curve_id,
      names_to = "time",
      values_to = "reconstructed"
    ) |>
    dplyr::mutate(time = as.numeric(.data$time))
  
  # Input curves
  input_tbl <- wide_mat |>
    dplyr::rename(curve_id = dplyr::all_of(curve_id_col)) |>
    tidyr::pivot_longer(
      cols = -curve_id,
      names_to = "time",
      values_to = "fitted"
    ) |>
    dplyr::mutate(time = as.numeric(.data$time))
  
  # Combine for reconstructed output
  recon_combined <- input_tbl |>
    dplyr::left_join(recon_tbl, by = c("curve_id", "time"))
  
  settings <- list(
    n_components = n_components,
    var_explained = var_explained,
    center = center,
    scale = scale,
    method = method,
    n_retained = k,
    time_var = .time,
    trt_var = .trt
  )
  
  out <- list(
    scores = scores_tbl,
    eigenfunctions = ef_tbl,
    variance = var_tbl,
    mean_curve = mean_curve_tbl,
    reconstructed = recon_combined,
    input_curves = input_tbl,
    pca = pca_res,
    settings = settings,
    call = match.call()
  )
  
  class(out) <- "r4pde_functional_pca"
  return(out)
}

test_that("functional_pca rejects invalid object classes", {
  invalid_obj <- list(curves = data.frame(time = 1:5, mu = 1:5))
  class(invalid_obj) <- "some_other_class"
  expect_error(functional_pca(invalid_obj), "must be of class 'functional_curves'")
})

test_that("functional_pca errors clearly when fitted curves are unavailable", {
  invalid_fc <- list(vars = list(time = "time", treatment = "trt"))
  class(invalid_fc) <- "functional_curves"
  expect_error(functional_pca(invalid_fc), "Fitted curves are not available")
})

test_that("functional_pca works properly on valid input", {
  # Mock a functional_curves object
  t_grid <- seq(1, 10, length.out = 10)
  trts <- c("A", "B", "C")
  
  curves_df <- expand.grid(time = t_grid, trt = trts)
  # Simple curves
  curves_df$mu <- ifelse(curves_df$trt == "A", curves_df$time * 0.1, 
                         ifelse(curves_df$trt == "B", curves_df$time * 0.2, curves_df$time * 0.3))
  
  mock_fc <- list(
    curves = curves_df,
    grid = t_grid,
    vars = list(time = "time", treatment = "trt")
  )
  class(mock_fc) <- "functional_curves"
  
  res <- functional_pca(mock_fc, center = TRUE, scale = FALSE)
  
  # 3. output inherits from "r4pde_functional_pca"
  expect_s3_class(res, "r4pde_functional_pca")
  
  # 4. scores have one row per curve
  expect_equal(nrow(res$scores), length(trts))
  expect_true("curve_id" %in% names(res$scores))
  
  # 5. eigenfunctions have values over the full time grid
  ef <- res$eigenfunctions
  expect_equal(length(unique(ef$time)), length(t_grid))
  expect_true(all(t_grid %in% ef$time))
  
  # 6. variance proportions sum to approximately 1 across all components
  # If we set n_components = NULL, var_explained = 1.0, it will return all.
  res_all <- functional_pca(mock_fc, center = TRUE, scale = FALSE, var_explained = 1.0)
  expect_equal(sum(res_all$variance$prop_var), 1, tolerance = 1e-5)
  
  # 7. retained components satisfy n_components or var_explained
  res_n1 <- functional_pca(mock_fc, n_components = 1)
  expect_equal(nrow(res_n1$variance), 1)
  expect_equal(res_n1$settings$n_retained, 1)
  
  # 8. reconstruction works and returns the expected columns
  expect_true(all(c("curve_id", "time", "fitted", "reconstructed") %in% names(res$reconstructed)))
  
  # 9. extractor functions return data frames
  scores_df <- get_fpca_scores(res)
  expect_s3_class(scores_df, "data.frame")
  
  ef_df <- get_fpca_eigenfunctions(res)
  expect_s3_class(ef_df, "data.frame")
  
  var_df <- get_fpca_variance(res)
  expect_s3_class(var_df, "data.frame")
  
  aug_df <- augment_functional_pca(res)
  expect_s3_class(aug_df, "data.frame")
  expect_true("residual" %in% names(aug_df))
  
  recon_df <- reconstruct_curves(res, components = 1)
  expect_s3_class(recon_df, "data.frame")
  
  # 10. plot methods return ggplot objects if ggplot2 is used
  p1 <- plot(res, type = "scree")
  expect_s3_class(p1, "ggplot")
  
  p2 <- plot(res, type = "components")
  expect_s3_class(p2, "ggplot")
  
  p3 <- plot(res, type = "reconstruction")
  expect_s3_class(p3, "ggplot")

  # Test scores plot with object retaining at least 2 components
  res_2comp <- functional_pca(mock_fc, n_components = 2)
  expect_true(all(c("curve_id", "FPC1", "FPC2") %in% names(res_2comp$scores)))
  p4 <- plot(res_2comp, type = "scores", components = c(1, 2))
  expect_s3_class(p4, "ggplot")
})

#' Instantaneous Disease Progress Rates from Functional Curves
#'
#' @description
#' Estimates instantaneous rates of plant disease progress (\eqn{S'(t) = dS(t)/dt})
#' and associated uncertainty from epidemic trajectories fitted by
#' \code{\link{functional_curves}}. Derivatives are computed using the GAM linear
#' predictor matrix (\code{lpmatrix}) with finite differences, supporting both
#' the original response scale (e.g., severity proportion/percentage points per day)
#' and the link (linear predictor) scale. Also computes epidemic rate phenotypes
#' including maximum rate (\eqn{r_{max}}), time of maximum rate (\eqn{t_{r_{max}}}),
#' growth duration, cumulative positive growth, and distinct growth windows.
#'
#' @param object An object of class \code{"functional_curves"}.
#' @param scale Character specifying the derivative scale: \code{"response"}
#'   (default; e.g., severity change per unit time) or \code{"link"} (linear predictor scale).
#' @param method Character specifying the finite difference method. Currently \code{"central"}
#'   (central difference in the interior with one-sided differences at temporal boundaries).
#' @param n_grid Integer; number of points in the regular temporal evaluation grid per group.
#'   Default is 200.
#' @param eps Numeric step size used in finite differences. If \code{NULL} (default),
#'   automatically chosen as \eqn{10^{-4} \times (t_{max} - t_{min})}.
#' @param interval Numeric confidence level for intervals. Default is 0.95.
#' @param threshold Numeric minimum rate threshold considered biologically relevant.
#'   Default is 0.
#' @param positive_only Logical; whether to restrict summarized growth descriptors
#'   to positive rates. Default is \code{TRUE}. Note that negative rates are always
#'   preserved in the detailed rate table (\code{data}).
#' @param uncertainty Logical; whether to compute standard errors and confidence
#'   intervals using the GAM covariance matrix. Default is \code{TRUE}.
#' @param growth_criterion Character specifying how active epidemic growth is defined:
#'   \code{"rate"} (default; \code{rate > threshold}) or \code{"significant"}
#'   (\code{rate_lower > threshold}, requiring \code{uncertainty = TRUE}).
#' @param newdata Optional user-supplied data frame with custom evaluation points.
#'   Must contain the time and treatment variables.
#' @param ... Additional arguments passed to methods.
#'
#' @details
#' \strong{Mathematical Formulation:}
#' Let \eqn{S(t)} denote the fitted disease trajectory and \eqn{\eta(t) = X(t)\beta}
#' the linear predictor from the GAM model, with \eqn{\mu(t) = g^{-1}(\eta(t))}.
#'
#' The derivative on the link scale is estimated by:
#' \deqn{D = \frac{X(t + \epsilon) - X(t - \epsilon)}{2\epsilon}}
#' \deqn{\eta'(t) = D \beta}
#' with variance \eqn{\operatorname{Var}(\eta'(t)) = \operatorname{diag}(D V_p D^T)},
#' where \eqn{V_p} is the Bayesian posterior covariance matrix of the GAM parameters.
#'
#' On the response scale (\code{scale = "response"}), the derivative of the mean trajectory is:
#' \deqn{S'(t) = \frac{\mu(\eta(t + \epsilon)) - \mu(\eta(t - \epsilon))}{2\epsilon}}
#' The approximate gradient vector \eqn{G} with respect to \eqn{\beta} is:
#' \deqn{G = \frac{\mu'(\eta(t + \epsilon)) X(t + \epsilon) - \mu'(\eta(t - \epsilon)) X(t - \epsilon)}{2\epsilon}}
#' where \eqn{\mu'(\eta) = d\mu/d\eta} is given by \code{family$mu.eta()}.
#' The uncertainty is then:
#' \deqn{\operatorname{Var}(S'(t)) = \operatorname{diag}(G V_p G^T)}
#'
#' \strong{Boundary Handling:}
#' At the observed boundaries \eqn{t_{min}} and \eqn{t_{max}}, one-sided forward and backward
#' differences are used to prevent unintended extrapolation beyond the observed domain.
#'
#' \strong{Time-Invariant Terms and Random Effects:}
#' Intercepts, main treatment effects, and terms constant with respect to time cancel
#' analytically in \eqn{X(t + \epsilon) - X(t - \epsilon)}. Environmental and experimental
#' unit random effects are excluded by default to yield population-adjusted treatment rates,
#' consistent with \code{\link{functional_curves}}.
#'
#' \strong{No-Epidemic and Flat Curves:}
#' Curves with all observed response values equal to zero (or estimated rates below
#' numerical tolerance \eqn{10^{-6}}) explicitly return \eqn{r_{max} = 0}, \eqn{t_{r_{max}} = \text{NA}},
#' \eqn{\text{growth\_duration} = 0}, and \eqn{\text{cumulative\_positive\_growth} = 0}.
#'
#' @return An object of class \code{"functional_rate"} containing:
#' \describe{
#'   \item{\code{data}}{A tibble of full evaluation grid with columns including
#'     treatment, time, \code{fitted}, \code{fitted_se}, \code{fitted_lower}, \code{fitted_upper},
#'     \code{rate}, \code{rate_se}, \code{rate_lower}, \code{rate_upper},
#'     \code{significant_increase}, \code{above_threshold}, and \code{significant_above_threshold}.}
#'   \item{\code{summary}}{A tibble of curve-level rate phenotypes:
#'     \code{r_max}, \code{t_r_max}, \code{r_mean}, \code{r_mean_positive},
#'     \code{t_growth_start}, \code{t_growth_end}, \code{growth_duration},
#'     \code{cumulative_positive_growth}, and \code{n_growth_windows}.}
#'   \item{\code{model}}{The underlying GAM model object.}
#'   \item{\code{call}}{The matched call.}
#'   \item{\code{settings}}{List of analysis settings used.}
#'   \item{\code{vars}}{List of variable names identified in the model.}
#'   \item{\code{family}}{Character string naming the GAM family.}
#' }
#'
#' @references
#' Madden, L. V., Hughes, G., & van den Bosch, F. (2007).
#' The Study of Plant Disease Epidemics. APS Press, St. Paul, MN.
#'
#' Wood, S. N. (2017). Generalized Additive Models: An Introduction with R (2nd ed.).
#' Chapman and Hall/CRC.
#'
#' Simpson, G. L. (2018). Modelling Palaeoecological Time Series Using Generalized Additive Models.
#' Frontiers in Ecology and Evolution, 6, 149.
#'
#' @examples
#' \dontrun{
#' set.seed(123)
#' df <- data.frame(
#'   time = rep(1:30, 3),
#'   treatment = rep(c("Early", "Late", "None"), each = 30),
#'   y = c(
#'     1 / (1 + exp(-(1:30 - 10) * 0.4)),  # Early
#'     1 / (1 + exp(-(1:30 - 20) * 0.4)),  # Late
#'     rep(0, 30)                          # None
#'   )
#' )
#'
#' curves <- functional_curves(
#'   data = df, time = "time", response = "y", treatment = "treatment",
#'   min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
#' )
#'
#' rates <- functional_rate(curves)
#' print(rates)
#' summary(rates)
#' plot(rates)
#' }
#'
#' @export
functional_rate <- function(
    object,
    scale = c("response", "link"),
    method = c("central"),
    n_grid = 200,
    eps = NULL,
    interval = 0.95,
    threshold = 0,
    positive_only = TRUE,
    uncertainty = TRUE,
    growth_criterion = c("rate", "significant"),
    newdata = NULL,
    ...
) {
  scale <- match.arg(scale)
  method <- match.arg(method)
  growth_criterion <- match.arg(growth_criterion)

  # Validation of input object
  if (!inherits(object, "functional_curves")) {
    stop("`object` must be of class 'functional_curves'.", call. = FALSE)
  }

  m_gam <- object$gam
  if (is.null(m_gam)) {
    stop("The GAM model is missing from `object`.", call. = FALSE)
  }

  .time <- object$vars$time
  .trt  <- object$vars$treatment
  .env  <- object$vars$environment
  .unit <- object$vars$unit
  covariates <- object$vars$covariates

  if (is.null(.time) || !is.character(.time) || length(.time) != 1) {
    stop("Time variable is not identifiable in `object`.", call. = FALSE)
  }
  if (is.null(.trt) || !is.character(.trt) || length(.trt) != 1) {
    stop("Treatment variable is not identifiable in `object`.", call. = FALSE)
  }

  # Validate arguments
  if (!is.numeric(n_grid) || length(n_grid) != 1 || is.na(n_grid) || n_grid < 2) {
    stop("`n_grid` must be a single integer >= 2.", call. = FALSE)
  }
  n_grid <- as.integer(n_grid)

  if (!is.null(eps)) {
    if (!is.numeric(eps) || length(eps) != 1 || is.na(eps) || eps <= 0) {
      stop("`eps` must be a single positive numeric value.", call. = FALSE)
    }
  }

  if (!is.numeric(interval) || length(interval) != 1 || is.na(interval) ||
      interval <= 0 || interval >= 1) {
    stop("`interval` must be a single numeric value in (0, 1).", call. = FALSE)
  }

  obs_df <- object$observed_data
  if (is.null(obs_df) && is.null(newdata)) {
    stop("`observed_data` is missing from `object` and `newdata` was not supplied.", call. = FALSE)
  }

  # Validate time domain in observed data if available
  if (!is.null(obs_df)) {
    times_by_trt <- obs_df |>
      dplyr::group_by(.data[[.trt]]) |>
      dplyr::summarise(
        n_times = dplyr::n_distinct(.data[[.time]]),
        t_min = min(.data[[.time]], na.rm = TRUE),
        t_max = max(.data[[.time]], na.rm = TRUE),
        .groups = "drop"
      )

    if (any(times_by_trt$n_times < 2)) {
      bad_trts <- times_by_trt |> dplyr::filter(.data$n_times < 2) |> dplyr::pull(.data[[.trt]])
      stop("The following groups have fewer than two distinct observed time points: ",
           paste(bad_trts, collapse = ", "), call. = FALSE)
    }

    if (any(times_by_trt$n_times < 4)) {
      warning("Some groups have very few (< 4) distinct observed time points; rate estimates may be unstable.",
              call. = FALSE)
    }
  }

  # Identify treatments with all-zero observed responses
  zero_epidemic_trts <- character(0)
  if (!is.null(obs_df)) {
    resp_col <- if (".y_raw" %in% names(obs_df)) {
      ".y_raw"
    } else if (!is.null(object$vars$response) && object$vars$response %in% names(obs_df)) {
      object$vars$response
    } else {
      NULL
    }

    if (!is.null(resp_col)) {
      zero_check <- obs_df |>
        dplyr::group_by(.data[[.trt]]) |>
        dplyr::summarise(
          all_zero = all(abs(.data[[resp_col]]) < 1e-6, na.rm = TRUE),
          .groups = "drop"
        )
      zero_epidemic_trts <- as.character(zero_check[[.trt]][zero_check$all_zero])
    }
  }

  # Extract model levels
  trt_var_levels <- if (!is.null(obs_df)) {
    levels(factor(obs_df[[.trt]]))
  } else {
    levels(factor(object$curves[[.trt]]))
  }
  env_var_levels <- if (!is.null(.env) && !is.null(obs_df)) levels(factor(obs_df[[.env]])) else NULL
  unit_var_levels <- if (!is.null(.unit) && !is.null(obs_df) && .unit %in% names(obs_df)) {
    levels(factor(obs_df[[.unit]]))
  } else {
    NULL
  }

  # Construct evaluation grid
  if (is.null(newdata)) {
    genotype_info <- object$genotype_info

    grid_list <- lapply(trt_var_levels, function(trt_i) {
      sub_obs <- if (!is.null(obs_df)) {
        obs_df[obs_df[[.trt]] == trt_i, , drop = FALSE]
      } else {
        object$curves[object$curves[[.trt]] == trt_i, , drop = FALSE]
      }

      tmin_i <- min(sub_obs[[.time]], na.rm = TRUE)
      tmax_i <- max(sub_obs[[.time]], na.rm = TRUE)

      if (!is.finite(tmin_i) || !is.finite(tmax_i) || tmax_i - tmin_i < 1e-6) {
        warning(sprintf("Group '%s' has an extremely short time span (span < 1e-6).", trt_i), call. = FALSE)
      }

      t_grid_i <- seq(tmin_i, tmax_i, length.out = n_grid)

      df_i <- tibble::tibble(
        !!.trt := factor(trt_i, levels = trt_var_levels),
        !!.time := t_grid_i,
        .t_min = tmin_i,
        .t_max = tmax_i
      )

      if (!is.null(covariates) && !is.null(genotype_info)) {
        cov_vals <- genotype_info |> dplyr::filter(.data[[.trt]] == trt_i)
        for (cov in covariates) {
          if (cov %in% names(cov_vals)) {
            df_i[[cov]] <- cov_vals[[cov]][1]
          }
        }
      }

      if (!is.null(.env)) {
        ref_env <- if (!is.null(object$settings$env_ref)) object$settings$env_ref else env_var_levels[1]
        df_i[[.env]] <- factor(ref_env, levels = env_var_levels)
      }

      if (!is.null(.unit) && !is.null(unit_var_levels) && length(unit_var_levels) > 0) {
        df_i[[.unit]] <- factor(unit_var_levels[1], levels = unit_var_levels)
      }

      df_i
    })

    grid_df <- dplyr::bind_rows(grid_list)

  } else {
    # Custom newdata provided
    req_cols <- c(.time, .trt)
    if (!is.null(.env)) req_cols <- c(req_cols, .env)
    if (!is.null(covariates)) req_cols <- c(req_cols, covariates)

    missing_cols <- setdiff(req_cols, names(newdata))
    if (length(missing_cols) > 0) {
      stop("`newdata` is missing required variables: ", paste(missing_cols, collapse = ", "), call. = FALSE)
    }

    grid_df <- tibble::as_tibble(newdata)
    grid_df[[.trt]] <- factor(grid_df[[.trt]], levels = trt_var_levels)
    grid_df[[.time]] <- as.numeric(grid_df[[.time]])

    if (!is.null(.env)) {
      grid_df[[.env]] <- factor(grid_df[[.env]], levels = env_var_levels)
    }
    if (!is.null(.unit) && !is.null(unit_var_levels) && length(unit_var_levels) > 0) {
      if (!(.unit %in% names(grid_df))) {
        grid_df[[.unit]] <- factor(unit_var_levels[1], levels = unit_var_levels)
      } else {
        grid_df[[.unit]] <- factor(grid_df[[.unit]], levels = unit_var_levels)
      }
    }

    # Check for extrapolation
    if (!is.null(obs_df)) {
      t_bounds <- obs_df |>
        dplyr::group_by(.data[[.trt]]) |>
        dplyr::summarise(
          t_min = min(.data[[.time]], na.rm = TRUE),
          t_max = max(.data[[.time]], na.rm = TRUE),
          .groups = "drop"
        )
      grid_df <- dplyr::left_join(grid_df, t_bounds, by = .trt)

      # Check bounds
      out_of_bounds <- grid_df[[.time]] < grid_df$t_min - 1e-7 | grid_df[[.time]] > grid_df$t_max + 1e-7
      if (any(out_of_bounds, na.rm = TRUE)) {
        stop("`newdata` contains time points outside the observed domain (extrapolation is not permitted).",
             call. = FALSE)
      }
      grid_df <- dplyr::rename(grid_df, .t_min = "t_min", .t_max = "t_max")
    } else {
      grid_df$.t_min <- min(grid_df[[.time]], na.rm = TRUE)
      grid_df$.t_max <- max(grid_df[[.time]], na.rm = TRUE)
    }
  }

  # Build perturbation grids for finite differences
  t_vals <- grid_df[[.time]]
  t_min_v <- grid_df$.t_min
  t_max_v <- grid_df$.t_max
  span_v <- pmax(t_max_v - t_min_v, 1e-6)

  eps_v <- if (!is.null(eps)) rep(eps, length(t_vals)) else 1e-4 * span_v
  eps_v <- pmin(eps_v, span_v / 4)

  # Boundary identification:
  # Left boundary: t - eps < t_min
  # Right boundary: t + eps > t_max
  # Interior: central difference
  is_left  <- (t_vals - eps_v < t_min_v - 1e-9)
  is_right <- (t_vals + eps_v > t_max_v + 1e-9)

  t1_v <- ifelse(is_left, t_vals, ifelse(is_right, t_vals - eps_v, t_vals - eps_v))
  t2_v <- ifelse(is_left, t_vals + eps_v, ifelse(is_right, t_vals, t_vals + eps_v))
  h_v  <- t2_v - t1_v

  # Data frames for GAM prediction
  df_0 <- grid_df |> dplyr::select(-".t_min", -".t_max")
  df_1 <- df_0
  df_2 <- df_0
  df_1[[.time]] <- t1_v
  df_2[[.time]] <- t2_v

  # Exclude random effect terms from prediction
  excl <- character()
  if (!is.null(.env)) excl <- c(excl, sprintf("s(%s)", .env))

  smooth_labels <- if (!is.null(m_gam$smooth)) {
    vapply(m_gam$smooth, function(s) s$label, character(1))
  } else {
    character(0)
  }
  if (!is.null(.unit) && any(grepl(sprintf("s(%s)", .unit), smooth_labels, fixed = TRUE))) {
    excl <- c(excl, sprintf("s(%s)", .unit))
  }

  # Predict lpmatrix
  X_0 <- mgcv::predict.gam(m_gam, newdata = df_0, type = "lpmatrix", exclude = excl)
  X_1 <- mgcv::predict.gam(m_gam, newdata = df_1, type = "lpmatrix", exclude = excl)
  X_2 <- mgcv::predict.gam(m_gam, newdata = df_2, type = "lpmatrix", exclude = excl)

  beta <- stats::coef(m_gam)
  Vp   <- m_gam$Vp
  if (is.null(Vp)) Vp <- stats::vcov(m_gam)

  fam     <- m_gam$family
  linkinv <- fam$linkinv
  mu.eta  <- fam$mu.eta

  if (is.null(linkinv) || is.null(mu.eta)) {
    stop("The model family does not provide valid `linkinv` or `mu.eta` functions.", call. = FALSE)
  }

  eta_0 <- as.numeric(X_0 %*% beta)
  eta_1 <- as.numeric(X_1 %*% beta)
  eta_2 <- as.numeric(X_2 %*% beta)

  mu_0 <- linkinv(eta_0)
  mu_1 <- linkinv(eta_1)
  mu_2 <- linkinv(eta_2)

  z <- stats::qnorm(1 - (1 - interval) / 2)

  if (scale == "link") {
    D <- (X_2 - X_1) / h_v
    rate <- (eta_2 - eta_1) / h_v
    fitted <- eta_0

    if (uncertainty) {
      fitted_se <- sqrt(pmax(0, rowSums((X_0 %*% Vp) * X_0)))
      fitted_lower <- fitted - z * fitted_se
      fitted_upper <- fitted + z * fitted_se

      var_rate <- pmax(0, rowSums((D %*% Vp) * D))
      rate_se  <- sqrt(var_rate)
      rate_lower <- rate - z * rate_se
      rate_upper <- rate + z * rate_se
    } else {
      fitted_se <- NA_real_
      fitted_lower <- NA_real_
      fitted_upper <- NA_real_
      rate_se <- NA_real_
      rate_lower <- NA_real_
      rate_upper <- NA_real_
    }
  } else {
    # scale == "response"
    rate <- (mu_2 - mu_1) / h_v
    fitted <- mu_0

    if (uncertainty) {
      se_eta_0 <- sqrt(pmax(0, rowSums((X_0 %*% Vp) * X_0)))
      fitted_se <- abs(mu.eta(eta_0)) * se_eta_0
      fitted_lower <- linkinv(eta_0 - z * se_eta_0)
      fitted_upper <- linkinv(eta_0 + z * se_eta_0)

      G <- (mu.eta(eta_2) * X_2 - mu.eta(eta_1) * X_1) / h_v
      var_rate <- pmax(0, rowSums((G %*% Vp) * G))
      rate_se  <- sqrt(var_rate)
      rate_lower <- rate - z * rate_se
      rate_upper <- rate + z * rate_se
    } else {
      fitted_se <- NA_real_
      fitted_lower <- NA_real_
      fitted_upper <- NA_real_
      rate_se <- NA_real_
      rate_lower <- NA_real_
      rate_upper <- NA_real_
    }
  }

  significant_increase <- if (uncertainty) (rate_lower > 0) else NA
  above_threshold      <- (rate > threshold)
  significant_above_threshold <- if (uncertainty) (rate_lower > threshold) else NA

  # Handle zero-epidemic curves explicitly in rate_data
  trt_vec <- as.character(df_0[[.trt]])
  for (zero_trt in zero_epidemic_trts) {
    idx_z <- which(trt_vec == zero_trt)
    if (length(idx_z) > 0) {
      fitted[idx_z] <- 0
      rate[idx_z]   <- 0
      if (uncertainty) {
        fitted_se[idx_z]    <- 0
        fitted_lower[idx_z] <- 0
        fitted_upper[idx_z] <- 0
        rate_se[idx_z]      <- 0
        rate_lower[idx_z]   <- 0
        rate_upper[idx_z]   <- 0
        significant_increase[idx_z] <- FALSE
        significant_above_threshold[idx_z] <- FALSE
      }
      above_threshold[idx_z] <- FALSE
    }
  }

  # Build full rate table
  rate_data <- df_0
  rate_data$fitted <- fitted
  rate_data$fitted_se <- fitted_se
  rate_data$fitted_lower <- fitted_lower
  rate_data$fitted_upper <- fitted_upper
  rate_data$rate <- rate
  rate_data$rate_se <- rate_se
  rate_data$rate_lower <- rate_lower
  rate_data$rate_upper <- rate_upper
  rate_data$significant_increase <- significant_increase
  rate_data$above_threshold <- above_threshold
  rate_data$significant_above_threshold <- significant_above_threshold

  # Warnings for negative rates and extreme uncertainty
  neg_prop <- mean(rate < 0, na.rm = TRUE)
  if (is.finite(neg_prop) && neg_prop > 0.30) {
    warning(sprintf("A large proportion (%.1f%%) of estimated rates are negative. Check model fit or smoothing dimension.",
                    neg_prop * 100), call. = FALSE)
  }

  if (uncertainty) {
    high_se <- which(is.finite(rate_se) & is.finite(rate) & (abs(rate) > 1e-4) & (rate_se / abs(rate) > 50))
    if (length(high_se) > 0) {
      warning("Very high uncertainty detected in some rate estimates (rate_se / |rate| > 50).", call. = FALSE)
    }
  }

  # Compute rate_summary per curve
  group_cols <- c(.trt)
  if (!is.null(.env) && .env %in% names(rate_data) && dplyr::n_distinct(rate_data[[.env]]) > 1) {
    group_cols <- c(group_cols, .env)
  }

  summary_rows <- rate_data |>
    dplyr::group_by(dplyr::across(dplyr::all_of(group_cols))) |>
    dplyr::group_modify(~ {
      sub_t   <- .x[[.time]]
      sub_r   <- .x$rate
      sub_low <- .x$rate_lower
      sub_fit <- .x$fitted
      trt_cur <- as.character(.y[[.trt]][1])

      is_zero_curve <- (trt_cur %in% zero_epidemic_trts) ||
                       all(abs(sub_r) < 1e-6, na.rm = TRUE) ||
                       all(abs(sub_fit) < 1e-6, na.rm = TRUE)

      # Evaluate active growth criterion
      is_growth <- if (growth_criterion == "significant") {
        if (!uncertainty) {
          warning("`growth_criterion = 'significant'` requires `uncertainty = TRUE`; falling back to `rate > threshold`.", call. = FALSE)
          sub_r > threshold
        } else {
          sub_low > threshold
        }
      } else {
        sub_r > threshold
      }

      # Duration and windows
      if (is_zero_curve || !any(is_growth, na.rm = TRUE)) {
        r_max <- if (any(sub_r > 0, na.rm = TRUE) && !is_zero_curve) max(sub_r, na.rm = TRUE) else 0
        if (r_max < 1e-6) r_max <- 0
        t_r_max <- if (r_max > 0) sub_t[which.max(sub_r)] else NA_real_

        span_t <- max(sub_t, na.rm = TRUE) - min(sub_t, na.rm = TRUE)
        r_mean <- if (span_t > 0) trapz_vec_internal(sub_t, sub_r) / span_t else mean(sub_r, na.rm = TRUE)
        r_mean_pos <- 0
        t_growth_start <- NA_real_
        t_growth_end   <- NA_real_
        growth_duration <- 0
        cum_growth <- 0
        n_windows <- 0
      } else {
        r_max <- max(sub_r, na.rm = TRUE)
        if (r_max < 1e-6) {
          r_max <- 0
          t_r_max <- NA_real_
        } else {
          t_r_max <- sub_t[which.max(sub_r)]
        }

        span_t <- max(sub_t, na.rm = TRUE) - min(sub_t, na.rm = TRUE)
        r_mean <- if (span_t > 0) trapz_vec_internal(sub_t, sub_r) / span_t else mean(sub_r, na.rm = TRUE)

        rates_pos <- sub_r[sub_r > threshold]
        r_mean_pos <- if (length(rates_pos) > 0) mean(rates_pos, na.rm = TRUE) else 0

        grow_idx <- which(is_growth)
        t_growth_start <- sub_t[grow_idx[1]]
        t_growth_end   <- sub_t[tail(grow_idx, 1)]

        # Runs of growth
        rle_growth <- rle(is_growth)
        starts <- c(1, cumsum(rle_growth$lengths) + 1)[1:length(rle_growth$lengths)]
        ends   <- cumsum(rle_growth$lengths)

        grow_runs <- which(rle_growth$values)
        n_windows <- length(grow_runs)

        if (n_windows > 0) {
          durations <- vapply(grow_runs, function(idx) {
            s_i <- starts[idx]
            e_i <- ends[idx]
            if (e_i > s_i) sub_t[e_i] - sub_t[s_i] else 0
          }, numeric(1))
          growth_duration <- sum(durations)
        } else {
          growth_duration <- 0
        }

        cum_growth <- trapz_vec_internal(sub_t, pmax(sub_r - threshold, 0))
        if (is.na(cum_growth)) cum_growth <- 0
      }

      tibble::tibble(
        r_max = r_max,
        t_r_max = t_r_max,
        r_mean = r_mean,
        r_mean_positive = r_mean_pos,
        t_growth_start = t_growth_start,
        t_growth_end = t_growth_end,
        growth_duration = growth_duration,
        cumulative_positive_growth = cum_growth,
        n_growth_windows = as.integer(n_windows)
      )
    }) |>
    dplyr::ungroup()

  res <- list(
    data = rate_data,
    summary = summary_rows,
    model = m_gam,
    call = match.call(),
    settings = list(
      scale = scale,
      method = method,
      n_grid = n_grid,
      eps = eps,
      interval = interval,
      threshold = threshold,
      positive_only = positive_only,
      uncertainty = uncertainty,
      growth_criterion = growth_criterion
    ),
    vars = object$vars,
    family = fam$family
  )

  class(res) <- "functional_rate"
  res
}

#' Print a functional_rate object
#'
#' @param x An object of class \code{"functional_rate"}.
#' @param ... Additional arguments.
#' @return The object \code{x}, invisibly.
#' @export
print.functional_rate <- function(x, ...) {
  cat("=========================================================\n")
  cat("Instantaneous Epidemic Rates (functional_rate)\n")
  cat("=========================================================\n")

  .trt  <- x$vars$treatment
  .time <- x$vars$time

  n_curves <- nrow(x$summary)
  t_min <- min(x$data[[.time]], na.rm = TRUE)
  t_max <- max(x$data[[.time]], na.rm = TRUE)

  n_detectable <- sum(x$summary$r_max > x$settings$threshold, na.rm = TRUE)

  cat(sprintf("Number of curves: %d\n", n_curves))
  cat(sprintf("Time domain: [%.2f, %.2f] (%s)\n", t_min, t_max, .time))
  cat(sprintf("Rate scale: %s\n", x$settings$scale))
  cat(sprintf("Method: %s finite difference\n", x$settings$method))
  cat(sprintf("Grid resolution: %d points per curve\n", x$settings$n_grid))
  cat(sprintf("Curves with detectable growth: %d of %d (threshold = %g)\n",
              n_detectable, n_curves, x$settings$threshold))
  cat("=========================================================\n")
  invisible(x)
}

#' Summarize a functional_rate object
#'
#' @param object An object of class \code{"functional_rate"}.
#' @param ... Additional arguments.
#' @return A tibble of curve-level rate phenotypes.
#' @export
summary.functional_rate <- function(object, ...) {
  object$summary
}

#' Plot functional_rate curves and rates
#'
#' @param x An object of class \code{"functional_rate"}.
#' @param which Optional character vector specifying which treatments/groups to display.
#' @param type Character specifying what to plot: \code{"both"} (default; fitted curve
#'   in top panel and derivative in bottom panel), \code{"curve"} (only trajectory),
#'   or \code{"rate"} (only instantaneous rate).
#' @param show_ci Logical; whether to show confidence ribbons. Default is \code{TRUE}.
#' @param show_threshold Logical; whether to show threshold and zero horizontal lines.
#'   Default is \code{TRUE}.
#' @param ncol Optional integer specifying number of facet columns.
#' @param ... Additional arguments.
#'
#' @return A \code{ggplot} or combined \code{patchwork} object.
#' @export
plot.functional_rate <- function(
    x,
    which = NULL,
    type = c("both", "curve", "rate"),
    show_ci = TRUE,
    show_threshold = TRUE,
    ncol = NULL,
    ...
) {
  type <- match.arg(type)

  .trt  <- x$vars$treatment
  .time <- x$vars$time

  df <- x$data

  if (!is.null(which)) {
    df <- df |> dplyr::filter(.data[[.trt]] %in% which)
    if (nrow(df) == 0) {
      stop("None of the specified treatments found in `x`.", call. = FALSE)
    }
  }

  y_label_curve <- if (x$settings$scale == "response") "Disease intensity, S(t)" else "Linear predictor, \u03b7(t)"
  y_label_rate  <- if (x$settings$scale == "response") "Instantaneous rate, S'(t)" else "Rate of linear change, \u03b7'(t)"

  # Plot curve S(t)
  p_curve <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[.time]], y = .data$fitted, color = .data[[.trt]], fill = .data[[.trt]]))

  if (show_ci && !all(is.na(df$fitted_lower))) {
    p_curve <- p_curve +
      ggplot2::geom_ribbon(ggplot2::aes(ymin = .data$fitted_lower, ymax = .data$fitted_upper),
                           alpha = 0.2, color = NA)
  }

  p_curve <- p_curve +
    ggplot2::geom_line(linewidth = 1) +
    ggplot2::labs(x = .time, y = y_label_curve, color = .trt, fill = .trt) +
    ggplot2::theme_classic(base_size = 12) +
    ggplot2::theme(legend.position = "top")

  if (type == "curve") return(p_curve)

  # Plot rate S'(t)
  p_rate <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[.time]], y = .data$rate, color = .data[[.trt]], fill = .data[[.trt]]))

  if (show_threshold) {
    p_rate <- p_rate + ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray50")
    if (x$settings$threshold != 0) {
      p_rate <- p_rate + ggplot2::geom_hline(yintercept = x$settings$threshold,
                                             linetype = "dotted", color = "firebrick")
    }
  }

  if (show_ci && !all(is.na(df$rate_lower))) {
    p_rate <- p_rate +
      ggplot2::geom_ribbon(ggplot2::aes(ymin = .data$rate_lower, ymax = .data$rate_upper),
                           alpha = 0.2, color = NA)
  }

  p_rate <- p_rate +
    ggplot2::geom_line(linewidth = 1) +
    ggplot2::labs(x = .time, y = y_label_rate, color = .trt, fill = .trt) +
    ggplot2::theme_classic(base_size = 12) +
    ggplot2::theme(legend.position = "bottom")

  if (type == "rate") return(p_rate)

  # Combine both
  if (requireNamespace("patchwork", quietly = TRUE)) {
    combined <- p_curve / p_rate + patchwork::plot_layout(guides = "collect") &
      ggplot2::theme(legend.position = "top")
    return(combined)
  } else if (requireNamespace("cowplot", quietly = TRUE)) {
    return(cowplot::plot_grid(p_curve, p_rate, ncol = 1, align = "v"))
  } else {
    message("Package 'patchwork' or 'cowplot' recommended to stack plots. Returning list.")
    return(list(curve = p_curve, rate = p_rate))
  }
}

#' Augment generic for functional objects
#'
#' @param x An object to augment.
#' @param ... Additional arguments.
#' @return An augmented data frame or tibble.
#' @export
augment <- function(x, ...) {
  UseMethod("augment")
}

#' Augment functional_rate
#'
#' @param x An object of class \code{"functional_rate"}.
#' @param ... Additional arguments.
#' @return A tibble of the full rate evaluation grid with predictions, derivatives,
#'   and confidence intervals.
#' @export
augment.functional_rate <- function(x, ...) {
  if (!inherits(x, "functional_rate")) {
    stop("`x` must be of class 'functional_rate'.", call. = FALSE)
  }
  x$data
}

#' Augment functional_rate (alias)
#'
#' @param x An object of class \code{"functional_rate"}.
#' @param ... Additional arguments.
#' @return A tibble of the full rate evaluation grid.
#' @export
augment_functional_rate <- function(x, ...) {
  augment.functional_rate(x, ...)
}

library(testthat)
library(r4pde)

# Helper function to generate simulated epidemic data
make_sim_data <- function() {
  set.seed(42)
  t_seq <- seq(1, 30, length.out = 15)
  df <- data.frame(
    time = rep(t_seq, 3),
    treatment = rep(c("Early", "Late", "None"), each = 15),
    y = c(
      1 / (1 + exp(-(t_seq - 10) * 0.4)),  # Early
      1 / (1 + exp(-(t_seq - 20) * 0.4)),  # Late
      rep(0, 15)                           # None
    )
  )
  df$y <- pmax(0, pmin(1, df$y + rnorm(nrow(df), 0, 0.005)))
  df$y[df$treatment == "None"] <- 0 # strictly zero
  df
}

test_that("1. Invalid inputs trigger informative errors", {
  expect_error(functional_rate("not_an_object"), "`object` must be of class 'functional_curves'")
  
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  # Invalid n_grid
  expect_error(functional_rate(fc, n_grid = 1), "`n_grid` must be a single integer >= 2")
  expect_error(functional_rate(fc, n_grid = -10), "`n_grid` must be a single integer >= 2")
  expect_error(functional_rate(fc, n_grid = "bad"), "`n_grid` must be a single integer >= 2")
  
  # Invalid eps
  expect_error(functional_rate(fc, eps = 0), "`eps` must be a single positive numeric value")
  expect_error(functional_rate(fc, eps = -0.5), "`eps` must be a single positive numeric value")
  
  # Invalid interval
  expect_error(functional_rate(fc, interval = 0), "`interval` must be a single numeric value in \\(0, 1\\)")
  expect_error(functional_rate(fc, interval = 1), "`interval` must be a single numeric value in \\(0, 1\\)")
  expect_error(functional_rate(fc, interval = 1.2), "`interval` must be a single numeric value in \\(0, 1\\)")
})

test_that("2. Linear curve with known derivative on link scale", {
  # Generate strictly linear logit progression: eta = -3 + 0.2 * time
  # On link scale, derivative should be identically 0.2 everywhere
  t_seq <- seq(5, 25, by = 1)
  eta <- -3 + 0.2 * t_seq
  p <- 1 / (1 + exp(-eta))
  
  df <- data.frame(
    time = rep(t_seq, 2),
    treatment = rep(c("A", "B"), each = length(t_seq)),
    y = rep(p, 2)
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial", k_smooth = 5
  )
  
  fr_link <- functional_rate(fc, scale = "link", n_grid = 50)
  
  # Mean estimated rate should be very close to true slope 0.2
  r_A <- fr_link$data$rate[fr_link$data$treatment == "A"]
  expect_equal(mean(r_A), 0.2, tolerance = 0.05)
})

test_that("3. Logistic curve with known analytical derivative", {
  # True logistic curve: S(t) = 1 / (1 + exp(-0.3*(t - 15)))
  # True derivative on response scale: S'(t) = 0.3 * S(t) * (1 - S(t))
  # Maximum rate occurs at t = 15, with value 0.3 * 0.5 * 0.5 = 0.075
  t_seq <- seq(2, 28, by = 1)
  p <- 1 / (1 + exp(-0.3 * (t_seq - 15)))
  
  df <- data.frame(
    time = rep(t_seq, 2),
    treatment = rep(c("Trt1", "Trt2"), each = length(t_seq)),
    y = rep(p, 2)
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial", k_smooth = 8
  )
  
  fr <- functional_rate(fc, scale = "response", n_grid = 100)
  
  # Maximum rate should be near 0.075 and t_r_max near 15
  summ <- summary(fr)
  expect_equal(summ$r_max[summ$treatment == "Trt1"], 0.075, tolerance = 0.015)
  expect_equal(summ$t_r_max[summ$treatment == "Trt1"], 15, tolerance = 1.0)
})

test_that("4. Multiple treatments handled properly", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc)
  summ <- summary(fr)
  
  expect_equal(nrow(summ), 3)
  expect_true(all(c("Early", "Late", "None") %in% summ$treatment))
})

test_that("5. Multiple environments support", {
  set.seed(12)
  t_seq <- seq(1, 20, length.out = 10)
  df <- expand.grid(time = t_seq, treatment = c("A", "B"), env = c("E1", "E2"))
  df$y <- 1 / (1 + exp(-(df$time - 10) * 0.3)) + rnorm(nrow(df), 0, 0.01)
  df$y <- pmax(0, pmin(1, df$y))
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    environment = "env", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc)
  expect_s3_class(fr, "functional_rate")
  expect_true("treatment" %in% names(fr$summary))
})

test_that("6. Scale 'link' vs 'response'", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr_resp <- functional_rate(fc, scale = "response")
  fr_link <- functional_rate(fc, scale = "link")
  
  expect_equal(fr_resp$settings$scale, "response")
  expect_equal(fr_link$settings$scale, "link")
  
  # Response rates are in proportion scale [0, 1] per day
  expect_true(all(fr_resp$data$rate <= 1.0))
  
  # Link scale rates can exceed 1
  expect_true(is.numeric(fr_link$data$rate))
})

test_that("7. Supported families (quasibinomial, betar, gaussian)", {
  sim_dat <- make_sim_data()
  # Quasibinomial
  fc_qb <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  fr_qb <- functional_rate(fc_qb)
  expect_s3_class(fr_qb, "functional_rate")
  
  # Beta family
  fc_beta <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "betar"
  )
  fr_beta <- functional_rate(fc_beta)
  expect_s3_class(fr_beta, "functional_rate")
})

test_that("8. Completely zero curve has r_max = 0, t_r_max = NA, growth_duration = 0", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc)
  summ <- summary(fr)
  
  none_row <- summ |> dplyr::filter(treatment == "None")
  expect_equal(none_row$r_max, 0)
  expect_true(is.na(none_row$t_r_max))
  expect_equal(none_row$growth_duration, 0)
  expect_equal(none_row$cumulative_positive_growth, 0)
  expect_equal(none_row$n_growth_windows, 0L)
  
  # Check that rate_data for None is zeroed
  none_data <- fr$data |> dplyr::filter(treatment == "None")
  expect_true(all(none_data$rate == 0))
  expect_true(all(none_data$fitted == 0))
})

test_that("9. Non-zero constant curve has derivative approximately zero", {
  set.seed(42)
  t_seq <- seq(1, 20, by = 1)
  df <- data.frame(
    time = rep(t_seq, 2),
    treatment = rep(c("Flat1", "Flat2"), each = length(t_seq)),
    y = rep(0.3, length(t_seq) * 2) + rnorm(length(t_seq) * 2, 0, 1e-4)
  )
  df$y <- pmax(0, pmin(1, df$y))
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc, scale = "response")
  expect_true(all(abs(fr$data$rate) < 0.01))
})

test_that("10. Negative rates preserved in rate_data without truncation", {
  # Decreasing curve
  t_seq <- seq(1, 20, by = 1)
  df <- data.frame(
    time = rep(t_seq, 2),
    treatment = rep(c("Dec1", "Dec2"), each = length(t_seq)),
    y = rep(1 - 1 / (1 + exp(-(t_seq - 10) * 0.4)), 2)
  )
  
  suppressWarnings({
    fc <- functional_curves(
      data = df, time = "time", response = "y", treatment = "treatment",
      min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
    )
    fr <- functional_rate(fc, scale = "response")
  })
  
  # Negative rates must be present in rate_data
  expect_true(any(fr$data$rate < 0))
})

test_that("11. Different temporal domains between groups without extrapolation", {
  df <- data.frame(
    time = c(1:15, 10:25),
    treatment = c(rep("Short", 15), rep("Shifted", 16)),
    y = c(
      1 / (1 + exp(-(1:15 - 8) * 0.4)),
      1 / (1 + exp(-(10:25 - 18) * 0.4))
    )
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc, n_grid = 30)
  
  short_data <- fr$data |> dplyr::filter(treatment == "Short")
  shifted_data <- fr$data |> dplyr::filter(treatment == "Shifted")
  
  expect_equal(min(short_data$time), 1)
  expect_equal(max(short_data$time), 15)
  expect_equal(min(shifted_data$time), 10)
  expect_equal(max(shifted_data$time), 25)
})

test_that("12. Custom newdata support and extrapolation guard", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  # Valid newdata
  custom_grid <- data.frame(
    time = rep(c(5, 10, 15, 20), 2),
    treatment = rep(c("Early", "Late"), each = 4)
  )
  
  fr_custom <- functional_rate(fc, newdata = custom_grid)
  expect_equal(nrow(fr_custom$data), 8)
  
  # Extrapolated newdata triggers error
  extrap_grid <- data.frame(
    time = c(0, 50),
    treatment = c("Early", "Late")
  )
  expect_error(functional_rate(fc, newdata = extrap_grid), "outside the observed domain")
})

test_that("13. Confidence intervals behavior", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr95 <- functional_rate(fc, interval = 0.95)
  fr90 <- functional_rate(fc, interval = 0.90)
  
  # rate_lower < rate < rate_upper for curves with positive uncertainty
  early_data <- fr95$data |> dplyr::filter(treatment == "Early")
  expect_true(all(early_data$rate_lower <= early_data$rate_upper))
  
  # 95% CI is wider than 90% CI
  ci95_width <- fr95$data$rate_upper - fr95$data$rate_lower
  ci90_width <- fr90$data$rate_upper - fr90$data$rate_lower
  expect_true(all(ci95_width >= ci90_width - 1e-8))
})

test_that("14. Integration of positive rate matches cumulative positive growth", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc, n_grid = 100, threshold = 0.01)
  summ <- summary(fr)
  
  early_data <- fr$data |> dplyr::filter(treatment == "Early")
  expected_growth <- trapz_vec_internal(early_data$time, pmax(early_data$rate - 0.01, 0))
  
  expect_equal(summ$cumulative_positive_growth[summ$treatment == "Early"], expected_growth, tolerance = 1e-4)
})

test_that("15. Correct identification of r_max and t_r_max in Early vs Late", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc, n_grid = 100)
  summ <- summary(fr)
  
  t_early <- summ$t_r_max[summ$treatment == "Early"]
  t_late  <- summ$t_r_max[summ$treatment == "Late"]
  
  expect_true(t_early < t_late)
  expect_equal(t_early, 10, tolerance = 2)
  expect_equal(t_late, 20, tolerance = 2)
})

test_that("16. Absence of extrapolation at boundaries", {
  sim_dat <- make_sim_data()
  t_min_orig <- min(sim_dat$time)
  t_max_orig <- max(sim_dat$time)
  
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc, n_grid = 50)
  
  expect_gte(min(fr$data$time), t_min_orig)
  expect_lte(max(fr$data$time), t_max_orig)
})

test_that("17. S3 methods print(), summary(), plot(), augment() work", {
  sim_dat <- make_sim_data()
  fc <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc)
  
  # print()
  expect_output(print(fr), "Instantaneous Epidemic Rates")
  
  # summary()
  s <- summary(fr)
  expect_s3_class(s, "tbl_df")
  expect_equal(nrow(s), 3)
  
  # plot()
  p_both <- plot(fr, type = "both")
  expect_true(inherits(p_both, "ggplot") || inherits(p_both, "patchwork") || is.list(p_both))
  
  p_curve <- plot(fr, type = "curve")
  expect_s3_class(p_curve, "ggplot")
  
  p_rate <- plot(fr, type = "rate")
  expect_s3_class(p_rate, "ggplot")
  
  # augment()
  aug <- augment(fr)
  expect_s3_class(aug, "tbl_df")
  expect_true("rate" %in% names(aug))
  
  aug2 <- augment_functional_rate(fr)
  expect_equal(aug, aug2)
})

test_that("18. Preservation of group names and factor levels", {
  sim_dat <- make_sim_data()
  names(sim_dat)[names(sim_dat) == "treatment"] <- "genotype_var"
  names(sim_dat)[names(sim_dat) == "time"] <- "day_after_emergence"
  
  fc <- functional_curves(
    data = sim_dat, time = "day_after_emergence", response = "y", treatment = "genotype_var",
    min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fr <- functional_rate(fc)
  
  expect_true("genotype_var" %in% names(fr$data))
  expect_true("day_after_emergence" %in% names(fr$data))
  expect_true("genotype_var" %in% names(fr$summary))
})

test_that("19. Compatibility with global_smooth = TRUE and FALSE", {
  sim_dat <- make_sim_data()
  
  fc_ind <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    global_smooth = FALSE, min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  fr_ind <- functional_rate(fc_ind)
  expect_s3_class(fr_ind, "functional_rate")
  
  fc_glob <- functional_curves(
    data = sim_dat, time = "time", response = "y", treatment = "treatment",
    global_smooth = TRUE, min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  fr_glob <- functional_rate(fc_glob)
  expect_s3_class(fr_glob, "functional_rate")
})

test_that("20. No alteration in existing functional functions", {
  set.seed(123)
  df <- data.frame(
    time = rep(1:5, 4),
    treatment = rep(c("A", "B"), each = 10),
    block = rep(rep(1:2, each = 5), 2),
    y = pmax(0, pmin(1, rep(c(0.1, 0.3, 0.5, 0.7, 0.9), 4) + rnorm(20, 0, 0.05)))
  )
  
  fc <- functional_curves(
    data = df, time = "time", response = "y", treatment = "treatment",
    block = "block", min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
  )
  
  fd <- functional_distances(fc, cluster_k = 2, show_progress = FALSE)
  expect_s3_class(fd, "functional_distances")
  expect_equal(fd$distance_matrix, t(fd$distance_matrix))
})

#' Functional Contrast (Disease Suppression Profiles)
#'
#' @description
#' Calculates functional contrast curves (Disease Suppression Profiles - DSP) between a 
#' reference treatment (e.g., untreated control) and other fungicide treatments over time. 
#' The DSP is defined by default as: \deqn{DSP(t) = y_{control}(t) - y_{treatment}(t)}
#' where \eqn{y(t)} is the disease intensity at time \eqn{t}. The goal is to evaluate
#' the intensity, timing, and persistence of the fungicide protection.
#'
#' @param data A data frame containing disease progress observations, or an object of class \code{"functional_curves"}.
#' @param time Character string naming the time variable.
#' @param response Character string naming the response variable.
#' @param treatment Character string naming the treatment variable.
#' @param reference Character string specifying the reference treatment level. Default is \code{"Control"}.
#' @param group Optional character vector of grouping variables (e.g., \code{c("experiment", "location")}).
#' @param smooth Logical; if \code{TRUE}, the function could apply smoothing (currently placeholder).
#' @param grid Optional numeric vector of common time points for interpolation. If provided, curves are interpolated.
#' @param contrast Character specifying the contrast type. Currently \code{"difference"} is implemented.
#' @param keep_reference Logical; if \code{TRUE}, includes the reference treatment in the output (with DSP = 0). Default is \code{FALSE}.
#' @param ... Additional arguments.
#'
#' @return An object of class \code{"functional_dsp"} (inheriting from \code{"tbl_df"}), containing:
#' \itemize{
#'   \item grouping variables (if any)
#'   \item treatment variable
#'   \item time variable
#'   \item \code{y_reference}: disease intensity of the reference
#'   \item \code{y_treatment}: disease intensity of the treatment
#'   \item \code{DSP}: the computed contrast
#'   \item \code{contrast_type}: type of contrast used
#'   \item \code{reference}: name of the reference treatment
#' }
#'
#' @seealso \code{\link{functional_summary}}, \code{\link{plot_dsp}}
#'
#' @export
functional_contrast <- function(
    data,
    time = "time",
    response = "response",
    treatment = "treatment",
    reference = "Control",
    group = NULL,
    smooth = FALSE,
    grid = NULL,
    contrast = c("difference", "relative"),
    keep_reference = FALSE,
    ...
) {
  contrast <- match.arg(contrast)
  
  if (inherits(data, "functional_curves")) {
    time_col <- data$vars$time
    trt_col <- data$vars$treatment
    resp_col <- "mu"
    df <- data$curves
    group_cols <- NULL
  } else {
    time_col <- time
    trt_col <- treatment
    resp_col <- response
    df <- tibble::as_tibble(data)
    group_cols <- group
  }
  
  if (!time_col %in% names(df)) stop(sprintf("Time column '%s' not found.", time_col))
  if (!trt_col %in% names(df)) stop(sprintf("Treatment column '%s' not found.", trt_col))
  if (!resp_col %in% names(df)) stop(sprintf("Response column '%s' not found.", resp_col))
  if (!is.null(group_cols)) {
    missing_groups <- setdiff(group_cols, names(df))
    if (length(missing_groups) > 0) {
      stop("Grouping columns not found: ", paste(missing_groups, collapse = ", "))
    }
  }
  
  # Ensure reference exists
  if (!reference %in% df[[trt_col]]) {
    stop(sprintf("Reference level '%s' not found in treatment column.", reference))
  }
  
  # Interpolation if grid is provided
  if (!is.null(grid)) {
    grid <- sort(unique(grid))
    df <- df |>
      dplyr::group_by(dplyr::across(dplyr::all_of(c(group_cols, trt_col)))) |>
      dplyr::reframe(
        !!resp_col := suppressWarnings(stats::approx(x = .data[[time_col]], y = .data[[resp_col]], xout = grid, rule = 2)$y),
        !!time_col := grid
      )
  } else {
    # If no grid, ensure same time points by interpolating treatments to match reference time points if needed
    # But usually data is already on a grid if from functional_curves.
    # For raw data, we will just use approx against the union of time points per group.
    
    df <- df |>
      dplyr::group_by(dplyr::across(dplyr::all_of(group_cols))) |>
      dplyr::group_modify(~ {
        common_times <- sort(unique(.x[[time_col]]))
        
        .x |>
          dplyr::group_by(.data[[trt_col]]) |>
          dplyr::reframe(
            !!resp_col := suppressWarnings(stats::approx(x = .data[[time_col]], y = .data[[resp_col]], xout = common_times, rule = 2)$y),
            !!time_col := common_times
          )
      }) |>
      dplyr::ungroup()
  }
  
  # Split reference
  ref_df <- df |>
    dplyr::filter(.data[[trt_col]] == reference) |>
    dplyr::select(dplyr::all_of(c(group_cols, time_col)), y_reference = dplyr::all_of(resp_col))
  
  # Check if reference exists for all groups
  if (!is.null(group_cols)) {
    ref_counts <- ref_df |> dplyr::group_by(dplyr::across(dplyr::all_of(group_cols))) |> dplyr::tally()
    all_groups <- df |> dplyr::distinct(dplyr::across(dplyr::all_of(group_cols)))
    if (nrow(ref_counts) < nrow(all_groups)) {
      stop("Reference treatment is missing in some groups.")
    }
  }
  
  # Calculate contrast
  res <- df |>
    dplyr::left_join(ref_df, by = c(group_cols, time_col)) |>
    dplyr::rename(y_treatment = dplyr::all_of(resp_col)) |>
    dplyr::mutate(
      DSP = if (contrast == "difference") {
        .data$y_reference - .data$y_treatment
      } else {
        (.data$y_reference - .data$y_treatment) / dplyr::na_if(.data$y_reference, 0)
      },
      contrast_type = contrast,
      reference = reference
    ) |>
    dplyr::arrange(dplyr::across(dplyr::all_of(c(group_cols, trt_col, time_col))))
  
  if (!keep_reference) {
    res <- res |> dplyr::filter(.data[[trt_col]] != reference)
  }
  
  # Create attributes for downstream methods
  attr(res, "dsp_vars") <- list(
    time = time_col,
    treatment = trt_col,
    response = "DSP",
    group = group_cols
  )
  class(res) <- c("functional_dsp", class(res))
  
  res
}

#' @keywords internal
trapz_vec_internal <- function(x, y) {
  ok <- !(is.na(x) | is.na(y))
  x <- x[ok]
  y <- y[ok]
  if (length(x) < 2) return(NA_real_)
  ord <- order(x)
  x <- x[ord]
  y <- y[ord]
  sum(diff(x) * (head(y, -1) + tail(y, -1)) / 2)
}

#' Summarize Disease Suppression Profiles
#'
#' @description
#' Extracts functional descriptors from Disease Suppression Profiles (DSPs).
#'
#' @param object An object of class \code{"functional_dsp"} or a data frame with a DSP column.
#' @param time Character string naming the time column.
#' @param response Character string naming the DSP column.
#' @param treatment Character string naming the treatment column.
#' @param group Optional character vector of grouping variables.
#' @param threshold Numeric; minimum suppression threshold for calculating persistence. Default is 0.
#' @param positive_only Logical; if \code{TRUE}, negative DSP values are set to 0 before area/centroid calculations. Default is \code{TRUE}.
#'
#' @return A \code{tibble} with one row per group/treatment containing:
#' \itemize{
#'   \item \code{protected_area}: Trapezoidal integral of DSP(t).
#'   \item \code{mean_suppression}: Temporal mean of DSP(t).
#'   \item \code{max_suppression}: Maximum value of DSP(t).
#'   \item \code{time_max_suppression}: Time at which maximum suppression occurs.
#'   \item \code{persistence}: Total duration where DSP(t) > threshold.
#'   \item \code{centroid}: Temporal centroid defined as \eqn{\int t \cdot DSP(t) dt / \int DSP(t) dt}.
#'   \item \code{energy}: Integral of \eqn{DSP(t)^2}.
#'   \item \code{early_area, mid_area, late_area}: Protected area in the first, second, and third tertiles of the time period.
#'   \item \code{late_decline_slope}: Slope of DSP(t) in the late period.
#'   \item \code{n_time_points}: Number of time points evaluated.
#' }
#' @examples
#' \dontrun{
#' # Using a generic data frame with custom column names
#' dat <- data.frame(
#'   DAA = c(0, 10, 20),
#'   fungicide = c("A", "A", "A"),
#'   contrast = c(0, 10, 0)
#' )
#' functional_summary(
#'   dat, 
#'   time = "DAA", 
#'   response = "contrast", 
#'   treatment = "fungicide"
#' )
#' }
#'
#' @export
functional_summary <- function(
    object,
    time = "time",
    response = "DSP",
    treatment = "treatment",
    group = NULL,
    threshold = 0,
    positive_only = TRUE
) {
  df <- tibble::as_tibble(object)
  
  if (inherits(object, "functional_dsp")) {
    vars <- attr(object, "dsp_vars")
    if (!is.null(vars)) {
      if (missing(time) && !is.null(vars$time)) time <- vars$time
      if (missing(response) && !is.null(vars$response)) response <- vars$response
      if (missing(treatment) && !is.null(vars$treatment)) treatment <- vars$treatment
      if (missing(group) && !is.null(vars$group)) group <- vars$group
    }
  }
  
  if (!all(c(time, response, treatment) %in% names(df))) {
    stop("Missing required columns in object.")
  }
  
  res <- df |>
    dplyr::group_by(dplyr::across(dplyr::all_of(c(group, treatment)))) |>
    dplyr::summarise(
      n_time_points = dplyr::n(),
      protected_area = {
        t <- .data[[time]]
        y <- .data[[response]]
        y_val <- if (positive_only) pmax(y, 0) else y
        trapz_vec_internal(t, y_val)
      },
      mean_suppression = {
        y <- .data[[response]]
        mean(y, na.rm = TRUE)
      },
      max_suppression = {
        y <- .data[[response]]
        max(y, na.rm = TRUE)
      },
      time_max_suppression = {
        t <- .data[[time]]
        y <- .data[[response]]
        idx <- which.max(y)
        if (length(idx) > 0) t[idx[1]] else NA_real_
      },
      persistence = {
        t <- .data[[time]]
        y <- .data[[response]]
        above <- y > threshold
        if (sum(above) < 2) {
          0
        } else {
          t_above <- t[above]
          max(t_above) - min(t_above)
        }
      },
      centroid = {
        t <- .data[[time]]
        y <- .data[[response]]
        y_val <- if (positive_only) pmax(y, 0) else y
        num <- trapz_vec_internal(t, t * y_val)
        den <- trapz_vec_internal(t, y_val)
        if (is.na(den) || den == 0) NA_real_ else num / den
      },
      energy = {
        t <- .data[[time]]
        y <- .data[[response]]
        trapz_vec_internal(t, y^2)
      },
      early_area = {
        t <- .data[[time]]
        y <- .data[[response]]
        y_val <- if (positive_only) pmax(y, 0) else y
        cuts <- stats::quantile(t, probs = c(1/3, 2/3), na.rm = TRUE)
        idx <- t <= cuts[1]
        trapz_vec_internal(t[idx], y_val[idx])
      },
      mid_area = {
        t <- .data[[time]]
        y <- .data[[response]]
        y_val <- if (positive_only) pmax(y, 0) else y
        cuts <- stats::quantile(t, probs = c(1/3, 2/3), na.rm = TRUE)
        idx <- t > cuts[1] & t <= cuts[2]
        trapz_vec_internal(t[idx], y_val[idx])
      },
      late_area = {
        t <- .data[[time]]
        y <- .data[[response]]
        y_val <- if (positive_only) pmax(y, 0) else y
        cuts <- stats::quantile(t, probs = c(1/3, 2/3), na.rm = TRUE)
        idx <- t > cuts[2]
        trapz_vec_internal(t[idx], y_val[idx])
      },
      late_decline_slope = {
        t <- .data[[time]]
        y <- .data[[response]]
        cuts <- stats::quantile(t, probs = c(1/3, 2/3), na.rm = TRUE)
        idx <- t > cuts[2]
        if (sum(idx) >= 2) {
          stats::coef(stats::lm(y[idx] ~ t[idx]))[2]
        } else {
          NA_real_
        }
      },
      .groups = "drop"
    )
    
  if (inherits(object, "functional_dsp")) {
    class(res) <- unique(c("functional_dsp", class(res)))
    attr(res, "dsp_vars") <- attr(object, "dsp_vars")
  }
  
  res
}

#' Plot Disease Suppression Profiles
#'
#' @description
#' Plots functional contrast curves (DSP) over time.
#'
#' @param object An object of class \code{"functional_dsp"}.
#' @param time Character string for the time column (auto-detected if object is \code{"functional_dsp"}).
#' @param response Character string for the DSP column.
#' @param treatment Character string for the treatment column.
#' @param group Optional character vector of grouping variables.
#' @param facet Optional character string specifying a variable for faceting.
#' @param show_zero Logical; whether to draw a horizontal reference line at y = 0. Default is \code{TRUE}.
#' @param linewidth Numeric; width of the lines.
#' @param alpha Numeric; transparency of the lines.
#' @param ylab Character string; label for the y-axis.
#' @param xlab Character string; label for the x-axis.
#'
#' @return A \code{ggplot} object.
#' @export
plot_dsp <- function(
    object,
    time = NULL,
    response = NULL,
    treatment = NULL,
    group = NULL,
    facet = NULL,
    show_zero = TRUE,
    linewidth = 1,
    alpha = 1,
    ylab = "Disease suppression profile",
    xlab = "Time"
) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) stop("ggplot2 is required.")
  
  if (inherits(object, "functional_dsp")) {
    vars <- attr(object, "dsp_vars")
    if (is.null(time)) time <- vars$time
    if (is.null(response)) response <- vars$response
    if (is.null(treatment)) treatment <- vars$treatment
    if (is.null(group) && length(vars$group) > 0) group <- vars$group
  } else {
    if (is.null(time)) time <- "time"
    if (is.null(response)) response <- "DSP"
    if (is.null(treatment)) treatment <- "treatment"
  }
  
  df <- tibble::as_tibble(object)
  
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[time]], y = .data[[response]], color = .data[[treatment]], group = .data[[treatment]]))
  
  if (show_zero) {
    p <- p + ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black")
  }
  
  p <- p + 
    ggplot2::geom_line(linewidth = linewidth, alpha = alpha) +
    ggplot2::labs(x = xlab, y = ylab, color = "Treatment") +
    ggplot2::theme_classic()
  
  if (!is.null(facet)) {
    p <- p + ggplot2::facet_wrap(stats::as.formula(paste("~", facet)))
  } else if (!is.null(group) && length(group) == 1) {
    p <- p + ggplot2::facet_wrap(stats::as.formula(paste("~", group[1])))
  }
  
  p
}

#' Rank Treatments based on Functional DSP Descriptors
#'
#' @description
#' Generates rankings of treatments based on extracted DSP functional descriptors.
#'
#' @param summary_data The tibble output from \code{\link{functional_summary}}.
#' @param metrics Character vector of metrics to rank.
#' @param higher_is_better Logical vector or named logical vector indicating if higher values are better for each metric. If a single logical, applies to all.
#' @param group Optional character vector of grouping variables.
#' @param ties.method Character string specifying how to handle ties. Default is \code{"average"}.
#'
#' @return A tibble with ranks for each specified metric, and an \code{average_rank}.
#' @export
rank_dsp <- function(
    summary_data,
    metrics = c("protected_area", "max_suppression", "persistence", "centroid"),
    higher_is_better = TRUE,
    group = NULL,
    ties.method = "average"
) {
  df <- tibble::as_tibble(summary_data)
  
  if (length(higher_is_better) == 1) {
    hib <- rep(higher_is_better, length(metrics))
    names(hib) <- metrics
  } else {
    if (is.null(names(higher_is_better))) {
      if (length(higher_is_better) != length(metrics)) stop("higher_is_better must have the same length as metrics if not named.")
      names(higher_is_better) <- metrics
      hib <- higher_is_better
    } else {
      hib <- higher_is_better
    }
  }
  
  if (is.null(group)) {
    grouped_df <- df |> dplyr::group_by()
  } else {
    grouped_df <- df |> dplyr::group_by(dplyr::across(dplyr::all_of(group)))
  }
  
  for (m in metrics) {
    if (!m %in% names(df)) stop(sprintf("Metric '%s' not found in summary_data.", m))
    
    grouped_df <- grouped_df |>
      dplyr::mutate(
        !!paste0("rank_", m) := {
          vals <- .data[[m]]
          if (hib[m]) {
            rank(-vals, ties.method = ties.method, na.last = "keep")
          } else {
            rank(vals, ties.method = ties.method, na.last = "keep")
          }
        }
      )
  }
  
  res <- grouped_df |> dplyr::ungroup()
  
  rank_cols <- paste0("rank_", metrics)
  res <- res |>
    dplyr::mutate(average_rank = rowMeans(dplyr::select(res, dplyr::all_of(rank_cols)), na.rm = TRUE))
  
  res
}

#' Plot DSP Rankings Heatmap
#'
#' @description
#' Creates a heatmap visualization of treatment rankings based on DSP descriptors.
#'
#' @param ranks_data Tibble output from \code{\link{rank_dsp}}.
#' @param treatment Character string naming the treatment column (for the y-axis).
#' @param group Optional character string for faceting by group.
#'
#' @return A \code{ggplot} object.
#' @export
plot_dsp_rank_heatmap <- function(
    ranks_data,
    treatment = "treatment",
    group = NULL
) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) stop("ggplot2 is required.")
  
  rank_cols <- grep("^rank_", names(ranks_data), value = TRUE)
  if (length(rank_cols) == 0) stop("No rank columns found. Did you run rank_dsp()?")
  
  long_df <- ranks_data |>
    dplyr::select(dplyr::all_of(c(treatment, group, rank_cols, "average_rank"))) |>
    tidyr::pivot_longer(
      cols = c(dplyr::all_of(rank_cols), "average_rank"),
      names_to = "metric",
      values_to = "rank"
    ) |>
    dplyr::mutate(
      metric = gsub("^rank_", "", .data$metric),
      metric = factor(.data$metric, levels = c(gsub("^rank_", "", rank_cols), "average_rank"))
    )
  
  p <- ggplot2::ggplot(long_df, ggplot2::aes(x = .data$metric, y = .data[[treatment]], fill = .data$rank)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::geom_text(ggplot2::aes(label = round(.data$rank, 1)), color = "black") +
    ggplot2::scale_fill_gradient2(low = "forestgreen", mid = "white", high = "red", midpoint = mean(long_df$rank, na.rm = TRUE)) +
    ggplot2::labs(x = "Metric", y = "Treatment", fill = "Rank") +
    ggplot2::theme_minimal() +
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))
  
  if (!is.null(group) && length(group) == 1) {
    p <- p + ggplot2::facet_wrap(stats::as.formula(paste("~", group[1])))
  }
  
  p
}

#' #' Simulate Disease Suppression Profile Data
#'
#' @description
#' Simulates a dataset for demonstrating functional analysis with Disease Suppression Profiles.
#' Returns a data frame with varying treatments and replicates over multiple time points.
#'
#' @return A \code{tibble} with columns \code{treatment}, \code{rep}, \code{time}, \code{y_control}, \code{dsp_true}, \code{severity_true}, and \code{severity}.
#' @export
simulate_dsp_data <- function() {
  set.seed(123)
  
  time_seq <- seq(0, 60, by = 5)
  
  control_curve <- function(t) {
    78 / (1 + exp(-0.13 * (t - 35)))
  }
  
  dsp_fun <- function(t, profile) {
    dplyr::case_when(
      profile == "Early suppression" ~ 
        156.7 * exp(-((t - 30)^2) / (2 * 12^2)),
      
      profile == "Late suppression" ~ 
        78.9 * exp(-((t - 58)^2) / (2 * 15^2)),
      
      profile == "Persistent suppression" ~ 
        43.9 * stats::plogis((t - 20) / 3) * (1 - stats::plogis((t - 65) / 5)),
      
      profile == "Control" ~ 0
    )
  }
  
  treatments <- tibble::tibble(
    treatment = c(
      "Control",
      "Early suppression",
      "Persistent suppression",
      "Late suppression"
    ),
    profile = c(
      "Control",
      "Early suppression",
      "Persistent suppression",
      "Late suppression"
    )
  )
  
  dat_concept <- tidyr::expand_grid(
    treatment = treatments$treatment,
    rep = 1:4,
    time = time_seq
  ) |>
    dplyr::left_join(treatments, by = "treatment") |>
    dplyr::mutate(
      y_control = control_curve(.data$time),
      dsp_true = dsp_fun(.data$time, .data$profile),
      severity_true = pmax(0, .data$y_control - .data$dsp_true)
    ) |>
    dplyr::group_by(.data$treatment, .data$rep) |>
    dplyr::mutate(
      severity_true = cummax(.data$severity_true),
      severity = pmin(100, pmax(0, .data$severity_true + stats::rnorm(dplyr::n(), 0, 2)))
    ) |>
    dplyr::ungroup() |>
    dplyr::select(-.data$profile)
    
  return(dat_concept)
}

library(testthat)

test_that("functional_contrast errors correctly", {
  dat <- simulate_dsp_data()
  
  # Error if control doesn't exist
  expect_error(
    functional_contrast(dat, time = "time", response = "severity", treatment = "treatment", reference = "NonExistent"),
    "Reference level 'NonExistent' not found"
  )
  
  # Error if control missing in a group
  dat_missing_control <- dat |> dplyr::filter(!(treatment == "Control" & rep == 1))
  expect_error(
    functional_contrast(dat_missing_control, time = "time", response = "severity", treatment = "treatment", reference = "Control", group = "rep"),
    "Reference treatment is missing in some groups"
  )
})

test_that("functional_contrast calculates DSP correctly", {
  # Simple manual data
  dat <- data.frame(
    time = c(1, 2, 3, 1, 2, 3),
    trt = c("Control", "Control", "Control", "A", "A", "A"),
    y = c(10, 20, 30, 5, 5, 10)
  )
  
  res <- functional_contrast(dat, time = "time", response = "y", treatment = "trt", reference = "Control")
  
  expect_s3_class(res, "functional_dsp")
  expect_true("DSP" %in% names(res))
  
  # For treatment A: (10-5), (20-5), (30-10) -> 5, 15, 20
  expect_equal(res$DSP, c(5, 15, 20))
  
  # Check class is preserved and it's a tibble
  expect_true(tibble::is_tibble(res))
})

test_that("functional_summary calculations are correct", {
  dat <- data.frame(
    time = c(0, 10, 20),
    trt = rep("A", 3),
    DSP = c(0, 10, 0) # Triangle with base 20, height 10. Area = 100
  )
  
  # Make it look like a functional_dsp object
  attr(dat, "dsp_vars") <- list(time = "time", treatment = "trt", response = "DSP", group = NULL)
  class(dat) <- c("functional_dsp", class(dat))
  
  summ <- functional_summary(dat)
  
  expect_true(tibble::is_tibble(summ))
  expect_equal(summ$protected_area, 100) # (10*10)/2 + (10*10)/2
  expect_equal(summ$max_suppression, 10)
  expect_equal(summ$time_max_suppression, 10)
  
  # Persistence with threshold 0 should be 20 (from 0 to 20, since > 0 is not true at boundaries? 
  # Wait, DSP > 0 is only true at time 10. 
  # So persistence > 0 might be 0 since only 1 point is > 0.
  # Let's test with a different threshold.
  
  dat2 <- data.frame(
    time = c(0, 5, 10, 15, 20),
    trt = rep("B", 5),
    DSP = c(0, 10, 10, 10, 0) 
  )
  attr(dat2, "dsp_vars") <- list(time = "time", treatment = "trt", response = "DSP", group = NULL)
  class(dat2) <- c("functional_dsp", class(dat2))
  
  summ2 <- functional_summary(dat2, threshold = 5)
  # >5 at 5, 10, 15. Persistence = 15 - 5 = 10.
  expect_equal(summ2$persistence, 10)
})

test_that("integration with plot and ranks works", {
  dat <- simulate_dsp_data()
  dsp <- functional_contrast(dat, time = "time", response = "severity", treatment = "treatment", reference = "Control", group = "rep")
  
  # Just group by treatment for summary to average out reps, or summarize per group
  summ <- functional_summary(dsp, group = "rep")
  
  ranks <- rank_dsp(summ)
  
  expect_true("rank_protected_area" %in% names(ranks))
  expect_true("average_rank" %in% names(ranks))
  
  # plot
  p1 <- plot_dsp(dsp)
  p2 <- plot_dsp_rank_heatmap(ranks, group = "rep")
  
  expect_s3_class(p1, "ggplot")
  expect_s3_class(p2, "ggplot")
})

test_that("functional_summary works with custom column names", {
  dat <- data.frame(
    days = c(0, 10, 20),
    fungicide = rep("A", 3),
    contrast = c(0, 10, 0)
  )
  
  summ <- functional_summary(dat, time = "days", response = "contrast", treatment = "fungicide")
  
  expect_true(tibble::is_tibble(summ))
  expect_equal(summ$protected_area, 100)
  expect_equal(summ$max_suppression, 10)
  expect_equal(summ$time_max_suppression, 10)
})

# Load necessary libraries
#' Custom ggplot2 theme based on cowplot::theme_half_open
#'
#' This function creates a new ggplot2 theme by modifying the cowplot::theme_half_open theme.
#' It sets a custom font size and changes the panel background color to gray96.
#'
#' @param font_size The base font size. Default is 16.
#'
#' @return A ggplot2 theme object.
#' @export
#' @family Miscellaneous

theme_r4pde <- function(font_size = 16) {
  base_theme <- cowplot::theme_half_open(font_size = font_size)
  modified_theme <- base_theme +
    theme(panel.background = element_rect(fill = "gray96"),
          strip.background = element_rect(colour = "white", fill = "white"))
  return(modified_theme)
}

#' Count the Number of Ones in Subareas of a Matrix
#'
#' This function takes a binary matrix (0s and 1s) and divides it into rectangular subareas,
#' counting the number of ones in each. Subareas are defined by the number of rows and columns
#' specified by the user. If the matrix dimensions are not perfectly divisible by the subarea size,
#' edge subareas may be smaller.
#'
#' @param matrix_data A matrix of 0s and 1s to analyze.
#' @param sub_rows Number of rows in each subarea.
#' @param sub_cols Number of columns in each subarea.
#'
#' @return A matrix where each cell corresponds to a subarea and contains the count of ones.
#'
#' @examples
#' set.seed(123)
#' mat <- matrix(sample(c(0, 1), 12 * 16, replace = TRUE), nrow = 16, ncol = 12)
#' count_matrix <- count_subareas(mat, sub_rows = 3, sub_cols = 3)
#' print(count_matrix)
#'
#' @export
#' @family Spatial analysis


# Function to count 1s in subareas
count_subareas <- function(matrix_data, sub_rows, sub_cols) {
  # Get matrix dimensions
  total_rows <- nrow(matrix_data)
  total_cols <- ncol(matrix_data)

  # Calculate number of subareas in each dimension
  n_sub_rows <- ceiling(total_rows / sub_rows)
  n_sub_cols <- ceiling(total_cols / sub_cols)

  # Initialize result matrix
  result <- matrix(NA, nrow = n_sub_rows, ncol = n_sub_cols)

  # Iterate through each subarea
  for (i in 1:n_sub_rows) {
    for (j in 1:n_sub_cols) {
      # Calculate row and column indices for current subarea
      row_start <- (i-1)*sub_rows + 1
      row_end <- min(i*sub_rows, total_rows)
      col_start <- (j-1)*sub_cols + 1
      col_end <- min(j*sub_cols, total_cols)

      # Extract submatrix
      submatrix <- matrix_data[row_start:row_end, col_start:col_end]

      # Count 1s in submatrix
      result[i,j] <- sum(submatrix == 1, na.rm = TRUE)
    }
  }

  return(result)
}

#' Runs Test
#'
#' Perform a runs test on the input data to test for clustering or randomness.
#'
#' @param x A numeric vector representing the input data
#' @examples
#' oruns_test(c(1, 0, 1, 1, 0, 1, 0, 0, 1, 1))
#'
#' @importFrom stats pnorm
#' @return an `r4pde.oruns` object.
#'
#' An `r4pde.oruns` object is a `list` containing:
#'   * U, number of runs,
#'   * EU, expected number of runs,
#'   * sU, standard deviation of the expected number of runs
#'   * Z, Z-score of the observed number of runs,
#'   * pvalue, the p-value of the Z-score, and
#'   * result, the test result of either "aggregation or clustering" or "randomness"
#'
#' @export
#' @family Spatial analysis
oruns_test <- function(x) {
  # S is the input data
  S <- x

  # U is the number of runs in the data
  U = max(cumsum(c(1, diff(S) != 0)))

  # m is the sum of the input data
  m = sum(S)

  # N is the length of the input data
  N = length(S)

  # EU is the expected number of runs
  EU = 1 + (2 * m * (N - m) / N)

  # sU is the standard deviation of the expected number of runs
  sU = sqrt(2 * m * (N - m) * (2 * m * (N - m) - N) / (N ^ 2 * (N - 1)))

  # Z is the Z-score of the observed number of runs
  Z = (U - EU) / sU

  # pvalue is the p-value of the Z-score
  pvalue <- (1 - pnorm(abs(Z), lower.tail = TRUE))

  result <- ifelse(Z < -1.64,
                   c("aggregation or clustering"),
                   c("randomness"))

  out <- list(U = U, EU = EU, sU = sU, Z = Z, pvalue = pvalue, result = result)
  class(out) <- union("r4pde.oruns_test", class(out))
  return(out)
}

#' Prints r4pde.oruns Object
#'
#' Custom [print()] method for `r4pde.oruns_test` objects.
#'
#' @param x a `r4pde.oruns_test` object
#' @param ... ignored
#' @export
#' @noRd

print.r4pde.oruns_test <- function(x, ...) {
  message(
    "Ordinary Runs Test of Data Sequence:\n",
    "-------------------------------------\n",
    sprintf("Total Number of Runs (U): %d\n", x$U),
    sprintf("Expected Number of Runs (EU): %.2f\n", x$EU),
    sprintf("Standard Deviation of Runs (sU): %.2f\n", x$sU),
    sprintf("Z-score: %.2f\n", x$Z),
    sprintf("P-value: %.4f\n\n", x$pvalue),
    "Interpretation:\n",
    sprintf("Based on the Z-score, the sequence exhibits '%s'.\n", x$result)
  )
  invisible(x)
}

#' Test for Spatial Join Count Statistics
#'
#' @description
#' The function `join_count` calculates spatial join count statistics for a binary matrix,
#' identifying patterns of aggregation or randomness.
#'
#' @param matrix_data A binary matrix (with elements 0 and 1) representing the spatial distribution
#'   of two types of points: 0 for healthy plants (H) and 1 for diseased plants (D). This matrix reflects
#'   the geographical distribution or layout of plants in the studied area.
#' @param verbose Logical. If TRUE (default), prints a formatted message to the console.
#' @return A comprehensive, rich-text formatted string of results that includes:
#'   \itemize{
#'     \item Statistical counts of specific binary sequences (e.g., "01 or 10", "11")
#'     \item Expected counts under the assumption of Complete Spatial Randomness (CSR)
#'     \item Standard deviations and Z-scores (ZHD for "01 or 10" sequences, ZDD for "11" sequences)
#'     \item Interpretation of whether the spatial distribution for each sequence type is
#'           "Aggregated" or "Not Aggregated" based on Z-scores
#'     \item A summary explaining the implications of these statistics and patterns
#'   }
#'   The return value aims to provide a clear understanding of the spatial arrangement's
#'   characteristics, aiding in further spatial analysis or research.
#'
#' @details
#' The function conducts an analysis by first counting the occurrence of specific sequences
#' ("01 or 10" and "11" - equivalent to HD and DD) in the binary matrix.
#'  It then calculates expected values, standard
#' deviations, and Z-scores to determine the spatial randomness or aggregation. The analysis
#' considers both horizontal and vertical adjacency (rook case) in the matrix.
#' @references
#' Madden, L. V., Hughes, G., & van den Bosch, F. (2007). The Study of Plant Disease Epidemics.
#'   The American Phytopathological Society.
#'
#' @export
#' @family Spatial analysis
join_count <- function(matrix_data, verbose = TRUE) {

  count_sequences <- function(matrix_data) {
    count_01_10 <- 0
    count_11 <- 0

    for (i in 1:nrow(matrix_data)) {
      for (j in 1:(ncol(matrix_data) - 1)) {
        if ((matrix_data[i, j] == 0 && matrix_data[i, j + 1] == 1) ||
            (matrix_data[i, j] == 1 && matrix_data[i, j + 1] == 0)) {
          count_01_10 <- count_01_10 + 1
        } else if (matrix_data[i, j] == 1 && matrix_data[i, j + 1] == 1) {
          count_11 <- count_11 + 1
        }
      }
    }

    for (j in 1:ncol(matrix_data)) {
      for (i in 1:(nrow(matrix_data) - 1)) {
        if ((matrix_data[i, j] == 0 && matrix_data[i + 1, j] == 1) ||
            (matrix_data[i, j] == 1 && matrix_data[i + 1, j] == 0)) {
          count_01_10 <- count_01_10 + 1
        } else if (matrix_data[i, j] == 1 && matrix_data[i + 1, j] == 1) {
          count_11 <- count_11 + 1
        }
      }
    }

    list(One_Zero_or_Zero_One = count_01_10, One_One = count_11)
  }

  results <- count_sequences(matrix_data)
  num_rows <- nrow(matrix_data)
  num_cols <- ncol(matrix_data)
  S0 <- 2 * ((2 * num_rows * num_cols) - num_rows - num_cols)
  S1 <- 2 * S0
  S2 <- 8 * ((8 * num_rows * num_cols) - (7 * num_rows) - (7 * num_cols) + 4)
  y <- mean(matrix_data)

  ER_01 <- S0 * y * (1 - y)
  sR_01 <- sqrt(S2 * y * (1 - y) + 4 * (S1 - S2) * y^2 * (1 - y)^2) / 2
  ER_11 <- S0 * y^2 / 2
  sR_11 <- sqrt(S1 * y^2 + (S2 - 2 * S1) * y^3 + (S1 - S2) * y^4) / 2

  ZHD <- (results$One_Zero_or_Zero_One - ER_01 - 0.5) / sR_01
  ZDD <- (results$One_One - ER_11 + 0.5) / sR_11

  pattern_HD <- ifelse(ZHD < -1.64, "aggregated", "not aggregated")
  pattern_DD <- ifelse(ZDD > 1.64, "aggregated", "not aggregated")

  output <- list(
    observed = list(HD = results$One_Zero_or_Zero_One, DD = results$One_One),
    expected = list(HD = ER_01, DD = ER_11),
    sd = list(HD = sR_01, DD = sR_11),
    zscore = list(HD = ZHD, DD = ZDD),
    pattern = list(HD = pattern_HD, DD = pattern_DD)
  )

  if (verbose) {
    message(
      "Join Count Analysis of Spatial Patterns of Plant Diseases:\n",
      "----------------------------------------------------------\n",
      "1) 'HD' Sequences:\n",
      sprintf("   - Observed Count: %d\n", output$observed$HD),
      sprintf("   - Expected Count: %.2f\n", output$expected$HD),
      sprintf("   - Standard Deviation: %.2f\n", output$sd$HD),
      sprintf("   - Z-score: %.2f\n", output$zscore$HD),
      sprintf("   - Pattern: %s\n\n", output$pattern$HD),
      "2) 'DD' Sequences:\n",
      sprintf("   - Observed Count: %d\n", output$observed$DD),
      sprintf("   - Expected Count: %.2f\n", output$expected$DD),
      sprintf("   - Standard Deviation: %.2f\n", output$sd$DD),
      sprintf("   - Z-score: %.2f\n", output$zscore$DD),
      sprintf("   - Pattern: %s\n", output$pattern$DD),
      "----------------------------------------------------------\n"
    )
  }

  invisible(output)
}

#' Windowpane Tests for Correlation Analysis
#'
#' This function performs bootstrapped correlation analysis for multiple predictors against a response variable.
#' It applies the Simes method for global significance testing and calculates individual correlations,
#' p-values, and bootstrap statistics.
#'
#' @param data A data frame containing the predictors and the response variable.
#' @param response_var A string representing the name of the response variable in the data frame.
#' @param corr_type A string specifying the correlation method to use; options are "spearman" (default),
#'        "pearson", or "kendall".
#' @param R An integer indicating the number of bootstrap replications. Default is 1000.
#' @param global_alpha A numeric value representing the global alpha level for the Simes correction.
#'        Default is 0.05.
#' @param individual_alpha A numeric value for the individual alpha threshold for testing individual predictors.
#'        Default is 0.005.
#'
#' @details The function calculates correlations between the response variable and each predictor in the
#' data frame, using bootstrapping to generate mean, standard deviation, and median estimates of the
#' correlation. The Simes correction is applied to control for multiple testing, providing a global
#' p-value (\emph{Pg}). The function also returns the maximum observed correlation.
#'
#' @return A list containing the following elements:
#' \item{results}{A data frame with columns: \code{variable}, \code{correlation}, \code{p_value},
#' \code{mean_corr}, \code{sd_corr}, \code{median_corr}, \code{rank}, \code{simes_threshold},
#' \code{significant_simes}, and \code{individual_significant}.}
#' \item{summary_table}{A data frame summarizing the global p-value (\emph{Pg}) and maximum correlation.}
#' \item{global_significant}{A logical value indicating whether the global test is significant.}
#'
#'
#' @export

windowpane_tests <- function(data, response_var, corr_type = "spearman", R = 1000, global_alpha = 0.05, individual_alpha = 0.005) {
  # Define predictors and response
  predictors <- setdiff(names(data), response_var)
  response <- data[[response_var]]

  # Ensure predictors are numeric
  data[predictors] <- lapply(data[predictors], as.numeric)

  # Initialize the results data frame
  results <- data.frame(variable = predictors,
                        correlation = NA,
                        p_value = NA,
                        mean_corr = NA,
                        sd_corr = NA,
                        median_corr = NA)

  # Define the internal function for bootstrapping
  calc_correlation <- function(data, indices, var, response_var) {
    # Subset data for bootstrap sample
    sample_data <- data[indices, ]
    var_data <- sample_data[[var]]
    response_data <- sample_data[[response_var]]

    # Calculate correlation for the given sample
    corr_result <- cor.test(var_data, response_data,
                            method = corr_type, exact = FALSE)

    # Return the correlation estimate and p-value
    return(c(as.numeric(corr_result$estimate), corr_result$p.value))
  }

  # Loop through each predictor
  for (var in predictors) {
    var_data <- data[[var]]

    # Ensure the variable is numeric and not constant
    if (is.numeric(var_data) && length(unique(var_data)) > 1) {
      # Check if enough complete cases exist
      complete_cases <- complete.cases(var_data, response)
      if (sum(complete_cases) > 5) {  # Minimum sample size of 5
        # Prepare data for bootstrapping
        data_boot <- data.frame(var_data = var_data[complete_cases], response = response[complete_cases])

        # Run the bootstrap (using boot package)
        boot_result <- boot::boot(data = data_boot, statistic = function(data, indices) {
          calc_correlation(data, indices, "var_data", "response")
        }, R = R)

        # Prepare bootstrap summary
        bootstrap_df <- as.data.frame(boot_result$t)
        colnames(bootstrap_df) <- c("correlation", "p_value")

        # Calculate mean, standard deviation, and median of the correlation estimates
        mean_corr <- mean(bootstrap_df$correlation, na.rm = TRUE)
        sd_corr <- sd(bootstrap_df$correlation, na.rm = TRUE)
        median_corr <- median(bootstrap_df$correlation, na.rm = TRUE)

        # Extract the initial correlation and p-value from the bootstrap
        corr <- boot_result$t0[1]
        p_value <- boot_result$t0[2]

        # Store results
        results[results$variable == var, c('correlation', 'p_value', 'mean_corr', 'sd_corr', 'median_corr')] <-
          c(corr, p_value, mean_corr, sd_corr, median_corr)
      }
    }
  }

  # Apply Simes method to adjust for multiple testing
  results <- results %>%
    dplyr::arrange(p_value) %>%
    mutate(rank = row_number(),
           m = n(),
           simes_threshold = global_alpha * rank / m,
           significant_simes = p_value <= simes_threshold,
           individual_significant = p_value <= individual_alpha)  # Use individual_alpha = 0.005

  # Calculate the global p-value (Pg) as the minimum of the Simes-adjusted p-values
  Pg <- min(results$p_value / (results$rank / results$m), na.rm = TRUE)

  # Determine global significance
  global_significant <- Pg < global_alpha

  # Find the maximum correlation
  max_correlation <- max(abs(results$correlation), na.rm = TRUE)

  # Add global Pg and max correlation as a separate row
  summary_table <- data.frame(
    Metric = c("Global P-value (Pg)", "Max Correlation"),
    Value = c(Pg, max_correlation)
  )

  # Return the results data frame and summary table
  return(list(
    results = results,
    summary_table = summary_table,
    global_significant = global_significant
  ))
}

#' Fit Gradient Models to Data
#'
#' This function fits three gradient models (exponential, power, and modified power) to given data.
#' It then ranks the models based on their R-squared values and returns diagnostic plots for each model.
#'
#' @param data A dataframe containing the data, with columns "x" representing distances and
#' "Y" representing the corresponding measurements or counts.
#' @param C A constant to be used in the modified power model. Defaults to 1.
#'
#' @return A list containing:
#' \item{data}{The input data, which will include an additional column 'mod_x'.}
#' \item{results_table}{A table of the model parameters and R-squared values.}
#' \item{plot_exponential}{Diagnostic plot for the exponential model.}
#' \item{plot_power}{Diagnostic plot for the power model.}
#' \item{plot_modified_power}{Diagnostic plot for the modified power model.}
#' \item{plot_exponential_original}{Plot of the original data with the exponential model fit.}
#' \item{plot_power_original}{Plot of the original data with the power model fit.}
#' \item{plot_modified_power_original}{Plot of the original data with the modified power model fit.}
#'
#' @examples
#' x <- c(0.8, 1.6, 2.4, 3.2, 4, 7.2, 12, 15.2, 21.6, 28.8)
#' Y <- c(184.9, 113.3, 113.3, 64.1, 25, 8, 4.3, 2.5, 1, 0.8)
#' grad1 <- data.frame(x = x, Y = Y)
#' library(ggplot2)
#' mg <- fit_gradients(grad1, C = 0.4)
#' mg$plot_power_original +
#'   labs(title = "", x = "Distance from focus (m)", y = "Count of lesions")
#'
#' @export
#' @family Spatial analysis

fit_gradients <- function(data, C = 1) {
  # Ensure column names are as expected
  if (!all(c("x", "Y") %in% colnames(data))) {
    stop("The dataframe should have columns named 'x' and 'Y'")
  }

  # Ensure ggplot2 is available
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("The 'ggplot2' package is required. Please install it with install.packages('ggplot2').")
  }

  # Fit the models
  exponential <- lm(log(Y) ~ x, data = data)
  power <- lm(log(Y) ~ log(x), data = data)
  modified_power <- lm(log(Y) ~ log(x + C), data = data)

  # Extract parameters and R-squared
  get_params <- function(model) {
    s <- summary(model)
    a <- round(coef(model)[1], 3)
    b <- round(coef(model)[2], 3)
    R2 <- round(s$r.squared, 3)
    a_back_transformed <- round(exp(a), 3)

    se_a <- round(s$coefficients["(Intercept)", "Std. Error"], 3)
    predictor_name <- names(coef(model))[2]
    se_b <- round(s$coefficients[predictor_name, "Std. Error"], 3)

    sig_symbol <- function(p_val) {
      if (p_val < 0.01) return("**")
      else if (p_val < 0.05) return("*")
      else return("")
    }

    sig_a <- sig_symbol(s$coefficients["(Intercept)", "Pr(>|t|)"])
    sig_b <- sig_symbol(s$coefficients[predictor_name, "Pr(>|t|)"])

    return(list(
      a = a, se_a = se_a, sig_a = sig_a,
      b = b, se_b = se_b, sig_b = sig_b,
      a_back = a_back_transformed, R2 = R2
    ))
  }

  exp_params <- get_params(exponential)
  power_params <- get_params(power)
  mod_power_params <- get_params(modified_power)

  # Combine results into a table
  results <- rbind(
    Exponential = unlist(exp_params),
    Power = unlist(power_params),
    Modified_Power = unlist(mod_power_params)
  )

  # Plotting
  plot_exponential <- ggplot2::ggplot(data, ggplot2::aes(x = x, y = log(Y))) +
    ggplot2::geom_point() +
    ggplot2::geom_smooth(method = "lm", se = FALSE, color = "black") +
    ggplot2::ggtitle("Exponential Model")

  plot_power <- ggplot2::ggplot(data, ggplot2::aes(x = log(x), y = log(Y))) +
    ggplot2::geom_point() +
    ggplot2::geom_smooth(method = "lm", se = FALSE, color = "black") +
    ggplot2::ggtitle("Power Model")

  data$mod_x <- log(data$x + C)
  plot_modified_power <- ggplot2::ggplot(data, ggplot2::aes(x = mod_x, y = log(Y))) +
    ggplot2::geom_point() +
    ggplot2::geom_smooth(method = "lm", se = FALSE, color = "black") +
    ggplot2::ggtitle("Modified Power Model")

  plot_exponential_original <- ggplot2::ggplot(data, ggplot2::aes(x = x, y = Y)) +
    ggplot2::geom_point() +
    ggplot2::stat_function(fun = function(x) exp_params$a_back * exp(exp_params$b * x), color = "black") +
    ggplot2::ggtitle("Exponential Model Fit")

  plot_power_original <- ggplot2::ggplot(data, ggplot2::aes(x = x, y = Y)) +
    ggplot2::geom_point() +
    ggplot2::stat_function(fun = function(x) power_params$a_back * (x^power_params$b), color = "black") +
    ggplot2::ggtitle("Power Model Fit")

  plot_modified_power_original <- ggplot2::ggplot(data, ggplot2::aes(x = x, y = Y)) +
    ggplot2::geom_point() +
    ggplot2::stat_function(fun = function(x) mod_power_params$a_back * ((x + C)^mod_power_params$b), color = "black") +
    ggplot2::ggtitle("Modified Power Model Fit")

  return(list(
    data = data,
    results_table = results,
    plot_exponential = plot_exponential,
    plot_power = plot_power,
    plot_modified_power = plot_modified_power,
    plot_exponential_original = plot_exponential_original,
    plot_power_original = plot_power_original,
    plot_modified_power_original = plot_modified_power_original
  ))
}

#' Print functional suppression profiles
#'
#' @param x An object of class \code{"functional_suppression_profiles"}.
#' @param ... Additional arguments.
#'
#' @export
print.functional_suppression_profiles <- function(x, ...) {
  n_treatments <- length(unique(x$clusters$treatment))
  n_profiles <- length(unique(x$clusters$cluster))
  
  cat("Functional Suppression Profiles\n")
  cat("--------------------------------\n")
  cat("Reference treatment:", x$reference, "\n")
  cat("Treatments:", n_treatments + 1, "\n") # +1 for reference
  cat("Profiles:", n_profiles, "\n")
  cat("Distance: functional distance among smoothed DSP curves\n")
  cat("Persistence threshold:", x$parameters$threshold, "\n\n")
  
  # Top treatments
  cat("Top treatments by mean rank:\n")
  top_n <- min(3, nrow(x$classification))
  top_df <- x$classification[1:top_n, ]
  
  trt_var <- x$parameters$treatment
  for (i in 1:top_n) {
    cat(sprintf("  %-3s %-12s mean rank = %.1f\n", 
                as.character(top_df[[trt_var]][i]), 
                as.character(top_df$profile[i]), 
                top_df$mean_rank[i]))
  }
  
  invisible(x)
}

#' Plot environment-adjusted epidemic curves by cluster
#'
#' @description
#' Plots environment-adjusted mean epidemic curves for each treatment,
#' colored according to functional cluster membership.
#'
#' The returned object is a \code{ggplot} and can be further modified
#' using standard ggplot2 layers.
#'
#' @param x An object of class \code{"functional_distances"}.
#' @param label_fun Optional function to modify treatment labels.
#' @param palette Optional named vector of colors for clusters.
#' @param alpha Line transparency.
#' @param linewidth Line width.
#' @param ... Additional arguments.
#'
#' @return A \code{ggplot} object.
#'
#' @export
plot_curves <- function(x, ...) {
  UseMethod("plot_curves")
}

#' @export
#' @rdname plot_curves
plot_curves.functional_distances <- function(
    x,
    label_fun = NULL,
    palette = NULL,
    alpha = 0.9,
    linewidth = 1.1,
    ...
){

  if(!requireNamespace("ggplot2", quietly = TRUE)) stop("Need ggplot2.")
  if(!requireNamespace("dplyr", quietly = TRUE)) stop("Need dplyr.")

  fc <- x$functional_curves
  trt <- fc$vars$treatment
  tim <- fc$vars$time
  pred_df <- fc$curves

  if(is.null(label_fun)) label_fun <- function(z) z

  df <- dplyr::left_join(pred_df, x$clusters, by = trt) |>
    dplyr::mutate(
      trt_lab = label_fun(.data[[trt]]),
      cluster = factor(cluster)
    )

  p <- ggplot2::ggplot(
    df,
    ggplot2::aes(
      x = .data[[tim]],
      y = mu,
      group = trt_lab,
      colour = cluster
    )
  ) +
    ggplot2::geom_line(
      linewidth = linewidth,
      alpha = alpha
    ) +
    ggplot2::expand_limits(y = c(0, 1)) +
    ggplot2::labs(
      x = tim,
      y = "Environment-adjusted mean severity",
      colour = "Cluster"
    ) +
    ggplot2::theme_classic(base_size = 13) +
    ggplot2::theme(legend.position = "bottom")

  if(!is.null(palette)){
    p <- p + ggplot2::scale_colour_manual(values = palette)
  }

  p
}

#' @export
#' @rdname plot_curves
plot_curves.functional_dsp <- function(x, ...) {
  plot_dsp(x, ...)
}

---
title: "Functional analysis of plant disease progress curves"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Functional analysis of plant disease progress curves}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5,
  fig.align = "center"
)
```

## Introduction

In plant disease epidemiology, temporal disease progress data are frequently condensed into single scalar summaries such as final severity or the Area Under the Disease Progress Curve (AUDPC). While convenient, these scalar reductions collapse the temporal dimension, discarding critical information about epidemic onset, peak progress rates, acceleration phases, and trajectory shapes.

The `{r4pde}` functional analysis framework treats disease progress curves as continuous functional data modeled via Generalized Additive Models (GAMs). This modular API enables researchers to fit smoothed trajectories, compute instantaneous rates of progress, measure whole-curve distances, perform functional PCA, quantify resistance indices, and evaluate environmental stability:

```text
observations (time, response, treatment, block/env)
    │
    ▼
functional_curves()
    │
    ├──► functional_rate()         (instantaneous velocity & rate phenotypes)
    │
    ├──► functional_distances()    (pairwise curve distances & clustering)
    │
    ├──► functional_pca()          (orthogonal modes of trajectory variation)
    │
    ├──► functional_resistance()   (Functional Resistance Index & ranking)
    │
    └──► functional_instability()  (genotype-by-environment trajectory stability)
```

```{r setup, message = FALSE}
library(r4pde)
library(ggplot2)
```

## Example Data: Soybean Bud Blight

To demonstrate this workflow, we use the package dataset `BudBlightSoybean`, which tracks bud blight incidence across 4 planting dates (`PD1` to `PD4`) evaluated at four assessment dates (30, 40, 50, and 60 days after planting) across 4 randomized blocks:

```{r data}
data("BudBlightSoybean", package = "r4pde")
head(BudBlightSoybean)
```

---

## 1. Fitting Disease Progress Curves: `functional_curves()`

**Scientific question:** *What are the continuous mean epidemic trajectories for each treatment over time, after accounting for experimental design structures such as blocking?*

`functional_curves()` uses penalized splines in a GAM framework (via `mgcv`) to estimate smooth, environment- and design-adjusted trajectories without forcing the data into restrictive parametric shapes (e.g., strictly logistic or Gompertz):

```{r curves, fig.alt = "Fitted mean disease progress curves over time for soybean bud blight treatments"}
fc <- functional_curves(
  data = BudBlightSoybean,
  time = "time",
  response = "y",
  treatment = "treat",
  block = "block",
  min_points = 3,
  family_try = "quasibinomial",
  show_progress = FALSE
)

# Plot fitted mean curves
plot(fc)
```

The fitted object `fc` contains the underlying GAM model, the predicted curves over a dense time grid, and model diagnostics.

---

## 2. Instantaneous Velocity and Growth Phenotypes: `functional_rate()`

**Scientific question:** *At what instantaneous rate is the epidemic progressing at any given point in time, when does maximum velocity occur ($t_{r_{\max}}$), and how long does the active epidemic growth window last?*

`functional_rate()` estimates the first temporal derivative $S'(t) = \frac{dS(t)}{dt}$ directly from the fitted GAM linear predictor matrix, propagating model covariance to compute pointwise confidence intervals and extracting key rate phenotypes:

```{r rate, fig.alt = "Instantaneous rate of disease progress and velocity phenotypes across treatments"}
fr <- functional_rate(fc, scale = "response", n_grid = 100)

# Summary of rate phenotypes (r_max, t_r_max, growth duration)
summary(fr)

# Visualize instantaneous progress rates over time
plot(fr)
```

Unlike finite differences on raw observations, `functional_rate()` provides smooth derivatives with rigorous uncertainty estimation, revealing whether treatments delay onset, reduce peak velocity, or shorten the active epidemic duration.

---

## 3. Whole-Trajectory Distances and Clustering: `functional_distances()`

**Scientific question:** *How different are treatments across the entire continuous temporal domain, and which treatments form cohesive clusters of epidemic behavior?*

`functional_distances()` calculates $L_2$ functional distances by integrating squared differences between fitted curves across the shared time domain:

$$D_{ij} = \sqrt{\int_T (f_i(t) - f_j(t))^2 dt}$$

It performs hierarchical clustering and profile identification:

```{r distances, fig.alt = c("Adjusted functional curves colored by functional cluster", "Hierarchical clustering dendrogram based on L2 functional distances")}
fd <- functional_distances(fc, cluster_k = 2, show_progress = FALSE)

# Plot environment-adjusted curves colored by functional cluster
plot_curves(fd)

# Plot hierarchical clustering dendrogram
plot_dendrogram(fd)
```

---

## 4. Orthogonal Trajectory Decomposition: `functional_pca()`

**Scientific question:** *What are the dominant modes of variation that distinguish epidemic curves across treatments?*

Functional Principal Component Analysis (FPCA) decomposes variation among epidemic trajectories into orthogonal temporal components (eigenfunctions), allowing researchers to differentiate between overall epidemic magnitude and timing/shape shifts:

```{r fpca, fig.alt = "Biplot of treatment trajectories in functional principal component score space"}
# Retain 2 components for score biplot
fpca <- functional_pca(fc, n_components = 2)

# Print variance explained by the functional principal components
print(fpca)

# Biplot of treatments in FPCA score space
plot(fpca, type = "scores")
```

---

## 5. Functional Resistance Profiling: `functional_resistance()`

**Scientific question:** *How do treatments rank in functional disease suppression relative to a susceptible reference, and are differences statistically supported?*

`functional_resistance()` computes the Functional Resistance Index (FRI) and Stability-Adjusted FRI (SAFRI) by benchmarking each curve's integral against a designated susceptible check:

```{r resistance}
# Using 'PD1' as the reference susceptible treatment
fres <- functional_resistance(
  fc,
  reference = "PD1",
  group_method = "quantile",
  n_groups = 2
)

# View resistance scores and rankings
fres$table
```

---

## 6. Environmental Instability: `functional_instability()`

**Scientific question:** *When multi-environment trials (locations, years) are available, which genotypes exhibit stable epidemic suppression versus high genotype-by-environment variability?*

When epidemics are monitored across multiple locations or seasons (using the `environment` parameter in `functional_curves()`), `functional_instability()` computes Normalized Functional Instability (NFI):

$$nFI_g = \frac{\frac{1}{E_g} \sum_{e=1}^{E_g} \int_T (f_{ge}(t) - \bar{f}_g(t))^2 dt}{\int_T \bar{f}_g(t)^2 dt}$$

It can also decompose instability into spatial (location-driven) and temporal (year-to-year) components via `env_sep`.

---

## Summary

The `{r4pde}` functional analysis API provides a coherent, principled alternative to scalar disease metrics. By chaining modular functions:

$$\text{data} \xrightarrow{\texttt{functional\_curves()}} \begin{cases}
\xrightarrow{\texttt{functional\_rate()}} \text{rate phenotypes & derivatives} \\
\xrightarrow{\texttt{functional\_distances()}} \text{hierarchical curve clustering} \\
\xrightarrow{\texttt{functional\_pca()}} \text{dominant trajectory modes} \\
\xrightarrow{\texttt{functional\_resistance()}} \text{standardized resistance ranking} \\
\xrightarrow{\texttt{functional\_instability()}} \text{G}\times\text{E stability decomposition}
\end{cases}$$

researchers gain deep insight into epidemic dynamics, treatment efficacy, and host resistance.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{get_fpca_variance}
\alias{get_fpca_variance}
\title{Get FPCA variance explained}
\usage{
get_fpca_variance(x)
}
\arguments{
\item{x}{An object of class \code{"r4pde_functional_pca"}.}
}
\description{
Get FPCA variance explained
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_instability.R
\name{functional_instability}
\alias{functional_instability}
\alias{functional_instability.functional_curves}
\alias{functional_instability.functional_dsp}
\title{Normalized functional instability from fitted functional disease curves}
\usage{
functional_instability(x, ...)

\method{functional_instability}{functional_curves}(
  x,
  n_time = 200,
  env_sep = NULL,
  env_names = c("location", "year"),
  return_curves = FALSE,
  ...
)

\method{functional_instability}{functional_dsp}(
  x,
  env_sep = NULL,
  env_names = c("location", "year"),
  return_curves = FALSE,
  ...
)
}
\arguments{
\item{x}{An object returned by \code{\link{functional_curves}} or a \code{functional_dsp} object.}

\item{...}{Additional arguments.}

\item{n_time}{Number of points in the prediction grid over the time domain.}

\item{env_sep}{Optional separator used to split \code{env} into spatial and
temporal components, for example \code{"_"} or \code{"-"}. If \code{NULL}, only the
overall NFI is returned.}

\item{env_names}{A character vector of length 2 giving the names of the
components obtained after splitting \code{env}. Default is
\code{c("location", "year")}.}

\item{return_curves}{Logical. If \code{TRUE}, the predicted genotype-by-environment
curves are also returned.}
}
\value{
If \code{return_curves = FALSE}, a tibble with one row per genotype and
columns:
\describe{
\item{geno}{Treatment or genotype identifier.}
\item{n_env}{Number of environments used in the calculation.}
\item{FI}{Absolute functional instability.}
\item{mean_energy}{Integrated squared mean epidemic curve.}
\item{nFI}{Normalized functional instability. Lower values indicate
greater stability.}
}

If \code{env_sep} is provided, the returned tibble also includes:
\describe{
\item{FI_space, mean_energy_space, nFI_space}{Spatial component of
instability.}
\item{FI_time, mean_energy_time, nFI_time}{Temporal component of
instability.}
}

If \code{return_curves = TRUE}, a list is returned with two elements:
\describe{
\item{metrics}{The tibble described above.}
\item{curves}{A tibble of predicted genotype-by-environment curves with
columns \code{geno}, \code{env}, \code{time}, \code{mu}, and \code{eta}.}
}
}
\description{
Computes normalized functional instability (NFI) for each treatment
based on genotype-by-environment predicted curves extracted from a
\code{\link{functional_curves}} object. Optionally, instability can be decomposed
into spatial and temporal components if the environment identifier can
be split into location and year.
}
\details{
The overall instability metric is defined as the mean integrated squared
deviation of each genotype-by-environment curve from the genotype-specific
mean curve across environments, normalized by the integrated squared mean
curve.

Let \eqn{f_{ge}(t)} denote the predicted epidemic trajectory of genotype
\eqn{g} in environment \eqn{e}, and let \eqn{\bar f_g(t)} denote the mean
trajectory of genotype \eqn{g} across environments. Functional instability
is computed as:

\deqn{
FI_g = \frac{1}{E_g}\sum_{e=1}^{E_g} \int_T \left(f_{ge}(t)-\bar f_g(t)\right)^2 dt
}

and normalized as:

\deqn{
nFI_g = \frac{FI_g}{\int_T \bar f_g(t)^2 dt}
}

Numerical integration is performed with the trapezoidal rule on a regular
prediction grid over the observed time domain.
}
\examples{
\dontrun{
m1 <- r4pde::functional_curves(
  data = dat_ready,
  time = "time",
  response = "y",
  treatment = "geno",
  environment = "env"
)

# Overall instability
res <- functional_instability(m1)
res

# Overall + spatial and temporal components
# Assuming env has values such as "PF_2021"
res2 <- functional_instability(m1, env_sep = "_")
res2

# Return curves used in the calculations
out <- functional_instability(m1, env_sep = "_", return_curves = TRUE)
out$metrics
head(out$curves)
}

}
\seealso{
\code{\link{functional_curves}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_era5.R
\name{get_era5}
\alias{get_era5}
\title{Fetch ERA5 Data from Open-Meteo for Multiple Locations}
\usage{
get_era5(
  data,
  days_around,
  date_col,
  study_col = "study",
  pars = c("temperature_2m", "relative_humidity_2m", "precipitation", "dewpoint_2m"),
  models = NULL,
  direction = "both"
)
}
\arguments{
\item{data}{A data frame containing the input data, including columns for \code{latitude}, \code{longitude},
and the date column specified by \code{date_col}.}

\item{days_around}{An integer specifying the number of days before and after the date in the date column
to download data.}

\item{date_col}{A character string specifying the name of the date column in the data frame.}

\item{study_col}{A character string specifying the name of the column containing the study identifier
in the input data frame (default: "study").}

\item{pars}{A character vector specifying the weather variables to fetch from Open-Meteo.
These are mapped to \code{nasapower} equivalents:
"temperature_2m" (T2M), "relative_humidity_2m" (RH2M), "precipitation" (PRECTOTCORR), etc.}

\item{models}{A character string specifying the reanalysis model(s) to use. If \code{NULL} (default),
Open-Meteo uses its "best_match" logic (ERA5, ERA5-Land, etc.).}

\item{direction}{Character string specifying the direction of the date range relative to the reference date.
Options are "both" (default), "back", or "forth".
"back" retrieves data from \code{date - days_around} to \code{date}.
"forth" retrieves data from \code{date} to \code{date + days_around}.
"both" retrieves data from \code{date - days_around} to \code{date + days_around}.}
}
\value{
A data frame (tibble) with daily weather data. Columns are named to match
\code{nasapower} conventions: \code{date}, \code{T2M}, \code{T2M_MAX}, \code{T2M_MIN}, \code{RH2M}, \code{PRECTOTCORR},
\code{T2MDEW}, \code{longitude}, \code{latitude}, and \code{study}.
}
\description{
This function downloads ERA5 historical weather data from the Open-Meteo API
for specified locations and dates. It mimics the functionality of \code{get_nasapower}
by fetching weather variables and returning a daily-aggregated data frame.
It uses the Open-Meteo Archive API and aggregates hourly data to daily values
to ensure all variables (including relative humidity and dew point) are available.
}
\details{
Since the Open-Meteo Archive API \code{daily} endpoint does not provide all variables
(like mean relative humidity) directly, this function fetches \code{hourly} data and
performs the daily aggregation manually:
\itemize{
\item \code{T2M}: Mean of hourly \code{temperature_2m}
\item \code{T2M_MAX}: Max of hourly \code{temperature_2m}
\item \code{T2M_MIN}: Min of hourly \code{temperature_2m}
\item \code{RH2M}: Mean of hourly \code{relative_humidity_2m}
\item \code{PRECTOTCORR}: Sum of hourly \code{precipitation}
\item \code{T2MDEW}: Mean of hourly \code{dewpoint_2m}
}
}
\seealso{
Other Disease modeling: 
\code{\link{get_brdwgd}()},
\code{\link{get_nasapower}()},
\code{\link{windowpane}()}
}
\concept{Disease modeling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{print.r4pde_curve_diagnostics}
\alias{print.r4pde_curve_diagnostics}
\title{Print method for curve diagnostics}
\usage{
\method{print}{r4pde_curve_diagnostics}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"r4pde_curve_diagnostics"}.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for curve diagnostics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_nasapower.R
\name{get_nasapower}
\alias{get_nasapower}
\title{Fetch NASA POWER Data for Multiple Locations with a Progress Bar}
\usage{
get_nasapower(
  data,
  days_around,
  date_col,
  study_col = "study",
  pars = c("T2M", "RH2M", "PRECTOTCORR", "T2M_MAX", "T2M_MIN", "T2MDEW"),
  direction = "both"
)
}
\arguments{
\item{data}{A data frame containing the input data, including columns for latitude, longitude, and
the date column.}

\item{days_around}{An integer specifying the number of days before and after the date in the date column
to download data.}

\item{date_col}{A character string specifying the name of the date column in the data frame.}

\item{study_col}{A character string specifying the name of the column containing the study identifier
in the input data frame (default: "study").}

\item{pars}{A character vector specifying the weather variables to fetch from NASA POWER
(default: c("T2M", "RH2M", "PRECTOTCORR", "T2M_MAX", "T2M_MIN", "T2MDEW")).}

\item{direction}{Character string specifying the direction of the date range relative to the reference date.
Options are "both" (default), "back", or "forth".
"back" retrieves data from \code{date - days_around} to \code{date}.
"forth" retrieves data from \code{date} to \code{date + days_around}.
"both" retrieves data from \code{date - days_around} to \code{date + days_around}.}
}
\value{
A data frame with the downloaded weather data from NASA POWER, combined for all specified locations.
Includes a variable \code{study} indicating the study identifier from the input data.
Returns an empty data frame if no data is retrieved.
}
\description{
This function downloads daily NASA POWER data for specified weather variables over a specified number of days
around a given date column for multiple locations. It includes a progress bar to show the download progress.
}
\details{
The function uses the \code{get_power} function from the \code{nasapower} package to fetch weather data for a range of
dates around the specified date column for each location. A progress bar is shown during the data download
process, and the results are combined into a single data frame.
}
\seealso{
Other Disease modeling: 
\code{\link{get_brdwgd}()},
\code{\link{get_era5}()},
\code{\link{windowpane}()}
}
\concept{Disease modeling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_dsp.R
\name{plot_dsp}
\alias{plot_dsp}
\title{Plot Disease Suppression Profiles}
\usage{
plot_dsp(
  object,
  time = NULL,
  response = NULL,
  treatment = NULL,
  group = NULL,
  facet = NULL,
  show_zero = TRUE,
  linewidth = 1,
  alpha = 1,
  ylab = "Disease suppression profile",
  xlab = "Time"
)
}
\arguments{
\item{object}{An object of class \code{"functional_dsp"}.}

\item{time}{Character string for the time column (auto-detected if object is \code{"functional_dsp"}).}

\item{response}{Character string for the DSP column.}

\item{treatment}{Character string for the treatment column.}

\item{group}{Optional character vector of grouping variables.}

\item{facet}{Optional character string specifying a variable for faceting.}

\item{show_zero}{Logical; whether to draw a horizontal reference line at y = 0. Default is \code{TRUE}.}

\item{linewidth}{Numeric; width of the lines.}

\item{alpha}{Numeric; transparency of the lines.}

\item{ylab}{Character string; label for the y-axis.}

\item{xlab}{Character string; label for the x-axis.}
}
\value{
A \code{ggplot} object.
}
\description{
Plots functional contrast curves (DSP) over time.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_diagnostics}
\alias{plot_diagnostics}
\title{Plot diagnostics for functional epidemic curve models}
\usage{
plot_diagnostics(
  diag,
  layout = c("2x2", "vertical", "horizontal"),
  show_curve = FALSE,
  rel_heights = c(1, 0.7)
)
}
\arguments{
\item{diag}{An object of class \code{"r4pde_curve_diagnostics"}.}

\item{layout}{Layout of diagnostic panels: \code{"2x2"}, \code{"vertical"},
or \code{"horizontal"}.}

\item{show_curve}{Logical; whether to include the example adjusted curve.}

\item{rel_heights}{Relative heights for panels when stacking.}
}
\value{
A composite plot object generated using \pkg{cowplot}.
}
\description{
Combines multiple diagnostic plots from
\code{\link{diagnose_curves}} into a multi-panel layout.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DSI.R
\name{DSI}
\alias{DSI}
\title{Calculate the Disease Severity Index (DSI) (class for each unit)}
\usage{
DSI(unit, class, max)
}
\arguments{
\item{unit}{A vector representing the units.}

\item{class}{A vector representing the classes corresponding to the units.}

\item{max}{A numeric value representing the maximum possible class value.}
}
\value{
Returns a single numeric value representing the DSI.
}
\description{
This function calculates the Disease Severity Index (DSI) based on the provided unit,
class, and maximum class value. The DSI is computed by aggregating the classes,
calculating weights by multiplying the frequency of each class by the class itself,
and then dividing the sum of these weights by the product of the total number of
entries and the maximum class value, then multiplying by 100.
}
\examples{
# Example usage:
unit <- c(1, 2, 3, 4, 5, 6)
class <- c(1, 2, 1, 2, 3, 1)
max <- 3
DSI(unit, class, max)

}
\seealso{
Other Disease quantification: 
\code{\link{CompMuCens}()},
\code{\link{DSI2}()}
}
\concept{Disease quantification}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{functional_pca}
\alias{functional_pca}
\alias{functional_pca.functional_curves}
\alias{functional_pca.functional_dsp}
\title{Functional principal component analysis of disease progress curves}
\usage{
functional_pca(object, ...)

\method{functional_pca}{functional_curves}(
  object,
  n_components = NULL,
  var_explained = 0.95,
  center = TRUE,
  scale = FALSE,
  method = c("pca_on_grid"),
  ...
)

\method{functional_pca}{functional_dsp}(
  object,
  n_components = NULL,
  var_explained = 0.95,
  center = TRUE,
  scale = FALSE,
  method = c("pca_on_grid"),
  ...
)
}
\arguments{
\item{object}{An object returned by \code{\link{functional_curves}}.}

\item{...}{Additional arguments for future extensions.}

\item{n_components}{Optional integer number of functional principal components to retain.}

\item{var_explained}{Cumulative variance threshold used when \code{n_components = NULL}.}

\item{center}{Logical; whether to center curves before PCA. Default TRUE.}

\item{scale}{Logical; whether to scale grid columns before PCA. Default FALSE.}

\item{method}{Character; method for FPCA, currently only \code{"pca_on_grid"} is supported.}
}
\value{
An object of class \code{"r4pde_functional_pca"} containing:
\itemize{
\item \code{scores}: Tibble of curve-level FPC scores.
\item \code{eigenfunctions}: Tibble of eigenfunction values across the time grid.
\item \code{variance}: Tibble of eigenvalues, variance explained, and cumulative variance.
\item \code{mean_curve}: Tibble of the mean curve.
\item \code{reconstructed}: Tibble of reconstructed curves using retained components.
\item \code{input_curves}: Tibble of original fitted curves.
\item \code{pca}: The underlying \code{prcomp} object.
\item \code{settings}: List of settings used.
\item \code{call}: The matched call.
}
}
\description{
Performs functional principal component analysis on fitted disease progress curves
returned by \code{\link{functional_curves}}. The function decomposes variation among epidemic
trajectories into orthogonal temporal components and returns curve-level scores,
eigenfunctions, variance explained, and reconstructed curves.
}
\details{
The function uses the fitted curves from \code{functional_curves()} and does not refit
the disease progress model. The first implementation uses PCA on a common prediction grid.
FPC scores can be analyzed as functional epidemiological traits in downstream models.

Interpretation:
\itemize{
\item FPC1 often captures the largest mode of variation, commonly overall epidemic intensity or speed.
\item Later FPCs may capture timing of disease onset, curve crossing, late acceleration, or other shape-related deviations.
\item Interpretation must be data-driven and should be based on eigenfunction plots and mean +/- perturbation plots.
}
}
\examples{
\dontrun{
curves <- functional_curves(...)
fpca <- functional_pca(curves, var_explained = 0.95)

print(fpca)
plot(fpca, type = "scree")
plot(fpca, type = "components")
plot(fpca, type = "scores", components = c(1, 2))

scores <- get_fpca_scores(fpca)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_dsp.R
\name{functional_summary}
\alias{functional_summary}
\title{Summarize Disease Suppression Profiles}
\usage{
functional_summary(
  object,
  time = "time",
  response = "DSP",
  treatment = "treatment",
  group = NULL,
  threshold = 0,
  positive_only = TRUE
)
}
\arguments{
\item{object}{An object of class \code{"functional_dsp"} or a data frame with a DSP column.}

\item{time}{Character string naming the time column.}

\item{response}{Character string naming the DSP column.}

\item{treatment}{Character string naming the treatment column.}

\item{group}{Optional character vector of grouping variables.}

\item{threshold}{Numeric; minimum suppression threshold for calculating persistence. Default is 0.}

\item{positive_only}{Logical; if \code{TRUE}, negative DSP values are set to 0 before area/centroid calculations. Default is \code{TRUE}.}
}
\value{
A \code{tibble} with one row per group/treatment containing:
\itemize{
\item \code{protected_area}: Trapezoidal integral of DSP(t).
\item \code{mean_suppression}: Temporal mean of DSP(t).
\item \code{max_suppression}: Maximum value of DSP(t).
\item \code{time_max_suppression}: Time at which maximum suppression occurs.
\item \code{persistence}: Total duration where DSP(t) > threshold.
\item \code{centroid}: Temporal centroid defined as \eqn{\int t \cdot DSP(t) dt / \int DSP(t) dt}.
\item \code{energy}: Integral of \eqn{DSP(t)^2}.
\item \code{early_area, mid_area, late_area}: Protected area in the first, second, and third tertiles of the time period.
\item \code{late_decline_slope}: Slope of DSP(t) in the late period.
\item \code{n_time_points}: Number of time points evaluated.
}
}
\description{
Extracts functional descriptors from Disease Suppression Profiles (DSPs).
}
\examples{
\dontrun{
# Using a generic data frame with custom column names
dat <- data.frame(
  DAA = c(0, 10, 20),
  fungicide = c("A", "A", "A"),
  contrast = c(0, 10, 0)
)
functional_summary(
  dat, 
  time = "DAA", 
  response = "contrast", 
  treatment = "fungicide"
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{plot.r4pde_functional_pca}
\alias{plot.r4pde_functional_pca}
\alias{autoplot.r4pde_functional_pca}
\title{Plot functional PCA results}
\usage{
\method{plot}{r4pde_functional_pca}(
  x,
  type = c("scree", "components", "scores", "reconstruction", "mean"),
  components = c(1, 2),
  curve_id = NULL,
  ...
)

autoplot.r4pde_functional_pca(object, ...)
}
\arguments{
\item{x}{An object of class \code{"r4pde_functional_pca"}.}

\item{type}{Type of plot: "scree", "components", "scores", "reconstruction", or "mean".}

\item{components}{Integer vector of length 2 indicating which components to plot for scores and mean perturbation.}

\item{curve_id}{Optional vector of curve IDs to include in reconstruction plot.}

\item{...}{Additional arguments.}

\item{object}{An object of class \code{"r4pde_functional_pca"}.}
}
\description{
Plot functional PCA results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_distances.R
\name{print.functional_distances}
\alias{print.functional_distances}
\title{Print functional_distances}
\usage{
\method{print}{functional_distances}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_distances"}.}

\item{...}{Additional arguments.}
}
\description{
Print functional_distances
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{BlastWheat}
\alias{BlastWheat}
\title{BlastWheat dataset}
\format{
A data frame with the following columns:
\describe{
\item{heading}{Date of heading}
\item{inc_mean}{Mean incidence}
\item{index_mean}{FHB index mean}
\item{latitude}{Latitude coordinate}
\item{location}{Experimental site name}
\item{longitude}{Longitude coordinate}
\item{state}{Brazilian state}
\item{study}{Study ID or code}
\item{year}{Crop year}
\item{yld_mean}{Mean yield}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
BlastWheat
}
\description{
Wheat blast dataset with severity and weather covariates.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{SpatialRandom}
\alias{SpatialRandom}
\title{SpatialRandom dataset}
\format{
A data frame with:
\describe{
\item{x}{x-coordinate}
\item{y}{y-coordinate}
}
}
\source{
Simulated example
}
\usage{
SpatialRandom
}
\description{
Simulated random spatial binary disease pattern.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{augment_functional_rate}
\alias{augment_functional_rate}
\title{Augment functional_rate (alias)}
\usage{
augment_functional_rate(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_rate"}.}

\item{...}{Additional arguments.}
}
\value{
A tibble of the full rate evaluation grid.
}
\description{
Augment functional_rate (alias)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{DidymellaWatermelon}
\alias{DidymellaWatermelon}
\title{DidymellaWatermelon dataset}
\format{
A data frame with:
\describe{
\item{EW_row}{Row position (east–west)}
\item{NS_col}{Column position (north–south)}
\item{dap}{Days after planting}
\item{severity}{Disease severity}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
DidymellaWatermelon
}
\description{
Assessment of Didymella symptoms in watermelon plots.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oruns_test_boustrophedon.R
\name{oruns_test_boustrophedon}
\alias{oruns_test_boustrophedon}
\title{Boustrophedon Run Test for Binary Matrix}
\usage{
oruns_test_boustrophedon(mat)
}
\arguments{
\item{mat}{A binary matrix (containing 0s and 1s, and possibly NAs).}
}
\value{
A list with two elements:
\item{rowwise_boustrophedon}{List containing the sequence and result of \code{oruns_test} for row-wise traversal.}
\item{colwise_boustrophedon}{List containing the sequence and result of \code{oruns_test} for column-wise traversal.}
}
\description{
Applies the ordinary runs test to a binary matrix using boustrophedon-style traversal.
The function supports two modes: row-wise and column-wise boustrophedon. Each traversal flattens the matrix into a 1D sequence
which is then tested using \code{oruns_test}.
}
\seealso{
\code{\link{oruns_test}}

Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_functional_suppression_profiles_list.R
\name{plot.functional_suppression_profiles_list}
\alias{plot.functional_suppression_profiles_list}
\title{Plot a list of functional suppression profiles}
\usage{
\method{plot}{functional_suppression_profiles_list}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_suppression_profiles_list"}.}

\item{...}{Additional arguments passed to \code{plot.functional_suppression_profiles}.}
}
\value{
A list of ggplot objects, one for each environment.
}
\description{
Plot a list of functional suppression profiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BPL.R
\name{BPL}
\alias{BPL}
\title{Binary Power Law Analysis for Spatial Disease Patterns}
\usage{
BPL(data)
}
\arguments{
\item{data}{A data frame containing the following columns:
\itemize{
\item \code{field}: The field identifier.
\item \code{n}: The number of observations in each quadrat.
\item \code{i}: The incidence count in each quadrat.
}}
}
\value{
A list containing the following elements:
\itemize{
\item \code{summary}: A data frame summarizing the input data by field, including total observations (\code{n_total}),
mean incidence (\code{incidence_mean}), observed variance (\code{V}), and binomial variance (\code{Vbin}).
\item \code{model_summary}: A summary of the linear model fitted to the log-transformed variances.
\item \code{hypothesis_test}: The result of the hypothesis test for the slope being equal to 1.
\item \code{ln_Ap}: The intercept of the linear model, representing the natural logarithm of the parameter \( A_p \).
\item \code{slope}: The slope of the linear model.
}
}
\description{
This function calculates the Binary Power Law (BPL) parameters for spatial disease patterns,
fits a linear model, and performs a hypothesis test for the slope.
}
\details{
The function performs the following steps:
\enumerate{
\item Summarizes the data by field to calculate the total number of observations (\code{n_total}),
mean incidence (\code{incidence_mean}), observed variance (\code{V}), and binomial variance (\code{Vbin}).
\item Log-transforms the variances.
\item Fits a linear model to the log-transformed variances.
\item Tests the hypothesis that the slope of the linear model is equal to 1.
}
}
\examples{
\donttest{
# Example usage with a sample data frame
result <- BPL(FHBWheat)
print(result$summary)
print(result$model_summary)
print(result$hypothesis_test)
print(paste("ln(Ap):", result$ln_Ap))
print(paste("Slope (b):", result$slope))
}
}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oruns_test.R
\name{oruns_test}
\alias{oruns_test}
\title{Runs Test}
\usage{
oruns_test(x)
}
\arguments{
\item{x}{A numeric vector representing the input data}
}
\value{
an \code{r4pde.oruns} object.

An \code{r4pde.oruns} object is a \code{list} containing:
\itemize{
\item U, number of runs,
\item EU, expected number of runs,
\item sU, standard deviation of the expected number of runs
\item Z, Z-score of the observed number of runs,
\item pvalue, the p-value of the Z-score, and
\item result, the test result of either "aggregation or clustering" or "randomness"
}
}
\description{
Perform a runs test on the input data to test for clustering or randomness.
}
\examples{
oruns_test(c(1, 0, 1, 1, 0, 1, 0, 0, 1, 1))

}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{augment.functional_rate}
\alias{augment.functional_rate}
\title{Augment functional_rate}
\usage{
\method{augment}{functional_rate}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_rate"}.}

\item{...}{Additional arguments.}
}
\value{
A tibble of the full rate evaluation grid with predictions, derivatives,
and confidence intervals.
}
\description{
Augment functional_rate
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_dendrogram.R
\name{plot_dendrogram}
\alias{plot_dendrogram}
\title{Plot functional dendrogram of epidemic curves}
\usage{
plot_dendrogram(
  x,
  label_fun = NULL,
  palette = NULL,
  show_cut = TRUE,
  label_size = 2.8
)
}
\arguments{
\item{x}{An object of class \code{"functional_distances"}.}

\item{label_fun}{Optional function to modify treatment labels.}

\item{palette}{Optional named vector of colors for clusters.}

\item{show_cut}{Logical; whether to display the cluster cut height.}

\item{label_size}{Numeric specifying the font size of the labels. Default is \code{2.8}.}
}
\value{
A \code{ggplot} object.
}
\description{
Visualizes the hierarchical clustering of treatments based on
functional distances among epidemic curves.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{functional_rate}
\alias{functional_rate}
\title{Instantaneous Disease Progress Rates from Functional Curves}
\usage{
functional_rate(
  object,
  scale = c("response", "link"),
  method = c("central"),
  n_grid = 200,
  eps = NULL,
  interval = 0.95,
  threshold = 0,
  positive_only = TRUE,
  uncertainty = TRUE,
  growth_criterion = c("rate", "significant"),
  newdata = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{"functional_curves"}.}

\item{scale}{Character specifying the derivative scale: \code{"response"}
(default; e.g., severity change per unit time) or \code{"link"} (linear predictor scale).}

\item{method}{Character specifying the finite difference method. Currently \code{"central"}
(central difference in the interior with one-sided differences at temporal boundaries).}

\item{n_grid}{Integer; number of points in the regular temporal evaluation grid per group.
Default is 200.}

\item{eps}{Numeric step size used in finite differences. If \code{NULL} (default),
automatically chosen as \eqn{10^{-4} \times (t_{max} - t_{min})}.}

\item{interval}{Numeric confidence level for intervals. Default is 0.95.}

\item{threshold}{Numeric minimum rate threshold considered biologically relevant.
Default is 0.}

\item{positive_only}{Logical; whether to restrict summarized growth descriptors
to positive rates. Default is \code{TRUE}. Note that negative rates are always
preserved in the detailed rate table (\code{data}).}

\item{uncertainty}{Logical; whether to compute standard errors and confidence
intervals using the GAM covariance matrix. Default is \code{TRUE}.}

\item{growth_criterion}{Character specifying how active epidemic growth is defined:
\code{"rate"} (default; \code{rate > threshold}) or \code{"significant"}
(\code{rate_lower > threshold}, requiring \code{uncertainty = TRUE}).}

\item{newdata}{Optional user-supplied data frame with custom evaluation points.
Must contain the time and treatment variables.}

\item{...}{Additional arguments passed to methods.}
}
\value{
An object of class \code{"functional_rate"} containing:
\describe{
\item{\code{data}}{A tibble of full evaluation grid with columns including
treatment, time, \code{fitted}, \code{fitted_se}, \code{fitted_lower}, \code{fitted_upper},
\code{rate}, \code{rate_se}, \code{rate_lower}, \code{rate_upper},
\code{significant_increase}, \code{above_threshold}, and \code{significant_above_threshold}.}
\item{\code{summary}}{A tibble of curve-level rate phenotypes:
\code{r_max}, \code{t_r_max}, \code{r_mean}, \code{r_mean_positive},
\code{t_growth_start}, \code{t_growth_end}, \code{growth_duration},
\code{cumulative_positive_growth}, and \code{n_growth_windows}.}
\item{\code{model}}{The underlying GAM model object.}
\item{\code{call}}{The matched call.}
\item{\code{settings}}{List of analysis settings used.}
\item{\code{vars}}{List of variable names identified in the model.}
\item{\code{family}}{Character string naming the GAM family.}
}
}
\description{
Estimates instantaneous rates of plant disease progress (\eqn{S'(t) = dS(t)/dt})
and associated uncertainty from epidemic trajectories fitted by
\code{\link{functional_curves}}. Derivatives are computed using the GAM linear
predictor matrix (\code{lpmatrix}) with finite differences, supporting both
the original response scale (e.g., severity proportion/percentage points per day)
and the link (linear predictor) scale. Also computes epidemic rate phenotypes
including maximum rate (\eqn{r_{max}}), time of maximum rate (\eqn{t_{r_{max}}}),
growth duration, cumulative positive growth, and distinct growth windows.
}
\details{
\strong{Mathematical Formulation:}
Let \eqn{S(t)} denote the fitted disease trajectory and \eqn{\eta(t) = X(t)\beta}
the linear predictor from the GAM model, with \eqn{\mu(t) = g^{-1}(\eta(t))}.

The derivative on the link scale is estimated by:
\deqn{D = \frac{X(t + \epsilon) - X(t - \epsilon)}{2\epsilon}}
\deqn{\eta'(t) = D \beta}
with variance \eqn{\operatorname{Var}(\eta'(t)) = \operatorname{diag}(D V_p D^T)},
where \eqn{V_p} is the Bayesian posterior covariance matrix of the GAM parameters.

On the response scale (\code{scale = "response"}), the derivative of the mean trajectory is:
\deqn{S'(t) = \frac{\mu(\eta(t + \epsilon)) - \mu(\eta(t - \epsilon))}{2\epsilon}}
The approximate gradient vector \eqn{G} with respect to \eqn{\beta} is:
\deqn{G = \frac{\mu'(\eta(t + \epsilon)) X(t + \epsilon) - \mu'(\eta(t - \epsilon)) X(t - \epsilon)}{2\epsilon}}
where \eqn{\mu'(\eta) = d\mu/d\eta} is given by \code{family$mu.eta()}.
The uncertainty is then:
\deqn{\operatorname{Var}(S'(t)) = \operatorname{diag}(G V_p G^T)}

\strong{Boundary Handling:}
At the observed boundaries \eqn{t_{min}} and \eqn{t_{max}}, one-sided forward and backward
differences are used to prevent unintended extrapolation beyond the observed domain.

\strong{Time-Invariant Terms and Random Effects:}
Intercepts, main treatment effects, and terms constant with respect to time cancel
analytically in \eqn{X(t + \epsilon) - X(t - \epsilon)}. Environmental and experimental
unit random effects are excluded by default to yield population-adjusted treatment rates,
consistent with \code{\link{functional_curves}}.

\strong{No-Epidemic and Flat Curves:}
Curves with all observed response values equal to zero (or estimated rates below
numerical tolerance \eqn{10^{-6}}) explicitly return \eqn{r_{max} = 0}, \eqn{t_{r_{max}} = \text{NA}},
\eqn{\text{growth\_duration} = 0}, and \eqn{\text{cumulative\_positive\_growth} = 0}.
}
\examples{
\dontrun{
set.seed(123)
df <- data.frame(
  time = rep(1:30, 3),
  treatment = rep(c("Early", "Late", "None"), each = 30),
  y = c(
    1 / (1 + exp(-(1:30 - 10) * 0.4)),  # Early
    1 / (1 + exp(-(1:30 - 20) * 0.4)),  # Late
    rep(0, 30)                          # None
  )
)

curves <- functional_curves(
  data = df, time = "time", response = "y", treatment = "treatment",
  min_points = 3, show_progress = FALSE, family_try = "quasibinomial"
)

rates <- functional_rate(curves)
print(rates)
summary(rates)
plot(rates)
}

}
\references{
Madden, L. V., Hughes, G., & van den Bosch, F. (2007).
The Study of Plant Disease Epidemics. APS Press, St. Paul, MN.

Wood, S. N. (2017). Generalized Additive Models: An Introduction with R (2nd ed.).
Chapman and Hall/CRC.

Simpson, G. L. (2018). Modelling Palaeoecological Time Series Using Generalized Additive Models.
Frontiers in Ecology and Evolution, 6, 149.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{diagnose_curves}
\alias{diagnose_curves}
\title{Diagnostic tools for functional epidemic curve models}
\usage{
diagnose_curves(x, grid_n = 200)
}
\arguments{
\item{x}{An object of class \code{"functional_curves"}.}

\item{grid_n}{Number of points used for the diagnostic smooth curve.}
}
\value{
An object of class \code{"r4pde_curve_diagnostics"} containing:
\itemize{
\item diagnostic data,
\item k-index checks,
\item concurvity estimates,
\item ggplot-based diagnostic panels.
}
}
\description{
Diagnostic tools for functional epidemic curve models fitted with \code{\link{functional_curves}()}.
Produces diagnostic plots without invoking base graphics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CompMuCens.R
\name{CompMuCens}
\alias{CompMuCens}
\title{Survival analysis for quantitative ordinal scale data}
\usage{
CompMuCens(dat, scale, grade = TRUE, ckData = FALSE)
}
\arguments{
\item{dat}{Data frame containing the data to be processed.}

\item{scale}{A numeric vector indicating the scale or order of classes.}

\item{grade}{Logical. If TRUE, uses the class value. If FALSE, uses the NPE (Non-Parametric Estimate).}

\item{ckData}{Logical. If TRUE, returns the input data along with the results.
If FALSE, returns only the results.}
}
\value{
A list containing the score statistic, hypothesis tests, adjusted significance level,
and conclusion based on pairwise comparisons.
}
\description{
Survival analysis for quantitative ordinal scale data
}
\details{
This function supports analysis of quantitative ordinal scale data via interval-censored methods.
The approach follows the workflow described by Chiang et al. (2023) and the reference implementation
provided in the CompMuCens repository.
}
\examples{
if (requireNamespace("interval", quietly = TRUE)) {
  trAs <- c(5,4,2,5,5,4,4,2,5,2,2,3,4,3,2,2,6,2,2,4,2,4,2,4,5,3,4,2,2,3)
  trBs <- c(5,3,2,4,4,5,4,5,4,4,6,4,5,5,5,2,6,2,3,5,2,6,4,3,2,5,3,5,4,5)
  trCs <- c(2,3,1,4,1,1,4,1,1,3,2,1,4,1,1,2,5,2,1,3,1,4,2,2,2,4,2,3,2,2)
  trDs <- c(5,5,4,5,5,6,6,4,6,4,3,5,5,6,4,6,5,6,5,4,5,5,5,3,5,6,5,5,5,6)
  inputData <- data.frame(
    treatment = c(rep("A",30), rep("B",30), rep("C",30), rep("D",30)),
    x = c(trAs, trBs, trCs, trDs)
  )
  CompMuCens(dat = inputData,
             scale = c(0,3,6,12,25,50,75,88,94,97,100,100),
             ckData = TRUE)
}

}
\references{
Chiang, K.S., Chang, Y.M., Liu, H.I., Lee, J.Y., El Jarroudi, M. and Bock, C. (2023).
Survival Analysis as a Basis to Test Hypotheses When Using Quantitative Ordinal Scale Disease Severity Data.
Phytopathology. \url{https://apsjournals.apsnet.org/doi/abs/10.1094/PHYTO-02-23-0055-R}
}
\seealso{
Other Disease quantification: 
\code{\link{DSI}()},
\code{\link{DSI2}()}
}
\concept{Disease quantification}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gradients.R
\name{fit_gradients}
\alias{fit_gradients}
\title{Fit Gradient Models to Data}
\usage{
fit_gradients(data, C = 1)
}
\arguments{
\item{data}{A dataframe containing the data, with columns "x" representing distances and
"Y" representing the corresponding measurements or counts.}

\item{C}{A constant to be used in the modified power model. Defaults to 1.}
}
\value{
A list containing:
\item{data}{The input data, which will include an additional column 'mod_x'.}
\item{results_table}{A table of the model parameters and R-squared values.}
\item{plot_exponential}{Diagnostic plot for the exponential model.}
\item{plot_power}{Diagnostic plot for the power model.}
\item{plot_modified_power}{Diagnostic plot for the modified power model.}
\item{plot_exponential_original}{Plot of the original data with the exponential model fit.}
\item{plot_power_original}{Plot of the original data with the power model fit.}
\item{plot_modified_power_original}{Plot of the original data with the modified power model fit.}
}
\description{
This function fits three gradient models (exponential, power, and modified power) to given data.
It then ranks the models based on their R-squared values and returns diagnostic plots for each model.
}
\examples{
x <- c(0.8, 1.6, 2.4, 3.2, 4, 7.2, 12, 15.2, 21.6, 28.8)
Y <- c(184.9, 113.3, 113.3, 64.1, 25, 8, 4.3, 2.5, 1, 0.8)
grad1 <- data.frame(x = x, Y = Y)
library(ggplot2)
mg <- fit_gradients(grad1, C = 0.4)
mg$plot_power_original +
  labs(title = "", x = "Distance from focus (m)", y = "Count of lesions")

}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{FHBWheat}
\alias{FHBWheat}
\title{FHBWheat dataset}
\format{
A data frame with:
\describe{
\item{field}{Field identifier}
\item{i}{Row position}
\item{n}{Column position}
\item{quadrat}{Quadrat ID}
\item{season}{Crop season}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
FHBWheat
}
\description{
Fusarium head blight quadrat assessments in wheat.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_dsp.R
\name{functional_contrast}
\alias{functional_contrast}
\title{Functional Contrast (Disease Suppression Profiles)}
\usage{
functional_contrast(
  data,
  time = "time",
  response = "response",
  treatment = "treatment",
  reference = "Control",
  group = NULL,
  smooth = FALSE,
  grid = NULL,
  contrast = c("difference", "relative"),
  keep_reference = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame containing disease progress observations, or an object of class \code{"functional_curves"}.}

\item{time}{Character string naming the time variable.}

\item{response}{Character string naming the response variable.}

\item{treatment}{Character string naming the treatment variable.}

\item{reference}{Character string specifying the reference treatment level. Default is \code{"Control"}.}

\item{group}{Optional character vector of grouping variables (e.g., \code{c("experiment", "location")}).}

\item{smooth}{Logical; if \code{TRUE}, the function could apply smoothing (currently placeholder).}

\item{grid}{Optional numeric vector of common time points for interpolation. If provided, curves are interpolated.}

\item{contrast}{Character specifying the contrast type. Currently \code{"difference"} is implemented.}

\item{keep_reference}{Logical; if \code{TRUE}, includes the reference treatment in the output (with DSP = 0). Default is \code{FALSE}.}

\item{...}{Additional arguments.}
}
\value{
An object of class \code{"functional_dsp"} (inheriting from \code{"tbl_df"}), containing:
\itemize{
\item grouping variables (if any)
\item treatment variable
\item time variable
\item \code{y_reference}: disease intensity of the reference
\item \code{y_treatment}: disease intensity of the treatment
\item \code{DSP}: the computed contrast
\item \code{contrast_type}: type of contrast used
\item \code{reference}: name of the reference treatment
}
}
\description{
Calculates functional contrast curves (Disease Suppression Profiles - DSP) between a
reference treatment (e.g., untreated control) and other fungicide treatments over time.
The DSP is defined by default as: \deqn{DSP(t) = y_{control}(t) - y_{treatment}(t)}
where \eqn{y(t)} is the disease intensity at time \eqn{t}. The goal is to evaluate
the intensity, timing, and persistence of the fungicide protection.
}
\seealso{
\code{\link{functional_summary}}, \code{\link{plot_dsp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functional_suppression_profiles.R
\name{plot.functional_suppression_profiles}
\alias{plot.functional_suppression_profiles}
\title{Plot functional suppression profiles}
\usage{
\method{plot}{functional_suppression_profiles}(
  x,
  type = c("dendrogram", "profiles", "heatmap", "rank", "all"),
  show_cut = TRUE,
  show_points = FALSE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"functional_suppression_profiles"}.}

\item{type}{Character string specifying the plot type. One of \code{"dendrogram"}, \code{"profiles"}, \code{"heatmap"}, \code{"rank"}, or \code{"all"}.}

\item{show_cut}{Logical; whether to display the cluster cut height in dendrogram. Default is \code{TRUE}.}

\item{show_points}{Logical; whether to overlay observed DSP values as points in the profiles plot. Default is \code{FALSE}.}

\item{...}{Additional arguments passed to specific plot functions.}
}
\value{
A \code{ggplot} object or a list of \code{ggplot} objects.
}
\description{
Plot functional suppression profiles
}
\details{
The plot method visualizes the components of a Functional Suppression Profile.
The dendrogram defines the Functional Suppression Profiles based on temporal similarity.
The heatmap is ordered by the dendrogram to help interpret these functional groups
using summary metrics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_dsp.R
\name{plot_dsp_rank_heatmap}
\alias{plot_dsp_rank_heatmap}
\title{Plot DSP Rankings Heatmap}
\usage{
plot_dsp_rank_heatmap(ranks_data, treatment = "treatment", group = NULL)
}
\arguments{
\item{ranks_data}{Tibble output from \code{\link{rank_dsp}}.}

\item{treatment}{Character string naming the treatment column (for the y-axis).}

\item{group}{Optional character string for faceting by group.}
}
\value{
A \code{ggplot} object.
}
\description{
Creates a heatmap visualization of treatment rankings based on DSP descriptors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oruns_test_byrowcol.R
\name{oruns_test_byrowcol}
\alias{oruns_test_byrowcol}
\title{Runs Test for Each Row and Column of a Binary Matrix}
\usage{
oruns_test_byrowcol(mat)
}
\arguments{
\item{mat}{A binary matrix (containing 0s and 1s, and possibly NAs).}
}
\value{
A list with four elements:
\item{row_results}{Data frame with test results for each row.}
\item{col_results}{Data frame with test results for each column.}
\item{row_summary}{Percentage summary of interpretation for rows.}
\item{col_summary}{Percentage summary of interpretation for columns.}
}
\description{
Applies the ordinary runs test to each row and column of a binary matrix individually.
}
\seealso{
\code{\link{oruns_test}}

Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_curves.R
\name{plot_curves}
\alias{plot_curves}
\alias{plot_curves.functional_distances}
\alias{plot_curves.functional_dsp}
\title{Plot environment-adjusted epidemic curves by cluster}
\usage{
plot_curves(x, ...)

\method{plot_curves}{functional_distances}(
  x,
  label_fun = NULL,
  palette = NULL,
  alpha = 0.9,
  linewidth = 1.1,
  ...
)

\method{plot_curves}{functional_dsp}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_distances"}.}

\item{...}{Additional arguments.}

\item{label_fun}{Optional function to modify treatment labels.}

\item{palette}{Optional named vector of colors for clusters.}

\item{alpha}{Line transparency.}

\item{linewidth}{Line width.}
}
\value{
A \code{ggplot} object.
}
\description{
Plots environment-adjusted mean epidemic curves for each treatment,
colored according to functional cluster membership.

The returned object is a \code{ggplot} and can be further modified
using standard ggplot2 layers.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{FusariumBanana}
\alias{FusariumBanana}
\title{FusariumBanana dataset}
\format{
A data frame with:
\describe{
\item{field}{Field ID}
\item{lat}{Latitude}
\item{lon}{Longitude}
\item{marker}{Infection marker presence}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
FusariumBanana
}
\description{
Observations of Fusarium symptoms in banana fields.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{BudBlightSoybean}
\alias{BudBlightSoybean}
\title{BudBlightSoybean dataset}
\format{
A data frame with the following columns:
\describe{
\item{block}{Block number}
\item{time}{Time point of assessment}
\item{treat}{Treatment name}
\item{y}{Incidence or severity value}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
BudBlightSoybean
}
\description{
Soybean bud blight incidence in experimental blocks.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_curves.R
\name{plot.functional_curves}
\alias{plot.functional_curves}
\title{Plot functional_curves}
\usage{
\method{plot}{functional_curves}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_curves"}.}

\item{...}{Additional arguments.}
}
\description{
Plot functional_curves
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_resistance.R
\name{functional_resistance}
\alias{functional_resistance}
\title{Compute functional resistance and stability-adjusted functional resistance}
\usage{
functional_resistance(
  object,
  instability = NULL,
  reference,
  lambda = 1,
  method = c("positive_area", "l2_difference"),
  scale_nfi = FALSE,
  n_groups = 4,
  group_method = c("none", "quantile", "bootstrap", "clustering"),
  time_var = NULL,
  genotype_var = NULL,
  n_boot = 1000,
  ci_level = 0.95,
  clustering_method = c("hclust", "kmeans"),
  adjust_by = NULL,
  reference_within_group = FALSE,
  group_within_adjust_by = TRUE,
  ...
)
}
\arguments{
\item{object}{An object of class \code{functional_curves}.}

\item{instability}{Optional output from \code{functional_instability()}.}

\item{reference}{Character string naming the reference genotype, or a named vector if \code{reference_within_group = TRUE}.}

\item{lambda}{Numeric penalty weight for instability. Default is 1.}

\item{method}{Character string for distance method.}

\item{scale_nfi}{Logical; if \code{TRUE}, normalizes nFI to a 0-1 scale before applying penalty.}

\item{n_groups}{Integer number of classes to group into.}

\item{group_method}{Character string for the grouping method.}

\item{time_var}{Optional variable name overrides.}

\item{genotype_var}{Optional variable name overrides.}

\item{n_boot}{Integer number of bootstrap iterations.}

\item{ci_level}{Numeric confidence level for bootstrap intervals.}

\item{clustering_method}{Character string for clustering method if \code{group_method = "clustering"}.}

\item{adjust_by}{Optional character string naming a genotype-level covariate to stratify by.}

\item{reference_within_group}{Logical; if \code{TRUE}, the reference is evaluated within each \code{adjust_by} group.}

\item{group_within_adjust_by}{Logical; if \code{TRUE}, resistance classes are assigned within each \code{adjust_by} group.}

\item{...}{Additional arguments.}
}
\value{
A list with a \code{table} containing genotype, FRI, nFI, SAFRI, rank, and classes,
and optionally bootstrap results.
}
\description{
Computes a functional resistance index (FRI) relative to a reference genotype curve.
Optionally adjusts FRI by a functional instability penalty to calculate a
stability-adjusted functional resistance index (SAFRI). Supports grouping
genotypes into classes based on quantiles, clustering, or bootstrap-supported differences.
}
\examples{
\dontrun{
fr <- functional_resistance(
  fc,
  reference = "Susceptible",
  group_method = "bootstrap"
)
print(fr)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{functional_pca_clusters}
\alias{functional_pca_clusters}
\title{Cluster curves based on FPCA scores}
\usage{
functional_pca_clusters(
  x,
  k = NULL,
  components = NULL,
  method = c("kmeans", "hclust"),
  choose_k = c("none", "silhouette", "elbow"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{"r4pde_functional_pca"}.}

\item{k}{Number of clusters.}

\item{components}{Integer vector of components to use.}

\item{method}{Clustering method: "kmeans" or "hclust".}

\item{choose_k}{Method for suggesting k: "none", "silhouette", or "elbow".}

\item{...}{Additional arguments passed to clustering functions.}
}
\description{
Cluster curves based on FPCA scores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suggest_k.R
\name{suggest_k}
\alias{suggest_k}
\title{Suggest GAM smoothing parameters for epidemic curve models}
\usage{
suggest_k(
  data = NULL,
  time = NULL,
  treatment = NULL,
  environment = NULL,
  n_time = NULL,
  n_env = NULL,
  rule = c("minimum", "median"),
  smoothness = c("conservative", "moderate", "flexible")
)
}
\arguments{
\item{data}{Optional data frame.  If supplied, \code{time}, \code{treatment},
and optionally \code{environment} are used to compute summaries of the
number of unique time points and environments per treatment.}

\item{time}{Unquoted column name for the time variable, or a character
string naming the column.}

\item{treatment}{Unquoted column name for the treatment / cultivar variable,
or a character string naming the column.}

\item{environment}{Optional unquoted column name for the environment
variable, or a character string naming the column.}

\item{n_time}{Integer.  Number of unique time points to use directly
(ignored when \code{data} is supplied).}

\item{n_env}{Integer.  Number of unique environments to use directly
(ignored when \code{data} is supplied, or when there is no environment
variable).}

\item{rule}{Character string. How to summarise the distribution of
replication counts across treatment-by-environment combinations:
\code{"minimum"} (default, conservative) or \code{"median"}.}

\item{smoothness}{Character string controlling how liberal the
recommendations are: \code{"conservative"} (default), \code{"moderate"},
or \code{"flexible"}.}
}
\value{
A named list with the following elements:
\describe{
\item{\code{time_summary}}{Named numeric vector with minimum, median, and
maximum unique time points per treatment-by-environment combination.}
\item{\code{environment_summary}}{Named numeric vector with minimum, median,
and maximum unique environments per treatment (or \code{NULL} when no
environment variable is given).}
\item{\code{effective_n_time}}{The effective number of time points chosen
by \code{rule}.}
\item{\code{effective_n_env}}{The effective number of environments chosen
by \code{rule}, or \code{NULL}.}
\item{\code{k_smooth}}{Recommended basis dimension for the global time smooth.}
\item{\code{k_trt}}{Recommended basis dimension for the treatment-specific smooth.}
\item{\code{k_env}}{Recommended basis dimension for the environment random effect.}
\item{\code{gamma}}{Recommended penalisation multiplier.}
\item{\code{message}}{A short interpretation message.}
}
}
\description{
A helper function to guide selection of the \code{k_smooth}, \code{k_trt},
\code{k_env}, and \code{gamma} parameters used by \code{\link{functional_curves}}.
Can infer the effective replication from a data frame or accept scalar values
directly when the data are not yet available.

For sparse epidemic data it is strongly recommended to use
\code{rule = "minimum"} so that k values are based on the least-replicated
treatment-by-environment combination, guarding against over-fitting.
}
\details{
The function computes, for every treatment-by-environment combination, the
number of unique time points at which observations are available.  It then
summarises these counts using the chosen \code{rule} to obtain
\code{effective_n_time}.  Similarly it computes, per treatment, the number
of unique environments, and summarises using the same \code{rule} to obtain
\code{effective_n_env}.

Recommended k values follow these heuristics:
\itemize{
\item \code{k_smooth}: global smooth over time; capped below
\code{effective_n_time - 1}.
\item \code{k_trt}: treatment-specific smooth; capped below
\code{k_smooth}.
\item \code{k_env}: environment random effect; capped below
\code{effective_n_env}.
\item \code{gamma}: penalty multiplier; higher values encourage
smoother fits.
}
}
\examples{
# Using explicit values
suggest_k(n_time = 5, n_env = 8)

# Inferring from a data frame (unquoted column names)
df <- data.frame(
  time      = rep(1:6, times = 6),
  cultivar  = rep(c("A", "B", "C"), each = 12),
  env       = rep(rep(c("E1", "E2"), each = 6), times = 3),
  severity  = runif(36, 0, 0.5)
)
suggest_k(
  data        = df,
  time        = time,
  treatment   = cultivar,
  environment = env,
  rule        = "minimum",
  smoothness  = "conservative"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_subareas.R
\name{count_subareas}
\alias{count_subareas}
\title{Count the Number of Ones in Subareas of a Matrix}
\usage{
count_subareas(matrix_data, sub_rows, sub_cols)
}
\arguments{
\item{matrix_data}{A matrix of 0s and 1s to analyze.}

\item{sub_rows}{Number of rows in each subarea.}

\item{sub_cols}{Number of columns in each subarea.}
}
\value{
A matrix where each cell corresponds to a subarea and contains the count of ones.
}
\description{
This function takes a binary matrix (0s and 1s) and divides it into rectangular subareas,
counting the number of ones in each. Subareas are defined by the number of rows and columns
specified by the user. If the matrix dimensions are not perfectly divisible by the subarea size,
edge subareas may be smaller.
}
\examples{
set.seed(123)
mat <- matrix(sample(c(0, 1), 12 * 16, replace = TRUE), nrow = 16, ncol = 12)
count_matrix <- count_subareas(mat, sub_rows = 3, sub_cols = 3)
print(count_matrix)

}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{reconstruct_curves}
\alias{reconstruct_curves}
\title{Reconstruct curves using specified FPCA components}
\usage{
reconstruct_curves(x, components = NULL)
}
\arguments{
\item{x}{An object of class \code{"r4pde_functional_pca"}.}

\item{components}{Integer vector of components to use for reconstruction. If NULL, uses all retained components.}
}
\description{
Reconstruct curves using specified FPCA components
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functional_instability.R
\name{plot_functional_instability}
\alias{plot_functional_instability}
\title{Plot method for functional instability results}
\usage{
plot_functional_instability(object, type = c("overall", "space", "time"), ...)
}
\arguments{
\item{object}{Output from \code{\link[=functional_instability]{functional_instability()}}.}

\item{type}{One of \code{"overall"}, \code{"space"}, or \code{"time"}.}

\item{...}{Further arguments passed to methods.}
}
\value{
A ggplot object.
}
\description{
Plot method for functional instability results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_curves.R
\name{functional_curves}
\alias{functional_curves}
\title{Fit genotype-specific epidemic trajectories using GAM}
\usage{
functional_curves(
  data,
  time,
  response,
  treatment,
  environment = NULL,
  block = NULL,
  unit = NULL,
  response_scale = c("proportion", "percent"),
  eps = 1e-04,
  min_points = 5,
  grid_n = 140,
  env_ref = NULL,
  k_smooth = 10,
  k_env = 4,
  k_trt = 6,
  gamma = 1.4,
  discrete = TRUE,
  family_try = c("betar", "quasibinomial"),
  show_progress = TRUE,
  covariates = NULL,
  include_covariates = FALSE,
  covariate_smooths = FALSE,
  global_smooth = FALSE,
  ...
)
}
\arguments{
\item{data}{A data.frame containing disease progress observations.}

\item{time}{Character string naming the time variable.}

\item{response}{Character string naming the response variable.}

\item{treatment}{Character string naming the treatment/cultivar factor.}

\item{environment}{Optional character string naming an environment factor.}

\item{block}{Optional character string naming a blocking factor.}

\item{unit}{Optional character string naming a unique experimental unit identifier.}

\item{response_scale}{Character string specifying the response scale: \code{"proportion"} or \code{"percent"}.}

\item{eps}{Numeric small constant retained for API compatibility.}

\item{min_points}{Minimum number of observations required per curve.}

\item{grid_n}{Number of time points for evaluating predicted curves.}

\item{env_ref}{Reference environment level used for adjusted predictions.}

\item{k_smooth}{Basis dimension for the global smooth of time.}

\item{k_env}{Basis dimension for the environment-specific smooth.}

\item{k_trt}{Basis dimension for the treatment-specific smooth.}

\item{gamma}{Penalization parameter passed to \code{mgcv::bam()}.}

\item{discrete}{Logical; whether to use discrete (approximate) fitting.}

\item{family_try}{Character string specifying the GAM family to try.}

\item{show_progress}{Logical; whether to show progress.}

\item{covariates}{Optional character vector of genotype-level covariates (e.g., \code{c("heading_group")}).}

\item{include_covariates}{Logical; whether to include covariates as fixed effects in the model.}

\item{covariate_smooths}{Logical; if TRUE and \code{include_covariates = TRUE}, adds smooth interactions \code{s(time, by=covariate)} for factor covariates.}

\item{global_smooth}{Logical; if \code{TRUE}, fits a shared global curve plus treatment deviations. If \code{FALSE} (default), fits independent smooths per treatment.}

\item{...}{Additional arguments.}
}
\value{
An object of class \code{"functional_curves"} containing:
\itemize{
\item \code{gam}: fitted GAM object and \code{family_used};
\item \code{curves}: environment-adjusted treatment mean curves on the common grid;
\item \code{grid}: the time grid used;
\item \code{observed_data}: the processed input data;
\item \code{genotype_info}: a tibble with one row per genotype containing covariates;
\item \code{vars}: variable names used;
\item \code{settings}: model settings;
\item \code{warnings_betar}: any warnings caught during beta fitting;
\item \code{plot_mean}: ggplot object of the mean curves.
}
}
\description{
Fits a generalized additive model (GAM) to replicated disease progress curves,
returning adjusted treatment mean curves evaluated on a common time grid.
}
\details{
Genotype-level covariates are descriptors that do not vary within a genotype, such as phenological groups.
For example, in wheat blast studies, a cultivar's \code{heading_group} (early, intermediate, late)
can be supplied. This helps distinguish between true genetic resistance and phenological escape, as
cultivars with different heading dates may encounter different infection-risk windows in the same
environment. When \code{include_covariates = TRUE}, the model accounts for these covariates,
yielding heading-adjusted functional resistance.
}
\examples{
\dontrun{
fc <- functional_curves(
  data = my_data,
  time = "time_var",
  response = "severity",
  treatment = "cultivar",
  environment = "env",
  covariates = c("heading_group"),
  include_covariates = TRUE,
  covariate_smooths = TRUE
)
plot(fc)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{get_fpca_eigenfunctions}
\alias{get_fpca_eigenfunctions}
\title{Get FPCA eigenfunctions}
\usage{
get_fpca_eigenfunctions(x, components = NULL)
}
\arguments{
\item{x}{An object of class \code{"r4pde_functional_pca"}.}

\item{components}{Optional integer vector of components to return.}
}
\description{
Get FPCA eigenfunctions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{plot.functional_rate}
\alias{plot.functional_rate}
\title{Plot functional_rate curves and rates}
\usage{
\method{plot}{functional_rate}(
  x,
  which = NULL,
  type = c("both", "curve", "rate"),
  show_ci = TRUE,
  show_threshold = TRUE,
  ncol = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"functional_rate"}.}

\item{which}{Optional character vector specifying which treatments/groups to display.}

\item{type}{Character specifying what to plot: \code{"both"} (default; fitted curve
in top panel and derivative in bottom panel), \code{"curve"} (only trajectory),
or \code{"rate"} (only instantaneous rate).}

\item{show_ci}{Logical; whether to show confidence ribbons. Default is \code{TRUE}.}

\item{show_threshold}{Logical; whether to show threshold and zero horizontal lines.
Default is \code{TRUE}.}

\item{ncol}{Optional integer specifying number of facet columns.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} or combined \code{patchwork} object.
}
\description{
Plot functional_rate curves and rates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DSI2.R
\name{DSI2}
\alias{DSI2}
\title{Calculate the Disease severity Index (DSI) (frequency of each class)}
\usage{
DSI2(class, freq, max)
}
\arguments{
\item{class}{A numeric vector representing the classes.}

\item{freq}{A numeric vector representing the frequency of each class.
Must be the same length as 'class'.}

\item{max}{A numeric value representing the maximum possible class value.}
}
\value{
Returns a single numeric value representing the DSI.
}
\description{
This function calculates the Disease Severity Index (DSI) given a vector of classes,
a vector of frequencies, and a maximum possible class value. The DSI is calculated
as a weighted sum of class values, where each class is multiplied by its corresponding
frequency, then divided by the product of the total frequency and maximum class value,
and finally multiplied by 100 to get a percentage.
}
\examples{
DSI2(c(0, 1, 2, 3, 4), c(2, 0, 5, 0, 5), 4)

}
\seealso{
Other Disease quantification: 
\code{\link{CompMuCens}()},
\code{\link{DSI}()}
}
\concept{Disease quantification}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AFSD.R
\name{AFSD}
\alias{AFSD}
\title{Analysis of foci structure and dynamics (AFSD)}
\usage{
AFSD(df)
}
\arguments{
\item{df}{A dataframe containing at least three columns: 'x', 'y', and 'i'. 'x' and 'y' represent spatial coordinates
and 'i' is a disease indicator (1 if diseased, otherwise 0).}
}
\value{
A list containing:
cluster_summary2: a dataframe summarizing the number and size of foci, and proportions of diseased plants.
cluster_df: a dataframe containing foci information, including size and number of rows and columns in each foci.
df_clustered: the original dataframe with an added 'focus_id' column, showing which foci each row belongs to.
}
\description{
This function performs the analysis of a simple method introduced by Nelson (1996) and expanded by Laranjeira et al. (1998).
The function assumes
the dataframe supplied as input has columns 'x', 'y', and 'i', where 'x' and 'y' are spatial coordinates
and 'i' is a disease indicator variable (1 if diseased, otherwise 0). The function performs several steps
including filtering rows where 'i' is 1, converting to an adjacency matrix, and creating foci using igraph.
It then calculates various statistics about the foci and returns these in a list.
}
\examples{
# Generate a sample dataframe
set.seed(123)
df <- data.frame(x = sample(1:100, 500, replace = TRUE),
                 y = sample(1:100, 500, replace = TRUE),
                 i = sample(0:1, 500, replace = TRUE, prob = c(0.7, 0.3)))

# Perform the AFSD
result <- AFSD(df)

}
\seealso{
Other Spatial analysis: 
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_dsp.R
\name{rank_dsp}
\alias{rank_dsp}
\title{Rank Treatments based on Functional DSP Descriptors}
\usage{
rank_dsp(
  summary_data,
  metrics = c("protected_area", "max_suppression", "persistence", "centroid"),
  higher_is_better = TRUE,
  group = NULL,
  ties.method = "average"
)
}
\arguments{
\item{summary_data}{The tibble output from \code{\link{functional_summary}}.}

\item{metrics}{Character vector of metrics to rank.}

\item{higher_is_better}{Logical vector or named logical vector indicating if higher values are better for each metric. If a single logical, applies to all.}

\item{group}{Optional character vector of grouping variables.}

\item{ties.method}{Character string specifying how to handle ties. Default is \code{"average"}.}
}
\value{
A tibble with ranks for each specified metric, and an \code{average_rank}.
}
\description{
Generates rankings of treatments based on extracted DSP functional descriptors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{WhiteMoldSoybean}
\alias{WhiteMoldSoybean}
\title{WhiteMoldSoybean dataset}
\format{
A data frame with:
\describe{
\item{country}{Country name}
\item{elevation}{Field elevation}
\item{elevation_class}{Elevation class}
\item{harvest_year}{Year of harvest}
\item{inc}{Incidence}
\item{inc_check}{Check plot incidence}
\item{inc_class}{Incidence class}
\item{location}{Location name}
\item{region}{Geographical region}
\item{scl}{Soybean canopy layer}
\item{season}{Crop season}
\item{state}{State name}
\item{study}{Study identifier}
\item{treat}{Treatment applied}
\item{yld}{Yield}
\item{yld_check}{Yield of untreated check}
\item{yld_class}{Yield class}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
WhiteMoldSoybean
}
\description{
National dataset of white mold severity and yield.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_functional_suppression_profiles.R
\name{print.summary.functional_suppression_profiles}
\alias{print.summary.functional_suppression_profiles}
\title{Print summary of functional suppression profiles}
\usage{
\method{print}{summary.functional_suppression_profiles}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.functional_suppression_profiles"}.}

\item{...}{Additional arguments.}
}
\description{
Print summary of functional suppression profiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{print.functional_rate}
\alias{print.functional_rate}
\title{Print a functional_rate object}
\usage{
\method{print}{functional_rate}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_rate"}.}

\item{...}{Additional arguments.}
}
\value{
The object \code{x}, invisibly.
}
\description{
Print a functional_rate object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_subareas_random.R
\name{count_subareas_random}
\alias{count_subareas_random}
\title{Random Subgrid Sampling of a Binary Matrix}
\usage{
count_subareas_random(matrix_data, sub_rows = 3, sub_cols = 3, n_samples = 100)
}
\arguments{
\item{matrix_data}{A binary matrix of 0s and 1s.}

\item{sub_rows}{Number of rows in each subgrid sample.}

\item{sub_cols}{Number of columns in each subgrid sample.}

\item{n_samples}{Number of subgrid samples to draw.}
}
\value{
A list of sampled subgrids. Each element is a list with:
\item{position}{Row and column start position of the sample.}
\item{submatrix}{The sampled subgrid matrix.}
\item{count}{Number of 1s in the sampled submatrix.}
}
\description{
Randomly samples submatrices (quadrats) of specified size from a binary matrix,
and returns the positions, submatrices, and count of 1s in each sampled quadrat.
}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_AFSD.R
\name{plot_AFSD}
\alias{plot_AFSD}
\title{Plot ASFD}
\usage{
plot_AFSD(df)
}
\arguments{
\item{df}{A dataframe containing at least three columns: 'x', 'y', and 'cluster_id'.
'x' and 'y' are spatial coordinates and 'cluster_id' is the cluster identifier
to which each cell belongs.}
}
\value{
A ggplot object with the scatter plot of foci (clusters).
}
\description{
This function creates a tile plot of the foci (cluster) identified by the AFSD function.
It colors each cell in a foci and labels the centroid of each cluster with the foci ID.
The 'ggplot2' package is used for the plot, and will be automatically installed if not already present.
}
\examples{
df <- data.frame(x = sample(1:100, 500, replace = TRUE),
                 y = sample(1:100, 500, replace = TRUE),
                 i = sample(0:1, 500, replace = TRUE, prob = c(0.7, 0.3)))

# Perform the AFSD
result <- AFSD(df)
# Plot the foci
plot_AFSD(result[[3]])

}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{join_count}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_dsp.R
\name{simulate_dsp_data}
\alias{simulate_dsp_data}
\title{#' Simulate Disease Suppression Profile Data}
\usage{
simulate_dsp_data()
}
\value{
A \code{tibble} with columns \code{treatment}, \code{rep}, \code{time}, \code{y_control}, \code{dsp_true}, \code{severity_true}, and \code{severity}.
}
\description{
Simulates a dataset for demonstrating functional analysis with Disease Suppression Profiles.
Returns a data frame with varying treatments and replicates over multiple time points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_brdwgd.R
\name{get_brdwgd}
\alias{get_brdwgd}
\title{Fetch BR-DWGD Data for Multiple Locations}
\usage{
get_brdwgd(
  data,
  days_around,
  date_col,
  study_col = "study",
  path = "netcdf_files",
  vars = c("pr", "Tmax", "Tmin", "Rs", "RH"),
  direction = "both"
)
}
\arguments{
\item{data}{A data frame containing the input data, including columns for \code{latitude}, \code{longitude},
and the date column specified by \code{date_col}.}

\item{days_around}{An integer specifying the number of days before and after the date in the date column
to download data.}

\item{date_col}{A character string specifying the name of the date column in the data frame.}

\item{study_col}{A character string specifying the name of the column containing the study identifier
in the input data frame (default: "study").}

\item{path}{A character string specifying the directory where the BR-DWGD NetCDF files are stored.}

\item{vars}{A character vector specifying the weather variables to fetch.
These are expected to be the prefix of the NetCDF files (e.g., "pr", "Tmax").
Default: \code{c("pr", "Tmax", "Tmin", "Rs", "RH")}.}

\item{direction}{Character string specifying the direction of the date range relative to the reference date.
Options are "both" (default), "back", or "forth".
"back" retrieves data from \code{date - days_around} to \code{date}.
"forth" retrieves data from \code{date} to \code{date + days_around}.
"both" retrieves data from \code{date - days_around} to \code{date + days_around}.}
}
\value{
A data frame (tibble) with daily weather data. Columns are named to match
\code{nasapower} conventions where possible: \code{date}, \code{PRECTOTCORR} (from \code{pr}),
\code{T2M_MAX} (from \code{Tmax}), \code{T2M_MIN} (from \code{Tmin}), \code{ALLSKY_SFC_SW_DWN} (from \code{Rs}),
\code{RH2M} (from \code{RH}), \code{longitude}, \code{latitude}, and \code{study}.
}
\description{
This function extracts daily weather data from the Brazilian Daily Weather Gridded Data (BR-DWGD)
NetCDF files for specified locations and dates. It mimics the functionality of \code{get_nasapower}
and \code{get_era5} by returning a daily-aggregated data frame with columns named according
to \code{nasapower} conventions.
}
\details{
The function requires the \code{terra} package to read NetCDF files and the \code{tidyr} package
for data reshaping. It expects NetCDF files to be named starting with the variable name
(e.g., \code{pr_*.nc}) and to contain the variable with the same name.
}
\seealso{
Other Disease modeling: 
\code{\link{get_era5}()},
\code{\link{get_nasapower}()},
\code{\link{windowpane}()}
}
\concept{Disease modeling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{RustSoybean}
\alias{RustSoybean}
\title{RustSoybean dataset}
\format{
A data frame with:
\describe{
\item{detection}{Detection score or date}
\item{epidemia}{Epidemic phase}
\item{latitude}{Latitude}
\item{local}{Location name}
\item{longitude}{Longitude}
\item{planting}{Planting date or stage}
\item{severity}{Disease severity}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
RustSoybean
}
\description{
Soybean rust severity and field metadata.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-doc.R
\docType{data}
\name{FHBWheatBrazil}
\alias{FHBWheatBrazil}
\title{FHBWheatBrazil dataset}
\format{
A data frame with the following columns:
\describe{
\item{study}{Study ID or code}
\item{year}{Crop year}
\item{location}{Experimental site name}
\item{state}{Brazilian state}
\item{lat}{Latitude coordinate}
\item{lon}{Longitude coordinate}
\item{cultivar}{Wheat cultivar}
\item{planting_date}{Planting date}
\item{flowering}{Flowering date}
\item{index}{FHB index}
}
}
\source{
Del Ponte Lab internal data
}
\usage{
FHBWheatBrazil
}
\description{
Fusarium head blight (FHB) index in wheat in Brazil.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{summary.functional_rate}
\alias{summary.functional_rate}
\title{Summarize a functional_rate object}
\usage{
\method{summary}{functional_rate}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"functional_rate"}.}

\item{...}{Additional arguments.}
}
\value{
A tibble of curve-level rate phenotypes.
}
\description{
Summarize a functional_rate object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_rate.R
\name{augment}
\alias{augment}
\title{Augment generic for functional objects}
\usage{
augment(x, ...)
}
\arguments{
\item{x}{An object to augment.}

\item{...}{Additional arguments.}
}
\value{
An augmented data frame or tibble.
}
\description{
Augment generic for functional objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_distances.R
\name{functional_distances}
\alias{functional_distances}
\title{Compute pairwise functional distances and cluster curves}
\usage{
functional_distances(
  object,
  cluster_k = 4,
  hc_method = "ward.D2",
  test_factor = NULL,
  n_perm = 999,
  test_mode = c("auto", "none", "global", "global_pairwise"),
  min_strata_for_pairwise = 6,
  perm_unit = NULL,
  perm_strata = NULL,
  bootstrap = FALSE,
  boot_B = 399,
  boot_seed = 1,
  boot_ci = c(0.025, 0.975),
  show_progress = TRUE,
  curve_level = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{functional_curves}.}

\item{cluster_k}{Number of clusters used to cut the hierarchical tree.}

\item{hc_method}{Clustering linkage method passed to \code{stats::hclust()}.}

\item{test_factor}{Optional a priori grouping used for a distance-based permutation test.}

\item{n_perm}{Number of permutations for the distance-based test.}

\item{test_mode}{Character; permutation test mode.}

\item{min_strata_for_pairwise}{Minimum number of strata levels required for pairwise tests.}

\item{perm_unit}{Optional character naming the unit at which group labels are permuted.}

\item{perm_strata}{Optional character naming a factor defining restricted permutation strata.}

\item{bootstrap}{Logical; if \code{TRUE}, compute bootstrap envelopes and distance summaries.}

\item{boot_B}{Integer number of bootstrap replicates.}

\item{boot_seed}{Integer seed for bootstrap resampling.}

\item{boot_ci}{Length-2 numeric vector of quantiles for bootstrap intervals.}

\item{show_progress}{Logical; whether to show progress.}

\item{curve_level}{Logical; whether to compute curve-level distance matrix.}

\item{...}{Additional arguments.}
}
\value{
An object of class \code{"functional_distances"} containing:
\itemize{
\item \code{distance}: treatment-by-treatment functional distance matrix;
\item \code{distance_table}: pairwise distances as a data frame;
\item \code{hc}: \code{hclust} object;
\item \code{clusters}: treatment cluster assignments;
\item \code{curve_distance}: curve-by-curve distance matrix;
\item \code{test}: permutation test results;
\item \code{bootstrap}: bootstrap results if requested.
}
}
\description{
Computes L2 functional distances among treatment mean trajectories evaluated on a
common time grid from a \code{functional_curves} object. Also computes curve-level
distances, performs hierarchical clustering, and optionally performs permutation testing.
}
\examples{
\dontrun{
fd <- functional_distances(fc, cluster_k = 4)
print(fd)
plot_dendrogram(fd)
plot_curves(fd)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/join_count.R
\name{join_count}
\alias{join_count}
\title{Test for Spatial Join Count Statistics}
\usage{
join_count(matrix_data, verbose = TRUE)
}
\arguments{
\item{matrix_data}{A binary matrix (with elements 0 and 1) representing the spatial distribution
of two types of points: 0 for healthy plants (H) and 1 for diseased plants (D). This matrix reflects
the geographical distribution or layout of plants in the studied area.}

\item{verbose}{Logical. If TRUE (default), prints a formatted message to the console.}
}
\value{
A comprehensive, rich-text formatted string of results that includes:
\itemize{
\item Statistical counts of specific binary sequences (e.g., "01 or 10", "11")
\item Expected counts under the assumption of Complete Spatial Randomness (CSR)
\item Standard deviations and Z-scores (ZHD for "01 or 10" sequences, ZDD for "11" sequences)
\item Interpretation of whether the spatial distribution for each sequence type is
"Aggregated" or "Not Aggregated" based on Z-scores
\item A summary explaining the implications of these statistics and patterns
}
The return value aims to provide a clear understanding of the spatial arrangement's
characteristics, aiding in further spatial analysis or research.
}
\description{
The function \code{join_count} calculates spatial join count statistics for a binary matrix,
identifying patterns of aggregation or randomness.
}
\details{
The function conducts an analysis by first counting the occurrence of specific sequences
("01 or 10" and "11" - equivalent to HD and DD) in the binary matrix.
It then calculates expected values, standard
deviations, and Z-scores to determine the spatial randomness or aggregation. The analysis
considers both horizontal and vertical adjacency (rook case) in the matrix.
}
\references{
Madden, L. V., Hughes, G., & van den Bosch, F. (2007). The Study of Plant Disease Epidemics.
The American Phytopathological Society.
}
\seealso{
Other Spatial analysis: 
\code{\link{AFSD}()},
\code{\link{BPL}()},
\code{\link{count_subareas}()},
\code{\link{count_subareas_random}()},
\code{\link{fit_gradients}()},
\code{\link{oruns_test}()},
\code{\link{oruns_test_boustrophedon}()},
\code{\link{oruns_test_byrowcol}()},
\code{\link{plot_AFSD}()}
}
\concept{Spatial analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/windowpane.R
\name{windowpane}
\alias{windowpane}
\title{Window Pane for Epidemiological Analysis}
\usage{
windowpane(
  data,
  end_date_col,
  date_col,
  variable,
  summary_type,
  threshold = NULL,
  window_lengths,
  direction = "backward",
  group_by_cols = NULL,
  date_format = "\%Y-\%m-\%d"
)
}
\arguments{
\item{data}{A data frame containing the input data.}

\item{end_date_col}{A string specifying the name of the column representing the end date.}

\item{date_col}{A string specifying the name of the column representing the date variable.}

\item{variable}{A string specifying the name of the column for which summary statistics are calculated.}

\item{summary_type}{A string specifying the type of summary to calculate. Options are "mean", "sum",
"above_threshold", or "below_threshold".}

\item{threshold}{Optional numeric value used when \code{summary_type} is "above_threshold" or "below_threshold".}

\item{window_lengths}{A numeric vector specifying the window lengths (in days) for the calculations.}

\item{direction}{A string specifying the direction of the window. Options are "backward" (default),
"forward", or "both".}

\item{group_by_cols}{Optional vector of strings specifying column names for grouping the data.}

\item{date_format}{A string specifying the format of the date columns. Default is "\%Y-\%m-\%d".}
}
\value{
A data frame with the calculated summary values for each window.
}
\description{
This function calculates summary statistics within specified windows around a given end date
in a dataset, facilitating epidemiological analysis. It allows backward, forward, or both
directions of window calculations based on a user-defined variable and window lengths.
}
\seealso{
Other Disease modeling: 
\code{\link{get_brdwgd}()},
\code{\link{get_era5}()},
\code{\link{get_nasapower}()}
}
\concept{Disease modeling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_pca.R
\name{augment_functional_pca}
\alias{augment_functional_pca}
\title{Augment functional PCA}
\usage{
augment_functional_pca(x)
}
\arguments{
\item{x}{An object of class \code{"r4pde_functional_pca"}.}
}
\description{
Augment functional PCA
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_resistance.R
\name{print.functional_resistance}
\alias{print.functional_resistance}
\title{Print functional_resistance}
\usage{
\method{print}{functional_resistance}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_resistance"}.}

\item{...}{Additional arguments.}
}
\description{
Print functional_resistance
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/windowpane_tests.R
\name{windowpane_tests}
\alias{windowpane_tests}
\title{Windowpane Tests for Correlation Analysis}
\usage{
windowpane_tests(
  data,
  response_var,
  corr_type = "spearman",
  R = 1000,
  global_alpha = 0.05,
  individual_alpha = 0.005
)
}
\arguments{
\item{data}{A data frame containing the predictors and the response variable.}

\item{response_var}{A string representing the name of the response variable in the data frame.}

\item{corr_type}{A string specifying the correlation method to use; options are "spearman" (default),
"pearson", or "kendall".}

\item{R}{An integer indicating the number of bootstrap replications. Default is 1000.}

\item{global_alpha}{A numeric value representing the global alpha level for the Simes correction.
Default is 0.05.}

\item{individual_alpha}{A numeric value for the individual alpha threshold for testing individual predictors.
Default is 0.005.}
}
\value{
A list containing the following elements:
\item{results}{A data frame with columns: \code{variable}, \code{correlation}, \code{p_value},
\code{mean_corr}, \code{sd_corr}, \code{median_corr}, \code{rank}, \code{simes_threshold},
\code{significant_simes}, and \code{individual_significant}.}
\item{summary_table}{A data frame summarizing the global p-value (\emph{Pg}) and maximum correlation.}
\item{global_significant}{A logical value indicating whether the global test is significant.}
}
\description{
This function performs bootstrapped correlation analysis for multiple predictors against a response variable.
It applies the Simes method for global significance testing and calculates individual correlations,
p-values, and bootstrap statistics.
}
\details{
The function calculates correlations between the response variable and each predictor in the
data frame, using bootstrapping to generate mean, standard deviation, and median estimates of the
correlation. The Simes correction is applied to control for multiple testing, providing a global
p-value (\emph{Pg}). The function also returns the maximum observed correlation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_functional_suppression_profiles.R
\name{summary.functional_suppression_profiles}
\alias{summary.functional_suppression_profiles}
\title{Summarize functional suppression profiles}
\usage{
\method{summary}{functional_suppression_profiles}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"functional_suppression_profiles"}.}

\item{...}{Additional arguments.}
}
\value{
A list of class \code{"summary.functional_suppression_profiles"}.
}
\description{
Summarize functional suppression profiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_suppression_profiles.R
\name{functional_suppression_profiles}
\alias{functional_suppression_profiles}
\title{Functional suppression profiles of disease control treatments}
\usage{
functional_suppression_profiles(
  data,
  reference,
  time = "time",
  response = "severity",
  treatment = "treatment",
  environment = NULL,
  by_environment = FALSE,
  env_ref = NULL,
  threshold = 5,
  metrics = c("protected_area", "max_suppression", "persistence", "centroid"),
  dist_method = "euclidean",
  hclust_method = "average",
  k = "auto",
  cut_height = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame or tibble containing the disease progress data.}

\item{reference}{Character string specifying the reference treatment name.}

\item{time}{Character string specifying the time column. Default is \code{"time"}.}

\item{response}{Character string specifying the response column. Default is \code{"severity"}.}

\item{treatment}{Character string specifying the treatment column. Default is \code{"treatment"}.}

\item{environment}{Character string specifying the environment column (optional).}

\item{by_environment}{Logical; if \code{TRUE}, the analysis is run separately for each environment. Default is \code{FALSE}.}

\item{env_ref}{Character string specifying the reference environment for joint analysis (optional).}

\item{threshold}{Numeric threshold for the persistence metric. Default is \code{5}.}

\item{metrics}{Character vector of metrics to include in the ranking. Default is \code{c("protected_area", "max_suppression", "persistence", "centroid")}.}

\item{dist_method}{Character string specifying the distance method. Default is \code{"euclidean"}.}

\item{hclust_method}{Character string specifying the hierarchical clustering method. Default is \code{"average"}.}

\item{k}{Integer specifying the number of functional profiles to create, or \code{"auto"} (default). When \code{"auto"}, the optimal number of profiles is estimated by maximizing the average silhouette width.}

\item{cut_height}{Numeric specifying the cut height for clustering. If provided, overrides \code{k}.}

\item{...}{Additional arguments passed to \code{\link{functional_contrast}}.}
}
\value{
A list of class \code{"functional_suppression_profiles"} containing:
\itemize{
\item \code{call}: The matched call.
\item \code{reference}: The reference treatment name.
\item \code{contrast}: The contrast data from \code{functional_contrast}.
\item \code{dsp_curves}: The fitted curves from \code{functional_curves}.
\item \code{distances}: The distances object from \code{functional_distances}.
\item \code{hclust}: The hierarchical clustering object.
\item \code{clusters}: A tibble with treatment cluster assignments (internal).
\item \code{profiles}: A lightweight tibble mapping treatments to profiles.
\item \code{classification}: A tibble with the final classification combining profiles, mean rank, and functional metrics, ordered by mean rank.
\item \code{summary}: A tibble with functional summary metrics.
\item \code{ranking}: A tibble with metric rankings.
\item \code{cluster_summary}: A tibble combining metrics, ranks, and cluster assignments.
\item \code{parameters}: A list of input parameters.
}
}
\description{
Computes disease suppression trajectories relative to a reference treatment, estimates functional distances among suppression curves, clusters treatments according to their temporal suppression profiles, and summarizes each profile using functional suppression metrics.
}
\details{
A Functional Suppression Profile (FSP) is the temporal trajectory of disease
suppression produced by a treatment relative to an untreated or reference control.
Functional distances and hierarchical clustering are computed from smoothed disease
suppression trajectories rather than raw observed suppression values. Functional
summary metrics are subsequently used to interpret the resulting suppression profiles.

The workflow includes contrasting treatments against a reference, fitting functional
curves, calculating distances, clustering, and summarizing the suppression using
metrics like protected area, maximum suppression, and persistence.

Treatments belonging to the same functional suppression profile are displayed using
the same colour throughout all graphical summaries. This visual consistency facilitates
interpretation of the relationship between suppression trajectories, functional clustering,
and suppression metrics.

The \code{classification} table combines functional profile membership, suppression metrics,
and mean functional rank. Functional profiles are defined from distances among smoothed
suppression trajectories, whereas mean rank summarizes the overall performance across
functional suppression metrics.
}
\examples{
\dontrun{
sim_dat <- tibble::tibble(
  treatment = rep(c("Control", "A", "B", "C"), each = 6),
  time = rep(seq(0, 25, by = 5), times = 4),
  severity = c(
    c(5, 10, 20, 35, 50, 65),
    c(3, 5, 10, 18, 30, 40),
    c(4, 6, 12, 22, 35, 45),
    c(2, 4, 8, 14, 22, 32)
  )
)
fsp <- functional_suppression_profiles(
  data = sim_dat,
  reference = "Control",
  time = "time",
  response = "severity",
  treatment = "treatment",
  threshold = 5,
  k = 2
)

fsp
summary(fsp)
plot(fsp, type = "dendrogram")
plot(fsp, type = "profiles")
plot(fsp, type = "heatmap")
plot(fsp, type = "rank")
}
}
\seealso{
\code{\link{functional_contrast}}, \code{\link{functional_curves}}, \code{\link{functional_distances}}, \code{\link{functional_summary}}, \code{\link{rank_dsp}}, \code{\link{plot_dendrogram}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_functional_suppression_profiles_list.R
\name{print.functional_suppression_profiles_list}
\alias{print.functional_suppression_profiles_list}
\title{Print a list of functional suppression profiles}
\usage{
\method{print}{functional_suppression_profiles_list}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"functional_suppression_profiles_list"}.}

\item{...}{Additional arguments.}
}
\description{
Print a list of functional suppression profiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_functional_suppression_profiles_list.R
\name{summary.functional_suppression_profiles_list}
\alias{summary.functional_suppression_profiles_list}
\title{Summarize a list of functional suppression profiles}
\usage{
\method{summary}{functional_suppression_profiles_list}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"functional_suppression_profiles_list"}.}

\item{...}{Additional arguments.}
}
\value{
A list of summaries, one for each environment.
}
\description{
Summarize a list of functional suppression profiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_r4pde.R
\name{theme_r4pde}
\alias{theme_r4pde}
\title{Custom ggplot2 theme based on cowplot::theme_half_open}
\usage{
theme_r4pde(font_size = 16)
}
\arguments{
\item{font_size}{The base font size. Default is 16.}
}
\value{
A ggplot2 theme object.
}
\description{
This function creates a new ggplot2 theme by modifying the cowplot::theme_half_open theme.
It sets a custom font size and changes the panel background color to gray96.
}
\concept{Miscellaneous}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suggest_k.R
\name{print.suggest_k}
\alias{print.suggest_k}
\title{Print method for suggest_k}
\usage{
\method{print}{suggest_k}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"suggest_k"}.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for suggest_k
}
